
/****************************************************************************************************//**
 * @file     S1C31.h
 *
 * @brief    CMSIS Cortex-M0PLUS Peripheral Access Layer Header File for
 *           S1C31 device family from Epson.
 *
 * @version  V2.2
 * @date     01 June 2020
 *
 * @note     Defined in the preprocessor. Default is W73
 *    
 *
 * @par      Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
 *
 *******************************************************************************************************/
#ifndef SE_S1C31_H
#define SE_S1C31_H

#ifdef __cplusplus
 extern "C" {
#endif 
	 
#include "S1C31W73.h"

#define ADC_module_header      "se_adc12a.h"
#define CLG_module_header      "se_clg.h"    
#define DMAC_module_header     "se_dmac.h"      
#define I2C_module_header      "se_i2c.h" 
#define LCD32C_module_header   "se_lcd32c.h"          
#define PPORT_module_header    "se_pport.h"      
#define QSPI_module_header     "se_qspi.h"      
#define REMC_module_header     "se_remc3.h"  
#define RFC_module_header      "se_rfc.h"    
#define RTC_module_header      "se_rtca.h"      
#define SND_module_header      "se_snda.h"      
#define SPI_module_header      "se_spia.h"      
#define SVD_module_header      "se_svd4.h"      
#define T16_module_header      "se_t16.h"      
#define T16B_module_header     "se_t16b.h"      
#define TSRVR_module_header    "se_tsrvr.h"      
#define UART_module_header     "se_uart3.h"   
#define USB_module_header      "se_usb.h"      
#define WDT_module_header      "se_wdt2.h"  
#define DMAUSB_module_header   "se_dmausb.h"
#define DMASPI_module_header   "se_dmaspi.h"
#define DMAQSPI_module_header  "se_dmaqspi.h"
#define SRC_module_header  "se_src.h"

//#define RFC_OSCMODE   

#define USB_PORT_ID        	   sePPORT_P44
#define USB_PORT_IRQ           PPORT_IRQn
#define PPORT_USB_INTF         PPORT->P4INTF
#define PPORT_USB_INTF_MSK     PPORT_P4INTF_P4IF4_Msk

#define seSRC(A)               ((seCLG_ClkSrc)((A)>>4))
#define seDIV(A)               ((uint8_t)((A)&0x0F))
#define se32k768HzSupplied()   (CLG->OSC_b.OSC1EN==1)

/** @addtogroup sePeriphLibrary
  * @{
  */

/** @defgroup CLG_Constants 
  * @{
  * @brief  Clock source 
  */  
  
/** 
  * @brief  Available Clock sources
  */
typedef enum {
  seCLG_OSC1  = 1,              ///< OSC1 as a SYSCLK source.
  seCLG_IOSC  = 0,              ///< IOSC as a SYSCLK source.
  seCLG_OSC3  = 2,              ///< OSC3 as a SYSCLK source.
  seCLG_EXOSC = 3               ///< EXOSC as a SYSCLK source.
} seCLG_ClkSrc;
 
/** 
  * @brief  IOSC oscilation set
  */ 
typedef enum {  
  seCLG_IOSC_1MHZ  =  1000000UL,
  seCLG_IOSC_2MHZ  =  2000000UL,
  seCLG_IOSC_8MHZ  =  8000000UL,
  seCLG_IOSC_12MHZ = 12000000UL,
  seCLG_IOSC_16MHZ = 16000000UL,
  seCLG_IOSC_24MHZ = 24000000UL,
  seCLG_IOSC_32MHZ = 32000000UL
} seCLG_IOSC_Freq;

/**
  * @}
  */


/** @addtogroup CLG_Constants
  * @{
  * @brief  Clock source and divider options  
  */
typedef enum {
  seCLG_IOSC_CLKDIV_1        = (seCLG_IOSC<<4 |  0),       ///< IOSC division ratio of 1/1.
  seCLG_IOSC_CLKDIV_2        = (seCLG_IOSC<<4 |  1),       ///< IOSC division ratio of 1/2.
  seCLG_IOSC_CLKDIV_4        = (seCLG_IOSC<<4 |  2),       ///< IOSC division ratio of 1/4.
  seCLG_IOSC_CLKDIV_8        = (seCLG_IOSC<<4 |  3),       ///< IOSC division ratio of 1/8.

  seCLG_OSC1_CLKDIV_1        = (seCLG_OSC1<<4 |  0),       ///< OSC1 division ratio of 1/1.
  seCLG_OSC1_CLKDIV_2        = (seCLG_OSC1<<4 |  1),       ///< OSC1 division ratio of 1/2.

  seCLG_OSC3_CLKDIV_1        = (seCLG_OSC3<<4 |  0),       ///< OSC3 division ratio of 1/1.
  seCLG_OSC3_CLKDIV_2        = (seCLG_OSC3<<4 |  1),       ///< OSC3 division ratio of 1/2.
  seCLG_OSC3_CLKDIV_8        = (seCLG_OSC3<<4 |  2),       ///< OSC3 division ratio of 1/8.
  seCLG_OSC3_CLKDIV_16       = (seCLG_OSC3<<4 |  3),       ///< OSC3 division ratio of 1/16.

  seCLG_EXOSC_CLKDIV_1       = (seCLG_EXOSC<<4 | 0),       ///< EXOSC division ratio of 1/1.
} seCLG_Clk;

#define seDEFAULT_CLG_CLK   seCLG_IOSC_CLKDIV_1
/**
  * @}
  */


/** @addtogroup I2C_Constants
  * @{
  * @brief  I2C Clock source and divider options  
  */
typedef enum {
  seI2C_IOSC_CLKDIV_1       = (seCLG_IOSC<<4 |  0),        ///< IOSC division ratio is 1/1.
  seI2C_IOSC_CLKDIV_2       = (seCLG_IOSC<<4 |  1),        ///< IOSC division ratio is 1/2.
  seI2C_IOSC_CLKDIV_4       = (seCLG_IOSC<<4 |  2),        ///< IOSC division ratio is 1/4.
  seI2C_IOSC_CLKDIV_8       = (seCLG_IOSC<<4 |  3),        ///< IOSC division ratio is 1/8.

  seI2C_OSC1_CLKDIV_1       = (seCLG_OSC1<<4 |  0),        ///< OSC1 division ratio is 1/1.

  seI2C_OSC3_CLKDIV_1       = (seCLG_OSC3<<4 |  0),        ///< OSC3 division ratio is 1/1.
  seI2C_OSC3_CLKDIV_2       = (seCLG_OSC3<<4 |  1),        ///< OSC3 division ratio is 1/2.
  seI2C_OSC3_CLKDIV_4       = (seCLG_OSC3<<4 |  2),        ///< OSC3 division ratio is 1/4.
  seI2C_OSC3_CLKDIV_8       = (seCLG_OSC3<<4 |  3),        ///< OSC3 division ratio is 1/8.

  seI2C_EXOSC_CLKDIV_1      = (seCLG_EXOSC<<4 | 0),        ///< EXOSC division ratio is 1/1.
} seI2C_Clk;

#define seDEFAULT_I2C_CLK   seI2C_IOSC_CLKDIV_1
/**
  * @}
  */

/** @addtogroup LCD32C_Constants 
  * @{
  * @brief  LCD32C Clock source and divider options  
  */
typedef enum {
  seLCD32C_IOSC_CLKDIV_16   = (seCLG_IOSC<<4 |  0),                       ///< IOSC division ratio is 1/16.
  seLCD32C_IOSC_CLKDIV_32   = (seCLG_IOSC<<4 |  1),                       ///< IOSC division ratio is 1/32.
  seLCD32C_IOSC_CLKDIV_64   = (seCLG_IOSC<<4 |  2),                       ///<  IOSC division ratio is 1/64.
  seLCD32C_IOSC_CLKDIV_128  = (seCLG_IOSC<<4 |  3),                      ///< IOSC division ratio is 1/128.
  seLCD32C_IOSC_CLKDIV_256  = (seCLG_IOSC<<4 |  4),                      ///< IOSC division ratio is 1/256.
  seLCD32C_IOSC_CLKDIV_512  = (seCLG_IOSC<<4 |  5),                       ///< IOSC division ratio is 1/512.

  seLCD32C_OSC1_CLKDIV_1    = (seCLG_OSC1<<4 |  0),                      ///< OSC1 division ratio is 1/1.

  seLCD32C_OSC3_CLKDIV_16   = (seCLG_OSC3<<4 |  0),                       ///< OSC3 division ratio is 1/4.
  seLCD32C_OSC3_CLKDIV_32   = (seCLG_OSC3<<4 |  1),                       ///< OSC3 division ratio is 1/8.
  seLCD32C_OSC3_CLKDIV_64   = (seCLG_OSC3<<4 |  2),                      ///< OSC3 division ratio is 1/16.
  seLCD32C_OSC3_CLKDIV_128  = (seCLG_OSC3<<4 |  3),                      ///< OSC3 division ratio is 1/32.
  seLCD32C_OSC3_CLKDIV_256  = (seCLG_OSC3<<4 |  4),                      ///< OSC3 division ratio is 1/64.
  seLCD32C_OSC3_CLKDIV_512  = (seCLG_OSC3<<4 |  5),                      ///< OSC3 division ratio is 1/128.

  seLCD32C_EXOSC_CLKDIV_1   = (seCLG_EXOSC<<4 | 0),                      ///< EXOSC division ratio is 1/1.
} seLCD32C_Clk;

#define seDEFAULT_LCD32C_CLK   seLCD32C_IOSC_CLKDIV_16
/**
  * @}
  */
  
/** @addtogroup PPORT_Constants 
  * @{
  * @brief  PPORT Clock source and divider options  
  */
typedef enum {
  sePPORT_IOSC_CLKDIV_1     = (seCLG_IOSC<<4 |  0),        ///< IOSC division ratio is 1/1.
  sePPORT_IOSC_CLKDIV_2     = (seCLG_IOSC<<4 |  1),        ///< IOSC division ratio is 1/2.
  sePPORT_IOSC_CLKDIV_4     = (seCLG_IOSC<<4 |  2),        ///< IOSC division ratio is 1/4.
  sePPORT_IOSC_CLKDIV_8     = (seCLG_IOSC<<4 |  3),        ///< IOSC division ratio is 1/8.
  sePPORT_IOSC_CLKDIV_16    = (seCLG_IOSC<<4 |  4),        ///< IOSC division ratio is 1/16.
  sePPORT_IOSC_CLKDIV_32    = (seCLG_IOSC<<4 |  5),        ///< IOSC division ratio is 1/32.
  sePPORT_IOSC_CLKDIV_64    = (seCLG_IOSC<<4 |  6),        ///< IOSC division ratio is 1/64.
  sePPORT_IOSC_CLKDIV_128   = (seCLG_IOSC<<4 |  7),        ///< IOSC division ratio is 1/128.
  sePPORT_IOSC_CLKDIV_256   = (seCLG_IOSC<<4 |  8),        ///< IOSC division ratio is 1/256.
  sePPORT_IOSC_CLKDIV_512   = (seCLG_IOSC<<4 |  9),        ///< IOSC division ratio is 1/512.
  sePPORT_IOSC_CLKDIV_1024  = (seCLG_IOSC<<4 | 10),        ///< IOSC division ratio is 1/1024.
  sePPORT_IOSC_CLKDIV_2048  = (seCLG_IOSC<<4 | 11),        ///< IOSC division ratio is 1/2048.
  sePPORT_IOSC_CLKDIV_4096  = (seCLG_IOSC<<4 | 12),        ///< IOSC division ratio is 1/4096
  sePPORT_IOSC_CLKDIV_8192  = (seCLG_IOSC<<4 | 13),        ///< IOSC division ratio is 1/8192.
  sePPORT_IOSC_CLKDIV_16384 = (seCLG_IOSC<<4 | 14),        ///< IOSC division ratio is 1/16384.
  sePPORT_IOSC_CLKDIV_32768 = (seCLG_IOSC<<4 | 15),        ///< IOSC division ratio is 1/32768.

  sePPORT_OSC1_CLKDIV_1     = (seCLG_OSC1<<4 |  0),        ///< OSC1 division ratio is 1/1.
  sePPORT_OSC1_CLKDIV_2     = (seCLG_OSC1<<4 |  1),        ///< OSC1 division ratio is 1/2.
  sePPORT_OSC1_CLKDIV_4     = (seCLG_OSC1<<4 |  2),        ///< OSC1 division ratio is 1/4.
  sePPORT_OSC1_CLKDIV_8     = (seCLG_OSC1<<4 |  3),        ///< OSC1 division ratio is 1/8.
  sePPORT_OSC1_CLKDIV_16    = (seCLG_OSC1<<4 |  4),        ///< OSC1 division ratio is 1/16.
  sePPORT_OSC1_CLKDIV_32    = (seCLG_OSC1<<4 |  5),        ///< OSC1 division ratio is 1/32.
  sePPORT_OSC1_CLKDIV_64    = (seCLG_OSC1<<4 |  6),        ///< OSC1 division ratio is 1/64.
  sePPORT_OSC1_CLKDIV_128   = (seCLG_OSC1<<4 |  7),        ///< OSC1 division ratio is 1/128.
  sePPORT_OSC1_CLKDIV_256   = (seCLG_OSC1<<4 |  8),        ///< OSC1 division ratio is 1/256.
  sePPORT_OSC1_CLKDIV_512   = (seCLG_OSC1<<4 |  9),        ///< OSC1 division ratio is 1/512.
  sePPORT_OSC1_CLKDIV_1024  = (seCLG_OSC1<<4 | 10),        ///< OSC1 division ratio is 1/1024.
  sePPORT_OSC1_CLKDIV_2048  = (seCLG_OSC1<<4 | 11),        ///< OSC1 division ratio is 1/2048.
  sePPORT_OSC1_CLKDIV_4096  = (seCLG_OSC1<<4 | 12),        ///< OSC1 division ratio is 1/4096
  sePPORT_OSC1_CLKDIV_8192  = (seCLG_OSC1<<4 | 13),        ///< OSC3 division ratio is 1/8192.
  sePPORT_OSC1_CLKDIV_16384 = (seCLG_OSC1<<4 | 14),        ///< OSC3 division ratio is 1/16384.
  sePPORT_OSC1_CLKDIV_32768 = (seCLG_OSC1<<4 | 15),        ///< OSC3 division ratio is 1/32768.

  sePPORT_OSC3_CLKDIV_1     = (seCLG_OSC3<<4 |  0),        ///< OSC3 division ratio is 1/1.
  sePPORT_OSC3_CLKDIV_2     = (seCLG_OSC3<<4 |  1),        ///< OSC3 division ratio is 1/2.
  sePPORT_OSC3_CLKDIV_4     = (seCLG_OSC3<<4 |  2),        ///< OSC3 division ratio is 1/4.
  sePPORT_OSC3_CLKDIV_8     = (seCLG_OSC3<<4 |  3),        ///< OSC3 division ratio is 1/8.
  sePPORT_OSC3_CLKDIV_16    = (seCLG_OSC3<<4 |  4),        ///< OSC1 division ratio is 1/16.
  sePPORT_OSC3_CLKDIV_32    = (seCLG_OSC3<<4 |  5),        ///< OSC1 division ratio is 1/32.
  sePPORT_OSC3_CLKDIV_64    = (seCLG_OSC3<<4 |  6),        ///< OSC1 division ratio is 1/64.
  sePPORT_OSC3_CLKDIV_128   = (seCLG_OSC3<<4 |  7),        ///< OSC1 division ratio is 1/128.
  sePPORT_OSC3_CLKDIV_256   = (seCLG_OSC3<<4 |  8),        ///< OSC1 division ratio is 1/256.
  sePPORT_OSC3_CLKDIV_512   = (seCLG_OSC3<<4 |  9),        ///< OSC1 division ratio is 1/512.
  sePPORT_OSC3_CLKDIV_1024  = (seCLG_OSC3<<4 | 10),        ///< OSC1 division ratio is 1/1024.
  sePPORT_OSC3_CLKDIV_2048  = (seCLG_OSC3<<4 | 11),        ///< OSC1 division ratio is 1/2048.
  sePPORT_OSC3_CLKDIV_4096  = (seCLG_OSC3<<4 | 12),        ///< OSC1 division ratio is 1/4096
  sePPORT_OSC3_CLKDIV_8192  = (seCLG_OSC3<<4 | 13),        ///< OSC3 division ratio is 1/8192.
  sePPORT_OSC3_CLKDIV_16384 = (seCLG_OSC3<<4 | 14),        ///< OSC3 division ratio is 1/16384.
  sePPORT_OSC3_CLKDIV_32768 = (seCLG_OSC3<<4 | 15),        ///< OSC3 division ratio is 1/32768.

  sePPORT_EXOSC_CLKDIV_1    = (seCLG_EXOSC<<4 | 0),        ///< EXOSC division ratio is 1/1.
} sePPORT_Clk;

#define seDEFAULT_PPORT_CLK   sePPORT_IOSC_CLKDIV_1
/**
  * @}
  */

/** @addtogroup REMC3_Constants
  * @{
  * @brief  REMC3 Clock source and divider options  
  */
typedef enum {
  seREMC_IOSC_CLKDIV_1       = (seCLG_IOSC<<4 |  0),       ///< IOSC division ratio is 1/1.
  seREMC_IOSC_CLKDIV_2       = (seCLG_IOSC<<4 |  1),       ///< IOSC division ratio is 1/2.
  seREMC_IOSC_CLKDIV_4       = (seCLG_IOSC<<4 |  2),       ///< IOSC division ratio is 1/4.
  seREMC_IOSC_CLKDIV_8       = (seCLG_IOSC<<4 |  3),       ///< IOSC division ratio is 1/8.
  seREMC_IOSC_CLKDIV_16      = (seCLG_IOSC<<4 |  4),       ///< IOSC division ratio is 1/16.
  seREMC_IOSC_CLKDIV_32      = (seCLG_IOSC<<4 |  5),       ///< IOSC division ratio is 1/32.
  seREMC_IOSC_CLKDIV_64      = (seCLG_IOSC<<4 |  6),       ///< IOSC division ratio is 1/64.
  seREMC_IOSC_CLKDIV_128     = (seCLG_IOSC<<4 |  7),       ///< IOSC division ratio is 1/128.
  seREMC_IOSC_CLKDIV_256     = (seCLG_IOSC<<4 |  8),       ///< IOSC division ratio is 1/256.
  seREMC_IOSC_CLKDIV_512     = (seCLG_IOSC<<4 |  9),       ///< IOSC division ratio is 1/512.
  seREMC_IOSC_CLKDIV_1024    = (seCLG_IOSC<<4 | 10),       ///< IOSC division ratio is 1/1024.
  seREMC_IOSC_CLKDIV_2048    = (seCLG_IOSC<<4 | 11),       ///< IOSC division ratio is 1/2048.
  seREMC_IOSC_CLKDIV_4096    = (seCLG_IOSC<<4 | 12),       ///< IOSC division ratio is 1/4096
  seREMC_IOSC_CLKDIV_8192    = (seCLG_IOSC<<4 | 13),       ///< IOSC division ratio is 1/8192.
  seREMC_IOSC_CLKDIV_16384   = (seCLG_IOSC<<4 | 14),       ///< IOSC division ratio is 1/16384.
  seREMC_IOSC_CLKDIV_32768   = (seCLG_IOSC<<4 | 15),       ///< IOSC division ratio is 1/32768.

  seREMC_OSC1_CLKDIV_1       = (seCLG_OSC1<<4 |  0),       ///< OSC1 division ratio is 1/1.
  seREMC_OSC1_CLKDIV_2       = (seCLG_OSC1<<4 |  1),       ///< OSC1 division ratio is 1/2.
  seREMC_OSC1_CLKDIV_4       = (seCLG_OSC1<<4 |  2),       ///< OSC1 division ratio is 1/4.
  seREMC_OSC1_CLKDIV_8       = (seCLG_OSC1<<4 |  3),       ///< OSC1 division ratio is 1/8.
  seREMC_OSC1_CLKDIV_16      = (seCLG_OSC1<<4 |  4),       ///< OSC1 division ratio is 1/16.
  seREMC_OSC1_CLKDIV_32      = (seCLG_OSC1<<4 |  5),       ///< OSC1 division ratio is 1/32.
  seREMC_OSC1_CLKDIV_64      = (seCLG_OSC1<<4 |  6),       ///< OSC1 division ratio is 1/64.
  seREMC_OSC1_CLKDIV_128     = (seCLG_OSC1<<4 |  7),       ///< OSC1 division ratio is 1/128.
  seREMC_OSC1_CLKDIV_256     = (seCLG_OSC1<<4 |  8),       ///< OSC1 division ratio is 1/256.

  seREMC_OSC3_CLKDIV_1       = (seCLG_OSC3<<4 |  0),       ///< OSC3 division ratio is 1/1.
  seREMC_OSC3_CLKDIV_2       = (seCLG_OSC3<<4 |  1),       ///< OSC3 division ratio is 1/2.
  seREMC_OSC3_CLKDIV_4       = (seCLG_OSC3<<4 |  2),       ///< OSC3 division ratio is 1/4.
  seREMC_OSC3_CLKDIV_8       = (seCLG_OSC3<<4 |  3),       ///< OSC3 division ratio is 1/8.
  seREMC_OSC3_CLKDIV_16      = (seCLG_OSC3<<4 |  4),       ///< OSC1 division ratio is 1/16.
  seREMC_OSC3_CLKDIV_32      = (seCLG_OSC3<<4 |  5),       ///< OSC1 division ratio is 1/32.
  seREMC_OSC3_CLKDIV_64      = (seCLG_OSC3<<4 |  6),       ///< OSC1 division ratio is 1/64.
  seREMC_OSC3_CLKDIV_128     = (seCLG_OSC3<<4 |  7),       ///< OSC1 division ratio is 1/128.
  seREMC_OSC3_CLKDIV_256     = (seCLG_OSC3<<4 |  8),       ///< OSC1 division ratio is 1/256.
  seREMC_OSC3_CLKDIV_512     = (seCLG_OSC3<<4 |  9),       ///< OSC1 division ratio is 1/512.
  seREMC_OSC3_CLKDIV_1024    = (seCLG_OSC3<<4 | 10),       ///< OSC1 division ratio is 1/1024.
  seREMC_OSC3_CLKDIV_2048    = (seCLG_OSC3<<4 | 11),       ///< OSC1 division ratio is 1/2048.
  seREMC_OSC3_CLKDIV_4096    = (seCLG_OSC3<<4 | 12),       ///< OSC1 division ratio is 1/4096
  seREMC_OSC3_CLKDIV_8192    = (seCLG_OSC3<<4 | 13),       ///< OSC3 division ratio is 1/8192.
  seREMC_OSC3_CLKDIV_16384   = (seCLG_OSC3<<4 | 14),       ///< OSC3 division ratio is 1/16384.
  seREMC_OSC3_CLKDIV_32768   = (seCLG_OSC3<<4 | 15),       ///< OSC3 division ratio is 1/32768.

  seREMC_EXOSC_CLKDIV_1      = (seCLG_EXOSC<<4 | 0)        ///< EXOSC division ratio is 1/1.
} seREMC_Clk;

#define seDEFAULT_REMC_CLK   seREMC_IOSC_CLKDIV_1
/**
  * @}
  */

/** @addtogroup RFC_Constants 
  * @{
  * @brief  RFC Clock source and divider options  
  */
typedef enum {
  seRFC_IOSC_CLKDIV_1       = (seCLG_IOSC<<4 |  0),        ///< IOSC division ratio is 1/1.
  seRFC_IOSC_CLKDIV_2       = (seCLG_IOSC<<4 |  1),        ///< IOSC division ratio is 1/2.
  seRFC_IOSC_CLKDIV_4       = (seCLG_IOSC<<4 |  2),        ///< IOSC division ratio is 1/4.
  seRFC_IOSC_CLKDIV_8       = (seCLG_IOSC<<4 |  3),        ///< IOSC division ratio is 1/8.

  seRFC_OSC1_CLKDIV_1       = (seCLG_OSC1<<4 |  0),        ///< OSC1 division ratio is 1/1.

  seRFC_OSC3_CLKDIV_1       = (seCLG_OSC3<<4 |  0),        ///< OSC3 division ratio is 1/1.
  seRFC_OSC3_CLKDIV_2       = (seCLG_OSC3<<4 |  1),        ///< OSC3 division ratio is 1/2.
  seRFC_OSC3_CLKDIV_4       = (seCLG_OSC3<<4 |  2),        ///< OSC3 division ratio is 1/4.
  seRFC_OSC3_CLKDIV_8       = (seCLG_OSC3<<4 |  3),        ///< OSC3 division ratio is 1/8.

  seRFC_EXOSC_CLKDIV_1      = (seCLG_EXOSC<<4 | 0)         ///< EXOSC division ratio is 1/1.
} seRFC_Clk;

#define seDEFAULT_RFC_CLK   seRFC_IOSC_CLKDIV_1
/**
  * @}
  */

/** @addtogroup SNDA_Constants
  * @{
  * @brief  SNDA Clock source and divider options  
  */

typedef enum {
  seSNDA_IOSC_CLKDIV_16     = (seCLG_IOSC<<4 |  0),           ///< IOSC division ratio is 1/16.
  seSNDA_IOSC_CLKDIV_32     = (seCLG_IOSC<<4 |  1),           ///< IOSC division ratio is 1/32.
  seSNDA_IOSC_CLKDIV_64     = (seCLG_IOSC<<4 |  2),           ///< IOSC division ratio is 1/64.
  seSNDA_IOSC_CLKDIV_128    = (seCLG_IOSC<<4 |  3),           ///< IOSC division ratio is 1/128.
  seSNDA_IOSC_CLKDIV_256    = (seCLG_IOSC<<4 |  4),           ///< IOSC division ratio is 1/256.
  seSNDA_IOSC_CLKDIV_512    = (seCLG_IOSC<<4 |  5),           ///< IOSC division ratio is 1/512.
  
  seSNDA_OSC1_CLKDIV_1      = (seCLG_OSC1<<4 |  0),           ///< OSC1 division ratio is 1/1.
  
  seSNDA_OSC3_CLKDIV_16     = (seCLG_OSC3<<4 |  0),           ///< OSC3 division ratio is 1/16.
  seSNDA_OSC3_CLKDIV_32     = (seCLG_OSC3<<4 |  1),           ///< OSC3 division ratio is 1/32.
  seSNDA_OSC3_CLKDIV_64     = (seCLG_OSC3<<4 |  2),           ///< OSC3 division ratio is 1/64.
  seSNDA_OSC3_CLKDIV_128    = (seCLG_OSC3<<4 |  3),           ///< OSC3 division ratio is 1/128.
  seSNDA_OSC3_CLKDIV_256    = (seCLG_OSC3<<4 |  4),           ///< OSC3 division ratio is 1/256.
  seSNDA_OSC3_CLKDIV_512    = (seCLG_OSC3<<4 |  5),           ///< OSC3 division ratio is 1/512.
  
  seSNDA_EXOSC_CLKDIV_1     = (seCLG_EXOSC<<4 | 0),           ///< EXOSC division ratio is 1/1.
  
} seSNDA_Clk;

#define seDEFAULT_SNDA_CLK   seSNDA_IOSC_CLKDIV_16
/**
  * @}
  */
  
/** @addtogroup SVD4_Constants
  * @{
  * @brief  SVD4 Clock source and divider options  
  */
typedef enum {
  seSVD_IOSC_CLKDIV_16       = (seCLG_IOSC<<4 |  0),       ///< IOSC division ratio is 1/16.
  seSVD_IOSC_CLKDIV_32       = (seCLG_IOSC<<4 |  1),       ///< IOSC division ratio is 1/32.
  seSVD_IOSC_CLKDIV_64       = (seCLG_IOSC<<4 |  2),       ///< IOSC division ratio is 1/64.
  seSVD_IOSC_CLKDIV_128      = (seCLG_IOSC<<4 |  3),       ///< IOSC division ratio is 1/128.
  seSVD_IOSC_CLKDIV_256      = (seCLG_IOSC<<4 |  4),       ///< IOSC division ratio is 1/256.
  seSVD_IOSC_CLKDIV_512      = (seCLG_IOSC<<4 |  5),       ///< IOSC division ratio is 1/512.
  seSVD_IOSC_CLKDIV_1024     = (seCLG_IOSC<<4 |  6),       ///< IOSC division ratio is 1/1024.

  seSVD_OSC1_CLKDIV_1        = (seCLG_OSC1<<4 |  0),       ///< OSC1 division ratio is 1/1.

  seSVD_OSC3_CLKDIV_16       = (seCLG_OSC3<<4 |  0),       ///< OSC3 division ratio is 1/16.
  seSVD_OSC3_CLKDIV_32       = (seCLG_OSC3<<4 |  1),       ///< OSC3 division ratio is 1/32.
  seSVD_OSC3_CLKDIV_64       = (seCLG_OSC3<<4 |  2),       ///< OSC3 division ratio is 1/64.
  seSVD_OSC3_CLKDIV_128      = (seCLG_OSC3<<4 |  3),       ///< OSC3 division ratio is 1/128.  
  seSVD_OSC3_CLKDIV_256      = (seCLG_OSC3<<4 |  4),       ///< OSC3 division ratio is 1/256.
  seSVD_OSC3_CLKDIV_512      = (seCLG_OSC3<<4 |  5),       ///< OSC3 division ratio is 1/512.
  seSVD_OSC3_CLKDIV_1024      = (seCLG_OSC3<<4 |  6),      ///< OSC3 division ratio is 1/1024.

  seSVD_EXOSC_CLKDIV_1       = (seCLG_EXOSC<<4 | 0)        ///< EXOSC division ratio is 1/1.
} seSVD_Clk;

#define seDEFAULT_SVD_CLK   seSVD_IOSC_CLKDIV_16
/**
  * @}
  */

/** @addtogroup T16_Constants 
  * @{
  * @brief  T16 Clock source and divider options  
  */
typedef enum {
  seT16_IOSC_CLKDIV_1       = (seCLG_IOSC<<4 |  0),        ///< IOSC division ratio is 1/1.
  seT16_IOSC_CLKDIV_2       = (seCLG_IOSC<<4 |  1),        ///< IOSC division ratio is 1/2.
  seT16_IOSC_CLKDIV_4       = (seCLG_IOSC<<4 |  2),        ///< IOSC division ratio is 1/4.
  seT16_IOSC_CLKDIV_8       = (seCLG_IOSC<<4 |  3),        ///< IOSC division ratio is 1/8.
  seT16_IOSC_CLKDIV_16      = (seCLG_IOSC<<4 |  4),        ///< IOSC division ratio is 1/16.
  seT16_IOSC_CLKDIV_32      = (seCLG_IOSC<<4 |  5),        ///< IOSC division ratio is 1/32.
  seT16_IOSC_CLKDIV_64      = (seCLG_IOSC<<4 |  6),        ///< IOSC division ratio is 1/64.
  seT16_IOSC_CLKDIV_128     = (seCLG_IOSC<<4 |  7),        ///< IOSC division ratio is 1/128.
  seT16_IOSC_CLKDIV_256     = (seCLG_IOSC<<4 |  8),        ///< IOSC division ratio is 1/256.
  seT16_IOSC_CLKDIV_512     = (seCLG_IOSC<<4 |  9),        ///< IOSC division ratio is 1/512.
  seT16_IOSC_CLKDIV_1024    = (seCLG_IOSC<<4 | 10),        ///< IOSC division ratio is 1/1024.
  seT16_IOSC_CLKDIV_2048    = (seCLG_IOSC<<4 | 11),        ///< IOSC division ratio is 1/2048.
  seT16_IOSC_CLKDIV_4096    = (seCLG_IOSC<<4 | 12),        ///< IOSC division ratio is 1/4096.
  seT16_IOSC_CLKDIV_8192    = (seCLG_IOSC<<4 | 13),        ///< IOSC division ratio is 1/8192.
  seT16_IOSC_CLKDIV_16384   = (seCLG_IOSC<<4 | 14),        ///< IOSC division ratio is 1/16384.
  seT16_IOSC_CLKDIV_32768   = (seCLG_IOSC<<4 | 15),        ///< IOSC division ratio is 1/32768.

  seT16_OSC1_CLKDIV_1       = (seCLG_OSC1<<4 |  0),        ///< OSC1 division ratio is 1/1.
  seT16_OSC1_CLKDIV_2       = (seCLG_OSC1<<4 |  1),        ///< OSC1 division ratio is 1/2.
  seT16_OSC1_CLKDIV_4       = (seCLG_OSC1<<4 |  2),        ///< OSC1 division ratio is 1/4.
  seT16_OSC1_CLKDIV_8       = (seCLG_OSC1<<4 |  3),        ///< OSC1 division ratio is 1/8.
  seT16_OSC1_CLKDIV_16      = (seCLG_OSC1<<4 |  4),        ///< OSC1 division ratio is 1/16.
  seT16_OSC1_CLKDIV_32      = (seCLG_OSC1<<4 |  5),        ///< OSC1 division ratio is 1/32.
  seT16_OSC1_CLKDIV_64      = (seCLG_OSC1<<4 |  6),        ///< OSC1 division ratio is 1/64.
  seT16_OSC1_CLKDIV_128     = (seCLG_OSC1<<4 |  7),        ///< OSC1 division ratio is 1/128.
  seT16_OSC1_CLKDIV_256     = (seCLG_OSC1<<4 |  8),        ///< OSC1 division ratio is 1/256.

  seT16_OSC3_CLKDIV_1       = (seCLG_OSC3<<4 |  0),        ///< OSC3 division ratio is 1/1.
  seT16_OSC3_CLKDIV_2       = (seCLG_OSC3<<4 |  1),        ///< OSC3 division ratio is 1/2.
  seT16_OSC3_CLKDIV_4       = (seCLG_OSC3<<4 |  2),        ///< OSC3 division ratio is 1/4.
  seT16_OSC3_CLKDIV_8       = (seCLG_OSC3<<4 |  3),        ///< OSC3 division ratio is 1/8.
  seT16_OSC3_CLKDIV_16      = (seCLG_OSC3<<4 |  4),        ///< OSC3 division ratio is 1/16.
  seT16_OSC3_CLKDIV_32      = (seCLG_OSC3<<4 |  5),        ///< OSC3 division ratio is 1/32.
  seT16_OSC3_CLKDIV_64      = (seCLG_OSC3<<4 |  6),        ///< OSC3 division ratio is 1/64.
  seT16_OSC3_CLKDIV_128     = (seCLG_OSC3<<4 |  7),        ///< OSC3 division ratio is 1/128.
  seT16_OSC3_CLKDIV_256     = (seCLG_OSC3<<4 |  8),        ///< OSC3 division ratio is 1/256.
  seT16_OSC3_CLKDIV_512     = (seCLG_OSC3<<4 |  9),        ///< OSC3 division ratio is 1/512.
  seT16_OSC3_CLKDIV_1024    = (seCLG_OSC3<<4 | 10),        ///< OSC3 division ratio is 1/1024.
  seT16_OSC3_CLKDIV_2048    = (seCLG_OSC3<<4 | 11),        ///< OSC3 division ratio is 1/2048.
  seT16_OSC3_CLKDIV_4096    = (seCLG_OSC3<<4 | 12),        ///< OSC3 division ratio is 1/4096.
  seT16_OSC3_CLKDIV_8192    = (seCLG_OSC3<<4 | 13),        ///< OSC3 division ratio is 1/8192.
  seT16_OSC3_CLKDIV_16384   = (seCLG_OSC3<<4 | 14),        ///< OSC3 division ratio is 1/16384.
  seT16_OSC3_CLKDIV_32768   = (seCLG_OSC3<<4 | 15),        ///< OSC3 division ratio is 1/32768.

  seT16_EXOSC_CLKDIV_1      = (seCLG_EXOSC<<4 | 0)         ///< EXOSC division ratio is 1/1.
} seT16_Clk;

#define seDEFAULT_T16_CLK   seT16_IOSC_CLKDIV_1
/**
  * @}
  */

/** @addtogroup ADC12A_Constants
  * @{
  * @brief  ADC12A Clock source and divider options  
  */
typedef seT16_Clk seADC12A_Clk;

#define seDEFAULT_ADC12A_CLK   seDEFAULT_T16_CLK
/**
  * @}
  */  
  
/** @addtogroup T16B_Constants
  * @{
  * @brief  T16B Clock source and divider options  
  */
typedef enum {

  seT16B_IOSC_CLKDIV_1       = (seCLG_IOSC<<4 |  0),       ///< IOSC division ratio is 1/1.
  seT16B_IOSC_CLKDIV_2       = (seCLG_IOSC<<4 |  1),       ///< IOSC division ratio is 1/2.
  seT16B_IOSC_CLKDIV_4       = (seCLG_IOSC<<4 |  2),       ///< IOSC division ratio is 1/4.
  seT16B_IOSC_CLKDIV_8       = (seCLG_IOSC<<4 |  3),       ///< IOSC division ratio is 1/8.
  seT16B_IOSC_CLKDIV_16      = (seCLG_IOSC<<4 |  4),       ///< IOSC division ratio is 1/16.
  seT16B_IOSC_CLKDIV_32      = (seCLG_IOSC<<4 |  5),       ///< IOSC division ratio is 1/32.
  seT16B_IOSC_CLKDIV_64      = (seCLG_IOSC<<4 |  6),       ///< IOSC division ratio is 1/64.
  seT16B_IOSC_CLKDIV_128     = (seCLG_IOSC<<4 |  7),       ///< IOSC division ratio is 1/128.
  seT16B_IOSC_CLKDIV_256     = (seCLG_IOSC<<4 |  8),       ///< IOSC division ratio is 1/256.
  seT16B_IOSC_CLKDIV_512     = (seCLG_IOSC<<4 |  9),       ///< IOSC division ratio is 1/512.
  seT16B_IOSC_CLKDIV_1024    = (seCLG_IOSC<<4 | 10),       ///< IOSC division ratio is 1/1024.
  seT16B_IOSC_CLKDIV_2048    = (seCLG_IOSC<<4 | 11),       ///< IOSC division ratio is 1/2048.
  seT16B_IOSC_CLKDIV_4096    = (seCLG_IOSC<<4 | 12),       ///< IOSC division ratio is 1/4096.
  seT16B_IOSC_CLKDIV_8192    = (seCLG_IOSC<<4 | 13),       ///< IOSC division ratio is 1/8192.
  seT16B_IOSC_CLKDIV_16384   = (seCLG_IOSC<<4 | 14),       ///< IOSC division ratio is 1/16384.
  seT16B_IOSC_CLKDIV_32768   = (seCLG_IOSC<<4 | 15),       ///< IOSC division ratio is 1/32768.

  seT16B_OSC1_CLKDIV_1       = (seCLG_OSC1<<4 |  0),       ///< OSC1 division ratio is 1/1.
  seT16B_OSC1_CLKDIV_2       = (seCLG_OSC1<<4 |  1),       ///< OSC1 division ratio is 1/2.
  seT16B_OSC1_CLKDIV_4       = (seCLG_OSC1<<4 |  2),       ///< OSC1 division ratio is 1/4.
  seT16B_OSC1_CLKDIV_8       = (seCLG_OSC1<<4 |  3),       ///< OSC1 division ratio is 1/8.
  seT16B_OSC1_CLKDIV_16      = (seCLG_OSC1<<4 |  4),       ///< OSC1 division ratio is 1/16.
  seT16B_OSC1_CLKDIV_32      = (seCLG_OSC1<<4 |  5),       ///< OSC1 division ratio is 1/32.
  seT16B_OSC1_CLKDIV_64      = (seCLG_OSC1<<4 |  6),       ///< OSC1 division ratio is 1/64.
  seT16B_OSC1_CLKDIV_128     = (seCLG_OSC1<<4 |  7),       ///< OSC1 division ratio is 1/128.
  seT16B_OSC1_CLKDIV_256     = (seCLG_OSC1<<4 |  8),       ///< OSC1 division ratio is 1/256.

  seT16B_OSC3_CLKDIV_1       = (seCLG_OSC3<<4 |  0),       ///< OSC3 division ratio is 1/1.
  seT16B_OSC3_CLKDIV_2       = (seCLG_OSC3<<4 |  1),       ///< OSC3 division ratio is 1/2.
  seT16B_OSC3_CLKDIV_4       = (seCLG_OSC3<<4 |  2),       ///< OSC3 division ratio is 1/4.
  seT16B_OSC3_CLKDIV_8       = (seCLG_OSC3<<4 |  3),       ///< OSC3 division ratio is 1/8.
  seT16B_OSC3_CLKDIV_16      = (seCLG_OSC3<<4 |  4),       ///< OSC3 division ratio is 1/16.
  seT16B_OSC3_CLKDIV_32      = (seCLG_OSC3<<4 |  5),       ///< OSC3 division ratio is 1/32.
  seT16B_OSC3_CLKDIV_64      = (seCLG_OSC3<<4 |  6),       ///< OSC3 division ratio is 1/64.
  seT16B_OSC3_CLKDIV_128     = (seCLG_OSC3<<4 |  7),       ///< OSC3 division ratio is 1/128.
  seT16B_OSC3_CLKDIV_256     = (seCLG_OSC3<<4 |  8),       ///< OSC3 division ratio is 1/256.
  seT16B_OSC3_CLKDIV_512     = (seCLG_OSC3<<4 |  9),       ///< OSC3 division ratio is 1/512.
  seT16B_OSC3_CLKDIV_1024    = (seCLG_OSC3<<4 | 10),       ///< OSC3 division ratio is 1/1024.
  seT16B_OSC3_CLKDIV_2048    = (seCLG_OSC3<<4 | 11),       ///< OSC3 division ratio is 1/2048.
  seT16B_OSC3_CLKDIV_4096    = (seCLG_OSC3<<4 | 12),       ///< OSC3 division ratio is 1/4096.
  seT16B_OSC3_CLKDIV_8192    = (seCLG_OSC3<<4 | 13),       ///< OSC3 division ratio is 1/8192.
  seT16B_OSC3_CLKDIV_16384   = (seCLG_OSC3<<4 | 14),       ///< OSC3 division ratio is 1/16384.
  seT16B_OSC3_CLKDIV_32768   = (seCLG_OSC3<<4 | 15),       ///< OSC3 division ratio is 1/32768.
  
  seT16B_EXOSC_CLKDIV_1      = (seCLG_EXOSC<<4 | 0),       ///< EXOSC division ratio is 1/1.
  seT16B_EXCLN0_CLKDIV_1     = (4<<4           | 0),       ///< EXCLN0 division ratio is 1/1.
  seT16B_EXCLN1_CLKDIV_1     = (5<<4           | 0),       ///< EXCLN1 division ratio is 1/1.
  seT16B_EXCLN0_INV_CLKDIV_1 = (6<<4           | 0),       ///< EXCLN0_INV division ratio is 1/1.
  seT16B_EXCLN1_INV_CLKDIV_1 = (7<<4           | 0),       ///< EXCLN1_INV division ratio is 1/1.
} seT16B_Clk;

#define seDEFAULT_T16B_CLK   seT16B_IOSC_CLKDIV_1
/**
  * @}
  */

/** @addtogroup UART3_Constants
  * @{
  * @brief  UART3 Clock source and divider options  
  */
typedef enum {
  seUART_IOSC_CLKDIV_1       = (seCLG_IOSC<<4 |  0),       ///< IOSC division ratio is 1/1.
  seUART_IOSC_CLKDIV_2       = (seCLG_IOSC<<4 |  1),       ///< IOSC division ratio is 1/2.
  seUART_IOSC_CLKDIV_4       = (seCLG_IOSC<<4 |  2),       ///< IOSC division ratio is 1/4.
  seUART_IOSC_CLKDIV_8       = (seCLG_IOSC<<4 |  3),       ///< IOSC division ratio is 1/8.

  seUART_OSC1_CLKDIV_1       = (seCLG_OSC1<<4 |  0),       ///< OSC1 division ratio is 1/1.

  seUART_OSC3_CLKDIV_1       = (seCLG_OSC3<<4 |  0),       ///< OSC3 division ratio is 1/1.
  seUART_OSC3_CLKDIV_2       = (seCLG_OSC3<<4 |  1),       ///< OSC3 division ratio is 1/2.
  seUART_OSC3_CLKDIV_4       = (seCLG_OSC3<<4 |  2),       ///< OSC3 division ratio is 1/4.
  seUART_OSC3_CLKDIV_8       = (seCLG_OSC3<<4 |  3),       ///< OSC3 division ratio is 1/8.

  seUART_EXOSC_CLKDIV_1      = (seCLG_EXOSC<<4 | 0)        ///< EXOSC division ratio is 1/1.
} seUART_Clk;

#define seDEFAULT_UART_CLK   seUART_IOSC_CLKDIV_1
/**
  * @}
  */
 
/** @addtogroup USB_Constants
  * @{
  * @brief  USB Clock source and divider options  
  */
typedef enum {
  seUSB_PLL                  = 0,                          ///< PLL clock input
} seUSB_ClkSrc;

#define seDEFAULT_USB_CLK     seUSB_PLL
/**
  * @}
  */
 
/** @addtogroup WDT2_Constants
  * @{
  * @brief  WDT2 Clock source and divider options  
  */
typedef enum {
  seWDT_IOSC_CLKDIV_8192     = (seCLG_IOSC<<4 |  0),       ///< IOSC division ratio is 1/8192.
  seWDT_IOSC_CLKDIV_16384    = (seCLG_IOSC<<4 |  1),       ///< IOSC division ratio is 1/16384.
  seWDT_IOSC_CLKDIV_32768    = (seCLG_IOSC<<4 |  2),       ///< IOSC division ratio is 1/32768.
  seWDT_IOSC_CLKDIV_65536    = (seCLG_IOSC<<4 |  3),       ///< IOSC division ratio is 1/65536.

  seWDT_OSC1_CLKDIV_128      = (seCLG_OSC1<<4 |  0),       ///< OSC1 division ratio is 1/128.

  seWDT_OSC3_CLKDIV_8192     = (seCLG_OSC3<<4 |  0),       ///< OSC3 division ratio is 1/8192.
  seWDT_OSC3_CLKDIV_16384    = (seCLG_OSC3<<4 |  1),       ///< OSC3 division ratio is 1/16384.
  seWDT_OSC3_CLKDIV_32768    = (seCLG_OSC3<<4 |  2),       ///< OSC3 division ratio is 1/32768.
  seWDT_OSC3_CLKDIV_65536    = (seCLG_OSC3<<4 |  3),       ///< OSC3 division ratio is 1/65536.

  seWDT_EXOSC_CLKDIV_1       = (seCLG_EXOSC<<4 | 0)        ///< EXOSC division ratio is 1/1.
} seWDT_Clk;

#define seDEFAULT_WDT_CLK    seWDT_IOSC_CLKDIV_8192
/**
  * @}
  */

/** @addtogroup PPORT_Constants
  * @{
  * @brief  PPORT options
  */
typedef enum {
  sePPORT_P00 = 0,                      ///< P00
  sePPORT_P01 = 1,                      ///< P01
  sePPORT_P02 = 2,                      ///< P02
  sePPORT_P03 = 3,                      ///< P03
  sePPORT_P04 = 4,                      ///< P04
  sePPORT_P05 = 5,                      ///< P05
  sePPORT_P06 = 6,                      ///< P06
  sePPORT_P07 = 7,                      ///< P07
  sePPORT_P10 = 8,                      ///< P10
  sePPORT_P11 = 9,                      ///< P11
  sePPORT_P12 = 10,                     ///< P12
  sePPORT_P13 = 11,                     ///< P13
  sePPORT_P14 = 12,                     ///< P14
  sePPORT_P15 = 13,                     ///< P15
  sePPORT_P16 = 14,                     ///< P16
  sePPORT_P17 = 15,                     ///< P17
  sePPORT_P20 = 16,                     ///< P20
  sePPORT_P21 = 17,                     ///< P21
  sePPORT_P22 = 18,                     ///< P22
  sePPORT_P23 = 19,                     ///< P23
  sePPORT_P24 = 20,                     ///< P24
  sePPORT_P25 = 21,                     ///< P25
  sePPORT_P26 = 22,                     ///< P26
  sePPORT_P27 = 23,                     ///< P27
  sePPORT_P30 = 24,                     ///< P30
  sePPORT_P31 = 25,                     ///< P31
  sePPORT_P32 = 26,                     ///< P32
  sePPORT_P33 = 27,                     ///< P33
  sePPORT_P34 = 28,                     ///< P34
  sePPORT_P35 = 29,                     ///< P35
  sePPORT_P36 = 30,                     ///< P36
  sePPORT_P37 = 31,                     ///< P37
  sePPORT_P40 = 32,                     ///< P40
  sePPORT_P41 = 33,                     ///< P41
  sePPORT_P42 = 34,                     ///< P42
  sePPORT_P43 = 35,                     ///< P43
  sePPORT_P44 = 36,                     ///< P44
  //sePPORT_P45 = 37,                     ///< P45
  //sePPORT_P46 = 38,                     ///< P46
  //sePPORT_P47 = 39,                     ///< P47
  sePPORT_P50 = 40,                     ///< P50
  sePPORT_P51 = 41,                     ///< P51
  sePPORT_P52 = 42,                     ///< P52
  sePPORT_P53 = 43,                     ///< P53
  sePPORT_P54 = 44,                     ///< P54
  sePPORT_P55 = 45,                     ///< P55
  sePPORT_P56 = 46,                     ///< P56
  sePPORT_P57 = 47,                     ///< P57
  sePPORT_P60 = 48,                     ///< P60
  sePPORT_P61 = 49,                     ///< P61
  sePPORT_P62 = 50,                     ///< P62
  sePPORT_P63 = 51,                     ///< P63
  sePPORT_P64 = 52,                     ///< P64
  sePPORT_P65 = 53,                     ///< P65
  sePPORT_P66 = 54,                     ///< P66
  sePPORT_P67 = 55,                     ///< P67
  sePPORT_P70 = 56,                     ///< P70
  sePPORT_P71 = 57,                     ///< P71
  sePPORT_P72 = 58,                     ///< P72
  sePPORT_P73 = 59,                     ///< P73
  sePPORT_P74 = 60,                     ///< P74
  sePPORT_P75 = 61,                     ///< P75
  sePPORT_P76 = 62,                     ///< P76
  sePPORT_P77 = 63,                     ///< P77
  sePPORT_P80 = 64,                     ///< P80
  sePPORT_P81 = 65,                     ///< P81
  sePPORT_P82 = 66,                     ///< P82
  sePPORT_P83 = 67,                     ///< P83
  sePPORT_P84 = 68,                     ///< P84
  sePPORT_P85 = 69,                     ///< P85
  sePPORT_P86 = 70,                     ///< P86 
  sePPORT_P87 = 71,                     ///< P87
  //sePPORT_P90 = 72,                     ///< P90
  sePPORT_Pd0 = 104,                    ///< Pd0
  sePPORT_Pd1 = 105,                    ///< Pd1
  sePPORT_Pd2 = 106,                    ///< Pd2
  sePPORT_Pd3 = 107,                    ///< Pd3
  
  sePPORT_Designated = 254,    ///<  Initialise Designated Peripheral Port
  sePPORT_None = 255           ///<  No Port Initialization 

} sePPORT_Id;


/** 
  * @brief  Designated OSC3 ports
  */
#define sePPORT_OSC3            sePPORT_Pd2 
#define sePPORT_OSC4            sePPORT_Pd3

/** 
  * @brief  Designated RFC ports
  */
#define sePPORT_RFC_RFCLKO0     sePPORT_P27	
#define sePPORT_RFC_SENB0       sePPORT_P04 
#define sePPORT_RFC_SENA0       sePPORT_P05 
#define sePPORT_RFC_REF0        sePPORT_P06 
#define sePPORT_RFC_RFIN0       sePPORT_P07 

/** 
  * @brief  Designated REMC ports
  */
#define sePPORT_REMC_REMO       sePPORT_P15 
#define sePPORT_REMC_CLPLS      sePPORT_P16 

/** 
  * @brief  Designated QSPI ports
  */
#define sePPORT_QSPI_0_CLK      sePPORT_P20 
#define sePPORT_QSPI_0_D0       sePPORT_P21 
#define sePPORT_QSPI_0_D1       sePPORT_P22 
#define sePPORT_QSPI_0_D2       sePPORT_P23 
#define sePPORT_QSPI_0_D3       sePPORT_P24 
#define sePPORT_QSPI_0_SS       sePPORT_P25 

/** 
  * @brief  Designated ADC12A ports
  */
#define sePPORT_ADC_0_IN6       sePPORT_P36
#define sePPORT_ADC_0_IN5       sePPORT_P35
#define sePPORT_ADC_0_IN4       sePPORT_P34
#define sePPORT_ADC_0_IN3       sePPORT_P33
#define sePPORT_ADC_0_IN2       sePPORT_P32
#define sePPORT_ADC_0_IN1       sePPORT_P31
#define sePPORT_ADC_0_IN0       sePPORT_P30
#define sePPORT_ADC_0_VREFA     sePPORT_P37
#define sePPORT_ADC_0_ADTRG     sePPORT_P41

/** 
  * @brief  Designated RTC ports
  */
#define sePPORT_RTC1S           sePPORT_P17 

/** 
  * @brief  Designated SPI ports
  */
#define sePPORT_SPI_0_DI       sePPORT_P00
#define sePPORT_SPI_0_DO       sePPORT_P01
#define sePPORT_SPI_0_CLK      sePPORT_P02 
#define sePPORT_SPI_0_SS       sePPORT_P03 

/** 
  * @brief  Designated SVD4 ports
  */
#define sePPORT_SVD_EXSVD0      sePPORT_P43   //< Supply voltage detector Ch.0 external voltage detection input
#define sePPORT_SVD_EXSVD1      sePPORT_P44   //< Supply voltage detector Ch.1 external voltage detection input (VBUS voltage detection input)

/** 
  * @brief  Designated EXOSC ports
  */
#define sePPORT_EXOSC           sePPORT_P26   //< Clock generator external clock input

/** 
  * @brief  Designated T16B Ch.0 ports
  */
#define sePPORT_T16B_0_EXCL0    sePPORT_P12 
#define sePPORT_T16B_0_EXCL1    sePPORT_P40 

/** 
  * @brief  Designated T16B Ch.1 ports
  */
#define sePPORT_T16B_1_EXCL0    sePPORT_P13
#define sePPORT_T16B_1_EXCL1    sePPORT_P41 

/** 
  * @brief  Designated SNDA ports
  */
#define sePPORT_BZOUT           sePPORT_P10
#define sePPORT_NBZOUT          sePPORT_P11 

/** 
  * @brief  PPORT groups
  */
typedef enum {
  sePPORT_G0 = ( 0 ),         /// Port group number P0x.
  sePPORT_G1 = ( 1 ),         ///< Port group number P1x.
  sePPORT_G2 = ( 2 ),         ///< Port group number P2x.
  sePPORT_G3 = ( 3 ),         ///< Port group number P3x.
  sePPORT_G4 = ( 4 ),         ///< Port group number P4x.
  sePPORT_G5 = ( 5 ),         ///< Port group number P5x.
  sePPORT_G6 = ( 6 ),         ///< Port group number P6x.
  sePPORT_G7 = ( 7 ),         ///< Port group number P7x.
  sePPORT_G8 = ( 8 ),         ///< Port group number P8x.
  sePPORT_G9 = ( 9 ),         ///< Port group number P9x.

  sePPORT_Ga = ( 10 ),        ///< Port group number Pax
  sePPORT_Gd = ( 11 ),        ///< Port group number Pdx
} sePPORT_PortGroup;
/**
  * @}
  */

/** 
  * @brief  Designated ADC Temperature chanel
  */
#define seADC12A_ADIN_TEMPERATURE   seADC12A_ADIN7            ///< ADIN7 is internaly connected to temperature sensor
#define seADC12A_INTE_TEMPERATURE   ADC12A_0_INTE_AD7CIE_Msk  ///< Interrupt mask  
#define seADC12A_INTF_TEMPERATURE   ADC12A_0_INTF_AD7CIF_Msk
/**
  * @}
  */

/** @defgroup Interrupts
  * @{
  * @brief  List of peripherial interrupts
  */
void  ADC12A_IRQHandler         ( void );  //< ADC12A
void  CLG_IRQHandler            ( void );  //< CLG
void  DMAC_IRQHandler           ( void );  //< DMAC
void  I2C_0_IRQHandler          ( void );  //< I2C CH0
void  I2C_1_IRQHandler          ( void );  //< I2C CH1
void  LCD32C_IRQHandler         ( void );  //< LCD32C
void  PPORT_IRQHandler          ( void );  //< PPORT
void  QSPI_0_IRQHandler         ( void );  //< QSPI
void  REMC3_IRQHandler          ( void );  //< REMC3
void  RFC_0_IRQHandler          ( void );  //< RFC
void  RTCA_IRQHandler           ( void );  //< RTCA
void  SNDA_IRQHandler           ( void );  //< SNDA
void  SPIA_0_IRQHandler         ( void );  //< SPIA CH0
void  SPIA_1_IRQHandler         ( void );  //< SPIA CH1
void  SVD4_0_IRQHandler         ( void );  //< SVD4 CH0
void  T16_0_IRQHandler          ( void );  //< T16 CH0
void  T16_1_IRQHandler          ( void );  //< T16 CH1
void  T16_2_IRQHandler          ( void );  //< T16 CH2 
void  T16_3_IRQHandler          ( void );  //< T16 CH3 
void  T16_4_IRQHandler          ( void );  //< T16 CH4
void  T16_5_IRQHandler          ( void );  //< T16 CH5 
void  T16_6_IRQHandler          ( void );  //< T16 CH6 
void  T16_7_IRQHandler          ( void );  //< T16 CH7 	
void  T16B_0_IRQHandler         ( void );  //< T16B CH0
void  T16B_1_IRQHandler         ( void );  //< T16B CH1
void  UART3_0_IRQHandler        ( void );  //< UART3 CH0
void  UART3_1_IRQHandler        ( void );  //< UART3 CH1
void  USB_IRQHandler            ( void );  //< USB
void  NMI_Handler               ( void );  //< NMI  
/**
  * @}
  */


/**
  * @}
  */ // sePeriphLibrary
  
//-------------- OSC3 clk  ----------------------------------------------
#define seGET_OSC3_FREQ(freq)      \
    freq = seGetOsc3XtalFreq()
#define seSET_OSC3_FREQ(freq)      

//-------------- IOSC clk  ----------------------------------------------
#define seGET_IOSC_FREQ(freq)      \
  switch ( CLG->IOSC_b.IOSCFQ ) {  \
  case 0:                          \
    freq =  seCLG_IOSC_1MHZ;       \
    break;                         \
  case 1:                          \
    freq =  seCLG_IOSC_2MHZ;       \
    break;                         \
  case 3:                          \
    freq =  seCLG_IOSC_8MHZ;       \
    break;                         \
  case 4:                          \
    freq = seCLG_IOSC_12MHZ;       \
    break;                         \
  case 5:                          \
    freq = seCLG_IOSC_16MHZ;       \
    break;                         \
  case 6:                          \
    freq = seCLG_IOSC_24MHZ;       \
    break;                         \
  case 7:                          \
    freq = seCLG_IOSC_32MHZ;       \
    break;                         \
  default:                         \
    freq = seCLG_IOSC_8MHZ;        \
    break;                         \
  }
#define seSET_IOSC_FREQ(freq)      \
  switch ( freq ) {                \
  case seCLG_IOSC_1MHZ:            \
    CLG->IOSC_b.IOSCFQ = 0;        \
    break;                         \
  case seCLG_IOSC_2MHZ:            \
    CLG->IOSC_b.IOSCFQ = 1;        \
    break;                         \
  case seCLG_IOSC_8MHZ:            \
    CLG->IOSC_b.IOSCFQ = 3;        \
    break;                         \
  case seCLG_IOSC_12MHZ:           \
    CLG->IOSC_b.IOSCFQ = 4;        \
    break;                         \
  case seCLG_IOSC_16MHZ:           \
    CLG->IOSC_b.IOSCFQ = 5;        \
    break;                         \
  case seCLG_IOSC_24MHZ:           \
    CLG->IOSC_b.IOSCFQ = 6;        \
    break;                         \
  case seCLG_IOSC_32MHZ:           \
    CLG->IOSC_b.IOSCFQ = 7;        \
    break;                         \
  default:                         \
    CLG->IOSC_b.IOSCFQ = 3;        \
    break;                         \
  }

//-------------- All interrupt flags and all interrupts enable -----------------
#define ADC12A_0_INTE_ALL_Msk  ( ADC12A_0_INTE_AD0CIE_Msk | ADC12A_0_INTE_AD1CIE_Msk | ADC12A_0_INTE_AD2CIE_Msk | \
                                 ADC12A_0_INTE_AD3CIE_Msk | ADC12A_0_INTE_AD4CIE_Msk | ADC12A_0_INTE_AD5CIE_Msk | \
                                 ADC12A_0_INTE_AD6CIE_Msk | ADC12A_0_INTE_AD7CIE_Msk | ADC12A_0_INTE_OVIE_Msk )


#define ADC12A_0_INTF_ALL_Msk  ( ADC12A_0_INTF_AD0CIF_Msk | ADC12A_0_INTF_AD1CIF_Msk | ADC12A_0_INTF_AD2CIF_Msk | \
                                 ADC12A_0_INTF_AD3CIF_Msk | ADC12A_0_INTF_AD4CIF_Msk | ADC12A_0_INTF_AD5CIF_Msk | \
                                 ADC12A_0_INTF_AD6CIF_Msk | ADC12A_0_INTF_AD7CIF_Msk | ADC12A_0_INTF_OVIF_Msk )
								 
#define CLG_INTF_ALL_Msk       ( CLG_INTF_IOSCSTAIF_Msk | CLG_INTF_OSC1STAIF_Msk | CLG_INTF_OSC3STAIF_Msk | \
                                 CLG_INTF_IOSCTEDIF_Msk | CLG_INTF_OSC1STPIF_Msk | CLG_INTF_IOSCTERIF_Msk )

#define CLG_INTE_ALL_Msk       ( CLG_INTE_IOSCSTAIE_Msk | CLG_INTE_OSC1STAIE_Msk | CLG_INTE_OSC3STAIE_Msk | \
                                 CLG_INTE_IOSCTEDIE_Msk | CLG_INTE_OSC1STPIE_Msk |  CLG_INTE_IOSCTERIE_Msk )
                                         
#define RFC_0_INTE_ALL_Msk     ( RFC_0_INTE_EREFIE_Msk | RFC_0_INTE_ESENAIE_Msk | RFC_0_INTE_ESENBIE_Msk | \
                                 RFC_0_INTE_OVMCIE_Msk | RFC_0_INTE_OVTCIE_Msk )

#define RFC_0_INTF_ALL_Msk     ( RFC_0_INTF_EREFIF_Msk | RFC_0_INTF_ESENAIF_Msk | RFC_0_INTF_ESENBIF_Msk | \
                                 RFC_0_INTF_OVMCIF_Msk | RFC_0_INTF_OVTCIF_Msk )
                                  
#define RTCA_INTE_ALL_Msk      ( RTCA_INTE_T1_32SECIE_Msk | RTCA_INTE_T1_8SECIE_Msk | RTCA_INTE_T1_4SECIE_Msk | \
                                 RTCA_INTE_T1_2SECIE_Msk | RTCA_INTE_T1SECIE_Msk | RTCA_INTE_T1MINIE_Msk | \
                                 RTCA_INTE_T1HURIE_Msk | RTCA_INTE_T1DAYIE_Msk | RTCA_INTE_ALARMIE_Msk | \
                                 RTCA_INTE_SW100IE_Msk | RTCA_INTE_SW10IE_Msk | RTCA_INTE_SW1IE_Msk | \
                                 RTCA_INTE_RTCTRMIE_Msk )

#define RTCA_INTF_ALL_Msk      ( RTCA_INTF_T1_32SECIF_Msk | RTCA_INTF_T1_8SECIF_Msk | RTCA_INTF_T1_4SECIF_Msk | \
                                 RTCA_INTF_T1_2SECIF_Msk | RTCA_INTF_T1SECIF_Msk | RTCA_INTF_T1MINIF_Msk | \
                                 RTCA_INTF_T1HURIF_Msk | RTCA_INTF_T1DAYIF_Msk | RTCA_INTF_ALARMIF_Msk | \
                                 RTCA_INTF_SW100IF_Msk | RTCA_INTF_SW10IF_Msk | RTCA_INTF_SW1IF_Msk | \
                                 RTCA_INTF_RTCTRMIF_Msk )
                                 
#define T16B_0_INTF_ALL_Msk    ( T16B_0_INTF_CNTZEROIF_Msk | T16B_0_INTF_CNTMAXIF_Msk | T16B_0_INTF_CMPCAP0IF_Msk | \
                                 T16B_0_INTF_CAPOW0IF_Msk | T16B_0_INTF_CMPCAP1IF_Msk | T16B_0_INTF_CAPOW1IF_Msk | \
                                 T16B_0_INTF_CMPCAP2IF_Msk | T16B_0_INTF_CAPOW2IF_Msk | T16B_0_INTF_CMPCAP3IF_Msk | \
                                 T16B_0_INTF_CAPOW3IF_Msk )

#define T16B_0_INTE_ALL_Msk    ( T16B_0_INTE_CNTZEROIE_Msk | T16B_0_INTE_CNTMAXIE_Msk | T16B_0_INTE_CMPCAP0IE_Msk | \
                                 T16B_0_INTE_CAPOW0IE_Msk | T16B_0_INTE_CMPCAP1IE_Msk | T16B_0_INTE_CAPOW1IE_Msk | \
                                 T16B_0_INTE_CMPCAP2IE_Msk | T16B_0_INTE_CAPOW2IE_Msk | T16B_0_INTE_CMPCAP3IE_Msk | \
                                 T16B_0_INTE_CAPOW3IE_Msk )                               
                       
#define UART3_0_INTF_ALL_Msk   ( UART3_0_INTF_TBEIF_Msk | UART3_0_INTF_RB1FIF_Msk | UART3_0_INTF_RB2FIF_Msk | \
                                 UART3_0_INTF_OEIF_Msk | UART3_0_INTF_PEIF_Msk | UART3_0_INTF_FEIF_Msk | \
                                 UART3_0_INTF_TENDIF_Msk | UART3_0_INTF_TBSY_Msk | UART3_0_INTF_RBSY_Msk )

#define UART3_0_INTE_ALL_Msk   ( UART3_0_INTE_TBEIE_Msk | UART3_0_INTE_RB1FIE_Msk | UART3_0_INTE_RB2FIE_Msk | \
                                 UART3_0_INTE_OEIE_Msk | UART3_0_INTE_PEIE_Msk | UART3_0_INTE_FEIE_Msk | \
                                 UART3_0_INTE_TENDIE_Msk )                              


#define SPIA_0_INTF_ALL_Msk    ( SPIA_0_INTF_TBEIF_Msk | SPIA_0_INTF_RBFIF_Msk | SPIA_0_INTF_TENDIF_Msk | \
                                 SPIA_0_INTF_OEIF_Msk | SPIA_0_INTF_BSY_Msk )

#define SPIA_0_INTE_ALL_Msk    ( SPIA_0_INTE_TBEIE_Msk | SPIA_0_INTE_RBFIE_Msk | SPIA_0_INTE_TENDIE_Msk | \
                                 SPIA_0_INTE_OEIE_Msk )

#define QSPI_0_INTF_ALL_Msk    ( QSPI_0_INTF_TBEIF_Msk | QSPI_0_INTF_RBFIF_Msk | QSPI_0_INTF_TENDIF_Msk | \
                                 QSPI_0_INTF_OEIF_Msk | QSPI_0_INTF_MMABSY_Msk | QSPI_0_INTF_BSY_Msk )

#define QSPI_0_INTE_ALL_Msk    ( QSPI_0_INTE_TBEIE_Msk | QSPI_0_INTE_RBFIE_Msk | QSPI_0_INTE_TENDIE_Msk | \
                                 QSPI_0_INTE_OEIE_Msk )



#ifdef __cplusplus
}
#endif

#endif /*SE_S1C31_H */
