/**************************************************************************//**
 * @file     system_C31W73.c
 * @brief    CMSIS Cortex-M0P Device Peripheral Access Layer Source File for
 *           Device C31W73
 * @version  V1.1
 * @date     22-December-2015
 *
 * @note
 *
 ******************************************************************************/
/* Copyright (c) 2012 ARM LIMITED

   All rights reserved.
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
   - Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
   - Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
   - Neither the name of ARM nor the names of its contributors may be used
     to endorse or promote products derived from this software without
     specific prior written permission.
   *
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDERS AND CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
   ---------------------------------------------------------------------------*/


#include <stdint.h>
#include <S1C31.h>
#include "se_clg.h"
#include <board.h>
#include <settings.h>

volatile unsigned g1msSysTicks;        // Timer tick
/*----------------------------------------------------------------------------
  DEFINES
 *----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
  Define clocks
 *----------------------------------------------------------------------------*/

#define __SYSTEM_CLOCK    (8000000UL)


/*----------------------------------------------------------------------------
  Clock Variable definitions
 *----------------------------------------------------------------------------*/

uint32_t SystemCoreClock = __SYSTEM_CLOCK;  /*!< System Clock Frequency (Core Clock)*/


/*----------------------------------------------------------------------------
  Clock functions
 *----------------------------------------------------------------------------*/
void SystemCoreClockUpdate (void)            /* Get Core Clock Frequency      */
{
  SystemCoreClock = seCLG_GetSysClk();
}

/**
 * Initialize the system
 *
 * @brief  Setup the microcontroller system.
 *         Initialize the System.
 */
void SystemInit (void)
{

  SystemCoreClock = __SYSTEM_CLOCK;
  BoardInit();

}

/**
 * @brief  Sets the sleep state.
 */
void setSleep( void )
{
#ifndef TICKLESS_ENABLED
  seSysTickStop();
#endif

  SCB->SCR |= SCB_SCR_SLEEPDEEP_Msk;

  __WFI();

#ifndef TICKLESS_ENABLED
  seSysTickStart();
#endif
}

/**
 * @brief  Sets the halt state.
 */
void setHalt( void )
{
#ifndef TICKLESS_ENABLED
  seSysTickStop();
#endif

  SCB->SCR &= ~SCB_SCR_SLEEPDEEP_Msk;

  __WFI();

#ifndef TICKLESS_ENABLED
  seSysTickStart();
#endif
}

/**
 * @brief  Sets Event Wait state.
 */
void setEventWait( void )
{
#ifdef TICKLESS_ENABLED
  __NOP();
#else
  __WFE();
#endif
}

void seSysTickStart( void ) {

  SysTick->CTRL |= SysTick_CTRL_ENABLE_Msk;

}

void seSysTickStop( void ) {

 SysTick->CTRL &= ~SysTick_CTRL_ENABLE_Msk;

}

uint32_t seSysTickInit( uint32_t ticks, uint32_t enableint ){
  uint32_t RetVal = seSTATUS_OK;

  if ((ticks - 1UL) > SysTick_LOAD_RELOAD_Msk) {
    RetVal = seSTATUS_NG;
  }
  else {
    SysTick->LOAD  = (uint32_t)(ticks - 1UL);
    SysTick->VAL   = 0UL;
    SysTick->CTRL  = SysTick_CTRL_CLKSOURCE_Msk | (enableint << SysTick_CTRL_TICKINT_Pos);
  }
  return RetVal;
}

#ifdef TICKLESS_ENABLED
uint32_t seSysGetTimeMS( void ) {

  SystemCoreClockUpdate();
  seSysTickInit( SystemCoreClock / TICK_PERSECOND, seDISABLE );
  seSysTickStart();
  while( !( SysTick->CTRL & SysTick_CTRL_COUNTFLAG_Msk )){}
  g1msSysTicks += 1000UL/TICK_PERSECOND;
  seSysTickStop();

  return g1msSysTicks;

}
void seSysSleepMS( uint32_t Milliseconds ) {

  SystemCoreClockUpdate();
  seSysTickInit( SystemCoreClock / TICK_PERSECOND, seDISABLE );
  seSysTickStart();
  do {
    while( !( SysTick->CTRL & SysTick_CTRL_COUNTFLAG_Msk )){}
    Milliseconds -= 1000UL/TICK_PERSECOND;
    g1msSysTicks += 1000UL/TICK_PERSECOND;
  }while ( Milliseconds > 0 );
  seSysTickStop();
}

#else // TICKLESS_ENABLED

uint32_t seSysGetTimeMS( void ) {

  return g1msSysTicks;

}

void seSysSleepMS( uint32_t Milliseconds ) {

  uint32_t newticks, cticks = seSysGetTimeMS();
  do {
    newticks = seSysGetTimeMS();
    if ( newticks < cticks ) {
        cticks = newticks;
    }
  } while ( ( newticks - cticks ) < Milliseconds );

} // TICKLESS_ENABLED
#endif

// 1ms clock. Just update the tick count.
void SysTick_Handler(void);
void SysTick_Handler(void) {
  g1msSysTicks += 1000UL/TICK_PERSECOND;
}

