/**
  ******************************************************************************
  * @file    board.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file contains all the board functions for the C31W73.
  *
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#include <stdio.h>
#include <string.h>
#include "board.h"

extern uint32_t __Vectors;               ///< Vector table
extern volatile unsigned g1msSysTicks;   ///< Timer tick

/**
  * @brief  Gets crystal frequency for OSC3.
  * @retval OSC3 crystal frequency in Hz.
  */
inline uint32_t seGetOsc3XtalFreq( void ) {
  
  return BOARD_CLG_OSC3_XTAL_FREQ;
}

/**
  * @brief  Gets crystal frequency for OSC1.
  * @retval OSC1 crystal frequency in Hz.
  */
inline uint32_t seGetOsc1XtalFreq( void ) {
  
  return BOARD_CLG_OSC1_XTAL_FREQ;
}

/**
  * @brief  Gets external frequency for EXOSC.
  * @retval EXOSC external frequency in Hz.
  */
inline uint32_t seGetExOscFreq( void ) {
  
  return BOARD_CLG_EXOSC_FREQUENCY;
}

#if defined( BOOT_LOADER )
/**
  * @brief  Jumps to the specified address of the loaded program.
  * @param  app_link_location: address
  * @retval None
  */
static void start_application( uint32_t app_link_location ){
  void (*pfMainApp)( void );
  uint32_t *pointer;

  __disable_interrupt();

  SCB->VTOR = app_link_location;
  pointer = (uint32_t *)app_link_location;
  __set_MSP(*pointer);
  pointer++;
  pfMainApp = (void (*)(void))(*pointer);
  pfMainApp();
}
#endif ///< BOOT_LOADER

#if  defined( UART_PRINTF )
/**
  * @brief  Initializes the UART peripheral according to the specified
  *         parameters in the seUART_InitStruct.
  * @param  UART_InitStruct: Pointer to an seUART_InitTypeDef structure which will be initialized.
  * @retval None
  */
void ConfigureDebugUART( seUART_InitTypeDef* InitStruct )
{
  ///< Stop supplying operating clock
  seUART_Disable( BOARD_UART_PRINTF_CH );

  ///< Configure the UPMUX directly instead of calling the sePPORT_UpMuxFunction
  ///< Since RAM has not been initialized yet
  ///< Set the USIN - P16, USOUT - P17
  PPORT->P1IOEN_b.P1IEN6     = 0;
  PPORT->P1IOEN_b.P1IEN7     = 0;
  PPORT->P1DAT_b.P1OUT6      = 1;
  PPORT->P1DAT_b.P1OUT7      = 1;
  PPORT->P1IOEN_b.P1OEN6     = 1;
  PPORT->P1IOEN_b.P1OEN7     = 1;
  PPORT->P1MODSEL_b.P1SEL6   = 0;
  PPORT->P1MODSEL_b.P1SEL7   = 0;
  UPMUX->P1MUX3_b.P16PERISEL = 3;   ///< UART.
  UPMUX->P1MUX3_b.P16PPFNC   = 1;   ///< USIN.
  UPMUX->P1MUX3_b.P16PERICH  = 0;   ///< Ch.0.
  UPMUX->P1MUX3_b.P17PERISEL = 3;   ///< UART.
  UPMUX->P1MUX3_b.P17PPFNC   = 2;   ///< USOUT.
  UPMUX->P1MUX3_b.P17PERICH  = 0;   ///< Ch.0.
  PPORT->P1FNCSEL_b.P16MUX   = 1;   ///< UPMUX.
  PPORT->P1MODSEL_b.P1SEL6   = 1;   ///< Enable peripheral I/O function.
  PPORT->P1FNCSEL_b.P17MUX   = 1;   ///< UPMUX.
  PPORT->P1MODSEL_b.P1SEL7   = 1;   ///< Enable peripheral I/O function.

  ///< Configure the UART Ch.n operating clock.
  seUART_ConfigureClock( UART3_0, InitStruct->Clk );

  ///< Select operation mode;
  seUART_ConfigureMode( UART3_0, InitStruct->Mode );

  ///< Initialize interrupt.
  seUART_DisableInt( UART3_0, UART3_0_INTE_ALL_Msk );
  seUART_ClearIntFlag( UART3_0, UART3_0_INTF_ALL_Msk );
}
#endif ///< UART_PRINTF

/**
  * @brief  Initializes the Board
  * @retval None
  */
seStatus BoardInit( void ) {
  g1msSysTicks = 0;
#if  defined( CACHE_ENABLED )
  /* Disable cache in case of a warm boot */

  ///< Disable write-protect.
  seProtectSys( seWRITE_PROTECT_OFF );
  CACHE->CTL_b.CACHEEN = 0;
  __asm("nop");
  __asm("nop");
  __asm("nop");
  __asm("nop");
  ///< Enable write-protect.
  seProtectSys( seWRITE_PROTECT_ON );
#endif ///< CACHE_ENABLED

  /* relocate vector table */
  __disable_irq();
  if ( (uint32_t) &__Vectors != 0 ) {
    SCB->VTOR = (uint32_t)&__Vectors;
    __DSB();
  }
  __enable_irq();

#ifdef EXECUTE_ON_OSC3
  seStatus fStatus;
  seCLG_Start( seCLG_OSC3 );
  fStatus = seCLG_SwitchSysClk( seCLG_OSC3_CLKDIV_1 ); 
  if ( fStatus == seSTATUS_NG ) {
    seASSERT( fStatus );
  }
#else ///< EXECUTE_ON_OSC3

  ///< Disable write-protect.
  seProtectSys( seWRITE_PROTECT_OFF );
  FLASHC->WAIT_b.RDWAIT = 2;
  ///< Enable write-protect.
  seProtectSys( seWRITE_PROTECT_ON );

  ///< Sets the IOSC frequency
  seCLG_SetIoscFreqSel( seCLG_IOSC_24MHZ );

#ifdef IOSC_AUTOTRIMMING_ON
  ///< Run Autotrimming of IOSC. Temporary switch to OSC3
  seCLG_Start( seCLG_OSC3 );
  seCLG_RunAutoTrimming( seCLG_IOSC, seCLG_OSC3 );
  seCLG_ClearIntFlag( seCLG_FLGS(( seCLG_IOSCTERIF | seCLG_IOSCTEDIF )));
  seCLG_Stop( seCLG_OSC3 );
#endif ///< IOSC_AUTOTRIMMING_ON

#endif ///< EXECUTE_ON_OSC3

  SystemCoreClockUpdate();

#ifndef TICKLESS_ENABLED
  seSysTickInit( SystemCoreClock / TICK_PERSECOND, seENABLE );
  seSysTickStart();
#endif ///< TICKLESS_ENABLED

#if  defined( UART_PRINTF )
  {
  seUART_InitTypeDef InitStruct;
  ///< Clear parameters
  memset( &InitStruct, 0, sizeof( InitStruct ) );
  ///< Init UART
  seCLG_Start( seSRC(BOARD_UART_PRINTF_CLK) );
  seUART_InitStruct( &InitStruct );
  InitStruct.Clk             = BOARD_UART_PRINTF_CLK;       ///< Set clock source for UART3
  InitStruct.Mode.reg_b.puen = seUART_MOD_PUEN_ENABLE;      ///< Enable USINn pin pull-up
  InitStruct.Mode.reg_b.chln = seUART_MOD_CHLN_8BIT;        ///< Set 7/8-bit data rate
  InitStruct.Mode.reg_b.stpb = seUART_MOD_STPB_1BIT;        ///< Select 1/2-bit stop bit length
  InitStruct.Mode.reg_b.pren = seUART_MOD_PREN_NO_PARITY;   ///< Enable parity function
  InitStruct.Mode.reg_b.prmd = seUART_MOD_PRMD_EVEN;        ///< Select even/odd parity function
  InitStruct.Mode.reg_b.irmd = seUART_MOD_IRMD_NO_IRDA;     ///< Enable Irda interface

  ConfigureDebugUART( &InitStruct );
  seUART_SetBaudRate( BOARD_UART_PRINTF_CH, seUART_BAUD_RATE_115200 );
 
  seUART_Enable( BOARD_UART_PRINTF_CH );
  }
#endif ///< UART_PRINTF

#if  defined( CACHE_ENABLED )
  /* Enable/Disable cache if configured */
  ///< Disable write-protect.
  seProtectSys( seWRITE_PROTECT_OFF );
  CACHE->CTL_b.CACHEEN = seENABLE;
  __asm("nop");
  __asm("nop");
  __asm("nop");
  __asm("nop");
  ///< Enable write-protect.
  seProtectSys( seWRITE_PROTECT_ON );
#endif  ///< CACHE_ENABLED
#if defined( BOOT_LOADER )
  start_application( APPLICATION_LINK_ADDRESS );
#endif ///< BOOT_LOADER

  return seSTATUS_OK;
}

#if  defined( UART_PRINTF )
/**
  * @brief  Receives a byte of data from UART receiver.
  * @note   This function is non blocking. It can be called from Interrupt Serice routine since it
  *         its implementation is deterministic. 
  * @retval one byte of data.
  */
int16_t seUART_GetCh( void ) {
 
  return BOARD_UART_PRINTF_CH->RXD;
}

/**
  * @brief  Receives a byte of data from UART receiver.
  * @note   This function is blocking.
  * @retval one byte of data.
  */
int16_t seUART_getchar( void ) {
  
  while( seINTERRUPT_OCCURRED != seUART_GetIntFlag( BOARD_UART_PRINTF_CH,  UART3_0_INTF_RB1FIF_Msk ) ) {
    ;
  }
  return BOARD_UART_PRINTF_CH->RXD;
}

/**
  * @brief  Transmitter a byte of data from UART transmitter.
  * @note   This function is blocking.
  * @retval one byte of data.
  */
int16_t seUART_putchar ( int16_t character ) {
  
  while( seINTERRUPT_OCCURRED != seUART_GetIntFlag( BOARD_UART_PRINTF_CH,  UART3_0_INTF_TBEIF_Msk ) ) { 
    ;
  }
  BOARD_UART_PRINTF_CH->TXD = character;

  while( seINTERRUPT_OCCURRED != seUART_GetIntFlag( BOARD_UART_PRINTF_CH, UART3_0_INTE_TENDIE_Msk ) ); 
  ///< Clear interrupt flag.
  seUART_ClearIntFlag( BOARD_UART_PRINTF_CH, UART3_0_INTE_TENDIE_Msk );
  return character;
}

/**
  * @brief  Returns a non-zero integer if a key is in the keyboard buffer.
  *         It will not wait for a key to be pressed. 
  * @note   This function is non blocking.
  */
int16_t seUART_kbhit( void ) {

  return seUART_GetIntFlag( BOARD_UART_PRINTF_CH,  UART3_0_INTF_RB1FIF_Msk );
}
#endif

/**
  * @brief  Displays the frequency of the system clock.
  * @retval None
  */
void ShowRunningClock( void ) {
/*
    if ( seCLG_OSC1 == seCLG_GetSysClkSrc() ) {
      printf ( " -CPU clock- seCLG_OSC1 (%d)\n", seCLG_GetSysClk() );
    } else if ( seCLG_OSC3 == seCLG_GetSysClkSrc() ) {
      printf ( " -CPU clock- seCLG_OSC3 (%d) \n", seCLG_GetSysClk() );
    } else if ( seCLG_IOSC == seCLG_GetSysClkSrc() ) {
      printf ( " -CPU clock- seCLG_IOSC (%d) \n", seCLG_GetSysClk() );
    } else {
      printf ( " - ??? Not expected state ??? \n" );
    }
*/
}

