/**
  ******************************************************************************
  * @file    se_adc12a.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file provides all the ADC12A hardware functions.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#include "se_adc12a.h"

/**
  * @brief  Initializes the ADC12A peripheral according to the specified
  *         parameters in the seADC12A_InitStruct.
  * @param  ADC12Ax: ADC12A peripheral
  * @param  InitStruct: pointer to a seADC12A_InitTypeDef structure that
  *         contains the configuration information for the specified ADC12A peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seADC12A_Init ( ADC12A_0_Type* ADC12Ax, seADC12A_InitTypeDef* InitStruct ) {

  ///< Check if selected clock is enabled
  seStatus fStatus = seCLG_IsEnabled( seSRC(InitStruct->Clk));
  
  ///< When Clk not started, do nothing.
  if ( fStatus == seSTATUS_OK ) { 
    //seAUX_ConfigurePortsForADC12a( ADC12Ax, &InitStruct->ports );

    ///< Configure the ADC12A operating clock.
    seAUX_ConfigureClockForADC12a( ADC12Ax, InitStruct->Clk );
    
    ///< Clear interrupt flags.
    seADC12A_ClearIntFlag( ADC12Ax, ADC12A_0_INTF_ALL_Msk );

    ///< Enable ADC12A
    ADC12Ax->CTL_b.MODEN = 1;

    ///< Configure parameters
    ADC12Ax->TRG_b.SMPCLK = InitStruct->SampleClock;      ///< Sample Clock in number of CLK_T16 cycles
    ADC12Ax->TRG_b.CNVTRG = InitStruct->TrigSrc;          ///< Conversion trigger source
    ADC12Ax->TRG_b.CNVMD = InitStruct->ConversionMode;    ///< Conversion mode
    ADC12Ax->TRG_b.STMD = InitStruct->StoreMode;          ///< Data store mode
    ADC12Ax->TRG_b.STAAIN = InitStruct->ports.StartAIN;   ///< Starting ADINx analog input to sample
    ADC12Ax->TRG_b.ENDAIN = InitStruct->ports.EndAIN;     ///< Ending ADINx analog input to sample
    ADC12Ax->CFG_b.VRANGE = InitStruct->VoltageRange;     ///< Conversion voltage range

  }

  return fStatus;
}

/**
  * @brief  Disable ADC12A interrupts
  * @param  ADC12Ax:  ADC12A peripheral
  * @param  irq:  interrupts mask
  * @retval None
  */
void seADC12A_DisableInt( ADC12A_0_Type* ADC12Ax, uint16_t irq ) {

  ADC12Ax->INTE &= ~irq;
}

/**
  * @brief  Enable ADC12A interrupts
  * @param  ADC12Ax:  ADC12A peripheral
  * @param  irq:  interrupts mask
  * @retval None
  */
void seADC12A_EnableInt( ADC12A_0_Type* ADC12Ax, uint16_t irq ) {

  ADC12Ax->INTE |= irq;
}

/**
  * @brief  Clear ADC12A interrupts
  * @param  ADC12Ax:  ADC12A peripheral
  * @param  irq:  interrupts mask
  * @retval None
  */
void seADC12A_ClearIntFlag( ADC12A_0_Type* ADC12Ax, uint16_t irq ) {

  ADC12Ax->INTF = irq;
}

/**
  * @brief  Disable ADC12A.  Stop conversion, voltage range off, module off.
  * @param  ADC12Ax:  ADC12A peripheral
  * @retval None
  */
void seADC12A_Disable( ADC12A_0_Type* ADC12Ax ) {

  ADC12Ax->CFG_b.VRANGE = seADC12A_VRANGE_OFF;  ///< Conversion voltage range OFF
  ADC12Ax->CTL = 0;                             ///< Disable conversion, turn off module
}

/**
  * @brief  Start ADC12A conversion.
  * @param  ADC12Ax:  ADC12A peripheral
  * @retval None
  */
void seADC12A_Start( ADC12A_0_Type* ADC12Ax ) {

  ADC12Ax->CTL_b.ADST = 1;
}

/**
  * @brief  Stop ADC12A conversion.
  * @param  ADC12Ax:  ADC12A peripheral
  * @retval None
  */
void seADC12A_Stop( ADC12A_0_Type* ADC12Ax ) {

  ADC12Ax->CTL_b.ADST = 0;
}

/**
  * @brief  Read ADC12A A/D value.
  * @param  ADC12Ax:  ADC12A peripheral
  * @retval A/D value
  */
uint16_t seADC12A_ReadADC( ADC12A_0_Type* ADC12Ax ) {

  return(ADC12Ax->ADD);
}

