/**
  ******************************************************************************
  * @file    se_adc12a.h
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file contains all the functions prototypes for the ADC12A firmware
  *          library.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#ifndef SE_ADC12A_H
#define SE_ADC12A_H

#include "se_common.h"

#ifdef __cplusplus
 extern "C" {
#endif

/** @addtogroup sePeriphLibrary
  * @{
  */

/** @defgroup ADC12A
  * @{
  * @brief The ADC12A is a 12-bit successive approximation analog-to-digital converter with a multiplexed input.
  *
  */

/** @defgroup ADC12A_Constants
  * @{
  */

typedef enum {
   seADC12A_STAT_IDLE           = 0U,     ///< ADC12A is idle
   seADC12A_STAT_CONVERTING     = 1U      ///< ADC12A is converting
} seADC12A_BSYSTAT;

typedef enum {
   seADC12A_ONETIME             = 0U,     ///< One-time conversion mode
   seADC12A_CONTINUOUS          = 1U      ///< Continuous conversion mode
} seADC12A_CNVMD;

typedef enum {
   seADC12A_RIGHT_JUSTIFY       = 0U,     ///< Right-justify data alignment
   seADC12A_LEFT_JUSTIFY        = 1U      ///< Left-justify data alignment
} seADC12A_STMD;

typedef enum {
   seADC12A_SWTRIG              = 0U,     ///< Software-triggered
   seADC12A_T16UNDERFLOW        = 1U,     ///< 16-bit timer underflow
   seADC12A_CNVTRG_RSV1         = 2U,     ///< Reserved
   seADC12A_EXTPIN              = 3U      ///< ADTRG external input pin
} seADC12A_CNVTRG;

typedef enum {
   seADC12A_ADIN0               = 0U,     ///< ADIN0 input
   seADC12A_ADIN1               = 1U,     ///< ADIN1 input
   seADC12A_ADIN2               = 2U,     ///< ADIN2 input
   seADC12A_ADIN3               = 3U,     ///< ADIN3 input
   seADC12A_ADIN4               = 4U,     ///< ADIN4 input
   seADC12A_ADIN5               = 5U,     ///< ADIN5 input
   seADC12A_ADIN6               = 6U,     ///< ADIN6 input
   seADC12A_ADIN7               = 7U      ///< ADIN7 input
} seADC12A_AIN;

typedef enum {
   seADC12A_SMPCLK_4CYC         = 0U,     ///< SMPCLK is 4 cycles of CLK_T16
   seADC12A_SMPCLK_5CYC         = 1U,     ///< SMPCLK is 5 cycles of CLK_T16
   seADC12A_SMPCLK_6CYC         = 2U,     ///< SMPCLK is 6 cycles of CLK_T16
   seADC12A_SMPCLK_7CYC         = 3U,     ///< SMPCLK is 7 cycles of CLK_T16
   seADC12A_SMPCLK_8CYC         = 4U,     ///< SMPCLK is 8 cycles of CLK_T16
   seADC12A_SMPCLK_9CYC         = 5U,     ///< SMPCLK is 9 cycles of CLK_T16
   seADC12A_SMPCLK_10CYC        = 6U,     ///< SMPCLK is 10 cycles of CLK_T16
   seADC12A_SMPCLK_11CYC        = 7U      ///< SMPCLK is 11 cycles of CLK_T16
} seADC12A_SMPCLK;

typedef enum {
   seADC12A_VRANGE_OFF          = 0U,     ///< Conversion disabled
   seADC12A_VRANGE_48_55        = 1U,     ///< Conversion range enabled, 4.8V to 5.5V
   seADC12A_VRANGE_36_55        = 2U,     ///< Conversion range enabled, 3.6V to 5.5V
   seADC12A_VRANGE_18_55        = 3U      ///< Conversion range enabled, 1.8V to 5.5V
} seADC12A_VRANGE;

/**
  * @}
  */  // ADC12A_Constants

/** @defgroup ADC12A_Types
  * @{
  */

typedef struct {
   seADC12A_AIN StartAIN;                 ///< Starting ADINx analog input to sample
   seADC12A_AIN EndAIN;                   ///< Ending ADINx analog input to sample
} seADC12a_Ports;

/** 
  * @brief  ADC12AInit structure definition
  */
typedef struct {   
   seADC12A_Clk Clk;                      ///< Specifies the timer clock source selection and the prescaler value used to divide the T16 clock.
   seADC12A_STMD StoreMode;               ///< Data store mode:  left-justify or right-justify
   seADC12A_CNVMD ConversionMode;         ///< Conversion mode:  one-time or continuous
   seADC12A_CNVTRG TrigSrc;               ///< Conversion trigger source.
   seADC12A_SMPCLK SampleClock;           ///< Sample Clock in number of CLK_T16 cycles
   seADC12A_VRANGE VoltageRange;          ///< Conversion voltage range
   seADC12a_Ports  ports;                 ///< Analog input pin
} seADC12A_InitTypeDef;

/**
  * @}
  */  // ADC12A_Types

/** @defgroup ADC12A_Functions
  * @{
  */

/**
  * @brief  Initializes the ADC12A peripheral according to the specified
  *         parameters in the seADC12A_InitStruct.
  * @param  ADC12Ax: ADC12A peripheral
  * @param  InitStruct: pointer to a seADC12A_InitTypeDef structure that
  *         contains the configuration information for the specified ADC12A peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seADC12A_Init ( ADC12A_0_Type* ADC12Ax, seADC12A_InitTypeDef* InitStruct );

/**
  * @brief  Configures ADC12A ports pins according to Start and End ADINx selection
  * @param  ADC12Ax: ADC12A peripheral
  * @param  StartAIN:  Starting ADINx
  * @param  EndAIN:  Ending ADINx
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seAUX_ConfigurePortsForADC12a( ADC12A_0_Type* ADC12Ax, seADC12a_Ports * ports );

/**
  * @brief  Configures ADC12A clock.
  * @param  ADC12Ax: ADC12A peripheral
  * @param  clock:  clock source and divider
  * @retval None
  */
seStatus seAUX_ConfigureClockForADC12a( ADC12A_0_Type* ADC12Ax, seADC12A_Clk clock );

/**
  * @brief  Disable ADC12A interrupts
  * @param  ADC12Ax:  ADC12A peripheral
  * @param  irq:  interrupts mask
  * @retval None
  */
void seADC12A_DisableInt( ADC12A_0_Type* ADC12Ax, uint16_t irq );

/**
  * @brief  Enable ADC12A interrupts
  * @param  ADC12Ax:  ADC12A peripheral
  * @param  irq:  interrupts mask
  * @retval None
  */
void seADC12A_EnableInt( ADC12A_0_Type* ADC12Ax, uint16_t irq );

/**
  * @brief  Clear ADC12A interrupts
  * @param  ADC12Ax:  ADC12A peripheral
  * @param  irq:  interrupts mask
  * @retval None
  */
void seADC12A_ClearIntFlag( ADC12A_0_Type* ADC12Ax, uint16_t irq );

/**
  * @brief  Disable ADC12A.  Stop conversion, voltage range off, module off.
  * @param  ADC12Ax:  ADC12A peripheral
  * @retval None
  */
void seADC12A_Disable( ADC12A_0_Type* ADC12Ax );

/**
  * @brief  Start ADC12A conversion.
  * @param  ADC12Ax:  ADC12A peripheral
  * @retval None
  */
void seADC12A_Start( ADC12A_0_Type* ADC12Ax );

/**
  * @brief  Stop ADC12A conversion.
  * @param  ADC12Ax:  ADC12A peripheral
  * @retval None
  */
void seADC12A_Stop( ADC12A_0_Type* ADC12Ax );

/**
  * @brief  Read ADC12A A/D value.
  * @param  ADC12Ax:  ADC12A peripheral
  * @retval A/D value
  */
uint16_t seADC12A_ReadADC( ADC12A_0_Type* ADC12Ax );

/**
  * @}
  */ // ADC12A_Functions

/**
  * @}
  */  // ADC12A

/**
  * @}
  */ // sePeriphLibrary

#ifdef __cplusplus
}
#endif
#endif  // SE_ADC12A_H

