/**
  ******************************************************************************
  * @file    se_aux.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file contains auxiliary functions for the sePeriphLibrary
  *          library for the S1C31W73.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2018-2020. All rights reserved.
  ******************************************************************************
  */

#include <stdint.h>
#include <stdio.h>
#include "se_common.h"
#include ADC_module_header
#include CLG_module_header
#include DMAC_module_header
#include I2C_module_header
#include LCD32C_module_header
#include PPORT_module_header
#include QSPI_module_header
#include REMC_module_header
#include RFC_module_header
#include RTC_module_header
#include SND_module_header
#include SPI_module_header
#include SVD_module_header
#include T16_module_header
#include T16B_module_header
#include UART_module_header
#include USB_module_header
#include WDT_module_header
#include DMAUSB_module_header
#include DMASPI_module_header
#include DMAQSPI_module_header
#include SRC_module_header

/*----------------------------------------------------------------------------
  Peripheral ports definitions
 *----------------------------------------------------------------------------*/

//-------------- Valid PPORT pins --------------------------------------------

const uint8_t ValidPortTable [14] =
{
  0xFF,            // Ports P07 - P00
  0xFF,            // Ports P17 - P10
  0xFF,            // Ports P27 - P20
  0xFF,            // Ports P37 - P30
  0x1F,            // Ports P47 - P40
  0xFF,            // Ports P57 - P50
  0xFF,            // Ports P67 - P60
  0xFF,            // Ports P77 - P70
  0xFF,            // Ports P87 - P80
  0x00,            // Ports P97 - P90
  0x00,            // Ports PA7 - PA0
  0x00,            // Ports PB7 - PB0
  0x00,            // Ports PC7 - PC0
  0x0F             // Ports PD7 - PD0
};

//-------------- Valid PPORT alternate functions -----------------------------
// Define valid alternate function settings for each PPort pin
// These values are descrete bits only for the table not for the register
#define altfunc0   0x01
#define altfunc1   0x02
#define altfunc2   0x04
#define altfunc3   0x08
const  uint8_t ValidAltFunctionTable[] =
{ 
  ( altfunc0 | altfunc1                     ), // P00   SDI1    / UPMUX
  ( altfunc0 | altfunc1                     ), // P01   SDO1    / UPMUX
  ( altfunc0 | altfunc1                     ), // P02   SPICLK1 / UPMUX
  ( altfunc0 | altfunc1                     ), // P03   #SPISS1 / UPMUX
  ( altfunc0 | altfunc1                     ), // P04   SENB0   / UPMUX
  ( altfunc0 | altfunc1                     ), // P05   SENA0   / UPMUX
  ( altfunc0 | altfunc1                     ), // P06   REF0    / UPMUX
  ( altfunc0 | altfunc1                     ), // P07   RFIN0   / UPMUX
  ( altfunc0 | altfunc1                     ), // P10   BZOUT   / UPMUX
  ( altfunc0 | altfunc1                     ), // P11   #BZOUT  / UPMUX
  ( altfunc0 | altfunc1                     ), // P12   EXCL00  / UPMUX
  ( altfunc0 | altfunc1                     ), // P13   EXCL10  / UPMUX
  ( altfunc0 | altfunc1                     ), // P14   FOUT    / UPMUX
  ( altfunc0 | altfunc1                     ), // P15   REMO    / UPMUX
  ( altfunc0 | altfunc1                     ), // P16   CLPLS   / UPMUX
  ( altfunc0 | altfunc1                     ), // P17   RTC1S   / UPMUX
  ( altfunc0 | altfunc1                     ), // P20   QSPICLK / UPMUX
  ( altfunc0 | altfunc1                     ), // P21   QSPIO0  / UPMUX
  ( altfunc0 | altfunc1                     ), // P22   QSPIO1  / UPMUX
  ( altfunc0 | altfunc1                     ), // P23   QSPIO2  / UPMUX
  ( altfunc0 | altfunc1                     ), // P24   QSPIO3  / UPMUX
  ( altfunc0 | altfunc1                     ), // P25   #QSPISS / UPMUX
  ( altfunc0 | altfunc1                     ), // P26   EXOSC   / UPMUX
  ( altfunc0 | altfunc1                     ), // P27   RFCLKO0 / UPMUX
  (            altfunc1 | altfunc2          ), // P30   UPMUX   / ADIN00
  (            altfunc1 | altfunc2          ), // P31   UPMUX   / ADIN01
  (            altfunc1 | altfunc2          ), // P32   UPMUX   / ADIN02
  (            altfunc1 | altfunc2          ), // P33   UPMUX   / ADIN03
  (            altfunc1 | altfunc2          ), // P34   UPMUX   / ADIN04
  (            altfunc1 | altfunc2          ), // P35   UPMUX   / ADIN05
  (            altfunc1 | altfunc2          ), // P36   UPMUX   / ADIN06
  (            altfunc1 | altfunc2          ), // P37   UPMUX   / VREFA
  ( altfunc0                                ), // P40   EXCL01
  ( altfunc0 | altfunc1                     ), // P41   EXCL11  / #ADTRG0
  ( altfunc0                                ), // P42   LFRO
  (                       altfunc2          ), // P43   EXSVD0
  (                       altfunc2          ), // P44   EXSVC1
  ( 0                                       ), // P45   None
  ( 0                                       ), // P46   None
  ( 0                                       ), // P47   None
  (                                altfunc3 ), // P50   COM16   / SEG95
  (                                altfunc3 ), // P51   COM17   / SEG94
  (                                altfunc3 ), // P52   COM18   / SEG93
  (                                altfunc3 ), // P53   COM19   / SEG92
  (                                altfunc3 ), // P54   COM20   / SEG91
  (                                altfunc3 ), // P55   COM21   / SEG90
  (                                altfunc3 ), // P56   COM22   / SEG89
  (                                altfunc3 ), // P57   COM23   / SEG88
  (                                altfunc3 ), // P60   COM24   / SEG87
  (                                altfunc3 ), // P61   COM25   / SEG86
  (                                altfunc3 ), // P62   COM26   / SEG85
  (                                altfunc3 ), // P63   COM27   / SEG84
  (                                altfunc3 ), // P64   COM28   / SEG83
  (                                altfunc3 ), // P65   COM29   / SEG82
  (                                altfunc3 ), // P66   COM30   / SEG81
  (                                altfunc3 ), // P67   COM31   / SEG80
  (                                altfunc3 ), // P70   SEG79
  (                                altfunc3 ), // P71   SEG78
  (                                altfunc3 ), // P72   SEG77
  (                                altfunc3 ), // P73   SEG76
  (                                altfunc3 ), // P74   SEG75
  (                                altfunc3 ), // P75   SEG74
  (                                altfunc3 ), // P76   SEG73
  (                                altfunc3 ), // P77   SEG72
  (                                altfunc3 ), // P80   SEG71
  (                                altfunc3 ), // P81   SEG70
  (                                altfunc3 ), // P82   SEG69
  (                                altfunc3 ), // P83   SEG68
  (                                altfunc3 ), // P84   SEG67
  (                                altfunc3 ), // P85   SEG66
  (                                altfunc3 ), // P86   SEG65
  (                                altfunc3 ), // P87   SEG64
  ( 0                                       ), // P90   None
  ( 0                                       ), // P91   None
  ( 0                                       ), // P92   None
  ( 0                                       ), // P93   None
  ( 0                                       ), // P94   None
  ( 0                                       ), // P95   None
  ( 0                                       ), // P96   None
  ( 0                                       ), // P97   None
  ( 0                                       ), // Pa0   None
  ( 0                                       ), // Pa1   None
  ( 0                                       ), // Pa2   None
  ( 0                                       ), // Pa3   None
  ( 0                                       ), // Pa4   None
  ( 0                                       ), // Pa5   None
  ( 0                                       ), // Pa6   None
  ( 0                                       ), // Pa7   None
  ( 0                                       ), // Pb0   None
  ( 0                                       ), // Pb1   None
  ( 0                                       ), // Pb2   None
  ( 0                                       ), // Pb3   None
  ( 0                                       ), // Pb4   None
  ( 0                                       ), // Pb5   None
  ( 0                                       ), // Pb6   None
  ( 0                                       ), // Pb7   None
  ( 0                                       ), // Pc0   None
  ( 0                                       ), // Pc1   None
  ( 0                                       ), // Pc2   None
  ( 0                                       ), // Pc3   None
  ( 0                                       ), // Pc4   None
  ( 0                                       ), // Pc5   None
  ( 0                                       ), // Pc6   None
  ( 0                                       ), // Pc7   None
  ( altfunc0                                ), // Pd0   SWCLK
  ( altfunc0                                ), // Pd1   SWD
  (                       altfunc2          ), // Pd2   OSC3
  (                       altfunc2          ), // Pd3   OSC5
  ( 0                                       ), // Pd4   None
  ( 0                                       ), // Pd5   None
  ( 0                                       ), // Pd6   None
  ( 0                                       ), // Pd7   None
};


//-------------- Valid UPMUX parameters tables -------------------------------
uint8_t ValidUpMuxPortTbl[] = {          // Valid UPMUX port pins (P00 to P37)
  0xFF,      // P07 - P00
  0xFF,      // P17 - P10
  0xFF,      // P27 - P20
  0xFF       // P37 - P30
};


uint8_t ValidUpMuxPeriph = 0x1F;         // invalid, invalid, invalid, T16B, UART, invalid, I2C, None

uint8_t ValidUpMuxChannelTbl[] = {       // Valid channels for each peripheral
  0xFF,      // "None" peripheral
  0x03,      // I2C CH 0 and 1 are valid
  0x01,      // SPIA CH 0 are valid
  0x03,      // UART3 CH 0 and 1 are valid
  0x03,      // T16B CH 0 and 1 are valid
  0x00,      // Reserved
  0x00,      // Reserved
  0x00       // Reserved
};

uint8_t ValidUpMuxPinFuncTbl[] = {       // Valid peripheral pin functions
  0xFF,      // "None" peripheral
  0x07,      // I2C has 3 pins valid
  0x1F,      // SPIA
  0x07,      // UART has 3 pins valid
  0x7F,      // T16B has 3 pins valid
  0x00,      // Reserved
  0x00,      // Reserved
  0x00       // Reserved
};


/**
  * @brief  Initialize for using a port as a general-purpose input port.
  * @param  portId: This parameter can be a value of @ref sePPORT_Id.
  * @param  config: This parameter can be a value of @ref config.
  * @retval State: can be a value of @ref seState
  */
seStatus sePPORT_Init(sePPORT_Id portId, uint32_t config)
{
  seStatus fStatus = seSTATUS_OK;
  
  if((config & 0x00000001) == 0)                               ///< GPIO
  {
    if(((config >> 12) & 0x00000001) == 0)                     ///< Dir = Input
	{
      sePPORT_InitAsInput((sePPORT_Id)portId);
      switch((config >> 16) & 0x00000007)                     ///< PullUp&down Resistor
      {
        case 0:
          sePPORT_EnablePullUpResistor((sePPORT_Id)portId);    ///< enable pullup control
          break;
        case 1:
          sePPORT_EnablePullDownResistor((sePPORT_Id)portId);  ///< enable pulldown control
          break;
        case 2:
          sePPORT_DisableBuiltInResistor((sePPORT_Id)portId);  ///< disable pullup&pulldown control
          break;
       }
       if(((config >> 14) & 0x00000001) == 0)                 ///< Chattering Filte
       {
         sePPORT_DisableChatteringFilter((sePPORT_Id)portId); ///< disable
       }
       else
       {
          sePPORT_EnableChatteringFilter((sePPORT_Id)portId); ///< enable
       }
    }
    else                                                      ///< Dir = Output
    {
      sePPORT_InitAsOutput((sePPORT_Id)portId);
      sePPORT_SetOutput((sePPORT_Id)portId, (sePPORT_Data)((config >> 13) & 0x00000001) );
    }
  }
  else                                                        ///< OTHER
  {
    if( ((config >> 1) & 0x0003) == 1)                        ///< Port Alt Function 1
    {
      ///< Program UPMUX first, if needed
      fStatus = sePPORT_UpMuxFunction( (sePPORT_Id)portId, (seUPMUX_Peripheral_Sel)((config >> 3) & 0x0007) 
						,(seUPMUX_Channel_Sel)((config >> 6) & 0x0003) , (uint8_t)((config >> 8) & 0x0007) );
			
			
    }
    ///< Port Alt Function 0
    ///< Port Alt Function 2
    ///< Port Alt Function 3
    if(fStatus == seSTATUS_OK)
    {
      fStatus = sePPORT_InitAsAltFunction( portId, (sePPORT_AltFunc)((config >> 1) & 0x0003) );
    }
  }
  return fStatus;
}


// ADC12A ---------------------------------------------------------------------
/**
  * @brief  Configures ADC12A clock.
  * @param  ADC12Ax: ADC12A peripheral
  * @param  clock:  clock source and divider
  * @retval None
  */
seStatus seAUX_ConfigureClockForADC12a( ADC12A_0_Type* ADC12Ax, seADC12A_Clk clock ) {
  
  T16_0_Type* T16x = 0;
  
  if ( ADC12Ax == ADC12A_0 ) {
    T16x = T16_7;
  } else {
    return seSTATUS_NG;
  }
  
  ///< Configure the ADC12A operating clock.
  T16x->CLK_b.CLKSRC = seSRC(clock); ///< Clock source selection
  T16x->CLK_b.CLKDIV = seDIV(clock); ///< Clock division ratio selection
  T16x->CLK_b.DBRUN  = seDBRUN;      ///< Whether clock supplied in DEBUG mode or not.
  
  return seSTATUS_OK;
}


// CLK ------------------------------------------------------------------------
/**
  * @brief  Configures ports for this module. Override this function to configure specific ports.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seAUX_ConfigurePortsForOSC3( void ) {

  seStatus fStatus = seSTATUS_OK;
  
  ///< Disable both the Input and Output
  PPORT->PDIOEN_b.PDIEN2 = 0;
  PPORT->PDIOEN_b.PDOEN2 = 0;
  PPORT->PDIOEN_b.PDIEN3 = 0;
  PPORT->PDIOEN_b.PDOEN3 = 0;
  
  ///< Clear the MODSEL bit to switch function to GPIO temporarily
  PPORT->PDMODSEL_b.PDSEL2 = 0;
  PPORT->PDMODSEL_b.PDSEL3 = 0;
  
  ///< Setup the requested Alternate Function
  PPORT->PDFNCSEL_b.PD2MUX = sePPORT_ALT_2;
  PPORT->PDFNCSEL_b.PD3MUX = sePPORT_ALT_2;
  
  ///< Switch to Alternate Function mode for that pin
  PPORT->PDMODSEL_b.PDSEL2 = 1;
  PPORT->PDMODSEL_b.PDSEL3 = 1;

  return fStatus;
}


// QSPI -----------------------------------------------------------------------
/**
  * @brief  Configures QSPICLKn frequency [Hz] (= baud rate [bps])
  * @param  QSPIx: QSPI peripheral
  * @param  speed: QSPICLKn frequency [Hz]
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seAUX_SetBusSpeedForQSPI( QSPI_0_Type* QSPIx, uint32_t speed ) {
  
  seStatus fStatus = seSTATUS_NG;
  T16_0_Type* T16x = 0;
  
  if ( QSPIx == QSPI_0 ) {
    T16x = T16_2;   
  } else {
    return seSTATUS_NG;
  }
  
  if ( speed != 0 ) {
    if ( QSPIx->MOD_b.NOCLKDIV == 0 ) {
      uint32_t qspiclk = seAUX_GetClkSpeedForT16( T16x );
      if ( qspiclk/2 >= speed ) {
        fStatus = seSTATUS_OK;
        T16x->TR = qspiclk/speed/2 - 1;
      }
      else{
        T16x->TR = 0; ///< Limit bus speed to max possible with this clocks
      }
    } 
  } 
  
  return fStatus;
}

/**
  * @brief  Discovers QSPICLKn frequency [Hz] (= baud rate [bps])
  * @param  QSPIx: QSPI peripheral
  * @retval QSPICLKn: frequency [Hz]
  */
uint32_t seAUX_GetBusSpeedForQSPI( QSPI_0_Type* QSPIx ) {
  
  uint32_t qspiclk = 0;
  uint32_t qspidiv = 1;
  T16_0_Type* T16x = 0;
  
  if ( QSPIx == QSPI_0 ) {
    T16x = T16_2;   
  } else {
    return 0;
  }  
  
  qspiclk = seAUX_GetClkSpeedForT16( T16x );

  switch ( QSPIx->MOD_b.NOCLKDIV ) {
  case 1: 
    break;
  case 0: 
    qspiclk = qspiclk/2/( T16x->TR + 1) ;
    break;
  default: 
    break; 
  }
  
  return ( qspiclk / qspidiv );
}

/**
  * @brief  Starts bus clock for QSPI
  * @param  QSPIx: QSPI peripheral
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seAUX_StartBusClockForQSPI( QSPI_0_Type* QSPIx ) {
  
  seStatus fStatus = seSTATUS_OK;
  T16_0_Type* T16x = 0;
  
  if ( QSPIx == QSPI_0 ) {
    T16x = T16_2;   
  } else {
    return seSTATUS_NG;
  }  
  
  seT16_Enable( T16x );
  if ( QSPIx->MOD_b.NOCLKDIV == seDISABLE ) {
    ///< Timer does not to be started when baud rate generator is not used
    seT16_Start( T16x );
  }
  
  return fStatus;
}

/**
  * @brief  Stops bus clock for QSPI
  * @param  QSPIx: QSPI peripheral
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seAUX_StopBusClockForQSPI( QSPI_0_Type* QSPIx ) {
    
  seStatus fStatus = seSTATUS_OK;
  T16_0_Type* T16x = 0;
  
  if ( QSPIx == QSPI_0 ) {
    T16x = T16_2;   
  } else {
    return seSTATUS_NG;
  }  
  
  seT16_Disable( T16x );
  if ( QSPIx->MOD_b.NOCLKDIV == seDISABLE ) {
    ///< Timer does not to be started when baud rate generator is not used
    seT16_Stop( T16x );
  }
  
  return fStatus;
}


// SNDA -----------------------------------------------------------------------
/**
  * @brief  Discovers SNDA frequency [Hz]
  * @param  SNDA: SNDA peripheral
  * @retval SNDACLK: frequency [Hz]
  */
uint32_t seAUX_GetClkSpeedForSNDA( SNDA_Type* SNDAx ) {

  uint32_t clk;
  uint32_t div;

  switch ( SNDAx->CLK_b.CLKSRC ) {
  case seCLG_IOSC:
  case seCLG_OSC3:
    div = 1 << SNDA->CLK_b.CLKDIV;
    break;
  case seCLG_OSC1:
  case seCLG_EXOSC:
  default:
      div = 1;
    break;
  }

  switch ( SNDA->CLK_b.CLKSRC ) {
  case seCLG_OSC1:
    clk = seGetOsc1XtalFreq();
    break;
  case seCLG_IOSC:
    clk = seCLG_GetIoscFreqSel();
    break;
  case seCLG_OSC3:
    clk = seGetOsc3XtalFreq();
    break;
  case seCLG_EXOSC:
    clk = seGetExOscFreq();
    break;
  default:
    clk = 0;
    break;
  }

  return ( clk / div );
}

// SPI ------------------------------------------------------------------------
/**
  * @brief  Configures SPICLKn frequency [Hz] (= baud rate [bps])
  * @param  SPIx: SPI peripheral
  * @param  speed: SPICLKn frequency [Hz]
  * @retval Status: can be a value of @ref seStatus
  */
seStatus  seAUX_SetBusSpeedForSPI( SPIA_0_Type* SPIx, uint32_t speed ) {
  
  seStatus fStatus = seSTATUS_NG;
  T16_0_Type* T16x = 0;
  uint32_t spiclk;
  
  if ( SPIx == SPIA_0 ) {
    T16x = T16_1;
  } else if ( SPIx == SPIA_1 ) {
    T16x = T16_6;  
  } else {
    return seSTATUS_NG;
  }
  
  if ( speed != 0 ) {
    if ( SPIx->MOD_b.NOCLKDIV == 0 ) {  
      spiclk = seAUX_GetClkSpeedForT16( T16x );
      if ( spiclk ) {
        fStatus = seSTATUS_OK;
        T16x->TR = spiclk/speed/2 - 1;
      }
    }
  }

  return fStatus;
}

/**
  * @brief  Discovers SPICLKn frequency [Hz] (= baud rate [bps])
  * @param  SPIx: SPI peripheral
  * @retval SPICLKn: frequency [Hz] 
  */
uint32_t seAUX_GetBusSpeedForSPI( SPIA_0_Type* SPIx ) {
  
  T16_0_Type* T16x = 0;
  uint32_t spiclk;
  
  if ( SPIx == SPIA_0 ) {
    T16x = T16_1;
  } else if ( SPIx == SPIA_1 ) {
    T16x = T16_6;
  } else {
    return 0;
  }
  
  spiclk = seAUX_GetClkSpeedForT16( T16x );
  
  if ( SPIx->MOD_b.NOCLKDIV == 0 && spiclk != 0 ) {
    spiclk /= 2;
    spiclk /= T16x->TR + 1;
  }
  
  return spiclk;
}

/**
  * @brief  Starts bus clock for SPI
  * @param  SPIx: SPI peripheral
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seAUX_StartBusClockForSPI( SPIA_0_Type* SPIx ) {
  
  seStatus fStatus = seSTATUS_OK;
  T16_0_Type* T16x = 0;
  
  if ( SPIx == SPIA_0 ) {
    T16x = T16_1;
  } else if ( SPIx == SPIA_1 ) {
    T16x = T16_6;
  } else {
    return seSTATUS_NG;
  }

    seT16_Enable( T16x );
    if ( SPIx->MOD_b.NOCLKDIV == seDISABLE ) {
      ///< Timer does not to be started when baud rate generator is not used
      seT16_Start( T16x );
    }  
 
  return fStatus;
}

/**
  * @brief  Stops bus clock for SPI
  * @param  SPIx: SPI peripheral
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seAUX_StopBusClockForSPI( SPIA_0_Type* SPIx ) {
  
  seStatus fStatus = seSTATUS_OK;
  T16_0_Type* T16x = 0;
  
  if ( SPIx == SPIA_0 ) {
    T16x = T16_1;
  } else if ( SPIx == SPIA_1 ) {
    T16x = T16_6;
  } else {
    return seSTATUS_NG;
  }

    seT16_Disable( T16x );
    if ( SPIx->MOD_b.NOCLKDIV == seDISABLE ) {
      ///< Timer does not to be started when baud rate generator is not used
      seT16_Stop( T16x );
    }  
 
  return fStatus;
}


// T16 channels ---------------------------------------------------------------
/**
  * @brief  Discovers T16 channel for a peripherial.
  * @param  PeriphAddr: pointer to a peripheral.
  * @retval T16 channel.
  */
T16_0_Type * seAUX_GetT16ch_ForPeripheral( void * PeriphAddr ) {
  
  T16_0_Type * T16x = 0;
  
  if ( (SPIA_0_Type*)PeriphAddr == SPIA_0 ) {
    T16x = T16_1;
  } else if ( (SPIA_0_Type*)PeriphAddr == SPIA_1 ) {
    T16x = T16_6;
  } else if ( (QSPI_0_Type*)PeriphAddr == QSPI_0 ) {
    T16x = T16_2; 
  }
  
  return T16x;
}

/**
  * @brief  Discovers T16 clock from registers.
  * @param  T16x: pointer to T16 peripheral.
  * @retval T16 clock.
  */
uint32_t seAUX_GetClkSpeedForT16( T16_0_Type* T16x ) { 

  uint32_t t16clk;
  uint32_t t16div;

  switch ( T16x->CLK_b.CLKSRC ) {
  case seCLG_IOSC:
  case seCLG_OSC3:
    t16div = 1 << T16x->CLK_b.CLKDIV;
    break;
  case seCLG_OSC1:
    if ( T16x->CLK_b.CLKDIV <= seT16_OSC1_CLKDIV_256 ) {
     t16div = 1 << T16x->CLK_b.CLKDIV;
     break;
  }
  case seCLG_EXOSC:
  default:
    t16div = 1;
    break;
  }

  switch ( T16x->CLK_b.CLKSRC ) {
  case seCLG_OSC1:
    t16clk = seGetOsc1XtalFreq();
    break;
  case seCLG_IOSC:
    t16clk = seCLG_GetIoscFreqSel();
    break;
  case seCLG_OSC3:
    t16clk = seGetOsc3XtalFreq();
    break;
  case seCLG_EXOSC:
    t16clk = seGetExOscFreq();
    break;
  default:
    t16clk = 0;
    break;
  }

  return ( t16clk / t16div );
}

// T16B -----------------------------------------------------------------------
/**
  * @brief  Discovers T16B clock from registers.
  * @param  T16Bx: Pointer to T16B peripheral, a timer channel.
  * @retval T16B clock.
  */
uint32_t seAUX_GetClkSpeedForT16B( T16B_0_Type* T16Bx ) { 

  uint32_t t16bclk;
  uint32_t t16bdiv;

  switch ( T16Bx->CLK_b.CLKSRC ) {
  case seCLG_IOSC:
  case seCLG_OSC3:
    t16bdiv = 1 << T16Bx->CLK_b.CLKDIV;
    break;
  case seCLG_OSC1:
    if ( T16Bx->CLK_b.CLKDIV <= seT16B_OSC1_CLKDIV_256 ) {
     t16bdiv = 1 << T16Bx->CLK_b.CLKDIV;
     break;
  }
  case seCLG_EXOSC:
  default:
    t16bdiv = 1;
    break;
  }

  switch ( T16Bx->CLK_b.CLKSRC ) {
  case seCLG_OSC1:
    t16bclk = seGetOsc1XtalFreq();
    break;
  case seCLG_IOSC:
    t16bclk = seCLG_GetIoscFreqSel();
    break;
  case seCLG_OSC3:
    t16bclk = seGetOsc3XtalFreq();
    break;
  case seCLG_EXOSC:
    t16bclk = seGetExOscFreq();
    break;
  default:
    t16bclk = 0;
    break;
  }

  return ( t16bclk / t16bdiv );
}

// UART -----------------------------------------------------------------------
/**
  * @brief  Discovers UART clock from registers.
  * @param  UARTx: This parameter defines a uart channel and can be a value of @ref UART3_0_Type.
  * @retval UART clock frequency to use in baud rate calculation.
  */
uint32_t seAUX_GetClkSpeedForUart( UART3_0_Type* UARTx ) {

  uint32_t uartclk;
  uint32_t uartdiv;

  switch ( UARTx->CLK_b.CLKSRC ) {
  case seCLG_IOSC:
  case seCLG_OSC3:
    uartdiv = 1 << UARTx->CLK_b.CLKDIV;
    break;
  case seCLG_EXOSC:
  case seCLG_OSC1:
  default:
    uartdiv = 1;
    break;
  }

  switch ( UARTx->CLK_b.CLKSRC ) {
  case seCLG_OSC1:
    uartclk = seGetOsc1XtalFreq();
    break;
  case seCLG_IOSC:
    uartclk = seCLG_GetIoscFreqSel();
    break;
  case seCLG_OSC3:
    uartclk = seGetOsc3XtalFreq();
    break;
  case seCLG_EXOSC:
    uartclk = seGetExOscFreq();
    break;
  default:
    uartclk = 0;
    break;
  }

  return ( uartclk / uartdiv );
}

// USB -----------------------------------------------------------------------
/**
  * @brief  Configures USB clock.
  * @param  ClkSrc: clock source and divider
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seAUX_SetClkForUSB( seUSB_ClkSrc ClkSrc ) {

  seStatus fStatus = seSTATUS_NG;

    seCLG_Start( seCLG_OSC3 );
    if ( CLG->OSC_b.OSC3EN ) {
      fStatus = seSTATUS_OK;  
    }

  return fStatus;
}

/**
  * @brief  Activate USB clock.
  * @retval None
  */
void seUSB_ActivateUSBCLK( void ) {

  ///< Disable write-protect.
  seProtectSys( seWRITE_PROTECT_OFF );
  USBMISC->CTL_b.USBWAIT = 1;    ///< (Setting Bus access cycle)
  
  USBMISC->CTL_b.REG18VEN = 1;   ///< (Turn 1.8 V regulator on)
  USBMISC->CTL_b.REG33VEN = 1;   ///< (Turn 3.3 V regulator on)

  seSysSleepMS(15);
  
  USBMISC->CTL_b.USBPLLEN = 1;     ///<(Turn PLL on)
  
  seSysSleepMS(4);

  USBMISC->CTL_b.USBRST = 1;       ///< Release USB reset
  USBMISC->CTL_b.VBUSDET = 1;      ///< Enable VBUS detect

  ///< Enable write-protect.
  seProtectSys( seWRITE_PROTECT_ON );
}

// WDT -----------------------------------------------------------------------
/**
  * @brief  Discovers WDT clock from registers.
  * @retval Hz: WDT clock in Hz.
  */
uint32_t seAUX_GetClkSpeedForWDT( void ) {

  uint32_t wdtclk;
  uint32_t wdtdiv; 

  switch ( WDT2->CLK_b.CLKSRC ) {
  case seCLG_IOSC:
    wdtdiv = ( 1 << ( WDT2->CLK_b.CLKDIV + 13 ) );
    break;
  case seCLG_OSC3:
    wdtdiv = ( 1 << ( WDT2->CLK_b.CLKDIV + 13 ) );
    break;
  case seCLG_OSC1:
    wdtdiv = 128;
    break;
  case seCLG_EXOSC:
  default:
    wdtdiv = 1;
    break;
  }

  switch ( WDT2->CLK_b.CLKSRC ) {
  case seCLG_OSC1:
    wdtclk = seGetOsc1XtalFreq();
    break;
  case seCLG_IOSC:
    wdtclk = seCLG_GetIoscFreqSel();
    break;
  case seCLG_OSC3:
    wdtclk = seGetOsc3XtalFreq();
    break;
  case seCLG_EXOSC:
    wdtclk = seGetExOscFreq();
    break;
  default:
    wdtclk = 0;
    break;
  }

  return ( wdtclk / wdtdiv );
}

// DMAC -----------------------------------------------------------------------
// Data Transmition
/**
  * @brief  For Memory-to-Peripheral DMAC transfer this function generates DMAC control data and sets it in the DMAC channel descriptor.
  *         Returns a register pointer further used to enable DMAC request.
  * @param  saddress: Source address. The size of the transfer defined in the function based on the peripheral type.
  * @param  periph: Pointer to a peripheral.
  * @param  size_m1: Transfer size minus one.
  * @param  chan: DMAC channel can be @ref seDMAC_CHANNEL
  * @retval Ptr: The register pointer further used to trigger DMAC transfer.
  */
volatile uint16_t* seAUX_ConfigMemToPeriphForDMAC( uint32_t saddress, uint32_t periph, uint32_t size_m1, seDMAC_CHANNEL chan ) { 
  
  uint32_t cdata1 = 0;
  volatile uint16_t* enable_trigger = NULL;
  
  switch ( periph ) {
    case (uint32_t)UART3_0:
    case (uint32_t)UART3_1:
      enable_trigger = &((UART3_0_Type*)periph)->TBEDMAEN;
      cdata1 = seDMAC_cdata( seDMAC_MODE_BASIC,size_m1,0UL,seDMAC_SIZE_BYTE,seDMAC_SIZE_BYTE,seDMAC_INC_1,seDMAC_INC_NO );
      seDMAC_SetChannel( cdata1, ( uint32_t )saddress+size_m1, ( uint32_t )&((UART3_0_Type*)periph)->TXD, chan ); 
      break; 
    
    case (uint32_t)T16B_0:
    case (uint32_t)T16B_1:
      enable_trigger = &((T16B_0_Type*)periph)->CC0DMAEN;
      cdata1 = seDMAC_cdata( seDMAC_MODE_BASIC,size_m1,0UL,seDMAC_SIZE_HALF_WORD,seDMAC_SIZE_HALF_WORD,seDMAC_INC_2,seDMAC_INC_NO );
      seDMAC_SetChannel( cdata1, ( uint32_t )saddress+size_m1*2, ( uint32_t )&((T16B_0_Type*)periph)->CCR0, chan );
      break; 
    
    case (uint32_t)SPIA_0:
    case (uint32_t)SPIA_1:
      enable_trigger = &((SPIA_0_Type*)periph)->TBEDMAEN;
      cdata1 = seDMAC_cdata( seDMAC_MODE_BASIC,size_m1,0UL,seDMAC_SIZE_HALF_WORD,seDMAC_SIZE_HALF_WORD,seDMAC_INC_2,seDMAC_INC_NO );
      seDMAC_SetChannel( cdata1, ( uint32_t )saddress+size_m1*2, ( uint32_t )&((SPIA_0_Type*)periph)->TXD, chan );
      break; 
        
    case (uint32_t)I2C_0:
    case (uint32_t)I2C_1:
      enable_trigger = &((I2C_0_Type*)periph)->TBEDMAEN;
      cdata1 = seDMAC_cdata( seDMAC_MODE_BASIC,size_m1,0UL,seDMAC_SIZE_BYTE,seDMAC_SIZE_BYTE,seDMAC_INC_1,seDMAC_INC_NO );
      seDMAC_SetChannel( cdata1, ( uint32_t )saddress+size_m1, ( uint32_t )&((I2C_0_Type*)periph)->TXD, chan );
      break; 
    
    case (uint32_t)QSPI_0:
      enable_trigger = &QSPI_0->TBEDMAEN;
      cdata1 = seDMAC_cdata( seDMAC_MODE_BASIC,size_m1,0UL,seDMAC_SIZE_HALF_WORD,seDMAC_SIZE_HALF_WORD,seDMAC_INC_2,seDMAC_INC_NO );
      seDMAC_SetChannel( cdata1, ( uint32_t )saddress+size_m1*2, ( uint32_t )&QSPI_0->TXD, chan );
      break;
 
    case (uint32_t)USB:
      enable_trigger = &USBMISC->WRDMAEN;
      cdata1 = seDMAC_cdata( seDMAC_MODE_BASIC,size_m1,0UL,seDMAC_SIZE_BYTE,seDMAC_SIZE_BYTE,seDMAC_INC_1,seDMAC_INC_NO );
      seDMAC_SetChannel( cdata1, ( uint32_t )saddress+size_m1, ( uint32_t )&USB->FIFODAT, chan );
      break; 
       
    case (uint32_t)SNDA:
      enable_trigger = &SNDA->EMDMAEN;
      cdata1 = seDMAC_cdata( seDMAC_MODE_BASIC,size_m1,0UL,seDMAC_SIZE_HALF_WORD,seDMAC_SIZE_HALF_WORD,seDMAC_INC_2,seDMAC_INC_NO );
      seDMAC_SetChannel( cdata1, ( uint32_t )saddress+size_m1*2, ( uint32_t )&SNDA->DAT, chan );
      break;
    
    default: break;
  }
  
  return enable_trigger;
}

// Data Reception
/**
  * @brief  For Peripheral-to-Memory DMAC transfer this function generates DMAC control data and sets it in the DMAC channel descriptor.
  *         Returns a register pointer further used to trigger DMAC transfer.
  * @param  periph: Pointer to a peripheral.
  * @param  daddress: Destination address. The size of the transfer defined in the function based on the peripheral type.
  * @param  size_m1: Transfer size minus one.
  * @param  chan: DMAC channel can be @ref seDMAC_CHANNEL
  * @retval Ptr: The register pointer further used to trigger DMAC transfer.
  */
volatile uint16_t* seAUX_ConfigPeriphToMemForDMAC( uint32_t periph,  uint32_t daddress, uint32_t size_m1, seDMAC_CHANNEL chan ) {

  uint32_t cdata1 = 0;
  volatile uint16_t*enable_trigger = NULL;
  
  switch ( periph ) {
    case (uint32_t)UART3_0:
    case (uint32_t)UART3_1:
      enable_trigger = &((UART3_0_Type*)periph)->RB1FDMAEN;
      cdata1 = seDMAC_cdata( seDMAC_MODE_BASIC,size_m1,0,seDMAC_SIZE_BYTE,seDMAC_SIZE_BYTE,seDMAC_INC_NO,seDMAC_INC_1);
      seDMAC_SetChannel( cdata1, ( uint32_t )&((UART3_0_Type*)periph)->RXD, daddress+size_m1, chan );
      break;
    
    case (uint32_t)T16B_0:
    case (uint32_t)T16B_1:
      enable_trigger = &((T16B_0_Type*)periph)->CC1DMAEN;
      cdata1 = seDMAC_cdata( seDMAC_MODE_BASIC,size_m1,0,seDMAC_SIZE_HALF_WORD,seDMAC_SIZE_HALF_WORD,seDMAC_INC_NO,seDMAC_INC_2 );
      seDMAC_SetChannel( cdata1, ( uint32_t )&((T16B_0_Type*)periph)->CCR1, daddress+size_m1*2, chan );
      break;
    
    case (uint32_t)SPIA_0:
	case (uint32_t)SPIA_1:
      enable_trigger = &((SPIA_0_Type*)periph)->RBFDMAEN;
      cdata1 = seDMAC_cdata( seDMAC_MODE_BASIC,size_m1,0,seDMAC_SIZE_HALF_WORD,seDMAC_SIZE_HALF_WORD,seDMAC_INC_NO,seDMAC_INC_2 );
      seDMAC_SetChannel( cdata1, ( uint32_t )&((SPIA_0_Type*)periph)->RXD, daddress+size_m1*2, chan );
      break;
    
    case (uint32_t)I2C_0:
    case (uint32_t)I2C_1:
      enable_trigger = &((I2C_0_Type*)periph)->RBFDMAEN;
      cdata1 = seDMAC_cdata( seDMAC_MODE_BASIC,size_m1,0,seDMAC_SIZE_BYTE,seDMAC_SIZE_BYTE,seDMAC_INC_NO,seDMAC_INC_1 );
      seDMAC_SetChannel( cdata1, ( uint32_t )&((I2C_0_Type*)periph)->RXD, daddress+size_m1, chan );
      break;
 
    case (uint32_t)QSPI_0:
      enable_trigger = &QSPI_0->RBFDMAEN;
      cdata1 = seDMAC_cdata( seDMAC_MODE_BASIC,size_m1,0,seDMAC_SIZE_HALF_WORD,seDMAC_SIZE_HALF_WORD,seDMAC_INC_NO,seDMAC_INC_2 );
      seDMAC_SetChannel( cdata1, ( uint32_t )&QSPI_0->RXD, daddress+size_m1*2, chan );
      break;
       
   case (uint32_t)USB:
      enable_trigger = &USBMISC->RDDMAEN;
      cdata1 = seDMAC_cdata( seDMAC_MODE_BASIC,size_m1,0,seDMAC_SIZE_BYTE,seDMAC_SIZE_BYTE,seDMAC_INC_NO,seDMAC_INC_1 );
      seDMAC_SetChannel( cdata1, ( uint32_t )&USB->FIFODAT, daddress+size_m1, chan );
      break;
   
    default: break;
  }

  return enable_trigger;
}

