/**
  ******************************************************************************
  * @file    se_clg.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file provides all the CLG hardware functions.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#include "se_clg.h"

/**
  * Static functions declarations
**/
static seStatus StartOsc1( void );
static void StopOsc1( void );
static void EnableOsc1StopDetection( void );
static void DisableOsc1StopDetection( void );
static seStatus StartOsc3( void );
static void StopOsc3( void );
static seStatus StartIosc( void );
static void StopIosc( void );
static seStatus RunIoscAutoTrimming( seCLG_ClkSrc temp_clock );
static seStatus NeedSwitch( seCLG_Clk clock );

/**
  * @brief  Initializes the CLG peripheral according to the specified
  *         parameters in the CLG_InitStruct.
  * @param  InitStruct: Pointer to a @ref seCLG_InitTypeDef structure that
  *         contains the configuration information for the specified CLG peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seCLG_Init( seCLG_InitTypeDef* InitStruct ) {

  seStatus fStatus = seSTATUS_OK;

  ///< Check if selected clock is system clock
  seStatus fSwitch = NeedSwitch( InitStruct->SysClk );

  ///< Selects a new clock source for the System clock
  if ( fSwitch ) {
    fStatus = seCLG_Start( seSRC(InitStruct->SysClk) );
  }

  ///< Sets Stop Clock mode used while in sleep
  if ( fStatus ) {
    fStatus = seCLG_SetOperInSlp( seSRC(InitStruct->SysClk), InitStruct->OperationInSlpEn );
  }

  ///< Sets SYSCLK switching function at wake-up
  if ( fStatus ) {
    fStatus = seCLG_SetWkUpSysClk( InitStruct->WkUpSysClk, InitStruct->SysClkSwitchOnWkUpEn );
  }

  ///< Selects a new clock source for the System clock
  if ( fStatus && fSwitch ) {
    seCLG_SwitchSysClk( InitStruct->SysClk );
  }

  return fStatus;
}

/**
  * @brief  Checks if clock is enabled.
  * @param  clock: Clock source see @ref seCLG_ClkSrc.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seCLG_IsEnabled(seCLG_ClkSrc clock) {
  
  seStatus fStatus = seSTATUS_NG;
  
  switch (clock) {
  case seCLG_IOSC:
    if ( CLG->OSC_b.IOSCEN ) {
      fStatus = seSTATUS_OK;
    }
    break;
    
  case seCLG_OSC1:
    if ( CLG->OSC_b.OSC1EN ) {
      fStatus = seSTATUS_OK;
    }
    break;

  case seCLG_OSC3:
    if ( CLG->OSC_b.OSC3EN ) {
      fStatus = seSTATUS_OK;
    }
    break;

  case seCLG_EXOSC:
    if ( CLG->OSC_b.EXOSCEN ) {
      fStatus = seSTATUS_OK;
    }
    break;
    
  default:
    break;
  }
  
  return fStatus;
}

/**
  * @brief  Selects a new clock source for the System clock.
  * @param  clock: Clock source see @ref seCLG_Clk.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seCLG_SwitchSysClk( seCLG_Clk clock ) {

  ///< Check if selected clock is enabled
  seStatus fStatus = seCLG_IsEnabled(seSRC(clock));

  ///< When Osc1 doesn't start, nothing is done.
  if ( fStatus == seSTATUS_OK ) {

    ///< Disable write-protect.
    seProtectSys( seWRITE_PROTECT_OFF );

    CLG->SCLK_b.CLKDIV = seDIV(clock);   ///< Set division ratio of the clock source.
    CLG->SCLK_b.CLKSRC = seSRC(clock);   ///< Set system clock source.

    ///< Enable write-protect.
    seProtectSys( seWRITE_PROTECT_ON );
  }

  return fStatus;
}

/**
  * @brief  Gets the SysClock frequency.
  * @retval Hz: Actual value of SysClock frequency in Hz.
  */
uint32_t seCLG_GetSysClk( void ) {

  uint32_t clk = 0;
  uint32_t cdiv = 1;

  ///< Gets the division ratio the SYSCLK frequency
  switch ( CLG->SCLK_b.CLKSRC ) {
  case seCLG_IOSC:
  case seCLG_OSC1:
    cdiv = 1 << CLG->SCLK_b.CLKDIV;
    break;
  case seCLG_EXOSC:
    cdiv = 1;
    break;
  case seCLG_OSC3:
    cdiv = CLG->SCLK_b.CLKDIV;
    if ( cdiv < 2 ) {
      cdiv = 1 << cdiv;
    } else {
      cdiv = 1 << (cdiv+1);
    }
    break;
  default:
    break;
  }

  ///< Gets the the SYSCLK frequency
  switch ( CLG->SCLK_b.CLKSRC ) {
  case seCLG_OSC1:
    clk = seGetOsc1XtalFreq();
    break;
  case seCLG_IOSC:
    clk = seCLG_GetIoscFreqSel();
    break;
  case seCLG_OSC3:
    clk = seGetOsc3XtalFreq();
    break;
  case seCLG_EXOSC:
    clk = seGetExOscFreq();
    break;
  default:
    break;
  }

  return ( clk / cdiv );
}

/**
  * @brief  Gets System clock source.
  * @retval Clock: a value of @ref seCLG_ClkSrc
  */
uint16_t seCLG_GetSysClkSrc( void ) {

  return CLG->SCLK_b.CLKSRC;
}

/**
  * @brief  Gets System clock divider.
  * @retval Divider: a value of @ref seCLG_ClkDiv
  */
uint16_t seCLG_GetSysClkDiv( void ) {

  return CLG->SCLK_b.CLKDIV;
}

/**
  * @brief  Gets the IOSC frequency.
  * @retval Hz: Actual value of IOSC frequency in Hz.
  */
uint32_t seCLG_GetIoscFreqSel( void ) {

  uint32_t freq;

  seGET_IOSC_FREQ(freq);

  return freq;
}

/**
  * @brief  Sets the IOSC frequency.
  * @param  freq: New frequency can be a value of @ref seCLG_IOSC_IoscFq.
  * @retval None
  */
void seCLG_SetIoscFreqSel( seCLG_IOSC_Freq freq ) {

  ///< Disable write-protect.
  seProtectSys( seWRITE_PROTECT_OFF );

  seSET_IOSC_FREQ( freq );

  ///< Enable write-protect.
  seProtectSys( seWRITE_PROTECT_ON );
}

/**
  * @brief  Starts Clock.
  * @param  clock: The clock to start.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seCLG_Start( seCLG_ClkSrc clock ) {

  seStatus fStatus = seSTATUS_NG;

  switch ( clock ) {
  case seCLG_IOSC:
    fStatus = StartIosc();
    break;

  case seCLG_OSC1:
    fStatus = StartOsc1();
    break;

  case seCLG_OSC3:
    fStatus = StartOsc3();
    break;

  default:
    break;
  }

  return fStatus;
}

/**
  * @brief  Stops Clock.
  * @param  clock: The clock @ref seCLG_ClkSrc to stop.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seCLG_Stop( seCLG_ClkSrc clock ) {

  seStatus fStatus = seSTATUS_NG;

  switch ( clock ) {
  case seCLG_IOSC:
    StopIosc();
    fStatus = seSTATUS_OK;
    break;

  case seCLG_OSC1:
    StopOsc1();
    fStatus = seSTATUS_OK;
    break;

  case seCLG_OSC3:
    StopOsc3();
    fStatus = seSTATUS_OK;
    break;

  default:
    break;
  }

  return fStatus;
}

/**
  * @brief  Enables/Disables oscillator stop detection function.
  * @note   The oscillation stop detection function restarts the OSC1 oscillator
  *         circuit when it detects the oscillation was stopped (due to under adverse
  *         environments, etc.).
  * @param  clock: This parameter can be a value of @ref seCLG_ClkSrc.
  * @param  StopDetectionEn: @ref seState. This enables/disables oscilator stop detection function.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seCLG_SetStopDetection( seCLG_ClkSrc clock, seState StopDetectionEn ) {

  seStatus fStatus = seSTATUS_NG;

  switch ( clock ) {

  case seCLG_OSC1:
    if ( StopDetectionEn ) {
      EnableOsc1StopDetection();
    }
    else {
      DisableOsc1StopDetection();
    }
    fStatus = seSTATUS_OK;
    break;

  case seCLG_IOSC:
  case seCLG_OSC3:
  default:
    break;
  }

  return fStatus;
}

/**
  * @brief  Set wake-up SYSCLK source and divide.
  * @param  WkUpSysClk: This parameter can be a value of @ref seCLG_Clk.
  * @param  SysClkSwitchOnWkUpEn: @ref seState When set to seDISABLE CPU, starts up with the same clock as one
  *         that used before SLEEP mode was entered. When set to seENABLE CPU, switches to
  *         a different wake-up clock that used before SLEEP mode was entered. The wake-up clock
  *         does not have to be enabled at this time. It will be enabled on the wake-up.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seCLG_SetWkUpSysClk( seCLG_Clk WkUpSysClk, seState SysClkSwitchOnWkUpEn ) {

  seStatus fStatus = seSTATUS_OK;

  ///< Disable write-protect.
  seProtectSys( seWRITE_PROTECT_OFF );

  CLG->SCLK_b.WUPSRC = seSRC(WkUpSysClk);        ///< Set the SYSCLK clock source at wake-up.
  CLG->SCLK_b.WUPDIV = seDIV(WkUpSysClk);        ///< Set the SYSCLK division ratio at wake-up.
  CLG->SCLK_b.WUPMD  = SysClkSwitchOnWkUpEn;     ///< Set SYSCLK switching function at wake-up.

  ///< Enable write-protect.
  seProtectSys( seWRITE_PROTECT_ON );

  return fStatus;
}

/**
 * @brief   The auto-trimming function adjusts the IOSCCLK clock frequency by
 *          trimming the clock with reference to the high precision OSC1CLK
 *          clock generated by the OSC1 oscillator circuit.
 *          As a side effect of this function SYSCLK is switched to different OSC.
 * @note    Although the triming time depends on the temperature, you will
 *          sacrifice some time when you use this function, likely quite a few ms.
 *          When IOSCCLK is being used as the system clock or a peripheral circuit clock,
 *          do not use the auto-trimming function.
 *          OSC1 is started as side effect of this function.
 * @param   clock: Clock to do the procedure on, this parameter can be a value of @ref seCLG_ClkSrc.
 * @param   temp_clock: Temporary clock to run CPU while doing the procedure, can be a value of @ref seCLG_ClkSrc.
 * @retval  Status: can be a value of @ref seStatus
 */
seStatus seCLG_RunAutoTrimming( seCLG_ClkSrc clock, seCLG_ClkSrc temp_clock ) {

  seStatus fStatus = seSTATUS_NG;

  switch ( clock ) {

  case seCLG_IOSC:
    fStatus = RunIoscAutoTrimming( temp_clock );
    break;

  case seCLG_OSC1:
  case seCLG_OSC3:
  default:
    break;
  }

  return fStatus;
}

/**
  * @brief  Sets Stop Clock mode used while in sleep.
  * @param  clock: This parameter can be a value of @ref seCLG_ClkSrc.
  * @param  SlpEnable: When this parameter is @ref seENABLE the clock stops in SLEEP mode.
  *         When this parameter is @ref seDISABLE the clock runs in SLEEP mode.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seCLG_SetOperInSlp( seCLG_ClkSrc clock, seState SlpEnable ) {

  seStatus fStatus = seSTATUS_NG;

  switch ( clock ) {
  case seCLG_IOSC:
    CLG->OSC_b.IOSCSLPC = SlpEnable;
    fStatus = seSTATUS_OK;
    break;

  case seCLG_OSC1:
    CLG->OSC_b.OSC1SLPC = SlpEnable;
    fStatus = seSTATUS_OK;
    break;

  case seCLG_OSC3:
    CLG->OSC_b.OSC3SLPC = SlpEnable;
    fStatus = seSTATUS_OK;
    break;

  default:
    break;
  }

  return fStatus;
}

/**
  * @brief  Gets Stop Clock mode used while in sleep.
  * @param  clock: This parameter can be a value of @ref seCLG_ClkSrc.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seCLG_GetOperInSlp( seCLG_ClkSrc clock ) {

  seStatus fStatus = seSTATUS_NG;

  switch ( clock ) {
  case seCLG_IOSC:
    if ( CLG->OSC_b.IOSCSLPC ) {
      fStatus = seSTATUS_OK;
    }
    break;

  case seCLG_OSC1:
    if ( CLG->OSC_b.OSC1SLPC ) {
      fStatus = seSTATUS_OK;
    }
    break;

  case seCLG_OSC3:
    if ( CLG->OSC_b.OSC3SLPC ) {
      fStatus = seSTATUS_OK;
    }
    break;

  default:
    break;
  }

  return fStatus;
}

/**
  * @brief  Enable CLG interrupt(s).
  * @param  irq: interrupt mask.
  * @retval None
  */
void seCLG_EnableInt( uint16_t irq ) {

  CLG->INTE |= irq;
}

/**
  * @brief  Disable CLG interrupt(s).
  * @param  irq: interrupt mask.
  * @retval None
  */
void seCLG_DisableInt( uint16_t irq ) {

  CLG->INTE &= ~irq;
}

/**
  * @brief  Returns CLG interrupt flag(s).
  * @param  flag: interrupt flags.
  * @retval InterruptStatus: can be a value of @ref seInterruptStatus
  */
seInterruptStatus seCLG_GetIntFlag( uint16_t flag ) {

  seInterruptStatus fStatus = seINTERRUPT_NOT_OCCURRED;
  uint16_t flagged = CLG->INTF;

  if ( flag ) {
    flagged &= flag;
    if ( flagged == flag ) {
      fStatus = seINTERRUPT_OCCURRED;
    }
  }

  return fStatus;
}

/**
  * @brief  Clears CLG interrupt(s).
  * @param  flag: interrupt flags.
  * @retval None
  */
void seCLG_ClearIntFlag( uint16_t flag ) {

  CLG->INTF = flag;
}


/**
  * Static functions declarations
**/


/**
  * @brief  Start Osc1.
  */
static seStatus StartOsc1( void ) {

   ///< Do nothing if not 32k768H Xtal
   if ( seGetOsc1XtalFreq() == 32768 ) {
    ///< It doesn't do at all when having already started.
    if ( !CLG->OSC_b.OSC1EN ) {
      ///< Disable write-protect.
      seProtectSys( seWRITE_PROTECT_OFF );

      ///< Clear interrupt flag.
      seCLG_ClearIntFlag( CLG_INTF_OSC1STAIF_Msk | CLG_INTF_OSC1STPIF_Msk );

      CLG->OSC_b.OSC1EN = 0;   ///< Stop Osc1.

      CLG->OSC1_b.INV1N  = 0;  ///< Set oscillation inverter gain(0 = Min).
      CLG->OSC1_b.CGI1   = 0;  ///< Set internal gate capacitor(0 = Min).
      CLG->OSC1_b.OSC1WT = 1;  ///< Set oscillation stabilization waiting time(1 = 4096 clocks).

      ///< Enable write-protect.
      seProtectSys( seWRITE_PROTECT_ON );

      CLG->OSC_b.OSC1EN = 1;   ///< Start oscillation.

      ///< Wait OSC1STAIF Interrupt for the oscillation stabilization
      while ( ! CLG->INTF_b.OSC1STAIF ) {
        ;
      }
    }
  }

  return ( CLG->OSC_b.OSC1EN==1 ) ? seSTATUS_OK : seSTATUS_NG;
}

/**
  * @brief  Stop Osc1.
  */
static void StopOsc1( void ) {

  ///< Stop Osc1.
  CLG->OSC_b.OSC1EN = 0;

  ///< Disable interrupt.
  CLG->INTE_b.OSC1STAIE = 0;

  ///< Clear interrupt flag.
  seCLG_ClearIntFlag( CLG_INTF_OSC1STAIF_Msk | CLG_INTF_OSC1STPIF_Msk );
}

/**
  * @brief  Enable oscillation stop detection function in Osc1.
  */
static void EnableOsc1StopDetection( void ) {

  ///< Disable write-protect.
  seProtectSys( seWRITE_PROTECT_OFF );

  ///< CLGOsc1
  CLG->OSC1_b.OSDRB = 1;  ///< Enable Osc1 restart function.
  CLG->OSC1_b.OSDEN = 1;  ///< Enable oscillation stop detection function.

  ///< Clear interrupt flag.
  seCLG_ClearIntFlag( CLG_INTF_OSC1STPIF_Msk );

  ///< Enable write-protect.
  seProtectSys( seWRITE_PROTECT_ON );
}

/**
  * @brief  Enable oscillation stop detection function in Osc1.
  */
static void DisableOsc1StopDetection( void ) {

  ///< Disable write-protect.
  seProtectSys( seWRITE_PROTECT_OFF );
  CLG->OSC1_b.OSDRB = 0;  /// Disable Osc1 restart function.
  CLG->OSC1_b.OSDEN = 0;  /// Disable oscillation stop detection function.

  ///< Disable interrupt.
  CLG->INTE_b.OSC1STPIE = 0;

  ///< Clear interrupt flag.
  seCLG_ClearIntFlag( CLG_INTF_OSC1STPIF_Msk );

  ///< Enable write-protect.
  seProtectSys( seWRITE_PROTECT_ON );
}

/**
  * @brief  Start Osc3.
  */
static seStatus StartOsc3( void ) {

  seStatus fStatus;

  ///< It doesn't do at all when having already started.
  if ( CLG->OSC_b.OSC3EN == 0 ) 
  {
    fStatus = seAUX_ConfigurePortsForOSC3();

    if (fStatus == seSTATUS_OK) 
    {
      ///< Disable write-protect.
      seProtectSys( seWRITE_PROTECT_OFF );

      ///< Clear interrupt flag.
      seCLG_ClearIntFlag( CLG_INTF_OSC3STAIF_Msk ); 

      CLG->OSC3_b.OSC3WT = 4;   ///< Set the oscillation stabilization waiting time(4 = 1,024 clocks).
      CLG->OSC3_b.OSC3INV = 3;  ///< Set oscillation inverter gain(3 = Max).

      ///< Enable write-protect.
      seProtectSys( seWRITE_PROTECT_ON );

      CLG->OSC_b.OSC3EN  = 1;   ///< Start oscillation.

      ///< Wait OSC3STAIF Interrupt for the oscillation stabilization
      while ( CLG->INTF_b.OSC3STAIF == 0 ) {
        __NOP();		/// wait ...
      }
    }
  }

  fStatus = ( CLG->OSC_b.OSC3EN==1 ) ? seSTATUS_OK : seSTATUS_NG;

  return fStatus;
}

/**
  * @brief  Stop Osc3.
  */
static void StopOsc3( void ) {

  ///< Stop Osc3.
  CLG->OSC_b.OSC3EN  = 0;

  ///< Disable interrupt.
  CLG->INTE_b.OSC3STAIE = 0;

  ///< Clear interrupt flag.
  seCLG_ClearIntFlag( CLG_INTF_OSC3STAIF_Msk );
}

/**
  * @brief  Start Iosc.
  */
static seStatus StartIosc( void ) {

  ///< It doesn't do at all when having already started.
  if ( CLG->OSC_b.IOSCEN == 0 ) {
    ///< Disable write-protect.
    seProtectSys( seWRITE_PROTECT_OFF );

    ///< Clear interrupt flag(CLG->INTF_b.IOSCSTAIF).
    seCLG_ClearIntFlag( CLG_INTF_IOSCSTAIF_Msk );

    ///< Enable write-protect.
    seProtectSys( seWRITE_PROTECT_ON );

    CLG->OSC_b.IOSCEN = 1;	///< Start oscillation.

    ///< Wait IOSCSTAIF Interrupt for the oscillation stabilization
    while ( CLG->INTF_b.IOSCSTAIF == 0 ) {
      __NOP();		/// wait ...
    }
  }

  return ( CLG->OSC_b.IOSCEN==1 ) ? seSTATUS_OK : seSTATUS_NG;
}

/**
  * @brief  Stop Iosc.
  */
static void StopIosc( void ) {

  ///< Stop Iosc.
  CLG->OSC_b.IOSCEN = 0;

  ///< Disable interrupt.
  CLG->INTE_b.IOSCSTAIE = 0;

  ///< Clear interrupt flag.
  seCLG_ClearIntFlag( CLG_INTF_IOSCSTAIF_Msk | CLG_INTF_IOSCTEDIF_Msk ); 
}

/**
  * @brief  The auto-trimming function adjusts the IoscCLK clock frequency by
  *         trimming the clock with reference to the high precision Osc1CLK
  *         clock generated by the Osc1 oscillator circuit.
  * @retval seSTATUS_OK Success.
  * @retval seSTATUS_NG Failed.
  */
static seStatus RunIoscAutoTrimming( seCLG_ClkSrc temp_clock ) {

  ///< Start Osc1.
  StartOsc1();

  ///< The system clock set to temp clock. It has to be started in user program
  seStatus fStatus = seCLG_SwitchSysClk( (seCLG_Clk)(temp_clock<<4) );

  if ( fStatus == seSTATUS_OK ) {
    ///< Stop and start Iosc.
    StopIosc();
    StartIosc();

    ///< Check Osc1 and Iosc operation has completed.
    if ( CLG->OSC_b.OSC1EN && CLG->OSC_b.IOSCEN ) {
      ///< Clear interrupt flag.
      seCLG_ClearIntFlag( CLG_INTF_IOSCTEDIF_Msk ); 

      ///< Disable write-protect.
      seProtectSys( seWRITE_PROTECT_OFF );

      ///< Enable Iosc oscillation auto-trimming.
      CLG->IOSC_b.IOSCSTM = 1;

      ///< Enable write-protect.
      seProtectSys( seWRITE_PROTECT_ON );

      ///< Wait Iosc oscillation auto-trimming.
      while ( CLG->INTF_b.IOSCTEDIF == 0 ) {
        __NOP();		/// wait ...
      }

      ///< The system clock set to Iosc.
      fStatus = seCLG_SwitchSysClk( seCLG_IOSC_CLKDIV_1 );

    }else{
      fStatus = seSTATUS_NG;
    }
  }

  return fStatus;
}

/**
  * @brief  Find out if change to the system clock required.
  * @retval seStatus
  */
static seStatus NeedSwitch( seCLG_Clk clock ) {

  seStatus fStatus = ( seCLG_GetSysClkSrc() != seSRC(clock) ) ? seSTATUS_OK : seSTATUS_NG;

  return fStatus;
}

/**
  * @brief  Gets crystal frequency for OSC3.
  *         This function must be overridden by user at board level.
  * @retval Status: crystal frequency in Hz.
  */
__weak uint32_t seGetOsc3XtalFreq( void ) {
  
  return 0; 
}

/**
  * @brief  Gets crystal frequency for OSC1 (expected crystal 32.768 kHz).
  *         This function must be overridden by user at board level.
  * @retval Status: crystal frequency in Hz.
  */
__weak uint32_t seGetOsc1XtalFreq( void ) {
  
  return 0;
}

/**
  * @brief  Gets external frequency for EXOSC.
  *         This function must be overridden by user at board level.
  * @retval Status: frequency in Hz.
  */
__weak uint32_t seGetExOscFreq( void ) {
  
  return 0;
}

