/**
  ******************************************************************************
  * @file    se_clg.h
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file contains all the functions prototypes for the CLG firmware
  *          library.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#ifndef SE_CLG_H
#define SE_CLG_H

#include "se_common.h"

#ifdef __cplusplus
 extern "C" {
#endif

/** @addtogroup sePeriphLibrary
  * @{
  */

/** @defgroup CLG
  * @{
  * @brief CLG module is the clock generator that controls the clock sources and manages clock supply to the CPU and the peripheral
  *        circuits
  */

/** @defgroup CLG_Constants
  * @{
  */

/**
  * @}
  */  // CLG_Constants

/** @defgroup CLG_Types
  * @{
  */

/**
  * @brief  CLG Init structure definition.
  */

typedef struct {
   seCLG_Clk SysClk;               ///< Specifies the System clock source.
   seCLG_Clk WkUpSysClk;           ///< Specifies the System clock source on Wakeup from Sleep.
   seState SysClkSwitchOnWkUpEn;   ///< Specifies if System clock switch on wakeup shoud be enabled
   seState OperationInSlpEn;       ///< Specifies the Operation in Sleep mode should be enabled.
} seCLG_InitTypeDef;

/**
  * @}
  */ // CLG_Types

/** @defgroup CLG_Functions
  * @{
  */

/**
  * @brief  Initializes the CLG peripheral according to the specified
  *         parameters in the CLG_InitStruct.
  * @param  InitStruct: Pointer to a @ref seCLG_InitTypeDef structure that
  *         contains the configuration information for the specified CLG peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seCLG_Init( seCLG_InitTypeDef* InitStruct );

/**
  * @brief  Checks if clock is enabled.
  * @param  clock: Clock source see @ref seCLG_ClkSrc.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seCLG_IsEnabled(seCLG_ClkSrc clock);

/**
  * @brief  Selects a new clock source for the System clock.
  * @param  clock: Clock source see @ref seCLG_Clk.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seCLG_SwitchSysClk( seCLG_Clk clock );

/**
  * @brief  Gets System clock source.
  * @retval Clock: a value of @ref seCLG_ClkSrc
  */
uint16_t seCLG_GetSysClkSrc( void );

/**
  * @brief  Gets System clock divider.
  * @retval Divider: a value of @ref seCLG_ClkDiv
  */
uint16_t seCLG_GetSysClkDiv( void );

/**
  * @brief  Sets the IOSC frequency.
  * @param  freq: New frequency can be a value of @ref seCLG_IOSC_IoscFq.
  * @retval None
  */
void seCLG_SetIoscFreqSel( seCLG_IOSC_Freq freq );

/**
  * @brief  Gets the IOSC frequency.
  * @retval Hz: Actual value of IOSC frequency in Hz.
  */
uint32_t seCLG_GetIoscFreqSel( void );

/**
  * @brief  Gets the SysClock frequency.
  * @retval Hz: Actual value of SysClock frequency in Hz.
  */
uint32_t seCLG_GetSysClk( void );

/**
  * @brief  Starts Clock.
  * @param  clock: The clock to start.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seCLG_Start( seCLG_ClkSrc clock );

/**
  * @brief  Stops Clock.
  * @param  clock: The clock @ref seCLG_ClkSrc to stop.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seCLG_Stop( seCLG_ClkSrc clock );

/**
  * @brief  Enables/Disables oscillator stop detection function.
  * @note   The oscillation stop detection function restarts the OSC1 oscillator
  *         circuit when it detects the oscillation was stopped (due to under adverse
  *         environments, etc.).
  * @param  clock: This parameter can be a value of @ref seCLG_ClkSrc.
  * @param  StopDetectionEn: @ref seState. This enables/disables oscilator stop detection function.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seCLG_SetStopDetection( seCLG_ClkSrc clock, seState StopDetectionEn );

/**
  * @brief  Set wake-up SYSCLK source and divide.
  * @param  WkUpSysClk: This parameter can be a value of @ref seCLG_Clk.
  * @param  SysClkSwitchOnWkUpEn: @ref seState When set to seDISABLE CPU, starts up with the same clock as one
  *         that used before SLEEP mode was entered. When set to seENABLE CPU, switches to
  *         a different wake-up clock that used before SLEEP mode was entered. The wake-up clock
  *         does not have to be enabled at this time. It will be enabled on the wake-up.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seCLG_SetWkUpSysClk( seCLG_Clk WkUpSysClk, seState SysClkSwitchOnWkUpEn );

/**
 * @brief   The auto-trimming function adjusts the IOSCCLK clock frequency by
 *          trimming the clock with reference to the high precision OSC1CLK
 *          clock generated by the OSC1 oscillator circuit.
 *          As a side effect of this function SYSCLK is switched to different OSC.
 * @note    Although the triming time depends on the temperature, you will
 *          sacrifice some time when you use this function, likely quite a few ms.
 *          When IOSCCLK is being used as the system clock or a peripheral circuit clock,
 *          do not use the auto-trimming function.
 *          OSC1 is started as side effect of this function.
 * @param   clock: Clock to do the procedure on, this parameter can be a value of @ref seCLG_ClkSrc.
 * @param   temp_clock: Temporary clock to run CPU while doing the procedure, can be a value of @ref seCLG_ClkSrc.
 * @retval  Status: can be a value of @ref seStatus
 */
seStatus seCLG_RunAutoTrimming( seCLG_ClkSrc clock, seCLG_ClkSrc temp_clock );

/**
  * @brief  Sets Stop Clock mode used while in sleep.
  * @param  clock: This parameter can be a value of @ref seCLG_ClkSrc.
  * @param  SlpEnable: When this parameter is @ref seENABLE the clock stops in SLEEP mode.
  *         When this parameter is @ref seDISABLE the clock runs in SLEEP mode.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seCLG_SetOperInSlp( seCLG_ClkSrc clock, seState SlpEnable );

/**
  * @brief  Gets Stop Clock mode used while in sleep.
  * @param  clock: This parameter can be a value of @ref seCLG_ClkSrc.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seCLG_GetOperInSlp( seCLG_ClkSrc clock );

/**
  * @brief  Enable CLG interrupt(s).
  * @param  irq: interrupt mask.
  * @retval None
  */
void seCLG_EnableInt( uint16_t irq );

/**
  * @brief  Disable CLG interrupt(s).
  * @param  irq: interrupt mask.
  * @retval None
  */
void seCLG_DisableInt( uint16_t irq );

/**
  * @brief  Returns CLG interrupt flag(s).
  * @param  flag: interrupt flags.
  * @retval InterruptStatus: can be a value of @ref seInterruptStatus
  */
seInterruptStatus seCLG_GetIntFlag( uint16_t flag );

/**
  * @brief  Clears CLG interrupt(s).
  * @param  flag: interrupt flags.
  * @retval None
  */
void seCLG_ClearIntFlag( uint16_t flag );

/**
  * @brief  Configures ports for this module. Override this function to configure specific ports.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seAUX_ConfigurePortsForOSC3( void );

/**
  * @brief  Gets crystal frequency for OSC1 (expected crystal 32.768 kHz).
  *         This function must be overridden by user at board level.
  * @retval Status: crystal frequency in Hz.
  */
uint32_t seGetOsc1XtalFreq( void );

/**
  * @brief  Gets crystal frequency for OSC3.
  *         This function must be overridden by user at board level.
  * @retval Status: crystal frequency in Hz.
  */
uint32_t seGetOsc3XtalFreq( void );

/**
  * @brief  Gets external frequency for EXOSC.
  *         This function must be overridden by user at board level.
  * @retval Status: frequency in Hz.
  */
uint32_t seGetExOscFreq( void );

/**
  * @}
  */ // CLG_Functions

/**
  * @}
  */ // CLG

/**
  * @}
  */ // sePeriphLibrary

#ifdef __cplusplus
}
#endif
#endif  // SE_CLG_H

