/**
  ******************************************************************************
  * @file    se_dmac.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file provides all the DMAC hardware functions.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#include <string.h>
#include "se_dmac.h"

/**
  * Static functions declarations
**/

/**
  * @brief  Initializes DMAC descriptors.
  * @param  dma_data_struc_ptr: Address of an alocated buffer for use for DMAC descriptors. Must be aligned.
  * @param  chnls: This parameter declares the maximum number of requested channels.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seDMAC_Init( uint32_t dma_data_struc_ptr, int chnls ) {
  
  seStatus fStatus = seSTATUS_NG;
  DMAC->CFG_b.MSTEN = 1;    ///< enable controller
  
  if ( chnls <= DMAC->STAT_b.CHNLS && !(dma_data_struc_ptr & 0x7F) ) {
    DMAC->CPTR  = dma_data_struc_ptr;      ///< Set the leading address of the data structure.
    memset( (void *)DMAC->CPTR, 0, 0x80 ); 
    DMAC->RMCLR = seDMAC_CH_ALL;           ///< Set Cancel mask state of DMA transfer requests 
    DMAC->PRCLR = seDMAC_CH_ALL;           ///< Set Decrease priority
    DMAC->PACLR = seDMAC_CH_ALL;           ///< Set Disable alternate data structure
    DMAC->ENCLR = seDMAC_CH_ALL;           ///< Set Disable DMAC channel
    fStatus = seSTATUS_OK;
  }
  
  return fStatus;
}

/**
  * @brief  Returns DMAC channel's data structure pointer.
  * @retval Ptr: A valid aligned memory pointer.
  */
uint32_t seDMAC_GetDataStrucPtr( void ) {
  
  return DMAC->CPTR;  
}

/**
  * @brief  Sets DMAC channel's data structure pointer.
  * @param  dma_data_struc_ptr: This parameter shall be a valid aligned memory pointer.
  * @retval None
  */
void seDMAC_SetDataStrucPtr( uint32_t dma_data_struc_ptr ) {
  
  DMAC->CPTR = dma_data_struc_ptr;
}

/**
  * @brief  Returns DMAC channel's alternate data structure pointer.
  * @retval Ptr: A valid aligned memory pointer.
  */
uint32_t seDMAC_GetAltDataStrucPtr( void ) {
  
  return DMAC->ACPTR;
}

/**
  * @brief  Initializes a DMAC channel.
  * @note   It does not check if the channel was already active.
  * @param  ctrl_data: This parameter is a DMA channel control data 32-bit value.
  * @param  transf_src_end: This parameter is a source end adddress.
  * @param  transf_dest_end: This parameter is a destination end adddress.
  * @param  chan: This parameter is a value of @ref seDMAC_CHANNEL.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seDMAC_SetChannel( uint32_t ctrl_data, uint32_t transf_src_end, uint32_t transf_dest_end, seDMAC_CHANNEL chan ) {

  seStatus fStatus = seSTATUS_NG;
  uint32_t index = seDMAC_IDX( chan ); 
  seDMAC_DataStruct * ptr = (seDMAC_DataStruct*)DMAC->CPTR;
  
  if ( ptr ) {
    ptr[index].control_data = ctrl_data;
    ptr[index].transfer_destination_end_pointer = transf_dest_end;
    ptr[index].transfer_source_end_pointer = transf_src_end;
    //ptr[index].request_num = reqnum;
    fStatus = seSTATUS_OK;
  }
  
  return fStatus;
}

/**
  * @brief  Gets DMAC mode from the valid data structure.
  *         Returns mode seDMAC_MODE_STOP if DMA structures are not initialized.
  * @param  chan: is a value of @ref seDMAC_CHANNEL.
  * @retval Mode Value matching @ref seDMAC_Mode
  */
uint32_t seDMAC_GetMode( seDMAC_CHANNEL chan ) {

  uint32_t mode = seDMAC_MODE_STOP;
  uint32_t index = seDMAC_IDX( chan ); 
  seDMAC_DataStruct * ptr = (seDMAC_DataStruct*)DMAC->CPTR;
  
  if ( ptr ) {
    mode = ptr[index].control_data & 7;
  }
  
  return mode;
}

/**
  * @brief  Gets DMAC Total number of units to transmit minus 1 from the data structure.
  * @param  chan: is a value of @ref seDMAC_CHANNEL.
  * @retval units: Total number of units to transmit minus 1.
  */
uint32_t seDMAC_GetNMinus1( seDMAC_CHANNEL chan ) {
  
  uint32_t nm = 0; 
  uint32_t index = seDMAC_IDX( chan ); 
  seDMAC_DataStruct * ptr = (seDMAC_DataStruct*)DMAC->CPTR;
  
  if ( ptr ) {
   nm = (( ptr[index].control_data ) >> 4 ) & 0x3FF;
  }
  
  return nm;
}

/**
  * @brief  Starts DMAC channel.
  * @param  chan: This parameter is a value of @ref seDMAC_CHANNEL. 
  * @retval Status: can be a value of @ref seStatus
  */
void seDMAC_Start( seDMAC_CHANNEL chan ) {
 
  DMAC->SWREQ = chan;
}

/**
  * @brief  Enables interrupt(s).
  * @param  src: This parameter is a value of @ref seDMAC_InterruptSrc. 
  * @param  flag: This parameter is a value of @ref seDMAC_CHANNEL.
  * @retval None
  */
void seDMAC_EnableInt( seDMAC_InterruptSrc src, seDMAC_IntFlag flag   ) {
  
  if ( (seDMAC_ERR_INT&src) == seDMAC_ERR_INT ) {
    DMAC->ERRIESET = flag ;
  } 
  if  ( (seDMAC_TRANSF_COMPL&src) == seDMAC_TRANSF_COMPL ) {
    DMAC->ENDIESET = flag ;
  }
}
  
/**
  * @brief  Disables interrupt(s).
  * @param  src: This parameter is a value of @ref seDMAC_InterruptSrc.
  * @param  flag: This parameter is a value of @ref seDMAC_CHANNEL.
  * @retval None
  */
void seDMAC_DisableInt( seDMAC_InterruptSrc src, seDMAC_IntFlag flag  ) {

  if ( (seDMAC_ERR_INT&src) == seDMAC_ERR_INT ) {
    DMAC->ERRIECLR = flag ;
  } 
  if  ( (seDMAC_TRANSF_COMPL&src) == seDMAC_TRANSF_COMPL ) {
    DMAC->ENDIECLR = flag ;
  }  
}

/**
  * @brief  Returns the status of interrupt flag for selected interrupt sources.
  *         Use caution when using this function with a combination of interrupt sources.
  * @param  src: This parameter is a value of @ref seDMAC_InterruptSrc.
  * @param  flag: This parameter is a value of @ref seDMAC_CHANNEL.
  * @retval InterruptStatus: can be a value of @ref seInterruptStatus.
  */
seInterruptStatus seDMAC_GetIntFlag( seDMAC_InterruptSrc src, seDMAC_IntFlag flag  ) {

  seInterruptStatus status = seINTERRUPT_NOT_OCCURRED;
  if ( ( seDMAC_TRANSF_COMPL&src ) == seDMAC_TRANSF_COMPL ) {
    status = ( ( DMAC->ENDIF & flag ) )? seINTERRUPT_OCCURRED:seINTERRUPT_NOT_OCCURRED;
  }
  if  ( ( status == seINTERRUPT_NOT_OCCURRED ) && ( seDMAC_ERR_INT&src ) == seDMAC_ERR_INT ) {
    status = ( DMAC->ERRIF )? seINTERRUPT_OCCURRED:seINTERRUPT_NOT_OCCURRED;
  }
  
  return status;
}

/**
  * @brief  Clears DMAC interrupt(s).
  * @param  src: This parameter is a value of @ref seDMAC_InterruptSrc.
  * @param  flag: This parameter is a value of @ref seDMAC_CHANNEL.
  * @retval None
  */
void seDMAC_ClearIntFlag( seDMAC_InterruptSrc src, seDMAC_IntFlag flag  ) {
  
  if ( ( src&seDMAC_TRANSF_COMPL ) == seDMAC_TRANSF_COMPL ) {
    DMAC->ENDIF = flag;
  }
  if  ( ( src&seDMAC_ERR_INT ) == seDMAC_ERR_INT ) {
    DMAC-> ERRIF = 1;
  } 
}

/**
  * @brief  Enables masking of the Peripheral devices.
  * @param  chnls: This parameter is a value of @ref seDMAC_CHANNEL.
  * @retval None
  */
void seDMAC_EnableRequestMask( seDMAC_CHANNEL chnls ) {
  
  DMAC->RMSET = chnls; 
}

/**
  * @brief  Disables masking of the Peripheral devices.
  * @param  chnls: This parameter is a value of @ref seDMAC_CHANNEL.
  * @retval None
  */
void seDMAC_DisableRequestMask( seDMAC_CHANNEL chnls ) {
  
  DMAC->RMCLR = chnls; 
}

/**
  * @brief  Enables DMAC channel.
  * @param  chnls: This parameter is a value of @ref seDMAC_CHANNEL.
  * @retval None
  */
void seDMAC_Enable( seDMAC_CHANNEL chnls ) {
  
  DMAC->ENSET = chnls; 
}

/**
  * @brief  Disables DMAC channel.
  * @param  chnls: This parameter is a value of @ref seDMAC_CHANNEL.
  * @retval None
  */
void seDMAC_Disable( seDMAC_CHANNEL chnls ) {
  
  DMAC->ENCLR = chnls; 
}

/**
  * @brief  Enables DMAC's alternate desriptors.
  * @param  chnls: This parameter is a value of @ref seDMAC_CHANNEL.
  * @retval None
  */
void seDMAC_AlternateEnable( seDMAC_CHANNEL chnls ) {
 
  DMAC->PASET = chnls; 
}

/**
  * @brief  Enables DMAC's primary desriptors.
  * @param  chnls: This parameter is a value of @ref seDMAC_CHANNEL.
  * @retval None
  */
void seDMAC_AlternateDisable( seDMAC_CHANNEL chnls ) {
 
  DMAC->PACLR = chnls; 
}

/**
  * @brief  Causes DMAC channel priority increase.
  * @param  chnls: This parameter is a value of @ref seDMAC_CHANNEL.
  * @retval None
  */
void seDMAC_PriorityIncrease( seDMAC_CHANNEL chnls ) { 
  
  DMAC->PRSET = chnls; 
}

/**
  * @brief  Causes DMAC channel priority decrease.
  * @param  chnls: This parameter is a value of @ref seDMAC_CHANNEL.
  * @retval None
  */
void seDMAC_PriorityDecrease( seDMAC_CHANNEL chnls ) { 
  
  DMAC->PRCLR = chnls; 
}

/**
  * @brief  For Peripheral-to-Memory DMAC transfer this function configures a DMAC channel
  *         and generates a DMAC request.
  *         Returns a register pointer further used to disable DMAC request.
  * @param  periph: Pointer to a peripheral.
  * @param  daddress: Destination address. The size of the transfer defined in the function based on the peripheral type.
  * @param  transfcount: Transfer size minus one.
  * @param  chan: DMAC channel can be @ref seDMAC_CHANNEL
  * @retval Ptr: The register pointer further used to trigger DMAC transfer.
  */
volatile uint16_t*  seDMAC_NonBlockTransfPeriphToMem( uint32_t periph, uint32_t daddress, uint32_t transfcount, seDMAC_CHANNEL chan ) {

  volatile uint16_t* enable_trigger = 0;
  if ( transfcount <= seDMAC_NM_MAX ) { 

    uint32_t size_m1 = transfcount-1;        
    enable_trigger = seAUX_ConfigPeriphToMemForDMAC( periph, daddress, size_m1, chan );

    if ( enable_trigger ) {
      seDMAC_Enable( chan );   
      seDMAC_DisableRequestMask( chan );
      seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, chan );
      *enable_trigger |= chan;
    }
  }
  
  return ( enable_trigger );
}

/**
  * @brief  For Memory-to-Peripheral DMAC transfer this function configures a DMAC channel
  *         and generates a DMAC request.
  *         Returns a register pointer further used to disable DMAC request.
  * @param  saddress: Source address. The size of the transfer defined in the function based on the peripheral type.
  * @param  periph: Pointer to a peripheral.
  * @param  transfcount: Transfer size minus one.
  * @param  chan: DMAC channel can be @ref seDMAC_CHANNEL
  * @retval Ptr: The register pointer further used to trigger DMAC transfer.
  */
volatile uint16_t* seDMAC_NonBlockTransfMemToPeriph( uint32_t saddress, uint32_t periph, uint32_t transfcount, seDMAC_CHANNEL chan ) {

  volatile uint16_t* enable_trigger = 0;

  if ( transfcount <= seDMAC_NM_MAX ) {    
    enable_trigger = seAUX_ConfigMemToPeriphForDMAC( saddress, periph, (transfcount-1), chan );

    if ( enable_trigger ) {
      seDMAC_Enable( chan );
      seDMAC_DisableRequestMask( chan );
      seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, chan );
      *enable_trigger |= chan;
    }
  }
  
  return ( enable_trigger );
}

/**
  * @brief  Memory fill 8-bit DMAC transfer.
  * @param  dstaddr: Destination address.
  * @param  nbytes: Number of bytes to fill/tranfer.
  * @param  fillbyte: Byte value to fill.
  * @param  chan: DMAC channel can be @ref seDMAC_CHANNEL
  * @retval None.
  */
void seDMAC_MemFill8 (uint32_t dstaddr, uint32_t nbytes, uint8_t fillbyte, seDMAC_CHANNEL chan)
{
    uint32_t k, m, cdata1;

    ///< Configure the primary data structure for the DMA channel
    DMAC->CFG_b.MSTEN = 1; ///< enable controller
    seDMAC_DisableRequestMask( chan );

    seDMAC_Enable( chan );
    DMAC->ENDIF = chan;

    m = nbytes / 1024;
    k = 0;
    for (k = 0; k < m; k++)
    {
        cdata1 = seDMAC_cdata( seDMAC_MODE_AUTO_REQ, 1024-1, 0UL,
                               seDMAC_SIZE_BYTE, seDMAC_SIZE_BYTE,
                               seDMAC_INC_NO, seDMAC_INC_1 );
        seDMAC_SetChannel( cdata1, ( uint32_t ) &(fillbyte), ( uint32_t )(dstaddr+(k*1024)+1024-1), chan );

        seDMAC_Enable(chan);
        seDMAC_Start(chan);

        ///< Wait transfer complete
        while((DMAC->ENDIF & chan) == 0);
        DMAC->ENDIF = chan;

        ///< Check that DMA is stopped
        while ( (seDMAC_GetMode( chan ) != seDMAC_MODE_STOP ) );
    }

    if ((nbytes % 1024) != 0)
    {
        cdata1 = seDMAC_cdata( seDMAC_MODE_AUTO_REQ, (nbytes%1024)-1, 0UL,
                               seDMAC_SIZE_BYTE, seDMAC_SIZE_BYTE,
                               seDMAC_INC_NO, seDMAC_INC_1 );

        seDMAC_SetChannel( cdata1, ( uint32_t ) &(fillbyte), ( uint32_t )(dstaddr+(k*1024)+(nbytes%1024)-1), chan );

        seDMAC_Enable(chan);
        seDMAC_Start(chan);

        ///< Wait transfer complete
        while((DMAC->ENDIF & chan) == 0);
        DMAC->ENDIF = chan;

        ///< Check that DMA is stopped
        while ( (seDMAC_GetMode( chan ) != seDMAC_MODE_STOP ) );
    }

    seDMAC_Disable(  chan );
}

/**
  * @brief  Memory fill 16-bit DMAC transfer.
  * @param  dstaddr: Destination address.
  * @param  nhwords: Number of half-words (16-bit) to fill/tranfer.
  * @param  fillhword: Half-word value to fill.
  * @param  chan: DMAC channel can be @ref seDMAC_CHANNEL
  * @retval None.
  */
void seDMAC_MemFill16 (uint32_t dstaddr, uint32_t nhwords, uint16_t fillhword, seDMAC_CHANNEL chan)
{
    uint32_t k, m, cdata1;

    ///< Configure the primary data structure for the DMA channel
    DMAC->CFG_b.MSTEN = 1; ///< enable controller
    seDMAC_DisableRequestMask( chan );

    seDMAC_Enable( chan );
    DMAC->ENDIF = chan;

    m = nhwords / 1024;
    k = 0;
    for (k = 0; k < m; k++)
    {
        cdata1 = seDMAC_cdata( seDMAC_MODE_AUTO_REQ, 1024-1, 0UL,
                               seDMAC_SIZE_HALF_WORD, seDMAC_SIZE_HALF_WORD,
                               seDMAC_INC_NO, seDMAC_INC_2 );
        seDMAC_SetChannel( cdata1, ( uint32_t ) &(fillhword), ( uint32_t )(dstaddr+(k*2048)+2048-2), chan );

        seDMAC_Enable(chan);
        seDMAC_Start(chan);

        ///< Wait transfer complete
        while((DMAC->ENDIF & chan) == 0);
        DMAC->ENDIF = chan;

        ///< Check that DMA is stopped
        while ( (seDMAC_GetMode( chan ) != seDMAC_MODE_STOP ) );
    }

    if ((nhwords % 1024) != 0)
    {
        cdata1 = seDMAC_cdata( seDMAC_MODE_AUTO_REQ, (nhwords%1024)-1, 0UL,
                               seDMAC_SIZE_HALF_WORD, seDMAC_SIZE_HALF_WORD,
                               seDMAC_INC_NO, seDMAC_INC_2 );

        seDMAC_SetChannel( cdata1, ( uint32_t ) &(fillhword), ( uint32_t )(dstaddr+(k*2048)+(nhwords%1024)*2-2), chan );

        seDMAC_Enable(chan);
        seDMAC_Start(chan);

        ///< Wait transfer complete
        while((DMAC->ENDIF & chan) == 0);
        DMAC->ENDIF = chan;

        ///< Check that DMA is stopped
        while ( (seDMAC_GetMode( chan ) != seDMAC_MODE_STOP ) );
    }

    seDMAC_Disable(  chan );
}

/**
  * @brief  Memory fill 32-bit DMAC transfer.
  * @param  dstaddr: Destination address.
  * @param  nwords: Number of words (32-bit) to fill/tranfer.
  * @param  fillword: Word value to fill.
  * @param  chan: DMAC channel can be @ref seDMAC_CHANNEL
  * @retval None.
  */
void seDMAC_MemFill32 (uint32_t dstaddr, uint32_t nwords, uint32_t fillword, seDMAC_CHANNEL chan)
{
    uint32_t k, m, cdata1;

    ///< Configure the primary data structure for the DMA channel
    DMAC->CFG_b.MSTEN = 1; ///< enable controller
    seDMAC_DisableRequestMask( chan );

    seDMAC_Enable( chan );
    DMAC->ENDIF = chan;

    m = nwords / 1024;
    k = 0;
    for (k = 0; k < m; k++)
    {
        cdata1 = seDMAC_cdata( seDMAC_MODE_AUTO_REQ, 1024-1, 0UL,
                               seDMAC_SIZE_WORD, seDMAC_SIZE_WORD,
                               seDMAC_INC_NO, seDMAC_INC_4 );
        seDMAC_SetChannel( cdata1, ( uint32_t ) &(fillword), ( uint32_t )(dstaddr+(k*4096)+4096-4), chan );

        seDMAC_Enable(chan);
        seDMAC_Start(chan);

        ///< Wait transfer complete
        while((DMAC->ENDIF & chan) == 0);
        DMAC->ENDIF = chan;

        ///< Check that DMA is stopped
        while ( (seDMAC_GetMode( chan ) != seDMAC_MODE_STOP ) );
    }

    if ((nwords % 1024) != 0)
    {
        cdata1 = seDMAC_cdata( seDMAC_MODE_AUTO_REQ, (nwords%1024)-1, 0UL,
                               seDMAC_SIZE_WORD, seDMAC_SIZE_WORD,
                               seDMAC_INC_NO, seDMAC_INC_4 );

        seDMAC_SetChannel( cdata1, ( uint32_t ) &(fillword), ( uint32_t )(dstaddr+(k*4096)+(nwords%1024)*4-4), chan );

        seDMAC_Enable(chan);
        seDMAC_Start(chan);

        ///< Wait transfer complete
        while((DMAC->ENDIF & chan) == 0);
        DMAC->ENDIF = chan;

        ///< Check that DMA is stopped
        while ( (seDMAC_GetMode( chan ) != seDMAC_MODE_STOP ) );
    }

    seDMAC_Disable(  chan );
}

/**
  * @brief  Memory-to-Memory 8-bit DMAC transfer.
  * @param  srcaddr: Source address.
  * @param  dstaddr: Destination address.
  * @param  nbytes: Number of bytes to transfer.
  * @param  chan: DMAC channel can be @ref seDMAC_CHANNEL
  * @retval None.
  */
void seDMAC_MemCpy8 (uint32_t srcaddr, uint32_t dstaddr, uint32_t nbytes, seDMAC_CHANNEL chan)
{
    uint32_t k, m, cdata1;

    ///< Configure the primary data structure for the DMA channel
    DMAC->CFG_b.MSTEN = 1; ///< enable controller
    seDMAC_DisableRequestMask( chan );

    seDMAC_Enable(chan);
    DMAC->ENDIF = chan;
    DMAC->ENDIESET = chan ;

    m = nbytes / 1024;
    k = 0;
    for (k = 0; k < m; k++)
    {
        cdata1 = seDMAC_cdata( seDMAC_MODE_AUTO_REQ, 1024-1, 0UL,
                               seDMAC_SIZE_BYTE, seDMAC_SIZE_BYTE,
                               seDMAC_INC_1, seDMAC_INC_1 );
        seDMAC_SetChannel( cdata1, ( uint32_t )(srcaddr+(k*1024)+1024-1), ( uint32_t )(dstaddr+(k*1024)+1024-1), chan );

        seDMAC_Enable(chan);
        seDMAC_Start(chan);

        ///< Wait transfer complete interrupt
        while((DMAC->ENDIF & chan) == 0);
        DMAC->ENDIF = chan;

        ///< Check that DMA is stopped
        while ( (seDMAC_GetMode( chan ) != seDMAC_MODE_STOP ) );
    }

    if ((nbytes % 1024) != 0)
    {
        cdata1 = seDMAC_cdata( seDMAC_MODE_AUTO_REQ, (nbytes%1024)-1, 0UL,
                               seDMAC_SIZE_BYTE, seDMAC_SIZE_BYTE,
                               seDMAC_INC_1, seDMAC_INC_1 );

        seDMAC_SetChannel( cdata1, ( uint32_t )(srcaddr+(k*1024)+(nbytes%1024)-1), ( uint32_t )(dstaddr+(k*1024)+(nbytes%1024)-1), chan );

        seDMAC_Enable(chan);
        seDMAC_Start(chan);

        ///< Wait transfer complete interrupt
        while((DMAC->ENDIF & chan) == 0);
        DMAC->ENDIF = chan;

        ///< Check that DMA is stopped
        while ( (seDMAC_GetMode( chan ) != seDMAC_MODE_STOP ) );
    }

    seDMAC_DisableInt( seDMAC_TRANSF_COMPL, chan   );
    seDMAC_EnableRequestMask( chan );
    seDMAC_Disable( chan );
}

/**
  * @brief  Memory-to-Memory 16-bit DMAC transfer.
  * @param  srcaddr: Source address.
  * @param  dstaddr: Destination address.
  * @param  nhwords: Number of half-words (16-bit) to transfer.
  * @param  chan: DMAC channel can be @ref seDMAC_CHANNEL
  * @retval None.
  */
void seDMAC_MemCpy16 (uint32_t srcaddr, uint32_t dstaddr, uint32_t nhwords, seDMAC_CHANNEL chan)
{
    uint32_t k, m, cdata1;

    ///< Configure the primary data structure for the DMA channel
    DMAC->CFG_b.MSTEN = 1; ///< enable controller
    seDMAC_DisableRequestMask( chan );

    seDMAC_Enable(chan);
    DMAC->ENDIF = chan;
    DMAC->ENDIESET = chan ;

    m = nhwords / 1024;
    k = 0;
    for (k = 0; k < m; k++)
    {
        cdata1 = seDMAC_cdata( seDMAC_MODE_AUTO_REQ, 1024-1, 0UL,
                               seDMAC_SIZE_HALF_WORD, seDMAC_SIZE_HALF_WORD,
                               seDMAC_INC_2, seDMAC_INC_2 );
        seDMAC_SetChannel( cdata1, ( uint32_t )(srcaddr+(k*2048)+2048-2), ( uint32_t )(dstaddr+(k*2048)+2048-2), chan );

        seDMAC_Enable(chan);
        seDMAC_Start(chan);

        ///< Wait transfer complete interrupt
        while((DMAC->ENDIF & chan) == 0);
        DMAC->ENDIF = chan;

        ///< Check that DMA is stopped
        while ( (seDMAC_GetMode( chan ) != seDMAC_MODE_STOP ) );
    }

    if ((nhwords % 1024) != 0)
    {
        cdata1 = seDMAC_cdata( seDMAC_MODE_AUTO_REQ, (nhwords%1024)-1, 0UL,
                               seDMAC_SIZE_HALF_WORD, seDMAC_SIZE_HALF_WORD,
                               seDMAC_INC_2, seDMAC_INC_2 );

        seDMAC_SetChannel( cdata1, ( uint32_t )(srcaddr+(k*2048)+(nhwords%1024)*2-2), ( uint32_t )(dstaddr+(k*2048)+(nhwords%1024)*2-2), chan );

        seDMAC_Enable(chan);
        seDMAC_Start(chan);

        ///< Wait transfer complete interrupt
        while((DMAC->ENDIF & chan) == 0);
        DMAC->ENDIF = chan;

        ///< Check that DMA is stopped
        while ( (seDMAC_GetMode( chan ) != seDMAC_MODE_STOP ) );
    }

    seDMAC_DisableInt( seDMAC_TRANSF_COMPL, chan   );
    seDMAC_EnableRequestMask( chan );
    seDMAC_Disable( chan );
}

/**
  * @brief  Memory-to-Memory 32-bit DMAC transfer.
  * @param  srcaddr: Source address.
  * @param  dstaddr: Destination address.
  * @param  nwords: Number of words (32-bit) to transfer.
  * @param  chan: DMAC channel can be @ref seDMAC_CHANNEL
  * @retval None.
  */
void seDMAC_MemCpy32 (uint32_t srcaddr, uint32_t dstaddr, uint32_t nwords, seDMAC_CHANNEL chan)
{
    uint32_t k, m, cdata1;

    ///< Configure the primary data structure for the DMA channel
    DMAC->CFG_b.MSTEN = 1; ///< enable controller
    seDMAC_DisableRequestMask( chan );

    seDMAC_Enable(chan);
    DMAC->ENDIF = chan;
    DMAC->ENDIESET = chan ;

    m = nwords / 1024;
    k = 0;
    for (k = 0; k < m; k++)
    {
        cdata1 = seDMAC_cdata( seDMAC_MODE_AUTO_REQ, 1024-1, 0UL,
                               seDMAC_SIZE_WORD, seDMAC_SIZE_WORD,
                               seDMAC_INC_4, seDMAC_INC_4 );
        seDMAC_SetChannel( cdata1, ( uint32_t )(srcaddr+(k*4096)+4096-4), ( uint32_t )(dstaddr+(k*4096)+4096-4), chan );

        seDMAC_Enable(chan);
        seDMAC_Start(chan);

        ///< Wait transfer complete interrupt
        while((DMAC->ENDIF & chan) == 0);
        DMAC->ENDIF = chan;

        ///< Check that DMA is stopped
        while ( (seDMAC_GetMode( chan ) != seDMAC_MODE_STOP ) );
    }

    if ((nwords % 1024) != 0)
    {
        cdata1 = seDMAC_cdata( seDMAC_MODE_AUTO_REQ, (nwords%1024)-1, 0UL,
                               seDMAC_SIZE_WORD, seDMAC_SIZE_WORD,
                               seDMAC_INC_4, seDMAC_INC_4 );

        seDMAC_SetChannel( cdata1, ( uint32_t )(srcaddr+(k*4096)+(nwords%1024)*4-4), ( uint32_t )(dstaddr+(k*4096)+(nwords%1024)*4-4), chan );

        seDMAC_Enable(chan);
        seDMAC_Start(chan);

        ///< Wait transfer complete interrupt
        while((DMAC->ENDIF & chan) == 0);
        DMAC->ENDIF = chan;

        ///< Check that DMA is stopped
        while ( (seDMAC_GetMode( chan ) != seDMAC_MODE_STOP ) );
    }

    seDMAC_DisableInt( seDMAC_TRANSF_COMPL, chan   );
    seDMAC_EnableRequestMask( chan );
    seDMAC_Disable( chan );
}

