/**
  ******************************************************************************
  * @file    se_dmaqspi.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file provides all the DMA for QSPI hardware functions.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2018-2020. All rights reserved.
  ******************************************************************************
  */

#include "se_dmac.h"
#include "se_qspi.h"
#include "se_dmaqspi.h"

/**
  * @brief  This function sends data by DMA through the QSPIx peripheral by DMA.
  * @note   DMAC must be initialized prior calling of this function.
  *         DMA Channel 0 is used by this function. It must be available for the duration of the function call.
  * @param  QSPIx:  QSPI to select the QSPI peripheral.
  * @param  data: Pointer to data to be transmitted.
  * @param  size: Data size in number of half words.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seDMAQSPI_TxHWords( QSPI_0_Type* QSPIx, uint16_t data[], uint32_t size ) {
  
  seStatus fStatus = seSTATUS_OK;
  
  if ( size <= seDMAC_NM_MAX ) {
    uint32_t size_m1 = size-1;
    uint32_t cdata1 = seDMAC_cdata( seDMAC_MODE_BASIC,size_m1,0UL,seDMAC_SIZE_HALF_WORD,seDMAC_SIZE_HALF_WORD,seDMAC_INC_2,seDMAC_INC_NO );  
    seDMAC_SetChannel( cdata1, ( uint32_t )data+size_m1*2, ( uint32_t )&QSPIx->TXD, seDMAC_CH0 ); 
    seDMAC_Enable( seDMAC_CH0 );   
    seDMAC_DisableRequestMask( seDMAC_CH0 );
    seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CH0 );
    QSPIx->TBEDMAEN |= seDMAC_CH0;  
    do {
      setEventWait(); 
      if ( seDMAC_GetIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CH0 ) ) {
        QSPIx->TBEDMAEN  &= ~seDMAC_CH0;
        seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CH0 );
      }
    } while (  seDMAC_GetMode( seDMAC_CH0 ) != seDMAC_MODE_STOP );
    
    seDMAC_EnableRequestMask( seDMAC_CH0 );
    seDMAC_Disable( seDMAC_CH0 );
    WHILE( QSPIx->INTF & QSPI_0_INTF_BSY_Msk, fStatus );
  } else {
    fStatus = seSTATUS_NG;
  }
  
  return fStatus;  
}

/**
  * @brief  This function returns the received data from the QSPIx peripheral by DMA.
  * @note   DMAC must be initialized prior calling of this function.
  *         DMA Channels 1,3 are used by this function. They must be available for duration of the function call.
  * @param  QSPIx: QSPI to select the QSPI peripheral.
  * @param  data: The received data pointer.
  * @param  size: Data size in number of half words.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seDMAQSPI_RxHWords( QSPI_0_Type* QSPIx, uint16_t data[], uint32_t size ) {

  uint16_t dummy = QSPIx->RXD; ///< dummy read
  seStatus fStatus = seSTATUS_OK;
  uint32_t size_m1 = size-1;

  if ( QSPIx->MOD_b.MST == 1 ) {
    ///< Configure the primary data structure for the DMA channel (Ch.1)
    ///< used for writing dummy byte to the TXD register
    uint32_t cdata1 = seDMAC_cdata( seDMAC_MODE_BASIC, size_m1, 0UL,
                                   seDMAC_SIZE_HALF_WORD, seDMAC_SIZE_HALF_WORD,
                                   seDMAC_INC_NO, seDMAC_INC_NO );  
    seDMAC_SetChannel( cdata1, ( uint32_t )&dummy, ( uint32_t )&QSPIx->TXD, seDMAC_CH1 );
    ///< Configure the primary data structure for the DMA channel (Ch.2)
    ///< used for reading data from the RXD
    cdata1 = seDMAC_cdata( seDMAC_MODE_BASIC, size_m1, 0UL,
                          seDMAC_SIZE_HALF_WORD, seDMAC_SIZE_HALF_WORD,
                          seDMAC_INC_NO, seDMAC_INC_2 );  
    seDMAC_SetChannel( cdata1, ( uint32_t )&QSPIx->RXD, ( uint32_t )data+size_m1*2, seDMAC_CH3 );
    seDMAC_Enable( seDMAC_CNLS( seDMAC_CH1 | seDMAC_CH3 ) );
    seDMAC_PriorityIncrease(  seDMAC_CH3 );
    seDMAC_DisableRequestMask( seDMAC_CNLS( seDMAC_CH1 | seDMAC_CH3 ) );
    seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CNLS( seDMAC_CH1 | seDMAC_CH3 ) );
    QSPIx->RBFDMAEN |= ( seDMAC_CNLS( seDMAC_CH1 | seDMAC_CH3 ) );
    ///< Issue a software DMA transfer request to trigger receiving
    seDMAC_Start( seDMAC_CH1 ); 
  } else {
    ///< Configure the primary data structure ONLY for the DMA channel (Ch.3)
    uint32_t cdata1 = seDMAC_cdata( seDMAC_MODE_BASIC, size_m1, 0UL,
                                   seDMAC_SIZE_HALF_WORD, seDMAC_SIZE_HALF_WORD,
                                   seDMAC_INC_NO, seDMAC_INC_1);  
    seDMAC_SetChannel( cdata1, ( uint32_t )&QSPIx->RXD, ( uint32_t )data+size_m1*2, seDMAC_CH3 ); 
    seDMAC_Enable( seDMAC_CH3 );
    seDMAC_DisableRequestMask( seDMAC_CH3 );
    seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CH3 );
    seDMAC_EnableInt( seDMAC_TRANSF_COMPL, seDMAC_CH3 ); 
    QSPIx->RBFDMAEN |= seDMAC_CH3;
  }  

  do {
    setEventWait(); 
    if ( seDMAC_GetIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CH3 ) ) {
      QSPIx->RBFDMAEN  &= ~seDMAC_CH3;
      seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CH3 );
    }
  } while ( (seDMAC_GetMode( seDMAC_CH3 ) != seDMAC_MODE_STOP ) );
  
  if ( QSPIx->MOD_b.MST == 1 ) {
    QSPIx->TBEDMAEN &= ~seDMAC_CH1; 
    seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CH1 );
    seDMAC_EnableRequestMask( seDMAC_CH1 );
    seDMAC_Disable( seDMAC_CH1 );
  } 
  seDMAC_EnableRequestMask( seDMAC_CH3 );
  seDMAC_PriorityDecrease(  seDMAC_CH3 );
  seDMAC_Disable( seDMAC_CH3 );
  WHILE( QSPIx->INTF & QSPI_0_INTF_BSY_Msk, fStatus );
  
  return fStatus;
}

/**
  * @brief  This function sends data by through the QSPIx peripheral by DMA.
  * @note   DMAC must be initialized prior calling of this function.
  *         DMA Channel 0 is used by this function. It must be available for duration of the function call.
  * @param  QSPIx:  QSPI to select the QSPI peripheral.
  * @param  data: Pointer to data to be transmitted.
  * @param  size: Data size in number of bytes.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seDMAQSPI_TxBytes( QSPI_0_Type* QSPIx, uint8_t data[], uint32_t size ) {  
  
  seStatus fStatus = seSTATUS_OK;
  
  if ( size <= seDMAC_NM_MAX ) {
    uint32_t size_m1 = size-1;
    uint32_t cdata1 = seDMAC_cdata( seDMAC_MODE_BASIC,size_m1,0UL,seDMAC_SIZE_BYTE,seDMAC_SIZE_BYTE,seDMAC_INC_1,seDMAC_INC_NO );
    seDMAC_SetChannel( cdata1, ( uint32_t )data+size_m1, ( uint32_t )&QSPIx->TXD, seDMAC_CH0 ); 
    seDMAC_Enable( seDMAC_CH0 );
    seDMAC_DisableRequestMask( seDMAC_CH0 );
    seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CH0 );
    QSPIx->TBEDMAEN |= seDMAC_CH0;  
    
    do {
      setEventWait();
      if ( seDMAC_GetIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CH0 ) ) {
        QSPIx->TBEDMAEN  &= ~seDMAC_CH0;
        seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CH0 );
      }
    } while (  seDMAC_GetMode( seDMAC_CH0 ) != seDMAC_MODE_STOP );
    
    seDMAC_EnableRequestMask( seDMAC_CH0 );
    seDMAC_Disable( seDMAC_CH0 );
    WHILE( QSPIx->INTF & QSPI_0_INTF_BSY_Msk, fStatus );
  } else {
    fStatus = seSTATUS_NG;
  }
  
  return fStatus;
}

/**
  * @brief  This function returns the received data from the QSPIx peripheral by DMA.
  * @note   DMAC must be initialized prior calling of this function.
  *         DMA Channels 1,3 are used by this function. They must be available for duration of the function call.
  * @param  QSPIx: QSPI to select the QSPI peripheral.
  * @param  data: The received data pointer.
  * @param  size: Data size in number of bytes.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seDMAQSPI_RxBytes( QSPI_0_Type* QSPIx, uint8_t data[], uint32_t size ) {

  uint16_t dummy = QSPIx->RXD; ///< dummy read
  seStatus fStatus = seSTATUS_OK;
  uint32_t size_m1 = size-1;

  if ( QSPIx->MOD_b.MST == 1 ) {
    ///< Configure the primary data structure for the DMA channel (Ch.1)
    ///< used for writing dummy byte to the TXD register
    uint32_t cdata1 = seDMAC_cdata( seDMAC_MODE_BASIC, size_m1, 0UL,
                                   seDMAC_SIZE_BYTE, seDMAC_SIZE_BYTE,
                                   seDMAC_INC_NO, seDMAC_INC_NO );  
    seDMAC_SetChannel( cdata1, ( uint32_t )&dummy, ( uint32_t )&QSPIx->TXD, seDMAC_CH1 );
    ///< Configure the primary data structure for the DMA channel (Ch.2) 
    ///< used for reading data from the RXD
    cdata1 = seDMAC_cdata( seDMAC_MODE_BASIC, size_m1, 0UL,
                          seDMAC_SIZE_BYTE, seDMAC_SIZE_BYTE, 
                          seDMAC_INC_NO, seDMAC_INC_1 );  
    seDMAC_SetChannel( cdata1, ( uint32_t )&QSPIx->RXD, ( uint32_t )data+size_m1, seDMAC_CH3 );
    seDMAC_Enable( seDMAC_CNLS( seDMAC_CH1 | seDMAC_CH3 ) );
    seDMAC_PriorityIncrease(  seDMAC_CH3 );
    seDMAC_DisableRequestMask( seDMAC_CNLS( seDMAC_CH1 | seDMAC_CH3 ) );
    seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CNLS( seDMAC_CH1 | seDMAC_CH3 ) );
    QSPIx->RBFDMAEN |= ( seDMAC_CNLS( seDMAC_CH1 | seDMAC_CH3 ) );
    ///< Issue a software DMA transfer request to trigger receiving
    seDMAC_Start( seDMAC_CH1 ); 
  } else {
    ///< Configure the primary data structure ONLY for the DMA channel (Ch.3)
    uint32_t cdata1 = seDMAC_cdata( seDMAC_MODE_BASIC, size_m1, 0UL,
                                   seDMAC_SIZE_BYTE, seDMAC_SIZE_BYTE,
                                   seDMAC_INC_NO, seDMAC_INC_1);  
    seDMAC_SetChannel( cdata1, ( uint32_t )&QSPIx->RXD, ( uint32_t )data+size_m1, seDMAC_CH3 ); 
    seDMAC_Enable( seDMAC_CH3 );
    seDMAC_DisableRequestMask( seDMAC_CH3 );
    seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CH3 );
    seDMAC_EnableInt( seDMAC_TRANSF_COMPL, seDMAC_CH3 ); 
    QSPIx->RBFDMAEN |= seDMAC_CH3;
  }  

  do {
    setEventWait(); 
    if ( seDMAC_GetIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CH3 ) ) {
      QSPIx->RBFDMAEN  &= ~seDMAC_CH3;
      seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CH3 );
    }
  } while ( (seDMAC_GetMode( seDMAC_CH3 ) != seDMAC_MODE_STOP ) );
  
  if ( QSPIx->MOD_b.MST == 1 ) {
    QSPIx->TBEDMAEN &= ~seDMAC_CH1; 
    seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CH1 );
    seDMAC_EnableRequestMask( seDMAC_CH1 );
    seDMAC_Disable( seDMAC_CH1 );
  } 
  seDMAC_EnableRequestMask( seDMAC_CH3 );
  seDMAC_PriorityDecrease(  seDMAC_CH3 );
  seDMAC_Disable( seDMAC_CH3 );
  
  WHILE( QSPIx->INTF & QSPI_0_INTF_BSY_Msk, fStatus );
  
  return fStatus;
}

/**
  * @brief  This function returns the received data by DMA from the QSPIx peripheral in memory mapped mode.
  * @note   DMAC must be initialized prior calling of this function. 
  *         DMA Channel 2 is used by this function. They must be available for duration of the function call.
  * @param  QSPIx: QSPI to select the QSPI peripheral.
  * @param  data: The received data pointer.
  * @param  size: Data size in number of long words.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seDMAQSPI_RxMmaWords( QSPI_0_Type* QSPIx, uint32_t offset, uint32_t data[], uint32_t size ) {
   
  uint32_t size_m1 = size-1;

  ///< Configure the primary data structure for the DMA channel (Ch.2) 
  uint32_t cdata1 = seDMAC_cdata( seDMAC_MODE_BASIC, size_m1, 0UL,
                                 seDMAC_SIZE_WORD, seDMAC_SIZE_WORD,
                                 seDMAC_INC_4, seDMAC_INC_4 );  
  seDMAC_SetChannel( cdata1,(offset&0x000FFFFFU)+size_m1*4,( uint32_t )data+size_m1*4, seDMAC_CH2 );
  seDMAC_Enable( seDMAC_CH2 );
  seDMAC_DisableRequestMask( seDMAC_CH2 );
  seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CH2 );
  ///< Enable the QSPI to issue DMA transfer requests to fifo
  QSPIx->FRLDMAEN |= seDMAC_CH2; 
  ///< Issue a software DMA transfer request
  seDMAC_Start( seDMAC_CH2 );
  do {
    setEventWait(); 
    if ( seDMAC_GetIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CH2 ) ) {
      QSPIx->FRLDMAEN  &= ~seDMAC_CH2;
      seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CH2 );
    }
  } while ( (seDMAC_GetMode( seDMAC_CH2 ) != seDMAC_MODE_STOP ) );

  seDMAC_EnableRequestMask( seDMAC_CH2 );
  seDMAC_Disable(  seDMAC_CH2 );

  return seSTATUS_OK;   
}

