/**
  ******************************************************************************
  * @file    se_dmaqspi.h
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file contains all the functions prototypes for the DMA for QSPI firmware
  *          library.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2018-2020. All rights reserved.
  ******************************************************************************
  */

#ifndef SE_DMAQSPI_H
#define SE_DMAQSPI_H

#include "se_common.h"

#ifdef __cplusplus
 extern "C" {
#endif

/** @addtogroup sePeriphLibrary
  * @{
  */

/** @addtogroup DMAC
  * @{
  */

/** @defgroup DMAQSPI_Functions
  * @{
  * @brief The Direct Memory Access functions related to QSPI.
  */

/**
  * @brief  This function sends data by through the QSPIx peripheral by DMA.
  * @note   DMAC must be initialized prior calling of this function.
  *         DMA Channel 0 is used by this function. It must be available for duration of the function call.
  * @param  QSPIx:  QSPI to select the QSPI peripheral.
  * @param  data: Pointer to data to be transmitted.
  * @param  size: Data size in number of bytes.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seDMAQSPI_TxBytes( QSPI_0_Type* QSPIx, uint8_t data[], uint32_t size );

/**
  * @brief  This function sends data by DMA through the QSPIx peripheral by DMA.
  * @note   DMAC must be initialized prior calling of this function.
  *         DMA Channel 0 is used by this function. It must be available for the duration of the function call.
  * @param  QSPIx:  QSPI to select the QSPI peripheral.
  * @param  data: Pointer to data to be transmitted.
  * @param  size: Data size in number of half words.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seDMAQSPI_TxHWords( QSPI_0_Type* QSPIx, uint16_t data[], uint32_t size );

/**
  * @brief  This function returns the received data from the QSPIx peripheral by DMA.
  * @note   DMAC must be initialized prior calling of this function.
  *         DMA Channels 1,3 are used by this function. They must be available for duration of the function call.
  * @param  QSPIx: QSPI to select the QSPI peripheral.
  * @param  data: The received data pointer.
  * @param  size: Data size in number of bytes.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seDMAQSPI_RxBytes( QSPI_0_Type* QSPIx, uint8_t data[], uint32_t size ); 

/**
  * @brief  This function returns the received data from the QSPIx peripheral by DMA.
  * @note   DMAC must be initialized prior calling of this function.
  *         DMA Channels 1,3 are used by this function. They must be available for duration of the function call.
  * @param  QSPIx: QSPI to select the QSPI peripheral.
  * @param  data: The received data pointer.
  * @param  size: Data size in number of half words.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seDMAQSPI_RxHWords( QSPI_0_Type* QSPIx, uint16_t data[], uint32_t size );

/**
  * @brief  This function returns the received data by DMA from the QSPIx peripheral in memory mapped mode.
  * @note   DMAC must be initialized prior calling of this function. 
  *         DMA Channel 2 is used by this function. They must be available for duration of the function call.
  * @param  QSPIx: QSPI to select the QSPI peripheral.
  * @param  data: The received data pointer.
  * @param  size: Data size in number of long words.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seDMAQSPI_RxMmaWords( QSPI_0_Type* QSPIx, uint32_t offset, uint32_t data[], uint32_t size );

/**
  * @}
  */ // DMAQSPI_Functions

/**
  * @}
  */ // DMAC

/**
  * @}
  */ // sePeriphLibrary

#ifdef __cplusplus
}
#endif
#endif  // SE_DMAQSPI_H

