/**
  ******************************************************************************
  * @file    se_dmaspi.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file provides all the DMA for SPI hardware functions.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2018-2020. All rights reserved.
  ******************************************************************************
  */

#include "se_dmac.h"
#include "se_spia.h"
#include "se_dmaspi.h"

/**
  * @brief  Sends a data through the SPIx peripheral by DMA.
  * @param  SPIx: SPI peripheral
  * @param  data: Data to be transmitted.
  * @param  size: Data size in number of words.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seDMASPI_TxHWords( SPIA_0_Type* SPIx, uint16_t data[], uint32_t size ) {

  seStatus fStatus = seSTATUS_OK;

  if ( size <= seDMAC_NM_MAX ) {
    uint32_t size_m1 = size-1;
    uint32_t cdata1 = seDMAC_cdata( seDMAC_MODE_BASIC,size_m1,0UL,seDMAC_SIZE_HALF_WORD,seDMAC_SIZE_HALF_WORD,seDMAC_INC_2,seDMAC_INC_NO );
    seDMAC_SetChannel( cdata1, ( uint32_t )data+size_m1*2, ( uint32_t )&SPIx->TXD, seDMAC_CH0 ); 
    seDMAC_Enable( seDMAC_CH0 );   
    seDMAC_DisableRequestMask( seDMAC_CH0 );
    seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CH0 );
    SPIx->TBEDMAEN |= seDMAC_CH0;
    do {
      setEventWait(); 
      if ( seDMAC_GetIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CH0 ) ) {
        SPIx->TBEDMAEN  &= ~seDMAC_CH0;
        seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CH0 );
      }
    } while (  seDMAC_GetMode( seDMAC_CH0 ) != seDMAC_MODE_STOP );
    
    SPIx->TBEDMAEN  &= ~seDMAC_CH0;
    seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CH0 );
    seDMAC_EnableRequestMask( seDMAC_CH0 );
    seDMAC_Disable( seDMAC_CH0 );
    WHILE( SPIx->INTF & SPIA_0_INTF_BSY_Msk, fStatus );
  } else {
    fStatus = seSTATUS_NG;
  }
  
  return fStatus;
}

/**
  * @brief  Returns the received data by the SPIx peripheral by DMA.
  * @param  SPIx: SPI peripheral
  * @param  data: The received data.
  * @param  size: Data size in number of words.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seDMASPI_RxHWords( SPIA_0_Type* SPIx, uint16_t data[], uint32_t size ) {
  
  uint16_t dummy = SPIx->RXD; ///< dummy read
  seStatus fStatus = seSTATUS_OK;
  uint32_t size_m1 = size-1;
  ///< Set transmit dummy data(Master mode only. Continuous data reception).
  if ( SPIx->MOD_b.MST == 1 ) {
    ///< Configure the primary data structure for the DMA channel (Ch.1) used for writing dummy byte to the TXD register
    uint32_t cdata1 = seDMAC_cdata( seDMAC_MODE_BASIC,size_m1,0UL,seDMAC_SIZE_HALF_WORD,seDMAC_SIZE_HALF_WORD,seDMAC_INC_NO,seDMAC_INC_NO );
    seDMAC_SetChannel( cdata1, ( uint32_t )&dummy, ( uint32_t )&SPIx->TXD, seDMAC_CH1 );
    ///< Configure the primary data structure for the DMA channel (Ch.2) used for reading data from the RXD
    cdata1 = seDMAC_cdata( seDMAC_MODE_BASIC,size_m1,0UL,seDMAC_SIZE_HALF_WORD,seDMAC_SIZE_HALF_WORD,seDMAC_INC_NO,seDMAC_INC_2);
    seDMAC_SetChannel( cdata1, ( uint32_t )&SPIx->RXD, ( uint32_t )data+size_m1*2, seDMAC_CH2 );
    ///< Enable both the DMA channels using the DMA controller register
    seDMAC_Enable( seDMAC_CNLS( seDMAC_CH1 | seDMAC_CH2 ) );
    ///< Increase the priority of the DMA channel used for reading data
    seDMAC_PriorityIncrease(  seDMAC_CH2 );
    ///< Clear the channel request masks for both the DMA channels
    seDMAC_DisableRequestMask( seDMAC_CNLS( seDMAC_CH1 | seDMAC_CH2 ) );
    ///< Clear the DMA transfer completion interrupt flags
    seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CNLS( seDMAC_CH1 | seDMAC_CH2 ) );
    ///< Enable only the DMA transfer completion interrupt of the DMA channel used for reading
    seDMAC_EnableInt( seDMAC_TRANSF_COMPL, seDMAC_CH2 );
    ///< Clear pending DMA interrupts in the CPU core
    ///< Enable pending DMA interrupts in the CPU core
    
    ///< Enable the SPI to issue DMA transfer requests to both the DMA channels
    SPIx->TBEDMAEN |= seDMAC_CH1;
    SPIx->RBFDMAEN |= seDMAC_CH2;
    
    do {
      setEventWait();
      if ( seDMAC_GetIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CH2 ) ) {
        SPIx->RBFDMAEN  &= ~seDMAC_CH2;
        seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CH2 );
      }
      } while ( (seDMAC_GetMode( seDMAC_CH2 ) != seDMAC_MODE_STOP ) );

    SPIx->RBFDMAEN  &= ~seDMAC_CH2;
    seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CH2 );
    SPIx->TBEDMAEN  &= ~seDMAC_CH1;
    seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CH1 );

    seDMAC_EnableRequestMask( seDMAC_CNLS( seDMAC_CH1 | seDMAC_CH2 ) );
    seDMAC_Disable( seDMAC_CNLS( seDMAC_CH1 | seDMAC_CH2 ) );
  } else {
  
    ///< Configure the primary data structure for the DMA channel (Ch.2) used for reading data from the RXD
    uint32_t cdata1 = seDMAC_cdata( seDMAC_MODE_BASIC,size_m1,0UL,seDMAC_SIZE_HALF_WORD,seDMAC_SIZE_HALF_WORD,seDMAC_INC_NO,seDMAC_INC_2);
    seDMAC_SetChannel( cdata1, ( uint32_t )&SPIx->RXD, ( uint32_t )data+size_m1*2, seDMAC_CH2 );
    ///< Enable the DMA channel
    seDMAC_Enable( seDMAC_CH2 );
    ///< Increase the priority of the DMA channel used for reading data
    seDMAC_PriorityIncrease( seDMAC_CH2 );
    ///< Clear the channel request masks for both the DMA channels
    seDMAC_DisableRequestMask( seDMAC_CH2 );
    ///< Clear the DMA transfer completion interrupt flags
    seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CH2 );
    ///< Enable only the DMA transfer completion interrupt of the DMA channel used for reading
    seDMAC_EnableInt( seDMAC_TRANSF_COMPL, seDMAC_CH2 );
    ///< Clear pending DMA interrupts in the CPU core
    ///< Enable pending DMA interrupts in the CPU core
    
    ///< Enable the SPI to issue DMA transfer requests to the DMA channel
    SPIx->RBFDMAEN |= seDMAC_CH2;
    
    do {
      setEventWait(); 
      if ( seDMAC_GetIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CH2 ) ) {
        SPIx->RBFDMAEN  &= ~seDMAC_CH2;
        seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CH2 ); 
      }
    } while ( (seDMAC_GetMode( seDMAC_CH2 ) != seDMAC_MODE_STOP ) );
 
    SPIx->RBFDMAEN  &= ~seDMAC_CH2;
    seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CH2 );
 
    seDMAC_EnableRequestMask( seDMAC_CH2 );
    seDMAC_Disable( seDMAC_CH2 );
  
  }
  ///< wait for idle state.
  WHILE( SPIx->INTF & SPIA_0_INTF_BSY_Msk, fStatus );
  
  return fStatus;
}

