/**
  ******************************************************************************
  * @file    se_dmausb.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file provides all the DMA for USB hardware functions.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2018-2020. All rights reserved.
  ******************************************************************************
  */

#include "se_dmac.h"
#include "se_usb.h"
#include "se_dmausb.h"

static seDMAC_CHANNEL s_DmaChIn  = seDMAC_CH_NONE; 
static seDMAC_CHANNEL s_DmaChOut = seDMAC_CH_NONE;

/**
  * @brief  Initializes DMA and prepares it for USB use.
  * @param  dma_data_struc_ptr: Address of an alocated buffer for use for DMAC descriptors. Must be aligned.
  * @param  DmaChIn: Input DMA channel.
  * @param  DmaChOut: Output DMA channel.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seDMAUSB_Init( uint32_t dma_data_struc_ptr, seDMAC_CHANNEL DmaChIn, seDMAC_CHANNEL DmaChOut ) {

  seStatus fStatus = seSTATUS_NG;

  if ( DmaChIn != seDMAC_CH_NONE || DmaChOut != seDMAC_CH_NONE ) { // &&?
    fStatus = seDMAC_Init( ( dma_data_struc_ptr + 0x80) & ~0x7FU, 3 );
    if ( DmaChIn ) {
      volatile uint16_t * enable_trigger_IN = seAUX_ConfigMemToPeriphForDMAC( NULL, (uint32_t)USB, 0, DmaChIn );
      if ( enable_trigger_IN ) {
        seDMAC_Enable( DmaChIn );
        seDMAC_DisableRequestMask( DmaChIn );
        seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, DmaChIn );
        uint32_t index = seDMAC_IDX( DmaChIn ); 
        volatile seDMAC_DataStruct * ptr = (seDMAC_DataStruct*)DMAC->CPTR;
        if ( ptr ) {
          ptr[index].transfer_destination_end_pointer = ( uint32_t )&USB->FIFODAT;
        }
      }
    }
    if ( DmaChOut ) {
      volatile uint16_t * enable_trigger_OUT = seAUX_ConfigPeriphToMemForDMAC( (uint32_t)USB, NULL, 0, DmaChOut );
      if ( enable_trigger_OUT ) {
        seDMAC_Enable( DmaChOut );
        seDMAC_DisableRequestMask( DmaChOut );
        seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, DmaChOut );
        uint32_t index = seDMAC_IDX( DmaChOut ); 
        volatile seDMAC_DataStruct * ptr = (seDMAC_DataStruct*)DMAC->CPTR;
        if ( ptr ) {
          ptr[index].transfer_source_end_pointer = ( uint32_t )&USB->FIFODAT;
        }
      }
    }

    s_DmaChIn = DmaChIn;
    s_DmaChOut = DmaChOut;
  }

  return fStatus;
}

/**
  * @brief  Reads available data from FIFO by DMA.
  * @param  EPNum: Endpoint number.
  * @param  buf: Pointer to a destination byte array.
  * @param  size: Destination array size.
  * @retval Returns a number of bytes read.
  */
uint16_t seDMAUSB_ReadFifo( uint16_t EPNum, uint8_t* buf, uint16_t size ) {

  ///< If the FIFO has available space for receiving data packets, the USB controller automatically responds to OUT
  ///< transactions to receive data. This enables software to perform OUT transfer without individual transaction control.
  ///< Note, however, that the USBEPmCTL.FNAK bit of the endpoint is set if a short packet is received (including
  ///< zero-length data packet) when the USBEPmCTL.AUTOFNAKDIS bit is cleared. Clear the USBEPmCTL.
  ///< FNAK bit when the next data transfer is ready.

  seUSB_SetReadDataEndpoint( EPNum );

  ///< Read the USBREMDATCNT.REMDAT[6:0] bits. (Check number of valid FIFO data)
  uint16_t count = USB->REMDATCNT_b.REMDAT;
  uint16_t transfcount = 0;
  ///< Read FIFO data through the USBFIFODAT.FIFODAT[7:0] bits.
  ///< By repeatedly executing Step 4, data in the FIFO of the specified endpoint
  ///< can be read by one byte at a time in the order of reception
  if ( count != 0 && size != 0 ) {
    transfcount = ( count <= size ) ? count : size;
    if ( transfcount >= seUSB_DMATRAN_THRESHOLD ) {
      if ( transfcount != 0 && s_DmaChOut != seDMAC_CH_NONE  ) {
        volatile seDMAC_DataStruct * ptr = (seDMAC_DataStruct*)DMAC->CPTR;
        //volatile uint16_t * enable_trigger_OUT = &USBMISC->RDDMAEN;
        if ( ptr ) {
          ptr += seDMAC_IDX(s_DmaChOut);
          ptr->control_data = seDMAC_cdata( seDMAC_MODE_BASIC,transfcount-1,0,
                                        seDMAC_SIZE_BYTE,seDMAC_SIZE_BYTE,
                                        seDMAC_INC_NO,seDMAC_INC_1 );
          ptr->transfer_destination_end_pointer = transfcount-1 + (uint32_t)buf;
          //*enable_trigger_OUT |= s_DmaChOut;
          USBMISC->RDDMAEN |= s_DmaChOut; ///< enable_trigger_OUT
          DMAC->ENSET = s_DmaChOut;
          while (( ptr->control_data & 7 ) != seDMAC_MODE_STOP );
        }
      }
    } else {
      int j = 0;
      do {
        buf[j++] = USB->FIFODAT_b.FIFODAT;
      }while ( j < transfcount );
    }
  }

  seUSB_ClearReadDataEndpoint(  EPNum  );
  
  return (transfcount);
}

/**
  * @brief  Writes data to available space in FIFO by DMA.
  * @param  EPNum: Endpoint number.
  * @param  buf: Pointer to a source byte array.
  * @param  size: Source array size.
  * @retval Returns a number of bytes written.
  */
uint16_t seDMAUSB_WriteFifo( uint16_t EPNum, const uint8_t* buf, uint16_t size ) {

  ///< Read the USBREMSPCCNT.REMSPC[6:0] bits.) (Check available space in FIFO)
  ///< Attention:
  ///< From the CPU, no writing is allowed to the same endpoint while the
  ///< USB host is writing data to the FIFO.
  ///< Set the USBFIFORWEN.FIFOWREN bit to 1. (Enable USBWRFIFOSEL register)
  ///< Set the USBWRFIFOSEL.EPmWR bit. (Select endpoint to write data)
  ///< before writing data, set the
  ///< USBEP0OCTL.FNAK or USBEPmCTL.FNAK bit so that the USB
  ///< controller will not receive data in an OUT transaction.
  seUSB_SetWriteDataEndpoint( EPNum );
  
  volatile uint16_t count = USB->REMSPCCNT_b.REMSPC;
  uint16_t transfcount = 0;

  ///< Write data to the FIFO through the USBFIFODAT.FIFODAT[7:0] bits.
  ///< By repeatedly executing Step 4, data can be written to the FIFO of the specified
  ///< endpoint by one byte at a time. The data written to the FIFO will be transmitted
  ///< in data packets in the order of writing.
  if ( count != 0 && size != 0 ) {
    transfcount  = ( size <= count ) ? size : count;
    if ( transfcount >= seUSB_DMATRAN_THRESHOLD ) {
        if ( transfcount != 0 && s_DmaChIn != seDMAC_CH_NONE ) {
            volatile seDMAC_DataStruct * ptr = (seDMAC_DataStruct*)DMAC->CPTR; 
            //volatile uint16_t * enable_trigger_IN = &USBMISC->WRDMAEN;
            if ( ptr ) {
              ptr += seDMAC_IDX(s_DmaChIn);
              ptr->control_data = seDMAC_cdata( seDMAC_MODE_BASIC,transfcount-1,0UL,
                                                seDMAC_SIZE_BYTE,seDMAC_SIZE_BYTE,
                                                seDMAC_INC_1,seDMAC_INC_NO ); 
              ptr->transfer_source_end_pointer = /*saddress +*/ transfcount-1  + (uint32_t)buf;
              //*enable_trigger_IN |= s_DmaChIn;
              USBMISC->WRDMAEN |= s_DmaChIn; ///< enable_trigger_IN
              DMAC->ENSET = s_DmaChIn;
              while ( (ptr->control_data & 7) != seDMAC_MODE_STOP );
            }
        }
    } else {
      int j = 0;
      do {
        USB->FIFODAT_b.FIFODAT = buf[j++];
      }while ( j < transfcount );
    }
  }

  if ( count != 0 ) {
    ///< Clear the USBEPmCTL. FNAK bit since the next data transfer is ready.
    seUSB_ClearWriteDataEndpoint(  EPNum, size  );
  }
  return transfcount;
}

