/**
  ******************************************************************************
  * @file    se_i2c.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file provides all the I2C hardware functions.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#include "se_i2c.h"

/**
  * Static functions declarations
  */
typedef enum {
   seI2C_STOP_CND      = 0x01,  ///< I2C status : STOP condition.
   seI2C_START_CND     = 0x02,  ///< I2C status : START condition.
   seI2C_TIME_OUT_CND  = 0x03   ///< I2C status : START/STOP condition timeout.
} seStartStopCondition;

static inline seStartStopCondition waitI2cSlvStartStopCondition( I2C_0_Type* I2Cx );
static seStatus setI2cMstStopCondition( I2C_0_Type* I2Cx );
static seStatus setI2cMstStartCondition( I2C_0_Type* I2Cx );

/**
  * @brief  Initializes the I2Cx peripheral according to the specified
  *         parameters in the InitStruct.
  * @param  I2Cx: I2C peripheral.
  * @param  InitStruct: Pointer to a @ref seI2C_InitTypeDef structure that
  *         contains the configuration information for the specified I2C peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seI2C_Init( I2C_0_Type * I2Cx, seI2C_InitTypeDef* InitStruct ) {  
  
  ///< Check if selected clock is enabled
  seStatus fStatus = seCLG_IsEnabled(seSRC(InitStruct->Clk));
    
  ///< When Clk not started, do nothing.
  if ( fStatus == seSTATUS_OK ) {   

    ///< Stop I2C.
    seI2C_Disable( I2Cx );

	///< Configuring the pins succeeded, so setup for Master or Slave mode
	if ( InitStruct->I2C_mode == seI2C_MODE_MASTER ) {
		///< Set the following I2C0CTL register.
		I2Cx->CTL_b.MST = InitStruct->I2C_mode;      ///< Set master mode(Master).

		///< Configure operating clock.
		I2Cx->CLK_b.CLKSRC = seSRC(InitStruct->Clk); ///< Set the clock source of the I2C.
		I2Cx->CLK_b.CLKDIV = seDIV(InitStruct->Clk); ///< Set the division ratio of the I2C operating clock.

		///< Set transfer rate( bps = CLK_I2C / (BRT + 3) x 2 ).
		///< 8.00MHz / 20 = 400000 bps.
		I2Cx->BR_b.BRT = InitStruct->BRT;
	} else {
		///< Set the following I2C0CTL register.
		I2Cx->CTL_b.MST = InitStruct->I2C_mode;       ///< Set slave mode(Slave).
		///< Set 10/7-bit address mode(0=7bit address mode / 1=10bit address mode).
		I2Cx->MOD_b.OADR10 = InitStruct->AddrMode;    ///< Set the number of own address bits for slave mode.
		///< Disable response to general call address.
		I2Cx->MOD_b.GCEN = InitStruct->RespGenCalls;  ///< Set whether to respond to master general calls in slave mode or not.
		///< Set slave address(7bit address mode).
		I2Cx->OADR_b.OADR = InitStruct->SlaveAddr;    ///< Set the own address for slave mode.
	}

	///< Clock supplied in DEBUG mode.
	I2Cx->CLK_b.DBRUN           = 1;    ///< Whether clock supplied in DEBUG mode or not.

	///< Disable I2C interrupts.
	I2Cx->INTE_b.TBEIE          = 0;    ///< Transmit buffer empty interrupt.
	I2Cx->INTE_b.RBFIE          = 0;    ///< Receive buffer full interrupt.
	I2Cx->INTE_b.ERRIE          = 0;    ///< Error detection interrupt.
	I2Cx->INTE_b.STARTIE        = 0;    ///< START condition interrupt.
	I2Cx->INTE_b.STOPIE         = 0;    ///< STOP condition interrupt.
	I2Cx->INTE_b.NACKIE         = 0;    ///< NACK reception interrupt.
	I2Cx->INTE_b.GCIE           = 0;    ///< General call address reception interrupt.
	I2Cx->INTE_b.BYTEENDIE      = 0;    ///< End of transfer interrupt.

  }

  return fStatus;
}

/**
  * @brief  Fills each InitStruct member with its default value for Master mode.
  * @param  InitStruct: Pointer to an @ref seI2C_InitTypeDef structure which will be initialized.
  * @retval None
  */
void seI2C_InitStructForMaster(seI2C_InitTypeDef* InitStruct) {
  
  InitStruct->I2C_mode      = seI2C_MODE_MASTER;
  InitStruct->Clk           = seDEFAULT_I2C_CLK;
  ///< Set transfer rate( bps = CLK_I2C / (BRT + 3) x 2 ).
  ///< 8.00MHz / 20 = 400000 bps.
  InitStruct->BRT           = 7;
}

/**
  * @brief  Fills each InitStruct member with its default value for Slave mode.
  * @param  InitStruct: Pointer to an @ref seI2C_InitTypeDef structure which will be initialized.
  * @retval None
  */
void seI2C_InitStructForSlave(seI2C_InitTypeDef* InitStruct) {
  
  InitStruct->I2C_mode      = seI2C_MODE_SLAVE;
  InitStruct->AddrMode      = seI2C_7BIT_SLV_ADDR;
  InitStruct->RespGenCalls  = seDISABLE;
  InitStruct->SlaveAddr     = 0x50;
}

/**
  * @brief  Resets the I2C channel.
  * @param  I2Cx: I2C peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seI2C_Reset( I2C_0_Type* I2Cx ) {
  
  seStatus fStatus = seSTATUS_OK;
  ///< Execute software reset.
  I2Cx->CTL_b.SFTRST = 1; 
  WHILE( I2Cx->CTL & I2C_0_CTL_SFTRST_Msk, fStatus );
  
  return fStatus;
}  

/**
  * @brief  Enables the specified I2C channel.
  * @param  I2Cx: I2C peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seI2C_Enable( I2C_0_Type* I2Cx ) {
  
  seStatus fStatus = seI2C_Reset( I2Cx );
  seASSERT( fStatus );
  
  ///< Enable I2C Ch.x operations.
  I2Cx->CTL_b.MODEN = 1; 

  return fStatus;  
}

/**
  * @brief  Disables the specified I2C channel.
  * @param  I2Cx: I2C peripheral.
  * @retval None
  */
void seI2C_Disable( I2C_0_Type* I2Cx ) {
     
  ///< Disable I2C Ch.x operations.
  I2Cx->CTL_b.MODEN = 0;
}

/**
  * @brief  Sends a data byte through the I2Cx peripheral if Master mode.
  * @param  I2Cx: I2C peripheral.
  * @param  address: I2C adddres, can be 10-bit address.
  * @param  data: Data to be transmitted.
  * @param  size: Number of bytes in data to send.
  * @param  stop_pending: if zero - send stop condition, if non zero - stop pending.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seI2C_MstSendData(I2C_0_Type* I2Cx, uint16_t address, uint8_t data[], uint32_t size, uint32_t stop_pending ) {

  ///< Set Start condition.
  seStatus fStatus = setI2cMstStartCondition( I2Cx );

  if ( fStatus == seSTATUS_OK ) {
    ///< Send slave address(+Write).
    if ( address > 0x7f ) {
      address &= 0x3ff;          //< extract 10-bit address
      address |= 0x1e << 10;     //< add leading '11110 to introduce the 10 bit addressing scheme
      I2Cx->TXD_b.TXD = (uint8_t)((address>>8) << 1) + 0;
    
      WHILE( (I2Cx->INTF & I2C_0_INTF_TBEIF_Msk) == 0, fStatus );
      if ( fStatus == seSTATUS_OK ) {
        I2Cx->TXD_b.TXD = (uint8_t)address;
      }
    } else {
      I2Cx->TXD_b.TXD = (address << 1) + 0;
    }
    
    ///< Send data.
    while ( size--  && (fStatus==seSTATUS_OK) ) {
      WHILE( (I2Cx->INTF&(I2C_0_INTF_TBEIF_Msk | I2C_0_INTF_NACKIF_Msk)) == 0, fStatus );
      ///< Check result.
      if ( fStatus == seSTATUS_OK ) {
        if ( I2Cx->INTF_b.TBEIF == 1 ) { 
          I2Cx->TXD_b.TXD = *data++; 
        }
        ///< The result is not checked at the last data.
        if ( I2Cx->INTF_b.NACKIF == 1 ) {
          seI2C_ClearIntFlag( I2Cx, I2C_0_INTF_NACKIF_Msk );
          fStatus = seSTATUS_NG;
          break;
        }
      }
    }

    ///< Set STOP condition.
    if ( ! stop_pending ) {
      if ( seSTATUS_NG == setI2cMstStopCondition( I2Cx ) ) {
        fStatus = seSTATUS_NG;
      }
    } 
  }
  
  return fStatus;
}

/**
  * @brief  Returns the most recent received data by the I2Cx peripheral if Master mode.
  * @param  I2Cx: I2C peripheral.
  * @param  address: I2C adddres, can be 10-bit address.
  * @param  data: ptr to place recieved data.
  * @param  size: Number of bytes in data to receive.
  * @param  stop_pending: if zero - send stop condition, if non zero - stop is pending
  * @retval Status: can be a value of @ref seStatus
  *         received data is store in data[].
  */
seStatus seI2C_MstReceiveData(I2C_0_Type* I2Cx, uint16_t address, uint8_t data[], uint32_t size, uint32_t stop_pending  ) {
  
  seStatus fStatus;
     
  if ( size == 1 ) {
    I2Cx->CTL_b.TXNACK = 1;
  }
  
  if ( address > 0x7f ) {
    fStatus = seI2C_MstSendData( I2Cx, address, data, 0, seENABLE ); 

    if ( fStatus == seSTATUS_OK ) {
    fStatus = setI2cMstStartCondition( I2Cx );

    if ( fStatus == seSTATUS_OK ) {
        address &= 0x3ff;          ///< extract 10-bit address
        address |= 0x1e << 10;     ///< add leading '11110 to introduce the 10 bit addressing scheme
        I2Cx->TXD_b.TXD = (uint8_t)((address>>8) << 1) + 1;
      }
    }
  } else {
    fStatus = setI2cMstStartCondition( I2Cx );
    if ( fStatus == seSTATUS_OK ) {
      ///< Send slave 7-bit address(7 to 1 bits) + Read(1)/Write(0)).
      I2Cx->TXD_b.TXD = (address << 1) + 1;
    }
  } 
  
  while ( size--  && (fStatus==seSTATUS_OK)  ) { 
    WHILE( (I2Cx->INTF&(I2C_0_INTF_RBFIF_Msk | I2C_0_INTF_NACKIF_Msk)) == 0, fStatus );
    if ( fStatus == seSTATUS_OK ) {  
	  if ( I2Cx->INTF_b.RBFIF == 1 ) {
	    ///< When the next-to-last data is received, send a NACK.
	    if ( size == 1 ) { 
	      I2Cx->CTL_b.TXNACK = 1; 
	    }
	    ///< receive Data.
	    *data++ = I2Cx->RXD_b.RXD;
	  } 
	  if ( I2Cx->INTF_b.NACKIF == 1 ) { 
	    seI2C_ClearIntFlag( I2Cx, I2C_0_INTF_NACKIF_Msk );
	    fStatus = seSTATUS_NG;
	    break;
	  }      
    }     
  }

  ///< Set STOP condition.
  if ( ! stop_pending ) {    
    if ( seSTATUS_NG == setI2cMstStopCondition( I2Cx ) ) {
      fStatus = seSTATUS_NG;
    }
  }    
  
  return fStatus; 
}

/**
  * @brief  Sends a data byte through the I2Cx peripheral if Slave mode.
  * @param  I2Cx: I2C peripheral.
  * @param  data: Data to be transmitted.
  * @param  size: Number of bytes in data to send.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seI2C_SlvSendData( I2C_0_Type* I2Cx, uint8_t data[], uint32_t size ) { 
  
  seStatus fStatus = seSTATUS_OK; 

  if ( I2Cx->MOD_b.OADR10 == seI2C_10BIT_SLV_ADDR ) {
    fStatus = seI2C_SlvReceiveData( I2Cx, data, 0 );    
  }
  
  if ( fStatus == seSTATUS_OK ) {
    ///< Wait Start flag condition.
    if ( seI2C_START_CND == waitI2cSlvStartStopCondition( I2Cx ) ) {

      ///< Check transmission mode(1:Transmit/0:Receive).
      if ( I2Cx->INTF_b.TR == 1 ) {
        // Transmit
        seI2C_ClearIntFlag( I2Cx, I2C_0_INTF_STARTIF_Msk ); 
        while ( size-- ) {
          
          I2Cx->TXD_b.TXD = *data++;
          
          ///< Wait for an interrupt request
          WHILE( (I2Cx->INTF&(I2C_0_INTF_TBEIF_Msk | 
                             I2C_0_INTF_NACKIF_Msk | 
                             I2C_0_INTF_STOPIF_Msk)) == 0, fStatus );
          if ( fStatus == seSTATUS_OK ) {    
            if ( I2Cx->INTF_b.NACKIF == 1 ) {
              ///< clear NACK.
              seI2C_ClearIntFlag( I2Cx, I2C_0_INTF_NACKIF_Msk );
              fStatus = seSTATUS_NG;
            } 
            if ( I2Cx->INTF_b.STOPIF == 1 ) { 
              fStatus = seSTATUS_NG;
            } 
          }
          if ( fStatus == seSTATUS_NG ) {
            break;
          }
        }

        if ( fStatus == seSTATUS_OK ) {
          WHILE( (I2Cx->INTF & I2C_0_INTF_TBEIF_Msk) == 0, fStatus );
          if ( fStatus == seSTATUS_OK ) I2Cx->TXD_b.TXD = 0;
        }
        
        ///< Wait for the last byte transmitted
        while ( seINTERRUPT_NOT_OCCURRED == seI2C_GetIntFlag( I2Cx, I2C_0_INTF_BYTEENDIF_Msk ) ) {
           ;
        }

        seI2C_ClearIntFlag( I2Cx, I2C_0_INTF_BYTEENDIF_Msk );
        
        waitI2cSlvStartStopCondition( I2Cx );

      } else {
        fStatus = seSTATUS_NG;
      }
    } else {
      fStatus = seSTATUS_NG;
    }
  }
  
  return fStatus;
}

/**
  * @brief  Returns the most recent received data by the I2Cx peripheral if Slave mode.
  * @param  I2Cx: I2C peripheral.
  * @param  data: ptr to be place received data.
  * @param  size: Number of bytes in data to receive.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seI2C_SlvReceiveData( I2C_0_Type* I2Cx, uint8_t data[], uint32_t size ) {
  
  seStatus fStatus = seSTATUS_NG;
  
  if ( size == 1 ) {
    I2Cx->CTL_b.TXNACK = 1; 
  }
  
  if ( seI2C_START_CND == waitI2cSlvStartStopCondition( I2Cx ) ) {

    ///< Check transmission mode(1:Transmit/0:Receive).
    if ( I2Cx->INTF_b.TR == 0 ) {       
      fStatus = seSTATUS_OK;
      
      ///< Clear START condition interrupt.
      seI2C_ClearIntFlag( I2Cx, I2C_0_INTF_STARTIF_Msk );

      ///< Receive data.
      while ( size-- ) {
        WHILE( (I2Cx->INTF&(I2C_0_INTF_RBFIF_Msk | I2C_0_INTF_BYTEENDIF_Msk)) == 0, fStatus );
        if ( fStatus == seSTATUS_NG )  {
          break;
        }        
        seI2C_ClearIntFlag( I2Cx, I2C_0_INTF_BYTEENDIF_Msk );
        if ( size == 1 ) {
          I2Cx->CTL_b.TXNACK = 1; 
        }
        if ( I2Cx->INTF_b.RBFIF == 1 ) {
          ///< receive Data.
          *data++ = I2Cx->RXD_b.RXD;
        }
      }
      ///< Wait for a STOP condition interrupt.
      waitI2cSlvStartStopCondition( I2Cx );
    }
  }

  return fStatus;
}

/**
  * @brief  Enables I2C channel interrupt.
  * @param  I2Cx: I2C peripheral.
  * @param  irq: The interrupt to enable, can be value of @ref seI2C_Interrupt.
  * @retval None
  */
void seI2C_EnableInt( I2C_0_Type* I2Cx, uint16_t irq ) {
  
  I2Cx->INTE |= irq;
}

/**
  * @brief  Disables I2C channel interrupt.
  * @param  I2Cx: I2C peripheral.
  * @param  irq: The interrupt to disable, can be value of @ref seI2C_Interrupt.
  * @retval None
  */
void seI2C_DisableInt( I2C_0_Type* I2Cx, uint16_t irq ) {
  
  I2Cx->INTE &= ~irq;
}

/**
  * @brief  Gets I2C channel interrupt flag.
  * @param  I2Cx: I2C peripheral.
  * @param  flag: The interrupt flags.
  * @retval InterruptStatus: can be a value of @ref seInterruptStatus
  */
seInterruptStatus seI2C_GetIntFlag( I2C_0_Type* I2Cx, uint16_t flag ) {
  
  seInterruptStatus fStatus = seINTERRUPT_NOT_OCCURRED;
  uint16_t flagged = I2Cx->INTF;
  
  if ( flag ) {
    flagged &= flag;
    if ( flagged == flag ) {
      fStatus = seINTERRUPT_OCCURRED;
    }
  }
  
  return fStatus;
}

/**
  * @brief  Clears I2C channel interrupt flag.
  * @param  I2Cx: I2C peripheral.
  * @param  flag: The interrupt flags.
  * @retval None
  */
void seI2C_ClearIntFlag( I2C_0_Type* I2Cx, uint16_t flag ) {
  
  I2Cx->INTF = flag;
}


/**
  * Static functions
  */


/**
  * @brief  Issue a START condition.
  * @retval Status
  */
static seStatus setI2cMstStartCondition( I2C_0_Type* I2Cx ) {
  
  seStatus fStatus = seSTATUS_OK; 
  ///< Clear START condition interrupt.
  seI2C_ClearIntFlag( I2Cx, I2C_0_INTF_STARTIF_Msk/*seI2C_STARTIF*/ );

  ///< Issue a START condition.
  I2Cx->CTL_b.TXSTART = 1; 

  ///< Wait START condition.
  WHILE( (I2Cx->INTF & I2C_0_INTF_STARTIF_Msk) == 0, fStatus );
  
  ///< Clear START condition interrupt.
  seI2C_ClearIntFlag( I2Cx, I2C_0_INTF_STARTIF_Msk/*seI2C_STARTIF*/ );
  
  return fStatus;
}

/**
  * @brief  Issue a STOP condition.
  * @retval Status
  */
static seStatus setI2cMstStopCondition( I2C_0_Type* I2Cx ) {
  
  seStatus fStatus = seSTATUS_OK; 
  ///< Clear STOP condition interrupt.
  seI2C_ClearIntFlag( I2Cx, I2C_0_INTF_STOPIF_Msk/*seI2C_STOPIF*/ );

  ///< Issue a STOP condition.
  I2Cx->CTL_b.TXSTOP = 1; 
  
  ///< Wait Stop flag condition.
  WHILE( (I2Cx->INTF & I2C_0_INTF_STOPIF_Msk) == 0, fStatus );
  
  ///< Clear STOP condition interrupt.
  seI2C_ClearIntFlag( I2Cx, I2C_0_INTF_STOPIF_Msk/*seI2C_STOPIF*/ );
  
  return fStatus;
}     

/**
  * @brief  wait I2C START/STOP condition.
  * @retval seI2C_status
  */
static inline seStartStopCondition waitI2cSlvStartStopCondition( I2C_0_Type* I2Cx ) {

  seStatus fStatus = seSTATUS_OK;
  seStartStopCondition ss_condition = seI2C_TIME_OUT_CND;
  ///< Wait for a START/STOP condition interrupt.
  WHILE( (I2Cx->INTF&(I2C_0_INTF_STOPIF_Msk | I2C_0_INTF_STARTIF_Msk)) == 0, fStatus );

  if ( fStatus == seSTATUS_OK ) {
    ///<  Check START condition interrupt.
    if ( I2Cx->INTF_b.STARTIF == 1 ) { 
      ///< Clear START condition interrupt.
      ss_condition = seI2C_START_CND;
    }
    ///< Check STOP condition interrupt.
    else if ( I2Cx->INTF_b.STOPIF == 1 ) { 
      ///< Clear STOP condition interrupt.
      seI2C_ClearIntFlag( I2Cx, I2C_0_INTF_STOPIF_Msk/*seI2C_STOPIF*/ );
      ///< Time-out START/STOP condition interrupt.
      ss_condition = seI2C_STOP_CND;
    } else { }
  } 
  
  return ss_condition;
}

