/**
  ******************************************************************************
  * @file    se_i2c.h
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file contains all the functions prototypes for the I2C firmware
  *          library.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#ifndef SE_I2C_H
#define SE_I2C_H

#include "se_common.h"

#ifdef __cplusplus
 extern "C" {
#endif

/** @addtogroup sePeriphLibrary
  * @{
  */

/** @defgroup I2C
  * @{
  * @brief The I2C module is a subset of the I2C bus interface
  */

/** @defgroup I2C_Constants
  * @{
  */

typedef enum {
   seI2C_MODE_SLAVE  = 0,                       ///< Specifies the I2C mode slave
   seI2C_MODE_MASTER = 1                        ///< Specifies the I2C mode master
} seI2C_mode;                                   ///< Master Mode Select

typedef enum {
   seI2C_7BIT_SLV_ADDR  = 0,                    ///< Specifies the I2C mode slave
   seI2C_10BIT_SLV_ADDR = 1                     ///< Specifies the I2C mode master
} seI2C_AddrMode;                               ///< Own addressing mode select

/**
  * @}
  */

/** @defgroup I2C_Types
  * @{
  */

/** 
  * @brief  I2C Init structure definition
  */

typedef struct {
   sePPORT_Id  SCL;
   sePPORT_Id  SDA;
} seI2C_Ports;

typedef struct {
   seI2C_mode            I2C_mode;         ///< Specifies Master/Slave mode, a value of @ref I2C_mode
   /* if Master mode */
   seI2C_Clk             Clk;              ///< Specifies the i2c clock source selection and the prescaler value used to divide the i2c clock.
   uint16_t              BRT;              ///< Specifies the baud rate( bps = CLK_I2C / (BRT + 3) x 2 ).
                                           ///< - 8.00MHz / 20 = 400000 bps.
                                           ///< Note: The I2C bus transfer rate is limited to 100 kbit/s in
                                           ///<       standard mode or 400 kbit/s in fast mode. Do not set a
                                           ///<       transfer rate exceeding the limit.
   /* if Slave mode */
   seI2C_AddrMode        AddrMode;         ///< Specifies if 7-bit or 10-bit address, a seI2C_AddrMode value.
   uint16_t              SlaveAddr;        ///< Specifies the slave address.
                                           ///< This parameter can be a 7-bit or 10-bit address.
   seState               RespGenCalls;     ///< Specifies if slave is to response to master's general calls.
} seI2C_InitTypeDef;

/**
  * @}
  */

/** @defgroup I2C_Functions
  * @{
  */

/**
  * @brief  Fills each InitStruct member with its default value for Master mode.
  * @param  InitStruct: Pointer to an @ref seI2C_InitTypeDef structure which will be initialized.
  * @retval None
  */
void seI2C_InitStructForMaster(seI2C_InitTypeDef* InitStruct);

/**
  * @brief  Fills each InitStruct member with its default value for Slave mode.
  * @param  InitStruct: Pointer to an @ref seI2C_InitTypeDef structure which will be initialized.
  * @retval None
  */
void seI2C_InitStructForSlave(seI2C_InitTypeDef* InitStruct);

/**
  * @brief  Initializes the I2Cx peripheral according to the specified
  *         parameters in the InitStruct.
  * @param  I2Cx: I2C peripheral.
  * @param  InitStruct: Pointer to a @ref seI2C_InitTypeDef structure that
  *         contains the configuration information for the specified I2C peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seI2C_Init( I2C_0_Type * I2Cx, seI2C_InitTypeDef* InitStruct );

/**
  * @brief  Enables the specified I2C channel.
  * @param  I2Cx: I2C peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seI2C_Enable( I2C_0_Type* I2Cx );

/**
  * @brief  Disables the specified I2C channel.
  * @param  I2Cx: I2C peripheral.
  * @retval None
  */
void seI2C_Disable( I2C_0_Type* I2Cx );

/**
  * @brief  Sends a data byte through the I2Cx peripheral if Master mode.
  * @param  I2Cx: I2C peripheral.
  * @param  address: I2C adddres, can be 10-bit address.
  * @param  data: Data to be transmitted.
  * @param  size: Number of bytes in data to send.
  * @param  stop_pending: if zero - send stop condition, if non zero - stop pending.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seI2C_MstSendData( I2C_0_Type* I2Cx, uint16_t address, uint8_t data[], uint32_t size, uint32_t stop_pending );

/**
  * @brief  Returns the most recent received data by the I2Cx peripheral if Master mode.
  * @param  I2Cx: I2C peripheral.
  * @param  address: I2C adddres, can be 10-bit address.
  * @param  data: ptr to place recieved data.
  * @param  size: Number of bytes in data to receive.
  * @param  stop_pending: if zero - send stop condition, if non zero - stop is pending
  * @retval Status: can be a value of @ref seStatus
  *         received data is store in data[].
  */
seStatus seI2C_MstReceiveData( I2C_0_Type* I2Cx, uint16_t address, uint8_t data[], uint32_t size, uint32_t stop_pending );

/**
  * @brief  Sends a data byte through the I2Cx peripheral if Slave mode.
  * @param  I2Cx: I2C peripheral.
  * @param  data: Data to be transmitted.
  * @param  size: Number of bytes in data to send.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seI2C_SlvSendData( I2C_0_Type* I2Cx, uint8_t data[], uint32_t size );

/**
  * @brief  Returns the most recent received data by the I2Cx peripheral if Slave mode.
  * @param  I2Cx: I2C peripheral.
  * @param  data: ptr to be place received data.
  * @param  size: Number of bytes in data to receive.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seI2C_SlvReceiveData( I2C_0_Type* I2Cx, uint8_t data[], uint32_t size );

/**
  * @brief  Resets the I2C channel.
  * @param  I2Cx: I2C peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seI2C_Reset( I2C_0_Type* I2Cx );

/**
  * @brief  Enables I2C channel interrupt.
  * @param  I2Cx: I2C peripheral.
  * @param  irq: The interrupt to enable, can be value of @ref seI2C_Interrupt.
  * @retval None
  */
void seI2C_EnableInt( I2C_0_Type* I2Cx, /*seI2C_Interrupt*/uint16_t irq );

/**
  * @brief  Disables I2C channel interrupt.
  * @param  I2Cx: I2C peripheral.
  * @param  irq: The interrupt to disable, can be value of @ref seI2C_Interrupt.
  * @retval None
  */
void seI2C_DisableInt( I2C_0_Type* I2Cx, /*seI2C_Interrupt*/uint16_t irq );

/**
  * @brief  Gets I2C channel interrupt flag.
  * @param  I2Cx: I2C peripheral.
  * @param  flag: The interrupt flags.
  * @retval InterruptStatus: can be a value of @ref seInterruptStatus
  */
seInterruptStatus seI2C_GetIntFlag( I2C_0_Type* I2Cx, uint16_t flag );

/**
  * @brief  Clears I2C channel interrupt flag.
  * @param  I2Cx: I2C peripheral.
  * @param  flag: The interrupt flags.
  * @retval None
  */
void seI2C_ClearIntFlag( I2C_0_Type* I2Cx, uint16_t flag );

/**
  * @brief  Configures ports for the I2C module.
  * @param  I2Cx: I2C peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seAUX_ConfigurePortsForI2C( I2C_0_Type* I2Cx, seI2C_Ports * ports );

/**
  * @}
  */ 

/**
  * @}
  */ 

/**
  * @}
  */ 

#ifdef __cplusplus
}
#endif
#endif // SE_I2C_H

