/**
  ******************************************************************************
  * @file    se_lcd32c.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file provides all the LCD32C hardware functions.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#include "string.h"
#include "se_lcd32c.h"

/**
  * @brief  Initializes the LCD32C peripheral according to the specified
  *         parameters in the LCD32C_InitStruct.
  * @note   This function configures the module, and module's interrupts.
  *         It clears module's interrupts but does not enable interrupt from the
  *         module to the CPU. This function enables module by start supplying
  *         operating clock. This module starts does not start display. It is user
  *         code responsibility to set appropriate display state.
  * @param  InitStruct: Pointer to a @ref seLCD32C_InitTypeDef structure that
  *         contains the configuration information for the LCD32C peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seLCD32C_Init( seLCD32C_InitTypeDef* InitStruct ) {
  
  ///< Check if selected clock is enabled
  seStatus fStatus = seCLG_IsEnabled(seSRC(InitStruct->Clk));
  
  ///< When Clk not started, do nothing.
  if ( fStatus == seSTATUS_OK ) {
  
    //fStatus = seAUX_ConfigurePortsForLcd32C();

    if ( fStatus == seSTATUS_OK ) {
      ///< Stop supplying operating clock
      seLCD32C_Disable();

      ///< Configure the LCD32C operating clock
      seLCD32C_ConfigureClock( InitStruct->Clk );

      ///< Enable operation clock; Start supplying operating clock
      seLCD32C_Enable( InitStruct->CtlLcdIoDischargeEn );
      
      ///< Configure desired frame frequency
      seLCD32C_SetFrameFreq( InitStruct );
      
      ///< Configure voltages
      seLCD32C_ConfigurePower( InitStruct );
      
      ///< Configure Display features
      seLCD32C_ConfigureDisplay( InitStruct );
      
      ///< Booster clock control
      seLCD32C_SetBoostClk( InitStruct->Tim2BoostClk );

      ///< Control SEG n line reverse drive selection
      LCD32C->TIM2_b.NLINE = InitStruct->Tim2Nline; ///<  Choices afrom 0 line to 31 lines

      ///< Set contrast
      seLCD32C_SetPanelContrast( seLCD32C_CONTRAST_LEVEL0 );

      ///< initialize interrupt.
      seLCD32C_DisableInt(); 
      
      ///< Clear interrupt flags.
      seLCD32C_ClearIntFlag();
    }
    
  } 
    
  return fStatus;
}

/**
  * @brief  Enables LCD32C by start supplying operating clock.
  * @param  IoDischarge: Controls if LCDIO discharge @ref seState.
  * @retval None
  */
void seLCD32C_Enable( seState IoDischarge  ) {

  ///< LCD IO discharge enable/disable
  LCD32C->CTL_b.LCDDIS = IoDischarge;
  
  ///< Enable LCD
  LCD32C->CTL_b.MODEN = 1;
}

/**
  * @brief  Disables LCD32C by stop supplying operating clock.
  * @retval None
  */
void seLCD32C_Disable( void ) {
  
  ///< Disable LCD
  LCD32C->CTL_b.MODEN = 0;
}

/**
  * @brief  Configures LCD32C clock source and clock divider.
  * @param  ClkSrc: Clock source can be a value of @ref seLCD32C_ClkSrc.
  * @param  ClkDivider: Clock divider a value of @ref seLCD32C_EXOSC_ClkDiv or @ref seLCD32C_IOSC_ClkDiv or ref seLCD32C_OSC3_ClkDiv or @ref seLCD32C_OSC1_ClkDiv
  *         depending on the selected clock source.
  * @retval None
  */
void seLCD32C_ConfigureClock( seLCD32C_Clk Clk ) {

  LCD32C->CLK_b.CLKSRC = seSRC(Clk);       ///< Clock source selection
  LCD32C->CLK_b.CLKDIV = seDIV(Clk);       ///< Clock division ratio selection
  LCD32C->CLK_b.DBRUN  = seDBRUN;          ///< Whether clock supplied in DEBUG mode or not.
}

/**
  * @brief  Sets frame rate by selecting the appropriate FRMCNT and LDUTY values.
  *         Refer to @ref LCD32C_Frame_Frequency_Control for details.
  * @param  InitStruct: pointer to a @ ref seLCD32C_InitTypeDef structure that
  *         contains the configuration information for the LCD32C peripheral.
  * @retval None
  */
void seLCD32C_SetFrameFreq( seLCD32C_InitTypeDef* InitStruct ) {

  ///< Set the drive duty
  LCD32C->TIM1_b.LDUTY = InitStruct->Tim1Duty;

  ///< Set the frame count 
  LCD32C->TIM1_b.FRMCNT = InitStruct->Tim1FrameCount;
}

/**
  * @brief  Sets Voltages.
  * @param  InitStruct: This parameter can be a value of @ref seLCD32C_InitTypeDef.
  * @retval None
  */
void seLCD32C_ConfigurePower( seLCD32C_InitTypeDef* InitStruct ) {

  ///< Set LCD Power Bias
  LCD32C->PWR_b.BIASSEL = InitStruct->PwrBiasSel;

  ///< Set LCD Power VC selection
  LCD32C->PWR_b.EXVCSEL = InitStruct->PwrExternalVcSel;
  
  ///< Set LCD voltage booster enable/disable
  LCD32C->PWR_b.BSTEN = InitStruct->PwrBoosterEn;
  
  ///< Set LCD doubler enable/disable
  LCD32C->PWR_b.DBLEN = InitStruct->PwrDoublerEn;
  
  ///< Set LCD heavy load protection mode enable/disable
  LCD32C->PWR_b.HVLD = InitStruct->PwrHeavyLoadProtectionModeEn;
  
  ///< Set LCD voltage regulator enable/disable
  LCD32C->PWR_b.VCEN = InitStruct->PwrVoltageRegulatorEn;
}

/**
  * @brief  Configres Display features.
  * @param  InitStruct: This parameter can be a value of @ref seLCD32C_InitTypeDef.
  * @retval None
  */
void seLCD32C_ConfigureDisplay( seLCD32C_InitTypeDef* InitStruct ) {

  ///< Select display area
  seLCD32C_SetDisplayArea( InitStruct->DspSelectArea );
  
  ///< Selects the common pin assignment direction
  LCD32C->DSP_b.COMREV  = InitStruct->DspSelComPinDir;
  
  ///< Selects the segment pin assignment direction.
  LCD32C->DSP_b.SEGREV = InitStruct->DspSelSegPinDir;
  
  ///< Select Display Inverted state
  seLCD32C_SetDisplayInvState( seLCD32C_DISP_NORMAL );
}

/**
  * @brief  Configres Display features.
  * @param  clk: This parameter can be a value of @ref seLCD32C_BoostClk.
  * @retval None
  */
void seLCD32C_SetBoostClk( seLCD32C_BoostClk clk ) {

  ///< Select the booster clock frequency for the LCD voltage booster.
  LCD32C->TIM2_b.BSTC = clk;
}

/**
  * @brief  Configres Display features.
  * @param  clk: This parameter can be a value of @ref seLCD32C_DoublerClk.
  * @retval None
  */
void seLCD32C_SetDoublerClk( seLCD32C_DoublerClk clk ) {
  
  ///< Select the booster clock frequency for the LCD circuit power supply voltage booster.
  LCD32C->TIM2_b.DBLC = clk;
}

/**
  * @brief  Sets Panel contrast.
  * @param  contrast: This parameter can be a value of @ref seLCD32C_Contrast.
  * @retval None
  */
void seLCD32C_SetPanelContrast( seLCD32C_Contrast contrast ) {
  
  ///< Set the LCD panel contrast.
  LCD32C->PWR_b.LC = contrast;
}

/**
  * @brief  Gets Panel contrast.
  * @retval Contrast: A value of @ref seLCD32C_Contrast.
  */
uint16_t seLCD32C_GetPanelContrast( void ) {
  
  ///< Get the LCD panel contrast.
  return LCD32C->PWR_b.LC;
}

/**
  * @brief  Sets Display state.
  * @param  state: This parameter can be a value of @ref seLCD32C_DisplayState.
  * @retval None
  */
void seLCD32C_SetDisplayState( seLCD32C_DispState state ) {

  LCD32C->DSP_b.DSPC = state;
}

/**
  * @brief  Gets Display state.
  * @retval A value of @ref seLCD32C_DispState.
  */
uint16_t seLCD32C_GetDisplayState( void ) {

  return LCD32C->DSP_b.DSPC;
}

/**
  * @brief  Sets Display Inverted state.
  * @param  state: This parameter can be a value of @ref seLCD32C_DispInvState.
  * @retval None
  */
void seLCD32C_SetDisplayInvState( seLCD32C_DispInvState state ) {

  LCD32C->DSP_b.DSPREV = state;
}

/**
  * @brief  Gets Display Inverted state.
  * @retval InvState: A value of @ref seLCD32C_DispInvState.
  */
uint16_t seLCD32C_GetDisplayInvState( void ) {
  
  return LCD32C->DSP_b.DSPREV;
}

/**
  * @brief  Gets Display Area.
  * @retval area: A value of @ref seLCD32C_DispSArea.
  */
uint16_t seLCD32C_GetDisplayArea( void ) {

  return LCD32C->DSP_b.DSPAR;
}

/**
  * @brief  Sets Display Area.
  * @param  area: This parameter can be a value of @ref seLCD32C_Area.
  * @retval None
  */
void seLCD32C_SetDisplayArea( seLCD32C_Area area ) {
  
  LCD32C->DSP_b.DSPAR = area;
}

/**
  * @brief  Clears Display Area.
  * @param  area: This parameter can be a value of @ref seLCD32C_Area.
  * @retval None
  */
void seLCD32C_ClearDisplayMemory( seLCD32C_Area area ) {

  void* segram =
    ( area == seLCD32C_DISPLAY_AREA0 ) ?
      (uint8_t*)seLCD32C_SEGRAM_TOP_ADDR_AREA0 :
        (uint8_t*)seLCD32C_SEGRAM_TOP_ADDR_AREA1;

  memset( segram, 0, seLCD32C_SEGRAM_SIZE );
}

/**
  * @brief  Sets Display Memory to a value.
  * @param  area: This parameter can be a value of @ref seLCD32C_Area.
  * @param  byte: Value to set.
  * @retval None
  */
void seLCD32C_SetDisplayMemory( seLCD32C_Area area, const uint8_t byte ) {

  void* segram =
    ( area == seLCD32C_DISPLAY_AREA0 ) ?
      (uint8_t*)seLCD32C_SEGRAM_TOP_ADDR_AREA0 :
        (uint8_t*)seLCD32C_SEGRAM_TOP_ADDR_AREA1;

  memset( segram, byte, seLCD32C_SEGRAM_SIZE );
}

/**
  * @brief  Copies user data to Display Area.
  * @param  area: This parameter can be a value of @ref seLCD32C_Area.
  * @param  buf: This parameter is the user data pointer.
  * @param  size: This parameter is the user data size.
  * @retval None
  */
void seLCD32C_CopyToDisplayArea( seLCD32C_Area area, const uint8_t *buf, uint16_t size ) {

  void *segram =
    ( area == seLCD32C_DISPLAY_AREA0 ) ?
      (uint8_t*)seLCD32C_SEGRAM_TOP_ADDR_AREA0 :
        (uint8_t*)seLCD32C_SEGRAM_TOP_ADDR_AREA1;
        
  if ( size <= seLCD32C_SEGRAM_SIZE ) {
    memcpy( segram, buf, size );
  }
}

/**
  * @brief  Enables Lcd interrupt.
  * @retval None
  */
void seLCD32C_EnableInt( void ) {
  
  LCD32C->INTE_b.FRMIE = seENABLE;
}

/**
  * @brief  Disables Lcd interrupt.
  * @retval None
  */
void seLCD32C_DisableInt( void ) {

  LCD32C->INTE_b.FRMIE = seDISABLE;
}

/**
  * @brief  Returns Lcd interrupt flag.
  * @retval InterruptStatus: can be a value of @ref seInterruptStatus
  */
seInterruptStatus seLCD32C_GetIntFlag( void ) {
  
  return LCD32C->INTF_b.FRMIF ? seINTERRUPT_OCCURRED : seINTERRUPT_NOT_OCCURRED;
}

/**
  * @brief  Clears Lcd interrupt.
  * @retval None
  */
void seLCD32C_ClearIntFlag( void ) {

  LCD32C->INTF = 0x01;
}

