/**
  ******************************************************************************
  * @file    se_lcd32c.h
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file contains all the functions prototypes for the LCD32C firmware
  *          library.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#ifndef SE_LCD32C_H
#define SE_LCD32C_H

#include "se_common.h"

#ifdef __cplusplus
 extern "C" {
#endif

/** @addtogroup sePeriphLibrary
  * @{
  *
  */

/** @defgroup LCD32C
  * @{
  *  @brief The LCD32C is a module to drive an LCD panel
 */

/** @defgroup LCD32C_Constants
  * @{
  */

/** @defgroup LCD32C_Clock
  * @{
  */

 /**
  * @brief  LCD32C Clock Source selection.
  */

/**
  * @brief  LCD32C Booster clock control options
  * @note   fCLK_LCD32C: LCD32C operating clock frequency [Hz]
  */
typedef enum {
   seLCD32C_FLCLK_DIV_64 = 3,                   ///< fLCLK 1/64.
   seLCD32C_FLCLK_DIV_32 = 2,                   ///< fLCLK 1/32.
   seLCD32C_FLCLK_DIV_16 = 1,                   ///< fLCLK 1/16.
   seLCD32C_FLCLK_DIV_4  = 0                    ///< fLCLK 1/4.
} seLCD32C_BoostClk;

/**
  * @brief  LCD32C Doubler clock control options
  * @note   fCLK_LCD32C: LCD32C operating clock frequency [Hz]
  */
typedef enum {
   seLCD32C_DBL_FLCLK_DIV_128 = 7,              ///< fLCLK 1/128.
   seLCD32C_DBL_FLCLK_DIV_64  = 6,              ///< fLCLK 1/64.
   seLCD32C_DBL_FLCLK_DIV_32  = 5,              ///< fLCLK 1/32.
   seLCD32C_DBL_FLCLK_DIV_16  = 4,              ///< fLCLK 1/16.
   seLCD32C_DBL_FLCLK_DIV_8   = 3,              ///< fLCLK 1/8.
   seLCD32C_DBL_FLCLK_DIV_4   = 2,              ///< fLCLK 1/4.
   seLCD32C_DBL_FLCLK_DIV_2   = 1,              ///< fLCLK 1/2.
   seLCD32C_DBL_FLCLK_DIV_1   = 0               ///< fLCLK 1/1.
} seLCD32C_DoublerClk;

/**
  * @}
  */

/** @defgroup LCD32C_DisplayState
  * The LCD panel display can be inverted without re-writing data RAM.
  * @{
  */

/**
  * @brief  LCD32C Display inversion.
  */
typedef enum {
   seLCD32C_DISP_INVERSE  = 0,                  ///< Display State is Inverted.
   seLCD32C_DISP_NORMAL   = 1                   ///< Display State is Normal.
} seLCD32C_DispInvState;

/** 
  * @brief  LCD32C Display state settings.
  */
typedef enum {
   seLCD32C_DISP_STATE_ALL_OFF  = 3,            ///< Display State is All off (Static Drive).
   seLCD32C_DISP_STATE_ALL_ON   = 2,            ///< Display State is All on.
   seLCD32C_DISP_STATE_NORMAL   = 1,            ///< Display State is Normal.
   seLCD32C_DISP_STATE_DISP_OFF = 0             ///< Display State is Display off.
} seLCD32C_DispState;

/**
  * @}
  */

/** @defgroup LCD32C_Frame_Frequency_Control
  * @{
  */

/**
  * @brief The frame frequency is determined by selecting a division ratio from 32 variations depending
  *        on the drive duty.
  *        The following equation is to calculate the frame frequency.
  *        fFR = 1/8 * fLCLK * 1/(FRMCNT + 1) * 1/(LDUTY + 1)
  *        Where
  *        fFR: Frame frequency [Hz]
  *        fLCLK: LCD32C operating clock frequency [Hz]
  *        FRMCNT: value (0 to 31)
  *        LDUTY:  value @ref seLCD32C_Duty
  * @note  To achieve a target frame rate the following formula can be used to calculate a value of FRMCNT
  *        based on selected fLCLK and LDUTY.
  *        FRMCNT=(1/8) * fLCLK * 1/(LDUTY+1) * (1/FR)-1.
  */
typedef enum {
   seLCD32C_DUTY_STATIC = 0,                    ///< Static duty
   seLCD32C_DUTY_1_2    = 1,                    ///< 1/2 duty
   seLCD32C_DUTY_1_3    = 2,                    ///< 1/3 duty
   seLCD32C_DUTY_1_4    = 3,                    ///< 1/4 duty
   seLCD32C_DUTY_1_5    = 4,                    ///< 1/5 duty
   seLCD32C_DUTY_1_6    = 5,                    ///< 1/6 duty
   seLCD32C_DUTY_1_7    = 6,                    ///< 1/7 duty
   seLCD32C_DUTY_1_8    = 7,                    ///< 1/8 duty
   seLCD32C_DUTY_1_9    = 8,                    ///< 1/9 duty
   seLCD32C_DUTY_1_10   = 9,                    ///< 1/10 duty
   seLCD32C_DUTY_1_11   = 10,                   ///< 1/11 duty
   seLCD32C_DUTY_1_12   = 11,                   ///< 1/12 duty
   seLCD32C_DUTY_1_13   = 12,                   ///< 1/13 duty
   seLCD32C_DUTY_1_14   = 13,                   ///< 1/14 duty
   seLCD32C_DUTY_1_15   = 14,                   ///< 1/15 duty
   seLCD32C_DUTY_1_16   = 15,                   ///< 1/16 duty
   seLCD32C_DUTY_1_17   = 16,                   ///< 1/17 duty
   seLCD32C_DUTY_1_18   = 17,                   ///< 1/18 duty
   seLCD32C_DUTY_1_19   = 18,                   ///< 1/19 duty
   seLCD32C_DUTY_1_20   = 19,                   ///< 1/20 duty
   seLCD32C_DUTY_1_21   = 20,                   ///< 1/21 duty
   seLCD32C_DUTY_1_22   = 21,                   ///< 1/22 duty
   seLCD32C_DUTY_1_23   = 22,                   ///< 1/23 duty
   seLCD32C_DUTY_1_24   = 23,                   ///< 1/24 duty
   seLCD32C_DUTY_1_25   = 24,                   ///< 1/25 duty
   seLCD32C_DUTY_1_26   = 25,                   ///< 1/26 duty
   seLCD32C_DUTY_1_27   = 26,                   ///< 1/27 duty
   seLCD32C_DUTY_1_28   = 27,                   ///< 1/28 duty
   seLCD32C_DUTY_1_29   = 28,                   ///< 1/29 duty
   seLCD32C_DUTY_1_30   = 29,                   ///< 1/30 duty
   seLCD32C_DUTY_1_31   = 30,                   ///< 1/31 duty
   seLCD32C_DUTY_1_32   = 31                    ///< 1/32 duty
} seLCD32C_Duty;


#define IS_LCD32C_LDUTY(LDUTY) (!((LDUTY) & ~LCD32C_TIM1_LDUTY_Msk))     ///< Lcd32c Duty parameter is within limits.

#define IS_LCD32C_FRMCNT(FRMCNT) (!((FRMCNT) & ~LCD32C_TIM1_FRMCNT_Msk)) ///< Lcd32c Frame Count is within limits.

/**
  * @}
  */

/** @defgroup LCD32C_Power_Settings
  * When using internal generation mode, select the reference voltage for boosting
  * voltage generated by the LCD voltage regulator according to the power supply voltage.
  *
  * Heavy load protection mode should be set when the display has inconsistencies in
  * density. Current consumption increases in heavy load protection mode,
  * therefore do not set heavy load protection mode if unnecessary.
  * @endverbatim
  * @{
  */

/** 
  * @brief  LCD32C drive bias.
  */
typedef enum {
   seLCD32C_PWR_BIAS_1_4   = 1,                 ///< Bias 1/4.
   seLCD32C_PWR_BIAS_1_5   = 0                  ///< Bias 1/5.
} seLCD32C_PwrBias;

/**
  * @}
  */

/** @defgroup LCD32C_Contrast.
  * @{
  */

/**
  * @brief Sets LCD panel contrast.
  */
typedef enum {
   seLCD32C_CONTRAST_LEVEL0  = 0,               ///< Contrast Level 0 (Light)
   seLCD32C_CONTRAST_LEVEL1  = 1,               ///< Contrast Level 1
   seLCD32C_CONTRAST_LEVEL2  = 2,               ///< Contrast Level 2
   seLCD32C_CONTRAST_LEVEL3  = 3,               ///< Contrast Level 3
   seLCD32C_CONTRAST_LEVEL4  = 4,               ///< Contrast Level 4
   seLCD32C_CONTRAST_LEVEL5  = 5,               ///< Contrast Level 5
   seLCD32C_CONTRAST_LEVEL6  = 6,               ///< Contrast Level 6
   seLCD32C_CONTRAST_LEVEL7  = 7,               ///< Contrast Level 7
   seLCD32C_CONTRAST_LEVEL8  = 8,               ///< Contrast Level 8 (Normal)
   seLCD32C_CONTRAST_LEVEL9  = 9,               ///< Contrast Level 9
   seLCD32C_CONTRAST_LEVEL10 = 10,              ///< Contrast Level 10
   seLCD32C_CONTRAST_LEVEL11 = 11,              ///< Contrast Level 11
   seLCD32C_CONTRAST_LEVEL12 = 12,              ///< Contrast Level 12
   seLCD32C_CONTRAST_LEVEL13 = 13,              ///< Contrast Level 13
   seLCD32C_CONTRAST_LEVEL14 = 14,              ///< Contrast Level 14
   seLCD32C_CONTRAST_LEVEL15 = 15               ///< Contrast Level 15 (Dark)
} seLCD32C_Contrast;

/**
  * @}
  */
/** @defgroup LCD32C_RAM_Area
  * @verbatim
  *  The correspondence between the memory bits of the display data RAM and the common/segment pins varies depending
  *  on the selected conditions below.
  *
  *  Drive duty (1/32 to 1/2 or static drive)
  *  Segment pin assignment (normal or inverse)
  *  Common pin assignment (normal or inverse)
  *
  *  The Hardware specification shows the correspondence between display data RAM and the common/segment pins in
  *  some drive duties.
  *
  *  Writing 1 to the display data RAM bit corresponding to a segment on the LCD panel turns the segment on, while
  *  writing 0 turns the segment off. Since the display memory is RAM, allowing reading and writing, bits can be controlled
  *  individually using logic operation instructions (read-modify-write instructions).
  *  @endverbatim
  * @{
  */
typedef enum {
   seLCD32C_DISPLAY_AREA0 = 0,                  ///< Area : Display Area 0.
   seLCD32C_DISPLAY_AREA1 = 1                   ///< Area : Display Area 1.
} seLCD32C_Area;

typedef enum {
   seLCD32C_SEGRAM_TOP_ADDR_AREA0 = 0x20200000, //< The display data RAM address(Area0).
   seLCD32C_SEGRAM_TOP_ADDR_AREA1 = 0x20200200, ///< The display data RAM address(Area1).
   seLCD32C_SEGRAM_SIZE  = 0x180                ///< The display data RAM size.
} seLCD32C_SegRamAddress;

/**
  * @}
  */

/**
  * @}
  */

/** @defgroup LCD32C_Types
  * @{ 
  */

/**
  * @brief  LCD32C Init structure definition.
  */
typedef struct {
   seLCD32C_Clk    Clk;                         ///< Specifies the System clock source and divider.
   seState         CtlLcdIoDischargeEn;         ///< Specifies if IO discharge should be enabled. When the panel is connected,
                                                ///< must be set to seENABLE even if display is turned off.
   uint16_t        Tim1FrameCount;              ///< Configures the LCD32C Frame Count used in frame frequency calculation.
   uint16_t        Tim1Duty;                    ///< Configures the LCD32C Duty used in frame frequency calculation.
   seLCD32C_BoostClk Tim2BoostClk;              ///< Configures the LCD32C Booster clock. It could be a value @ref seLCD32C_BoostClk
   uint16_t        Tim2Nline;                   ///< Configures the LCD32C SEG n line reverse drive selection.
   seState         PwrVoltageRegulatorEn;       ///< Configures voltage regulator enable/disable.
   seState         PwrHeavyLoadProtectionModeEn;///< Configures heavy load protection mode enable/diable.
   seState         PwrBoosterEn;                ///< Configures voltage booster enable/disable.
   seState         PwrDoublerEn;                ///< Configures doubler booster enable/disable.
   seState         PwrExternalVcSel;            ///< Configures the LCD drive voltage supply mode.
   seLCD32C_PwrBias PwrBiasSel;                 ///< Configures the LCD drive bias.It could be a value @ref seLCD32C_PwrBias.
   seLCD32C_Area   DspSelectArea;               ///< Select one of the display areas.
   uint16_t        DspSelComPinDir;             ///< Select COM pins assignment direction. If 1, memory bits are assigned to common pins in ascending order.
                                                ///< If 0, memory bits are assigned to common pins in descending order.
   uint16_t        DspSelSegPinDir;             ///< Select SEG ins assignment direction. If 1, memory addresses are assigned to segment pins in ascending order.
                                                ///< If 0, memory addresses are assigned to segment pins in descending order.
} seLCD32C_InitTypeDef;

/**
  * @}
  */

/** @defgroup LCD32C_Functions
  * @{
  */

/**
  * @brief  Initializes the LCD32C peripheral according to the specified
  *         parameters in the LCD32C_InitStruct.
  * @note   This function configures the module, and module's interrupts.
  *         It clears module's interrupts but does not enable interrupt from the
  *         module to the CPU. This function enables module by start supplying
  *         operating clock. This module starts does not start display. It is user
  *         code responsibility to set appropriate display state.
  * @param  InitStruct: Pointer to a @ref seLCD32C_InitTypeDef structure that
  *         contains the configuration information for the LCD32C peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seLCD32C_Init( seLCD32C_InitTypeDef* InitStruct );

/**
  * @brief  Enables LCD32C by start supplying operating clock.
  * @param  IoDischarge: Controls if LCDIO discharge @ref seState.
  * @retval None
  */
void seLCD32C_Enable( seState IoDischarge );

/**
  * @brief  Disables LCD32C by stop supplying operating clock.
  * @retval None
  */
void seLCD32C_Disable( void );

/**
  * @brief  Configures LCD32C clock source and clock divider.
  * @param  ClkSrc: Clock source can be a value of @ref seLCD32C_ClkSrc.
  * @param  ClkDivider: Clock divider a value of @ref seLCD32C_EXOSC_ClkDiv or @ref seLCD32C_IOSC_ClkDiv or ref seLCD32C_OSC3_ClkDiv or @ref seLCD32C_OSC1_ClkDiv
  *         depending on the selected clock source.
  * @retval None
  */
void seLCD32C_ConfigureClock( seLCD32C_Clk Clk );

/**
  * @brief  Sets frame rate by selecting the appropriate FRMCNT and LDUTY values.
  *         Refer to @ref LCD32C_Frame_Frequency_Control for details.
  * @param  InitStruct: pointer to a @ ref seLCD32C_InitTypeDef structure that
  *         contains the configuration information for the LCD32C peripheral.
  * @retval None
  */
void seLCD32C_SetFrameFreq( seLCD32C_InitTypeDef* InitStruct );

/**
  * @brief  Sets Voltages.
  * @param  InitStruct: This parameter can be a value of @ref seLCD32C_InitTypeDef.
  * @retval None
  */
void seLCD32C_ConfigurePower( seLCD32C_InitTypeDef* InitStruct );

/**
  * @brief  Configres Display features.
  * @param  InitStruct: This parameter can be a value of @ref seLCD32C_InitTypeDef.
  * @retval None
  */
void seLCD32C_ConfigureDisplay( seLCD32C_InitTypeDef* InitStruct );

/**
  * @brief  Configres Display features.
  * @param  clk: This parameter can be a value of @ref seLCD32C_BoostClk.
  * @retval None
  */
void seLCD32C_SetBoostClk( seLCD32C_BoostClk clk );

/**
  * @brief  Configres Display features.
  * @param  clk: This parameter can be a value of @ref seLCD32C_DoublerClk.
  * @retval None
  */
void seLCD32C_SetDoublerClk( seLCD32C_DoublerClk clk );

/**
  * @brief  Sets Panel contrast.
  * @param  contrast: This parameter can be a value of @ref seLCD32C_Contrast.
  * @retval None
  */
void seLCD32C_SetPanelContrast( seLCD32C_Contrast contrast );

/**
  * @brief  Gets Panel contrast.
  * @retval Contrast: A value of @ref seLCD32C_Contrast.
  */
uint16_t seLCD32C_GetPanelContrast( void );

/**
  * @brief  Sets Display state.
  * @param  state: This parameter can be a value of @ref seLCD32C_DisplayState.
  * @retval None
  */
void seLCD32C_SetDisplayState( seLCD32C_DispState state );

/**
  * @brief  Gets Display state.
  * @retval A value of @ref seLCD32C_DispState.
  */
uint16_t seLCD32C_GetDisplayState( void );

/**
  * @brief  Sets Display Inverted state.
  * @param  state: This parameter can be a value of @ref seLCD32C_DispInvState.
  * @retval None
  */
void seLCD32C_SetDisplayInvState( seLCD32C_DispInvState state );

/**
  * @brief  Gets Display Inverted state.
  * @retval InvState: A value of @ref seLCD32C_DispInvState.
  */
uint16_t seLCD32C_GetDisplayInvState( void );

/**
  * @brief  Sets Display Area.
  * @param  area: This parameter can be a value of @ref seLCD32C_Area.
  * @retval None
  */
void seLCD32C_SetDisplayArea( seLCD32C_Area area );

/**
  * @brief  Gets Display Area.
  * @retval area: A value of @ref seLCD32C_DispSArea.
  */
uint16_t seLCD32C_GetDisplayArea( void );

/**
  * @brief  Clears Display Area.
  * @param  area: This parameter can be a value of @ref seLCD32C_Area.
  * @retval None
  */
void seLCD32C_ClearDisplayMemory( seLCD32C_Area area );

/**
  * @brief  Sets Display Memory to a value.
  * @param  area: This parameter can be a value of @ref seLCD32C_Area.
  * @param  byte: Value to set.
  * @retval None
  */
void seLCD32C_SetDisplayMemory( seLCD32C_Area area, const uint8_t byte );

/**
  * @brief  Copies user data to Display Area.
  * @param  area: This parameter can be a value of @ref seLCD32C_Area.
  * @param  buf: This parameter is the user data pointer.
  * @param  size: This parameter is the user data size.
  * @retval None
  */
void seLCD32C_CopyToDisplayArea( seLCD32C_Area area, const uint8_t *buf, uint16_t size );

/**
  * @brief  Enables Lcd interrupt.
  * @retval None
  */
void seLCD32C_EnableInt( void );

/**
  * @brief  Disables Lcd interrupt.
  * @retval None
  */
void seLCD32C_DisableInt( void );

/**
  * @brief  Returns Lcd interrupt flag.
  * @retval InterruptStatus: can be a value of @ref seInterruptStatus
  */
seInterruptStatus seLCD32C_GetIntFlag( void );

/**
  * @brief  Clears Lcd interrupt.
  * @retval None
  */
void seLCD32C_ClearIntFlag( void );

/**
  * @brief  Configures ports for this module. Override this function to configure specific ports.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seAUX_ConfigurePortsForLcd32C( void );

/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */
#ifdef __cplusplus
}
#endif
#endif  // SE_LCD32C_H

