/**
  ******************************************************************************
  * @file    se_pport.h
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file contains all the functions prototypes for the PPORT firmware
  *          library.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#ifndef SE_PPORT_H
#define SE_PPORT_H

#include "se_common.h"

#ifdef __cplusplus
 extern "C" {
#endif

/** @addtogroup sePeriphLibrary
  * @{
  */

/** 
  * @defgroup PPORT
  * @{
  * @brief The PPORT (I/O Ports) circuit.
  *        This circuit allows the selection of I/O functions on processor pins between
  *        GPIO, and several alternate functions. Selected PPORT groups also can be routed
  *        to the UPMUX (Universal Port Multiplexer) where additional Peripheral Circuits
  *        can be selected to go a variety of pins.
  */ 

/** @defgroup PPORT_Constants 
  * @{
  */

typedef enum {
   sePPORT_P0 = ( 0 ),                   ///< Port number Px0.
   sePPORT_P1 = ( 1 ),                   ///< Port number Px1.
   sePPORT_P2 = ( 2 ),                   ///< Port number Px2.
   sePPORT_P3 = ( 3 ),                   ///< Port number Px3.
   sePPORT_P4 = ( 4 ),                   ///< Port number Px4.
   sePPORT_P5 = ( 5 ),                   ///< Port number Px5.
   sePPORT_P6 = ( 6 ),                   ///< Port number Px6.
   sePPORT_P7 = ( 7 )                    ///< Port number Px7.
} sePPORT_PortNumber;

typedef enum {
   sePPORT_ALT_0 = ( 0 ),                ///< Port Alt Function 0
   sePPORT_ALT_1 = ( 1 ),                ///< Port Alt Function 1
   sePPORT_ALT_2 = ( 2 ),                ///< Port Alt Function 2
   sePPORT_ALT_3 = ( 3 )                 ///< Port Alt Function 3
} sePPORT_AltFunc;


typedef enum {
   sePPORT_DATA_LOW = ( 0 ),             ///< Port Data:LOW.
   sePPORT_DATA_HIGH= ( 1 )              ///< Port Data:HIGH.
} sePPORT_Data;

typedef enum {
   sePPORT_INT_EDGE_RISING  = ( 0 ),     ///< Input signal:Rising edge.
   sePPORT_INT_EDGE_FALLING = ( 1 )      ///< Input signal:Falling edge.
} sePPORT_Edge;

typedef enum {
   seUPMUX_PER_DISABLE = ( 0 ),          ///< UpMux Disabled
   seUPMUX_PER_I2C = ( 1 ),              ///< UpMux I2C Peripheral
   seUPMUX_PER_SPIA = ( 2 ),             ///< UpMux SPIA Peripheral
   seUPMUX_PER_UART = ( 3 ),             ///< UpMux UART Peripheral
   seUPMUX_PER_T16B = ( 4 )              ///< UpMux T16B Peripheral
} seUPMUX_Peripheral_Sel;

typedef enum {
   seUPMUX_CH_0 = ( 0 ),                 ///< UpMux Ch 0
   seUPMUX_CH_1 = ( 1 ),                 ///< UpMux Ch 1
   seUPMUX_CH_2 = ( 2 )                  ///< UpMux Ch 2
} seUPMUX_Channel_Sel;

typedef enum {
   seUPMUX_SPIA_SDI = ( 1 ),             ///< UpMux Fnc SPIA SDI
   seUPMUX_SPIA_SDO = ( 2 ),             ///< UpMux Fnc SPIA SDO
   seUPMUX_SPIA_SPICLK = ( 3 ),          ///< UpMux Fnc SPIA SPICLK
   seUPMUX_SPIA_SPISS = ( 4 )            ///< UpMux Fnc SPIA SPISS
} seUPMUX_SPIA_Fnc;

typedef enum {
   seUPMUX_I2C_SCL = ( 1 ),              ///< UpMux Fnc I2C SCL
   seUPMUX_I2C_SDA = ( 2 )               ///< UpMux Fnc I2C SDA
} seUPMUX_I2C_Fnc;

typedef enum {
   seUPMUX_UART_USIN = ( 1 ),            ///< UpMux Fnc UART USIN
   seUPMUX_UART_USOUT = ( 2 )            ///< UpMux Fnc UART USOUT
} seUPMUX_UART_Fnc;

typedef enum {
   seUPMUX_T16B_TOUT_CAP0 = ( 1 ),       ///< UpMux Fnc T16B TOUT_CAP0
   seUPMUX_T16B_TOUT_CAP1 = ( 2 ),       ///< UpMux Fnc T16B TOUT_CAP1
   seUPMUX_T16B_TOUT_CAP2 = ( 3 ),       ///< UpMux Fnc T16B TOUT_CAP2
   seUPMUX_T16B_TOUT_CAP3 = ( 4 ),       ///< UpMux Fnc T16B TOUT_CAP3
   seUPMUX_T16B_TOUT_CAP4 = ( 5 ),       ///< UpMux Fnc T16B TOUT_CAP4
   seUPMUX_T16B_TOUT_CAP5 = ( 6 )        ///< UpMux Fnc T16B TOUT_CAP5
} seUPMUX_T16B_Fnc;

/**
  * @}
  */ 

/** @defgroup PPORT_Types
  * @{ 
  */

/** 
  * @brief  PPORTInit structure definition
  */

typedef struct {                        ///< PPORT Structure
   __IO uint16_t  DAT;                  ///< PPort Data Register
   __IO uint16_t  IOEN;                 ///< PPort Enable Register 
   __IO uint16_t  RCTL;                 ///< PPort Pull-up/down Control Register
   __IO uint16_t  INTF;                 ///< PPort Interrupt Flag Register
   __IO uint16_t  INTCTL;               ///< PPort Interrupt Control Register
   __IO uint16_t  CHATEN;               ///< PPort Chattering Filter Enable Register
   __IO uint16_t  MODSEL;               ///< PPort Mode Select Register
   __IO uint16_t  FNCSEL;               ///< PPort Function Select Register
} sePPORT_Group;

/**
  * @brief UPMUX Setting Register bit positions 
  */
#define PERISEL_Pos 0
#define PERICH_Pos  3
#define PFNC_Pos    5

/**
  * @}
  */

/** @defgroup PPORT_Functions
  * @{
  */

/**
  * @brief  Initialize for using a port as a general-purpose input port.
  * @param  portId: This parameter can be a value of @ref sePPORT_Id.
  * @param  config: This parameter can be a value of @ref config.
  * @retval State: can be a value of @ref seState
  */
seStatus sePPORT_Init(sePPORT_Id portId, uint32_t config);

/**
  * @brief  Configures PPORT timer clock source and clock divider.
  * @param  clock: This parameter can be a value of @ref sePPORT_Clk.
  * @note   If Chattering filter is enabled application shall clear interrupts when changing the clock settings.
  * @retval None
  */
void sePPORT_ConfigureClock( sePPORT_Clk clock );

/**
  * @brief  Initialize for using a port as a general-purpose input port.
  * @param  portId: This parameter can be a value of @ref sePPORT_Id.
  * @retval None
  */
void sePPORT_InitAsInput( sePPORT_Id portId );

/**
  * @brief  Enables the pull-up resistor.
  * @param  portId: This parameter can be a value of @ref sePPORT_Id.
  * @retval None
  */
void sePPORT_EnablePullUpResistor( sePPORT_Id portId );

/**
  * @brief  Enables the pull-down resistor.
  * @param  portId: This parameter can be a value of @ref sePPORT_Id.
  * @retval None
  */
void sePPORT_EnablePullDownResistor( sePPORT_Id portId );

/**
  * @brief  Disables built-in resistor.
  * @param  portId: This parameter can be a value of @ref sePPORT_Id.
  * @retval None
  */
void sePPORT_DisableBuiltInResistor( sePPORT_Id portId );

/**
  * @brief  Initialize for using a port as a general-purpose output port.
  * @param  portId: This parameter can be a value of @ref sePPORT_Id.
  * @retval None
  */
void sePPORT_InitAsOutput( sePPORT_Id portId );

/**
  * @brief  Enable port interrupt.
  * @param  portId: This parameter can be a value of @ref sePPORT_Id.
  * @param  edge: This parameter can be a value of @ref sePPORT_Edge.
  * @retval None
  */
void sePPORT_EnableInt( sePPORT_Id portId, sePPORT_Edge edge );

/**
  * @brief  Disable port interrupt.
  * @param  portId: This parameter can be a value of @ref sePPORT_Id.
  * @retval None
  */
void sePPORT_DisableInt( sePPORT_Id portId );

/**
  * @brief  Get port interrupt.
  * @param  portId: This parameter can be a value of @ref sePPORT_Id.
  * @retval InterruptStatus: can be a value of @ref seInterruptStatus.
  */
seInterruptStatus sePPORT_GetIntFlag( sePPORT_Id portId );

/**
  * @brief  Clear port interrupt.
  * @param  portId: This parameter can be a value of @ref sePPORT_Id.
  * @retval None
  */
void sePPORT_ClearIntFlag( sePPORT_Id portId );

/**
  * @brief  Get port output value.
  * @param  portId: This parameter can be a value of @ref sePPORT_Id.
  * @retval value: can be a value of @ref sePPORT_Data.
  */
sePPORT_Data sePPORT_GetOutput( sePPORT_Id portId );

/**
  * @brief  Set port output data.
  * @param  portId: This parameter can be a value of @ref sePPORT_Id.
  * @param  value: can be a value of @ref sePPORT_Data.
  * @retval None
  */
void sePPORT_SetOutput( sePPORT_Id portId, sePPORT_Data data );

/**
  * @brief  Get port input value.
  * @param  portId: This parameter can be a value of @ref sePPORT_Id.
  * @retval value: can be a value of @ref sePPORT_Data.
  */
sePPORT_Data sePPORT_GetInput( sePPORT_Id portId );

/**
  * @brief  Enable port Chattering Filter.
  * @param  portId: This parameter can be a value of @ref sePPORT_Id.
  * @note   Application shall clear interrupts when changing Chattering Filter.
  * @retval None
  */
void sePPORT_EnableChatteringFilter( sePPORT_Id portId );

/**
  * @brief  Disable port Chattering Filter.
  * @param  portId: This parameter can be a value of @ref sePPORT_Id.
  * @note   Application shall clear interrupts when changing Chattering Filter.
  * @retval None
  */
void sePPORT_DisableChatteringFilter( sePPORT_Id portId );

/**
  * @brief  Gets the Chattering Filter.
  * @param  portId: This parameter can be a value of @ref sePPORT_Id.
  * @retval State: can be a value of @ref seState
  */
seState sePPORT_GetChatteringFilter( sePPORT_Id portId );

/**
  * @brief  Initialize for using a port as HiZ that is GPIO with input and
  *         output disabled, also disables the Interrupt
  * @param  portId: This parameter can be a value of @ref sePPORT_Id.
  * @retval None
  */
void sePPORT_InitAsHiZ( sePPORT_Id portId );

/**
  * @brief  Initialize for using UpMux use for the requested function
  * @param  portId: This parameter can be a value of @ref sePPORT_Id.
  * @param  peripheralNo: This parameter can be a value of @ref seUPMUX_Peripheral_Sel
  * @param  channelNo: This parameter can be a value of @ref seUPMUX_Channel_Sel
  * @param  ioFuncNo: This parameter selects I/O function
  * @retval Status: can be a value of @ref seStatus
  */
seStatus sePPORT_UpMuxFunction( sePPORT_Id portId, 
                                seUPMUX_Peripheral_Sel peripheralNo, 
                                seUPMUX_Channel_Sel channelNo, 
                                uint8_t ioFuncNo );
/**
  * @brief  Initialize for using a port as an alternate function.
  * @param  portId: This parameter can be a value of @ref sePPORT_Id.
  * @param  funcNo: This parameter can be a value of @ref sePPORT_AltFunc.
  * @retval Status: can be a value of @ref seStatus
  */ 
seStatus sePPORT_InitAsAltFunction( sePPORT_Id portId, 
                                    sePPORT_AltFunc funcNo );
/**
  * @}
  */

/**
 * Internal definition macro
 */

#define PORT_select_gpio        (0)     ///< Port GPIO
#define PORT_select_other       (1)     ///< Port OTHER

#define MUX_no_0                (0)     ///< Port Alt Function 0
#define MUX_no_1                (1)     ///< Port Alt Function 1
#define MUX_no_2                (2)     ///< Port Alt Function 2
#define MUX_no_3                (3)     ///< Port Alt Function 3

#define MUX_persel_0            (0)     ///< UpMux Disabled
#define MUX_persel_1            (1)     ///< UpMux I2C Peripheral
#define MUX_persel_2            (2)     ///< UpMux SPIA Peripheral
#define MUX_persel_3            (3)     ///< UpMux UART Peripheral
#define MUX_persel_4            (4)     ///< UpMux T16B Peripheral
  
#define MUX_ch0                 (0)     ///< UpMux Ch 0
#define MUX_ch1                 (1)     ///< UpMux Ch 1
#define MUX_ch2                 (2)     ///< UpMux Ch 2

#define MUX_fuction_0           (0)     ///< UpMux Fnc None
#define MUX_fuction_1           (1)     ///< UpMux Fnc SCL,SDI   ,USIN ,TOUT0
#define MUX_fuction_2           (2)     ///< UpMux Fnc SDA,SDO   ,USOUT,TOUT1
#define MUX_fuction_3           (3)     ///< UpMux Fnc    ,SPICLK,     ,TOUT2
#define MUX_fuction_4           (4)     ///< UpMux Fnc    ,SPISS ,     ,TOUT3

#define PORT_dir_input          (0)     ///< Port INPUT
#define PORT_dir_output         (1)     ///< Port OUTPUT

#define PORT_output_low         (0)     ///< Port Output Low
#define PORT_output_high        (1)     ///< Port Output High

#define PORT_chatta_disable     (0)     ///< PORT chattaring disable
#define PORT_chatta_enable      (1)     ///< PORT chattaring enable

#define PORT_pullup             (0)     ///< PORT pullup enable
#define PORT_pulldown           (1)     ///< PORT pulldon enable
#define PORT_none               (2)     ///< PORT pullup & pulldown disable

#define PORT_INIT(a, b, c, d ,e , f ,g ,h ,i)   (((PORT_##a) << 0) + ((MUX_##b) << 1) + ((MUX_##c) << 3) + ((MUX_##d) << 6) + ((MUX_##e) << 8) + ((PORT_##f) << 12) + ((PORT_##g) << 13)  + ((PORT_##h) << 14) + ((PORT_##g) << 16))

/**
  * @}
  */

/**
  * @}
  */

#ifdef __cplusplus
}

#endif
#endif /*SE_PPORT_H */

