/**
  ******************************************************************************
  * @file    se_qspi.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file provides all the QSPI hardware functions.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#include "string.h"
#include "se_qspi.h"

/**
  * @brief  This function initializes the QSPIx peripheral according to the specified
  *         parameters in the QSPI_InitStruct.
  * @param  QSPIx: QSPI to select the QSPI peripheral.
  * @param  QSPI_InitStruct: pointer to a @ref seQSPI_InitTypeDef structure that
  *         contains the configuration information for the specified QSPI peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seQSPI_Init( QSPI_0_Type* QSPIx, seQSPI_InitTypeDef* QSPI_InitStruct ) {

  seStatus fStatus = seSTATUS_NG;

  ///< Stop QSPI.
  seQSPI_Stop( QSPIx );

  ///< Set QSPI ch.x
  QSPIx->MOD_b.PUEN        = QSPI_InitStruct->PUEN;     ///< Enable/Disable pull-up/down.
  QSPIx->MOD_b.NOCLKDIV    = QSPI_InitStruct->NOCLKDIV; ///< Set master mode operating clock.
  QSPIx->MOD_b.LSBFST      = QSPI_InitStruct->LSBFST;   ///< Set MSB first/LSB first.
  QSPIx->MOD_b.CPHA        = QSPI_InitStruct->CPHA;     ///< Set clock phase.
  QSPIx->MOD_b.CPOL        = QSPI_InitStruct->CPOL;     ///< Set clock polarity.
  QSPIx->MOD_b.MST         = QSPI_InitStruct->MST;      ///< Set master/slave mode.
  QSPIx->MOD_b.TMOD        = QSPI_InitStruct->TMOD;     ///< Set Transfer Mode
  QSPIx->MOD_b.CHLN        = QSPI_InitStruct->CHLN;     ///< Set number of clocks for data transfer
  QSPIx->MOD_b.CHDL        = QSPI_InitStruct->CHDL;     ///< Set the number of clocks to drive the serial data lines
  
  ///< If using memory mapped access mode
  QSPIx->MMACFG1_b.TCSH    = QSPI_InitStruct->TCSH;     ///< Set slave select signal negation period
  QSPIx->RMADRH_b.RMADR    = QSPI_InitStruct->RMADR;    ///< Set remapping address
  QSPIx->MMACFG2_b.DUMDL   = QSPI_InitStruct->DUMDL;    ///< Set dummy cycle drive length
  QSPIx->MMACFG2_b.DUMLN   = QSPI_InitStruct->DUMLN;    ///< Set dummy cycle length
  QSPIx->MMACFG2_b.DATTMOD = QSPI_InitStruct->DATTMOD;  ///< Set data cycle transfer mode
  QSPIx->MMACFG2_b.DUMTMOD = QSPI_InitStruct->DUMTMOD;  ///< Set dummy cycle transfer mode
  QSPIx->MMACFG2_b.ADRTMOD = QSPI_InitStruct->ADRTMOD;  ///< Set address cycle transfer mode
  QSPIx->MMACFG2_b.ADRCYC  = QSPI_InitStruct->ADRCYC;   ///< Set 24 or 32-bit address cycle
  QSPIx->MB_b.XIPACT       = QSPI_InitStruct->XIPACT;   ///< Set XIP activation mode byte
  QSPIx->MB_b.XIPEXT       = QSPI_InitStruct->XIPEXT;   ///< Set XIP termination mode byte
  
  ///< Assign QSPI I/O function to the ports
  //fStatus = seAUX_ConfigurePortsForQSPI( QSPIx, &QSPI_InitStruct->ports );
    
  if (fStatus == seSTATUS_OK) {
    ///< Execute software reset.
    fStatus = seQSPI_Reset( QSPIx );
    QSPIx->CTL_b.MODEN  = 1;                             ///< Enable QSPI operation
       
    ///< Set the following bits when using interrupt
    QSPIx->INTF = QSPI_0_INTF_ALL_Msk;                   ///< Clear interrupt flags
    QSPIx->INTE = QSPI_InitStruct->INTE;                 ///< Enable interrupts
  }
    
  return fStatus; 
}

/**
  * @brief  This function initializes the QSPIx peripheral as Master according to the specified
  *         parameters in the QSPI_InitStruct.
  * @param  QSPI_InitStruct: pointer to a seQSPI_InitTypeDef structure that
  *         contains the configuration information for the specified QSPI peripheral.
  * @retval None
  */
void seQSPI_InitStructForMaster( seQSPI_InitTypeDef* QSPI_InitStruct ) { 
  
  ///< Set QSPI variables
  QSPI_InitStruct->CHLN     = seQSPI_08CLK;       ///< Set number of clocks for data transfer
  QSPI_InitStruct->CHDL     = seQSPI_08CLK;       ///< Set the number of clocks to drive the serial data lines
  QSPI_InitStruct->CPHA     = seQSPI_PH_RISE;     ///< Set clock phase.
  QSPI_InitStruct->CPOL     = seQSPI_POL_LOW;     ///< Set clock polarity.
  QSPI_InitStruct->LSBFST   = seQSPI_MSB_FST;     ///< Set MSB first/LSB first.
  QSPI_InitStruct->MST      = seQSPI_MODE_MASTER; ///< Set master/slave mode.
  QSPI_InitStruct->TMOD     = seQSPI_MODE_SINGLE; ///< Set Transfer Mode
  QSPI_InitStruct->NOCLKDIV = seDISABLE;          ///< Set master mode operating clock.
  QSPI_InitStruct->PUEN     = seENABLE;           ///< Enable/Disable pull-up/down.

  //====== If using MMA mode ======//
  QSPI_InitStruct->TCSH     = seQSPI_08CLK;       ///< Set slave select signal negation period
  QSPI_InitStruct->RMADR    = 0;                  ///< Set remapping address
  QSPI_InitStruct->DUMDL    = seQSPI_08CLK;       ///< Set dummy cycle drive length
  QSPI_InitStruct->DUMLN    = seQSPI_08CLK;       ///< Set dummy cycle length
  QSPI_InitStruct->DATTMOD  = seQSPI_MODE_SINGLE; ///< Set data cycle transfer mode
  QSPI_InitStruct->DUMTMOD  = seQSPI_MODE_SINGLE; ///< Set dummy cycle transfer mode
  QSPI_InitStruct->ADRTMOD  = seQSPI_MODE_SINGLE; ///< Set address cycle transfer mode
  QSPI_InitStruct->ADRCYC   = seQSPI_24BIT_ADDR;  ///< Set 24 or 32-bit address cycle
  QSPI_InitStruct->XIPACT   = 0;                  ///< Set XIP activation mode byte
  QSPI_InitStruct->XIPEXT   = 0;                  ///< Set XIP termination mode byte
  
  //====== If interrupt use ======//
  QSPI_InitStruct->INTE     = 0;                  ///< No interrupt
  
  
}

/**
  * @brief  This function initializes the QSPIx peripheral as Slave according to the specified
  *         parameters in the QSPI_InitStruct.
  * @param  QSPI_InitStruct: pointer to a s@ref eQSPI_InitTypeDef structure that
  *         contains the configuration information for the specified QSPI peripheral.
  * @retval None
  */
void seQSPI_InitStructForSlave( seQSPI_InitTypeDef* QSPI_InitStruct ) {
  
  QSPI_InitStruct->CHLN     = seQSPI_08CLK;       ///< Set number of clocks for data transfer
  QSPI_InitStruct->CHDL     = seQSPI_08CLK;       ///< Set the number of clocks to drive the serial data lines (CHDL should be equal to CHLN for Slave)
  QSPI_InitStruct->CPHA     = seQSPI_PH_RISE;     ///< Set clock phase.
  QSPI_InitStruct->CPOL     = seQSPI_POL_LOW;     ///< Set clock polarity.
  QSPI_InitStruct->LSBFST   = seQSPI_MSB_FST;     ///< Set MSB first/LSB first.
  QSPI_InitStruct->MST      = seQSPI_MODE_SLAVE;  ///< Set master/slave mode.
  QSPI_InitStruct->TMOD     = seQSPI_MODE_SINGLE; ///< Set Transfer Mode
  QSPI_InitStruct->PUEN     = seENABLE;           ///< Enable/Disable pull-up/down.
  
  //====== If using MMA mode ======//
  QSPI_InitStruct->TCSH     = seQSPI_08CLK;       ///< Set slave select signal negation period
  QSPI_InitStruct->RMADR    = 0;                  ///< Set remapping address
  QSPI_InitStruct->DUMDL    = seQSPI_08CLK;       ///< Set dummy cycle drive length
  QSPI_InitStruct->DUMLN    = seQSPI_08CLK;       ///< Set dummy cycle length
  QSPI_InitStruct->DATTMOD  = seQSPI_MODE_SINGLE; ///< Set data cycle transfer mode
  QSPI_InitStruct->DUMTMOD  = seQSPI_MODE_SINGLE; ///< Set dummy cycle transfer mode
  QSPI_InitStruct->ADRTMOD  = seQSPI_MODE_SINGLE; ///< Set address cycle transfer mode
  QSPI_InitStruct->ADRCYC   = seQSPI_24BIT_ADDR;  ///< Set 24 or 32-bit address cycle
  QSPI_InitStruct->XIPACT   = 0;                  ///< Set XIP activation mode byte
  QSPI_InitStruct->XIPEXT   = 0;                  ///< Set XIP termination mode byte
  
  //====== If interrupt use ======//
  QSPI_InitStruct->INTE     = 0;                  ///< No interrupt
  
}

/**
  * @brief  This function starts or enables the specified QSPI channel.
  * @param  QSPIx: QSPI to select the QSPI peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seQSPI_Start( QSPI_0_Type* QSPIx ) {

  seStatus fStatus = seSTATUS_NG;

  ///< Start QSPI
  ///< Enable Bus Clock(Master mode only).
  if ( QSPIx->MOD_b.MST == 1 ) {
    fStatus = seAUX_StartBusClockForQSPI( QSPIx );
  }

  ///< Execute software reset.
  fStatus = seQSPI_Reset( QSPIx );
  
  ///< Enable QSPI
  QSPIx->CTL_b.MODEN = 1;

  return fStatus;
}

/**
  * @brief  This function stops or disables the specified QSPI channel.
  * @param  QSPIx: QSPI to select the QSPI peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seQSPI_Stop( QSPI_0_Type* QSPIx ) {
  
  seStatus fStatus = seSTATUS_OK;
  ///< Stop QSPI 
  ///< Disable QSPI.
  QSPIx->CTL_b.MODEN = 0;

  ///< Disable Bus Clock(Master mode only).
  if ( QSPIx->MOD_b.MST == 1 ) {
    fStatus = seAUX_StopBusClockForQSPI( QSPIx );
  }
  
  return fStatus;
}

/**
  * @brief  This function  is software reset of the specified QSPI channel.
  * @param  QSPIx: QSPI to select the QSPI peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seQSPI_Reset( QSPI_0_Type* QSPIx ) { 

  seStatus fStatus = seSTATUS_OK;
  ///< Execute software reset.
  QSPIx->CTL_b.SFTRST = 1; 
  WHILE( QSPIx->CTL & QSPI_0_CTL_SFTRST_Msk, fStatus );

  return fStatus;
}

/**
  * @brief  This function sends single data value through the QSPIx peripheral.
  * @param  QSPIx:  QSPI to select the QSPI peripheral.
  * @param  value: Data value to be transmitted.
  * @param  count: Data size in number of bytes.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seQSPI_TxValue( QSPI_0_Type* QSPIx, uint8_t value, uint32_t count ) {
    
  seStatus fStatus = seSTATUS_OK;
  while ( count-- ) {
      ///< Send data.
      WHILE( (QSPIx->INTF & QSPI_0_INTF_TBEIF_Msk) == 0, fStatus );
      if ( fStatus != seSTATUS_OK ) return fStatus;
      QSPIx->TXD = value;
  } 
  
  WHILE( QSPIx->INTF & QSPI_0_INTF_BSY_Msk, fStatus ); 
  
  return fStatus;
}

/**
  * @brief  This function sends data through the QSPIx peripheral.
  * @param  QSPIx:  QSPI to select the QSPI peripheral.
  * @param  data: Pointer to data to be transmitted.
  * @param  size: Data size in number of bytes.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seQSPI_TxBytes( QSPI_0_Type* QSPIx, uint8_t data[], uint32_t size ) {
    
  seStatus fStatus = seSTATUS_OK;
  
  while ( size-- ) {
     ///< Send data.
     WHILE( (QSPIx->INTF & QSPI_0_INTF_TBEIF_Msk) == 0, fStatus );
     SANITY_CHECK( fStatus );
     QSPIx->TXD = *data++;
  }

  WHILE( QSPIx->INTF & QSPI_0_INTF_BSY_Msk, fStatus );   
  
  return fStatus;
}

/**
  * @brief  This function sends data through the QSPIx peripheral.
  * @param  QSPIx:  QSPI to select the QSPI peripheral.
  * @param  data: Pointer to data to be transmitted.
  * @param  size: Data size in number of half words.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seQSPI_TxHWords( QSPI_0_Type* QSPIx, uint16_t data[], uint32_t size ) {  
  
  seStatus fStatus = seSTATUS_OK;
  
  while ( size-- ) {
     ///< Send data.
     WHILE( (QSPIx->INTF & QSPI_0_INTF_TBEIF_Msk) == 0, fStatus );
     SANITY_CHECK( fStatus );
     QSPIx->TXD = *data++;
  } 
  
  WHILE( QSPIx->INTF & QSPI_0_INTF_BSY_Msk, fStatus );   
  
  return fStatus;
}

/**
  * @brief  This function configures I/O for Rx or Tx direction (Master mode only).
  * @param  QSPIx: QSPIx_CH0 to select the QSPI peripheral.
  * @retval None
  */
void seQSPI_SetIO( QSPI_0_Type* QSPIx, seQSPI_IO direction ) {
  
  QSPIx->CTL_b.DIR = direction;
}

/**
  * @brief  This function returns the received data from the QSPIx peripheral
  * @param  QSPIx: QSPI to select the QSPI peripheral.
  * @param  data: The received data pointer.
  * @param  size: Data size in number of bytes. 
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seQSPI_RxBytes( QSPI_0_Type* QSPIx, uint8_t data[], uint32_t size ) {

  seStatus fStatus = seSTATUS_OK;
  uint8_t dummy = QSPIx->RXD; ///< dummy read
    
  if ( QSPIx->MOD_b.MST == 1 ) {
    ///< Master mode only. Continuous data reception.
    if ( size == 0 ) size = 1;		///< If size is zero we still read 1 byte.
    ///< Check transmit buffer empty.
    WHILE( (QSPIx->INTF & QSPI_0_INTF_TBEIF_Msk) == 0, fStatus );  
    SANITY_CHECK( fStatus );
    ///< Receive data.
    do {
      ///< Set transmit dummy data(Master mode only).
      QSPIx->TXD = dummy;
      ///< Check receive buffer full.
      WHILE( (QSPIx->INTF & QSPI_0_INTF_RBFIF_Msk) == 0, fStatus );
      SANITY_CHECK( fStatus );
      ///< Read receive data.
      *data++ = QSPIx->RXD;
    } while ( --size );
    
  } else { 
    ///< Slave mode only. Continuous data reception.
    ///< Receive data.
    while ( size-- ) {
      ///< Check receive buffer full.
      WHILE( (QSPIx->INTF & QSPI_0_INTF_RBFIF_Msk) == 0, fStatus );
      SANITY_CHECK( fStatus );
      ///< Read receive data.
      *data++ = QSPIx->RXD;
    } 
      
  }

  ///< wait for idle state.
  WHILE( QSPIx->INTF & QSPI_0_INTF_BSY_Msk, fStatus );  
  
  return fStatus;
}

/**
  * @brief  This function returns the received data by the QSPIx peripheral.
  * @param  QSPIx: QSPI to select the QSPI peripheral.
  * @param  data: The received data pointer.
  * @param  size: Data size in number of half words. 
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seQSPI_RxHWords( QSPI_0_Type* QSPIx, uint16_t data[], uint32_t size ) {

  seStatus fStatus = seSTATUS_OK;
  uint8_t dummy = QSPIx->RXD; ///< dummy read
    
  if ( QSPIx->MOD_b.MST == 1 ) {
    ///< Master mode only. Continuous data reception.
    if ( size == 0 ) size = 1;		///< If size is zero we still read 1 byte.
    ///< Check transmit buffer empty.
    WHILE( (QSPIx->INTF & QSPI_0_INTF_TBEIF_Msk) == 0, fStatus ); 
    SANITY_CHECK( fStatus );
    ///< Receive data.
    do {
      ///< Set transmit dummy data(Master mode only).
      QSPIx->TXD = dummy;    
      ///< Check receive buffer full.
      WHILE( (QSPIx->INTF & QSPI_0_INTF_RBFIF_Msk) == 0, fStatus );
      SANITY_CHECK( fStatus );
      ///< Read receive data.
      *data++ = QSPIx->RXD;
    } while ( --size );
    
  } else { 
    ///< Slave mode only. Continuous data reception.
    ///< Receive data.
    while ( size-- ) {
      ///< Check receive buffer full.
      WHILE( (QSPIx->INTF & QSPI_0_INTF_RBFIF_Msk) == 0, fStatus );
      SANITY_CHECK( fStatus );
      ///< Read receive data.
      *data++ = QSPIx->RXD;
    }  

  }

  ///< wait for idle state.
  WHILE( QSPIx->INTF & QSPI_0_INTF_BSY_Msk, fStatus );  
    
  return fStatus;
}

/**
  * @brief  This function enables QSPI channel interrupt.
  * @param  QSPIx: QSPI to select the QSPI peripheral.
  * @param  irq: Interrupt to be enabled.
  * @retval None
  */
void seQSPI_EnableInt( QSPI_0_Type* QSPIx, uint16_t irq ) {

  QSPIx->INTE |= irq;
}

/**
  * @brief  This function disables QSPI channel interrupt.
  * @param  QSPIx: QSPI to select the QSPI peripheral.
  * @param  irq: Interrupt to be enabled.
  * @retval None
  */
void seQSPI_DisableInt( QSPI_0_Type* QSPIx, uint16_t irq ) {

  QSPIx->INTE &= ~irq;
}

/**
  * @brief  This function gets QSPI channel interrupt flag.
  * @param  QSPIx: QSPI to select the QSPI peripheral.
  * @param  flag: Interrupt flag to check.
  * @retval seInterruptStatus see @ref seInterruptStatus.
  */
seInterruptStatus seQSPI_GetIntFlag( QSPI_0_Type* QSPIx, uint16_t flag ) {

  seInterruptStatus fStatus = seINTERRUPT_NOT_OCCURRED;
  uint16_t flagged = QSPIx->INTF;
  
  if ( flag ) {
    flagged &= flag;
    if ( flagged == flag ) {
      fStatus = seINTERRUPT_OCCURRED;
    }
  }
  
  return fStatus;
}

/**
  * @brief  This function sets the mapping mode for reading from External flash.
  * @param  QSPIx: QSPIx_CH0 to select the QSPI peripheral.
  * @param  raddr: Remapping address.
  * @param  rcmd: Read command defined by flash type used for mapping.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seQSPI_SetMasterRxMMA( QSPI_0_Type* QSPIx, uint32_t raddr, uint16_t flash_rcmd ) {
  
  seStatus fStatus = seSTATUS_OK;
  
  if ( QSPIx->MMACFG2_b.MMAEN ) {
    fStatus = seSTATUS_NG;
  } else {
    seQSPI_ASSERT_MST_CS0();
    WHILE( (QSPIx->INTF & QSPI_0_INTF_TBEIF_Msk) == 0, fStatus );
    SANITY_CHECK( fStatus );
    QSPIx->TXD = flash_rcmd; 
    WHILE( (QSPIx->INTF & QSPI_0_INTF_TBEIF_Msk) == 0, fStatus );
    SANITY_CHECK( fStatus );
    WHILE( (QSPIx->INTF & QSPI_0_INTF_TENDIF_Msk) == 0, fStatus );
    QSPIx->INTF = QSPI_0_INTF_TENDIF_Msk;
    if ( fStatus == seSTATUS_OK ) {
      QSPIx->RMADRH = 0xFFF0&(raddr >> 16); 
      QSPIx->MMACFG2_b.MMAEN = 1;
    }
  }
  
  return fStatus;
}

/**
  * @brief  This function ends mapping mode for reading from External flash.
  * @param  QSPIx: QSPIx_CH0 to select the QSPI peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seQSPI_ClearMasterRxMMA( QSPI_0_Type* QSPIx ) {
  
  seStatus fStatus = seSTATUS_OK;
  seQSPI_NEGATE_MST_CS0();
  QSPIx->MMACFG2_b.MMAEN = 0;
  WHILE( QSPIx->INTF & QSPI_0_INTF_MMABSY_Msk, fStatus );
  
  return fStatus;
}

/**
  * @brief  This function terminates QSPI module.
  * @param  QSPIx:  QSPI to select the QSPI peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seQSPI_TermMasterTx( QSPI_0_Type* QSPIx ) {
  
  seStatus fStatus = seSTATUS_OK;
  WHILE( (QSPIx->INTF & QSPI_0_INTF_TENDIF_Msk) == 0, fStatus );
  QSPIx->INTF = QSPI_0_INTF_TENDIF_Msk;
  seQSPI_Stop( QSPIx );
  
  return fStatus;
}

/**
  * @brief  This function clears QSPI channel interrupt flag.
  * @param  QSPIx: QSPI to select the QSPI peripheral.
  * @param  flag: Interrupt flag to clear.
  * @retval None
  */
void seQSPI_ClearIntFlag( QSPI_0_Type* QSPIx, uint16_t flag ) {
  
  QSPIx->INTF = flag;  
}

/**
  * @brief  This function sets QSPI mode (single, dual, quad).
  * @param  QSPIx: QSPI to select the QSPI peripheral.
  * @param  mode: Sets single, dual or quad, see @ref seQSPI_TransferMode.
  * @param  chln: Number of clocks (seQSPI_01CLK is prohibited) for data transfer, see @ref seQSPI_Clocks.
  * @param  chdl: Number of clocks to drive the serial data lines, see @ref seQSPI_Clocks.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seQSPI_SetMode( QSPI_0_Type* QSPIx, 
                       seQSPI_TransferMode mode, 
                       seQSPI_Clocks chln, seQSPI_Clocks chdl ) {
  
  seStatus fStatus = seSTATUS_NG;
  
  if ( chln > 0 ) {
    QSPIx->MOD_b.TMOD = mode;        ///< Select Transfer Mode
    QSPIx->MOD_b.CHLN = chln;        ///< data transfer clocks
    QSPIx->MOD_b.CHDL = chdl;        ///< serial data lines clocks
    
    fStatus = seSTATUS_OK;
  }
  
  return fStatus;
}

/**
  * @brief  This function  asserts Chip Slave Select 0 (Master mode only)
  *         It controls a native QSPI slave select line that can be used with MMA.
  * @retval None
  */
void seQSPI_ASSERT_MST_CS0( void ) {

  QSPI_0->CTL_b.MSTSSO = 0;
  // Wait...
  __NOP();  
}

/**
  * @brief  This function negates Chip Slave Select 0 (Master mode only)
  *         It controls a native QSPI slave select line that can be used with MMA.
  * @retval None
  */
void seQSPI_NEGATE_MST_CS0( void ) {

  QSPI_0->CTL_b.MSTSSO = 1;
  // Wait...
  __NOP(); 
}

