/**
  ******************************************************************************
  * @file    se_qspi.h
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file contains all the functions prototypes for the QSPI firmware
  *          library.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#ifndef SE_QSPI_H
#define SE_QSPI_H

#include "se_common.h"

#ifdef __cplusplus
 extern "C" {
#endif

/** @addtogroup sePeriphLibrary
  * @{
  */

/** @defgroup QSPI
  * @{
  * @brief The QSPI module is a subset of the QSPI bus interface
  */

/** @defgroup QSPI_Constants
  * @{
  * @brief The QSPI module exported constants
  */ 

typedef enum {
   seQSPI_16CLK = 0xf,          ///< Number of clocks = 16 clocks.
   seQSPI_15CLK = 0xe,          ///< Number of clocks = 15 clocks.
   seQSPI_14CLK = 0xd,          ///< Number of clocks = 14 clocks.
   seQSPI_13CLK = 0xc,          ///< Number of clocks = 13 clocks.
   seQSPI_12CLK = 0xb,          ///< Number of clocks = 12 clocks.
   seQSPI_11CLK = 0xa,          ///< Number of clocks = 11 clocks.
   seQSPI_10CLK = 0x9,          ///< Number of clocks = 10 clocks.
   seQSPI_09CLK = 0x8,          ///< Number of clocks =  9 clocks.
   seQSPI_08CLK = 0x7,          ///< Number of clocks =  8 clocks.
   seQSPI_07CLK = 0x6,          ///< Number of clocks =  7 clocks.
   seQSPI_06CLK = 0x5,          ///< Number of clocks =  6 clocks.
   seQSPI_05CLK = 0x4,          ///< Number of clocks =  5 clocks.
   seQSPI_04CLK = 0x3,          ///< Number of clocks =  4 clocks.
   seQSPI_03CLK = 0x2,          ///< Number of clocks =  3 clocks.
   seQSPI_02CLK = 0x1,          ///< Number of clocks =  2 clocks.
   seQSPI_01CLK = 0x0           ///< Number of clocks =  1 clocks.
} seQSPI_Clocks;

typedef enum {
   seQSPI_MSB_FST  = 0,         ///< Specify the data format (input/output permutation) MSB first.
   seQSPI_LSB_FST  = 1          ///< Specify the data format (input/output permutation) LSB first.
} seQSPI_Format;

typedef enum {
   seQSPI_POL_LOW  = 0,         ///< Output is low when clock is off.
   seQSPI_POL_HIGH = 1          ///< Output is high when clock is off.
} seQSPI_Polarity;

typedef enum {
   seQSPI_PH_RISE = 0,          ///< Triggers on positive edge of clock.
   seQSPI_PH_FALL = 1           ///< Triggers on negative edge of clock.
} seQSPI_Phase;

typedef enum {
   seQSPI_MODE_SLAVE  = 0,      ///< Specifies the QSPI mode slave.
   seQSPI_MODE_MASTER = 1       ///< Specifies the QSPI mode master.
} seQSPI_OperMode;

typedef enum {
   seQSPI_MODE_SINGLE  = 0,     ///< Single transfer mode.
   seQSPI_MODE_DUAL    = 1,     ///< Dual transfer mode.
   seQSPI_MODE_QUAD    = 2      ///< Quad transfer mode.
} seQSPI_TransferMode;

typedef enum {
   seQSPI_Output = 0,           ///< Specifies IO direction as output.
   seQSPI_Input  = 1            ///< Specifies IO direction as input.
} seQSPI_IO;

typedef enum {
   seQSPI_24BIT_ADDR = 0,       ///< Specifies IO direction as output.
   seQSPI_32BIT_ADDR = 1        ///< Specifies IO direction as input.
} seQSPI_AddrMode;

/**
  * @}
  */

/** @defgroup QSPI_Types
  * @{
  */

/** 
  * @brief  QSPI Init structure definition
  */

typedef struct {
  sePPORT_Id          QSPID0;   ///< QSPISD0 pin port definition
  sePPORT_Id          QSPID1;   ///< QSPISD1 pin port definition
  sePPORT_Id          QSPID2;   ///< QSPISD2 pin port definition
  sePPORT_Id          QSPID3;   ///< QSPISD3 pin port definition
  sePPORT_Id          QSPICLK;  ///< QSPICLK pin port definition
  sePPORT_Id          QSPISS;   ///< QSPISS# pin port definition
}seQSPI_Ports;

typedef struct {
  seQSPI_Clocks       CHDL;     ///< Set a number of clocks to drive the serial data lines.
  seQSPI_Clocks       CHLN;     ///< Set a number of clocks for data transfer.
  seState             PUEN;     ///< Set input pin pull-up/down.
  seState             NOCLKDIV; ///< Set master mode operating clock.
  seQSPI_Format       LSBFST;   ///< Set MSB first/LSB first.
  seQSPI_Phase        CPHA;     ///< Set clock phase.
  seQSPI_Polarity     CPOL;     ///< Set clock polarity.
  seQSPI_OperMode     MST;      ///< Set master/slave mode.
  seQSPI_TransferMode TMOD;     ///< Set transfer mode (single/dual/quad).
  
  //====== If DMA transfer use ======//
  seQSPI_Clocks       TCSH;     ///< Set slave select signal negation period.
  uint16_t            RMADR;    ///< Set an offset, remapping start address high reg.
  seQSPI_Clocks       DUMDL;    ///< Set dummy cycle drive length.
  seQSPI_Clocks       DUMLN;    ///< Set dummy cycle length.
  seQSPI_TransferMode DATTMOD;  ///< Set data cycle transfer mode.
  seQSPI_TransferMode DUMTMOD;  ///< Set dummy cycle transfer mode.
  seQSPI_TransferMode ADRTMOD;  ///< Set address cycle transfer mode.
  seQSPI_AddrMode     ADRCYC;   ///< Set 24- or 32-bit address cycle.
  uint8_t             XIPACT;   ///< Set XIP activation mode byte.
  uint8_t             XIPEXT;   ///< Set XIP termination mode byte.
  
  //====== If interrupt use ======//
  uint16_t            INTE;
  
}seQSPI_InitTypeDef;

/**
  * @}
  */

/** @defgroup QSPI_Functions
  * @{
  */

/**
  * @brief  This function initializes the QSPIx peripheral as Master according to the specified
  *         parameters in the QSPI_InitStruct.
  * @param  QSPI_InitStruct: pointer to a seQSPI_InitTypeDef structure that
  *         contains the configuration information for the specified QSPI peripheral.
  * @retval None
  */
void seQSPI_InitStructForMaster( seQSPI_InitTypeDef* QSPI_InitStruct );

/**
  * @brief  This function initializes the QSPIx peripheral as Slave according to the specified
  *         parameters in the QSPI_InitStruct.
  * @param  QSPI_InitStruct: pointer to a s@ref eQSPI_InitTypeDef structure that
  *         contains the configuration information for the specified QSPI peripheral.
  * @retval None
  */
void seQSPI_InitStructForSlave( seQSPI_InitTypeDef* QSPI_InitStruct );

/**
  * @brief  This function initializes the QSPIx peripheral according to the specified
  *         parameters in the QSPI_InitStruct.
  * @param  QSPIx: QSPI to select the QSPI peripheral.
  * @param  QSPI_InitStruct: pointer to a @ref seQSPI_InitTypeDef structure that
  *         contains the configuration information for the specified QSPI peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seQSPI_Init( QSPI_0_Type* QSPIx, seQSPI_InitTypeDef* QSPI_InitStruct );

/**
  * @brief  This function starts or enables the specified QSPI channel.
  * @param  QSPIx: QSPI to select the QSPI peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seQSPI_Start( QSPI_0_Type* QSPIx );

/**
  * @brief  This function stops or disables the specified QSPI channel.
  * @param  QSPIx: QSPI to select the QSPI peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seQSPI_Stop( QSPI_0_Type* QSPIx );

/**
  * @brief  This function  is software reset of the specified QSPI channel.
  * @param  QSPIx: QSPI to select the QSPI peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seQSPI_Reset( QSPI_0_Type* QSPIx );

/**
  * @brief  This function sends data through the QSPIx peripheral.
  * @param  QSPIx:  QSPI to select the QSPI peripheral.
  * @param  data: Pointer to data to be transmitted.
  * @param  size: Data size in number of half words.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seQSPI_TxHWords( QSPI_0_Type* QSPIx, uint16_t data[], uint32_t size );

/**
  * @brief  This function sends data through the QSPIx peripheral.
  * @param  QSPIx:  QSPI to select the QSPI peripheral.
  * @param  data: Pointer to data to be transmitted.
  * @param  size: Data size in number of bytes.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seQSPI_TxBytes( QSPI_0_Type* QSPIx, uint8_t data[], uint32_t size );

/**
  * @brief  This function sends single data value through the QSPIx peripheral.
  * @param  QSPIx:  QSPI to select the QSPI peripheral.
  * @param  value: Data value to be transmitted.
  * @param  count: Data size in number of bytes.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seQSPI_TxValue( QSPI_0_Type* QSPIx, uint8_t value, uint32_t count );

/**
  * @brief  This function returns the received data by the QSPIx peripheral.
  * @param  QSPIx: QSPI to select the QSPI peripheral.
  * @param  data: The received data pointer.
  * @param  size: Data size in number of half words. 
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seQSPI_RxHWords( QSPI_0_Type* QSPIx, uint16_t data[], uint32_t size );

/**
  * @brief  This function returns the received data from the QSPIx peripheral
  * @param  QSPIx: QSPI to select the QSPI peripheral.
  * @param  data: The received data pointer.
  * @param  size: Data size in number of bytes. 
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seQSPI_RxBytes( QSPI_0_Type* QSPIx, uint8_t data[], uint32_t size );

/**
  * @brief  This function sets QSPI mode (single, dual, quad).
  * @param  QSPIx: QSPI to select the QSPI peripheral.
  * @param  mode: Sets single, dual or quad, see @ref seQSPI_TransferMode.
  * @param  chln: Number of clocks (seQSPI_01CLK is prohibited) for data transfer, see @ref seQSPI_Clocks.
  * @param  chdl: Number of clocks to drive the serial data lines, see @ref seQSPI_Clocks.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seQSPI_SetMode( QSPI_0_Type* QSPIx, seQSPI_TransferMode mode, 
                                       seQSPI_Clocks chln, seQSPI_Clocks chdl ); 

/**
  * @brief  This function enables QSPI channel interrupt.
  * @param  QSPIx: QSPI to select the QSPI peripheral.
  * @param  irq: Interrupt to be enabled.
  * @retval None
  */
void seQSPI_EnableInt( QSPI_0_Type* QSPIx, uint16_t irq );

/**
  * @brief  This function disables QSPI channel interrupt.
  * @param  QSPIx: QSPI to select the QSPI peripheral.
  * @param  irq: Interrupt to be enabled.
  * @retval None
  */
void seQSPI_DisableInt( QSPI_0_Type* QSPIx, uint16_t irq );

/**
  * @brief  This function gets QSPI channel interrupt flag.
  * @param  QSPIx: QSPI to select the QSPI peripheral.
  * @param  flag: Interrupt flag to check.
  * @retval seInterruptStatus see @ref seInterruptStatus.
  */
seInterruptStatus seQSPI_GetIntFlag( QSPI_0_Type* QSPIx, uint16_t flag );

/**
  * @brief  This function clears QSPI channel interrupt flag.
  * @param  QSPIx: QSPI to select the QSPI peripheral.
  * @param  flag: Interrupt flag to clear.
  * @retval None
  */
void seQSPI_ClearIntFlag( QSPI_0_Type* QSPIx, uint16_t flag );

/**
  * @brief  This function sets the mapping mode for reading from External flash.
  * @param  QSPIx: QSPIx_CH0 to select the QSPI peripheral.
  * @param  raddr: Remapping address.
  * @param  rcmd: Read command defined by flash type used for mapping.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seQSPI_SetMasterRxMMA( QSPI_0_Type* QSPIx, uint32_t raddr, uint16_t flash_rcmd );

/**
  * @brief  This function ends mapping mode for reading from External flash.
  * @param  QSPIx: QSPIx_CH0 to select the QSPI peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seQSPI_ClearMasterRxMMA( QSPI_0_Type* QSPIx );

/**
  * @brief  This function terminates QSPI module.
  * @param  QSPIx:  QSPI to select the QSPI peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seQSPI_TermMasterTx( QSPI_0_Type* QSPIx );

/**
  * @brief  This function configures I/O for Rx or Tx direction (Master mode only).
  * @param  QSPIx: QSPIx_CH0 to select the QSPI peripheral.
  * @retval None
  */
void seQSPI_SetIO( QSPI_0_Type* QSPIx, seQSPI_IO direction );

/**
  * @brief  This function  asserts Chip Slave Select 0 (Master mode only)
  *         It controls a native QSPI slave select line that can be used with MMA.
  * @retval None
  */
void seQSPI_ASSERT_MST_CS0( void );

/**
  * @brief  This function negates Chip Slave Select 0 (Master mode only)
  *         It controls a native QSPI slave select line that can be used with MMA.
  * @retval None
  */
void seQSPI_NEGATE_MST_CS0( void );

/**
  * @brief  Configures ports for this module.
  * @param  QSPIx:  QSPI to select the QSPI peripheral.
  * @param  ports:  QSPI ports to configure.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seAUX_ConfigurePortsForQSPI( QSPI_0_Type* QSPIx, seQSPI_Ports * ports );

/**
  * @brief  Starts bus clock for QSPI
  * @param  QSPIx: QSPI peripheral
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seAUX_StartBusClockForQSPI( QSPI_0_Type* QSPIx );

/**
  * @brief  Stops bus clock for QSPI
  * @param  QSPIx: QSPI peripheral
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seAUX_StopBusClockForQSPI( QSPI_0_Type* QSPIx );

/**
  * @brief  Discovers QSPICLKn frequency [Hz] (= baud rate [bps])
  * @param  QSPIx: QSPI peripheral
  * @retval QSPICLKn: frequency [Hz]
  */
uint32_t seAUX_GetBusSpeedForQSPI( QSPI_0_Type* QSPIx );

/**
  * @brief  Configures QSPICLKn frequency [Hz] (= baud rate [bps])
  * @param  QSPIx: QSPI peripheral
  * @param  speed: QSPICLKn frequency [Hz]
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seAUX_SetBusSpeedForQSPI( QSPI_0_Type* QSPIx, uint32_t speed );

/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */

#ifdef __cplusplus
}
#endif
#endif // SE_QSPI_H

