/**
  ******************************************************************************
  * @file    se_remc3.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file provides all the REMC hardware functions.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#include "se_remc3.h"

/**
  * @brief  Initializes the REMC peripheral according to the specified
  *         parameters in the seREMC_InitStruct. Stop REMC before initializing.
  * @param  REMC3x: REMC3 peripheral.
  * @param  InitStruct: pointer to a seREMC_InitTypeDef structure that
  *         contains the configuration information for the specified REMC peripheral.@ref seREMC_InitTypeDef.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seREMC_Init( REMC3_Type* REMC3x, seREMC_InitTypeDef* InitStruct ) {
  
  seStatus fStatus = seCLG_IsEnabled( seSRC(InitStruct->Clk) );
  
  ///< Check if selected clock is enabled
  if ( fStatus == seSTATUS_OK ) { 
    ///<< Stop REMC 
    seREMC_Stop(REMC3x);
    
    ///< Configure REMO pin
    //fStatus = seAUX_ConfigurePortsForREMC( REMC3x, &InitStruct->ports );
    if (fStatus == seSTATUS_OK) {
      ///< Configure the REMC operating clock.
      seREMC_ConfigureClock( REMC3x, InitStruct->Clk );
                            
      REMC3x->DBCTL_b.REMCRST = 1;                              ///< Reset REMC.
      REMC3x->DBCTL = 0x0001;                                   ///< ----.----.---0.00-1 OINV=0(norm) BUFEN=0(disable) TRMD=0(repeat) MODEN=1
      REMC3x->CARR = InitStruct->carr;                          ///< Set carrier waveform.
      REMC3x->CCTL_b.CARREN = 1;                                ///< Enable carrier modulation.
      REMC3x->INTF = REMC3_INTF_DBIF_Msk | REMC3_INTF_APIF_Msk; ///< DBIF=1 APIF=1  Clear interrupt flag.
      REMC3x->INTE = REMC3_INTE_DBIE_Msk;                       ///< DBIF=1 APIF=0  Enable interrupt.
    }
  }
  
  return fStatus;
}

/**
  * @brief  Configures REMC timer clock source and clock divider.
  * @param  REMC3x: REMC3 peripheral.
  * @param  clock: This parameter can be a value of @ref seREMC_ClkSrc.
  * @param  divider: This parameter can be a value of @ref seREMC_ClkDiv.
  * @retval None
  */
void seREMC_ConfigureClock( REMC3_Type* REMC3x, seREMC_Clk clock ) {
  
  ///< Configure the REMC operating clock.
  REMC3x->CLK_b.CLKSRC = seSRC(clock); ///< Clock source selection
  REMC3x->CLK_b.CLKDIV = seDIV(clock); ///< Clock division ratio selection
  REMC3x->CLK_b.DBRUN  = seDBRUN;      ///< Whether clock supplied in DEBUG mode or not. 
}


/**
  * @brief  Starts REMC by supplying the operating clock.
  * @param  REMC3x: REMC3 peripheral.
  * @param  aplen: Sets data bit active pulse length.
  * @param  dblen: Set data bit length.
  * @retval None
  */
void seREMC_Start( REMC3_Type* REMC3x, uint16_t aplen, uint16_t dblen ) {
 
  REMC3x->DBCTL = 0x0001;    ///< REMOINV=0(norm) BUFEN=0(disable) TRMD=0(repeat) MODEN=1
  REMC3x->APLEN = aplen;     ///< Set data bit active pulse length.
  REMC3x->DBLEN = dblen;     ///< Set data bit length.
  REMC3x->DBCTL = 0x0002;    ///< REMCRST=1  Reset REMC.
  REMC3x->DBCTL = 0x0009;    ///< REMOINV=0(norm) BUFEN=1(enable) TRMD=0(repeat) MODEN=1  Enable compare buffer.
  REMC3x->DBCTL = 0x0309;    ///< PRESET=1 PRUN=1 REMOINV=0(norm) BUFEN=1(enable) TRMD=0(repeat) MODEN=1  Preset and start counter.
}

/**
  * @brief  Stops the REMC module.
  * @param  REMC3x: REMC3 peripheral.
  * @retval None
  */
void seREMC_Stop( REMC3_Type* REMC3x ) {

  REMC3x->DBCTL = 0x0009;    ///< PRUN=0 REMOINV=0(norm) BUFEN=1(enable) TRMD=0(repeat) MODEN=1  Stop counter.
  REMC3x->DBCTL = 0x0008;    ///< PRUN=0 REMOINV=0(norm) BUFEN=1(enable) TRMD=0(repeat) MODEN=0  Disable counter clock.
}

