/**
  ******************************************************************************
  * @file    se_remc3.h
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file contains all the functions prototypes for the REMC firmware
  *          library.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#ifndef SE_REMC3_H
#define SE_REMC3_H

#include "se_common.h"

#ifdef __cplusplus
 extern "C" {
#endif

/** @addtogroup sePeriphLibrary
  * @{
  */

/** 
  * @defgroup REMC3
  * @{
  * @brief The REMC3 circuit generates infrared remote control output signals. 
  *        This circuit can also be applicable to an EL lamp drive circuit by adding a simple external circuit.
  */

/** @defgroup REMC3_Constants
  * @{
  */

/**
  * @}
  */ 

/** @defgroup REMC_Types
  * @{ 
  */

/** 
  * @brief  REMCInit structure definition
  */

typedef struct {
   sePPORT_Id REMO;           ///< REMO pin port definition
   sePPORT_Id CLPLS;          ///< CLPLS pin port definition
} seREMC_Ports;

typedef struct{
   seREMC_Clk Clk;            ///< Specifies the timer clock source and divide selection.
   uint16_t carr;             ///< Carrier Waveform Register
} seREMC_InitTypeDef;

/**
  * @}
  */ 

/** @defgroup REMC3_Functions
  * @{
  */

/**
  * @brief  Initializes the REMC peripheral according to the specified
  *         parameters in the seREMC_InitStruct. Stop REMC before initializing.
  * @param  REMC3x: REMC3 peripheral.
  * @param  InitStruct: pointer to a seREMC_InitTypeDef structure that
  *         contains the configuration information for the specified REMC peripheral.@ref seREMC_InitTypeDef.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seREMC_Init( REMC3_Type* REMC3x, seREMC_InitTypeDef* InitStruct );

/**
  * @brief  Configures REMC timer clock source and clock divider.
  * @param  REMC3x: REMC3 peripheral.
  * @param  clock: This parameter can be a value of @ref seREMC_ClkSrc.
  * @param  divider: This parameter can be a value of @ref seREMC_ClkDiv.
  * @retval None
  */
void seREMC_ConfigureClock( REMC3_Type* REMC3x, seREMC_Clk clock );

/**
  * @brief  Starts REMC by supplying the operating clock.
  * @param  REMC3x: REMC3 peripheral.
  * @param  aplen: Sets data bit active pulse length.
  * @param  dblen: Set data bit length.
  * @retval None
  */
void seREMC_Start( REMC3_Type* REMC3x, uint16_t aplen, uint16_t dblen );

/**
  * @brief  Stops the REMC module.
  * @param  REMC3x: REMC3 peripheral.
  * @retval None
  */
void seREMC_Stop( REMC3_Type* REMC3x );

/**
  * @brief  Configures ports for this module.
  * @param  REMC3x:  REMC3 peripheral.
  * @param  ports:  REMC3 ports to configure.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seAUX_ConfigurePortsForREMC( REMC3_Type* REMC3x, seREMC_Ports * ports );

/**
  * @}
  */ 

/**
  * @}
  */

/**
  * @}
  */

#ifdef __cplusplus
}
#endif
#endif /*SE_REMC3_H */

