/**
  ******************************************************************************
  * @file    se_rfc.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file provides all the RFC hardware functions.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#include "se_rfc.h"

/**
  * @brief  Initializes the RFC peripheral according to the specified 
  *         parameters in the seRFC_InitStruct. Stop RFC before initializing.
  * @param  RFCx:  RFC peripheral
  * @param  InitStruct: pointer to a seRFC_InitTypeDef structure that
  *         contains the configuration information for the specified RFC peripheral.@ref seRFC_InitTypeDef.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seRFC_Init( RFC_0_Type* RFCx, seRFC_InitTypeDef* InitStruct ) {
  
  seStatus fStatus = seCLG_IsEnabled(seSRC(InitStruct->Clk));
  
  ///< Check if selected clock is enabled
  if ( fStatus == seSTATUS_OK ) {
    ///< Stop RFC
    seRFC_Stop(RFCx);
    
    //fStatus = seAUX_ConfigurePortsForRFC( RFCx, &InitStruct->ports );

    if (fStatus == seSTATUS_OK) {
      ///< Configure the RFC operating clock.
      seRFC_ConfigureClock( RFCx, InitStruct->Clk );
      
      ///< Disable interrupt.
      seRFC_DisableInt( RFCx, RFC_0_INTE_ALL_Msk );

      ///< Clear interrupt flag.
      seRFC_ClearIntFlag( RFCx, RFC_0_INTF_ALL_Msk );

      ///< Disable continuous oscillation.
      RFCx->CTL_b.CONEN = seDISABLE;

      ///< Set external clock input mode.
      RFCx->CTL_b.EVTEN = InitStruct->EnableExternClockInput;
#ifdef RFC_OSCMODE
      ///< Configure the oscillation mode.
      RFCx->CTL_b.SMODE = InitStruct->OscMode;
#endif
    }
  }

  return fStatus;
}

/**
  * @brief  Configures RFC timer clock source and clock divider.
  * @param  RFCx:  RFC peripheral
  * @param  clock: This parameter can be a value of @ref seRFC_Clk.
  * @retval None
  */
void seRFC_ConfigureClock( RFC_0_Type* RFCx, seRFC_Clk clock ) {
  
  ///< Configure the RFC operating clock.
  RFCx->CLK_b.CLKSRC = seSRC(clock);   ///< Clock source selection
  RFCx->CLK_b.CLKDIV = seDIV(clock);   ///< Clock division ratio selection
  RFCx->CLK_b.DBRUN  = seDBRUN;        ///< Whether clock supplied in DEBUG mode or not.
}

/**
  * @brief  Starts RFC by supplying the operating clock.
  * @param  RFCx:  RFC peripheral
  * @retval None
  */
void seRFC_Start( RFC_0_Type* RFCx ) {

  ///< Disable interrupts.
  RFCx->INTE = 0x0000;
  ///< Clear interrupt flag.
  RFCx->INTF = RFC_0_INTF_ALL_Msk; 
  ///< Start RFC.
  RFCx->CTL_b.MODEN = 1;
}

/**
  * @brief  Stops the RFC module.
  * @param  RFCx:  RFC peripheral
  * @retval None
  */
void seRFC_Stop( RFC_0_Type* RFCx ) {
  
  ///< Stop RFC.
  RFCx->CTL_b.MODEN = 0;
  ///< Disable interrupt.
  RFCx->INTE = 0x0000;
  ///< Clear interrupt flag.
  RFCx->INTF = RFC_0_INTF_ALL_Msk;
}

/**
  * @brief  Set measurement counter data.
  * @param  RFCx:  RFC peripheral
  * @param  count: Measurement counter data.
  */  
void seRFC_SetMeasurementCounter( RFC_0_Type* RFCx, uint32_t count ) {
  
  ///< Set measurement counter data.
  RFCx->MCL = ( uint16_t )( ( count >>  0 ) & 0xFFFF );
  RFCx->MCH = ( uint16_t )( ( count >> 16 ) & 0x00FF );
}

/**
  * @brief  Get measurement counter data.
  * @param  RFCx:  RFC peripheral
  * @retval Measurement counter data.
  */ 
uint32_t seRFC_GetMeasurementCounter( RFC_0_Type* RFCx ) {
  
  uint32_t result = 0;

  ///< Get measurement counter data.
  result = (( RFCx->MCH & 0x00FFUL ) << 16 ); ///< TPK check 32 bit promotion
  result = result | RFCx->MCL;

  return result;
}

/**
  * @brief  Set Time Base Counter.
  * @param  RFCx:  RFC peripheral
  * @param  count: Measurement count.
  */ 
void seRFC_SetTimeBaseCounter( RFC_0_Type* RFCx, uint32_t count ) {
  
  ///< Set Time base counter data.
  RFCx->TCL = ( uint16_t )( ( count >>  0) & 0xFFFF );
  RFCx->TCH = ( uint16_t )( ( count >> 16) & 0x00FF );
}

/**
  * @brief  Get Time Base Counter.
  * @param  RFCx:  RFC peripheral
  * @retval Measurement count.
  */ 
uint32_t seRFC_GetTimeBaseCounter( RFC_0_Type* RFCx ) {
  
  uint32_t result = 0;

  ///< Get Time base counter data.
  result = ( ( RFCx->TCH & 0x00FFUL ) << 16 ); ///< TPK Check 32-bit promotion
  result = result | RFCx->TCL;

  return result;
}

/**
  * @brief  Executes converting operation.
  * @param  RFCx:  RFC peripheral
  * @param  startCount: Start Count.
  * @param  sensorCount: Sensor Count.
  * @retval Measurement count.
  */
int16_t seRFC_RunConvertingOperation( RFC_0_Type* RFCx, uint32_t startCount, uint32_t * sensorCount ) {
  
  int16_t result = RFC_STS_CONV_ERR_OVMC;
  
  ///< Set measurement counter data.
  seRFC_SetMeasurementCounter( RFCx, startCount );
  ///< Set Time base counter data.
  seRFC_SetTimeBaseCounter( RFCx, 0x0UL );
  ///< Clear interrupt flag.
  RFCx->INTF = RFC_0_INTF_OVTCIF_Msk | RFC_0_INTF_EREFIF_Msk;
  ///< Start CR oscillation.
  RFCx->TRG_b.SREF = 1;
  while ( RFCx->INTF_b.EREFIF == 0 ) {
    ///< Time base counter overflow error.
    if ( RFCx->INTF_b.OVTCIF == 1 ) {
      ///< RFCx->INTF_b.OVTCIF = 1;
      RFCx->INTF = RFC_0_INTF_OVTCIF_Msk;
      result =  RFC_STS_CONV_ERR_OVTC;
      break;
    }
  }
  
  if ( result !=  RFC_STS_CONV_ERR_OVTC ) {
    ///< Clear interrupt flag.
    RFCx->INTF	= RFC_0_INTF_OVTCIF_Msk | RFC_0_INTF_ESENAIF_Msk | RFC_0_INTF_ESENBIF_Msk;
    ///< Start Sensor A/B oscillation.
    RFCx->TRG_b.SSENA = 1;

    ///< Wait for interrupts.
    do  {
      ///< Sensor A oscillation completion interrupt.
      if ( RFCx->INTF_b.ESENAIF == 1 ) {
        RFCx->INTF = RFC_0_INTF_ESENAIF_Msk;
        *sensorCount = seRFC_GetMeasurementCounter(RFCx);
        result = RFC_STS_CONV_SEN_A;
      }
      ///< Sensor B oscillation completion interrupt.
      else if ( RFCx->INTF_b.ESENBIF == 1 ) {
        RFCx->INTF = RFC_0_INTF_ESENBIF_Msk;
        *sensorCount = seRFC_GetMeasurementCounter(RFCx);
        result = RFC_STS_CONV_SEN_B;
      }
      ///< Measurement counter overflow error.
      else if ( RFCx->INTF_b.OVMCIF == 1 ) {
        RFCx->INTF = RFC_0_INTF_OVTCIF_Msk;
        result = RFC_STS_CONV_ERR_OVMC;
      }
      else { 
        result = 0;
      }
    } while ( result == 0 );
  }
  
  return result;
}

/**
  * @brief  Disables the RFC interrupts.
  * @param  RFCx:  RFC peripheral
  * @param  irq: Interrupt mask to disable.
  * @retval None
  */
void seRFC_DisableInt( RFC_0_Type* RFCx, uint16_t irq ) {

  RFCx->INTE &= ~irq;
}

/**
  * @brief  Enables the RFC interrupts.
  * @param  RFCx:  RFC peripheral
  * @param  irq: Interrupt mask to enable.
  * @retval None
  */
void seRFC_EnableInt( RFC_0_Type* RFCx, uint16_t irq ) {

  RFCx->INTE |= irq;
}

/**
  * @brief  Clears Interrupt flags.
  * @param  RFCx:  RFC peripheral
  * @param  irq: Interrupt flags to clear.
  * @retval None
  */
void seRFC_ClearIntFlag( RFC_0_Type* RFCx, uint16_t irq ) {

  RFCx->INTF = irq;
}

