/**
  ******************************************************************************
  * @file    se_rfc.h
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file contains all the functions prototypes for the RFC firmware
  *          library.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#ifndef SE_RFC_H
#define SE_RFC_H

#include "se_common.h"

#ifdef __cplusplus
 extern "C" {
#endif

/** @addtogroup sePeriphLibrary
  * @{
  */

/** 
  * @defgroup RFC
  * @{
  * @brief RFC is a R/F Converter module.
  */ 

/** @defgroup RFC_Constants 
  * @{
  */

#define RFC_STS_CONV_SEN_A       (  1 )  ///< Status : Sensor A oscillation completion interrupt.
#define RFC_STS_CONV_SEN_B       (  2 )  ///< Status : Sensor B oscillation completion interrupt.
#define RFC_STS_CONV_ERR_OVTC    ( -1 )  ///< Status : Time base counter overflow error.
#define RFC_STS_CONV_ERR_OVMC    ( -2 )  ///< Status : Measurement counter overflow error.

typedef  seCLG_ClkSrc seRFC_ClkSrc;

typedef enum {
   seRFC_DcMode = 0,  ///< DC oscillation mode for resistive sensor measurements.
   seRFC_AcMode = 1   ///< AC oscillation mode for resistive sensor measurements.
} seRFC_OscMode;

/**
  * @}
  */ 

/** @defgroup RFC_Types
  * @{ 
  */

/** 
  * @brief  RFCInit structure definition
  */

typedef struct {
   sePPORT_Id  SENA;                ///< SENA pin port definition
   sePPORT_Id  SENB;                ///< SENB pin port definition
   sePPORT_Id  REF;                 ///< REF pin port definition
   sePPORT_Id  RFIN;                ///< RFIN pin port definition
   sePPORT_Id  RFCLKO;              ///< RFCLKO pin port definition
} seRFC_Ports;

typedef struct {
   seRFC_Clk Clk;                   ///< Specifies the timer clock source and divide selection.
   seState StopInSleep;             ///< Specifies if operation stops or continues in sleep mode.
   seState EnableExternClockInput;  ///< Enables external clock inputmode.
   seRFC_OscMode OscMode;           ///< Configures an oscillation mode for resistive measurements.
} seRFC_InitTypeDef;

/**
  * @}
  */ 

/** @defgroup RFC_Functions
  * @{
  */

/**
  * @brief  Initializes the RFC peripheral according to the specified 
  *         parameters in the seRFC_InitStruct. Stop RFC before initializing.
  * @param  RFCx:  RFC peripheral
  * @param  InitStruct: pointer to a seRFC_InitTypeDef structure that
  *         contains the configuration information for the specified RFC peripheral.@ref seRFC_InitTypeDef.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seRFC_Init( RFC_0_Type* RFCx, seRFC_InitTypeDef* InitStruct );

/**
  * @brief  Configures RFC timer clock source and clock divider.
  * @param  RFCx:  RFC peripheral
  * @param  clock: This parameter can be a value of @ref seRFC_Clk.
  * @retval None
  */
void seRFC_ConfigureClock( RFC_0_Type* RFCx, seRFC_Clk clock );

/**
  * @brief  Starts RFC by supplying the operating clock.
  * @param  RFCx:  RFC peripheral
  * @retval None
  */
void seRFC_Start( RFC_0_Type* RFCx );

/**
  * @brief  Stops the RFC module.
  * @param  RFCx:  RFC peripheral
  * @retval None
  */
void seRFC_Stop( RFC_0_Type* RFCx );

/**
  * @brief  Disables the RFC interrupts.
  * @param  RFCx:  RFC peripheral
  * @param  irq: Interrupt mask to disable.
  * @retval None
  */
void seRFC_DisableInt( RFC_0_Type* RFCx, uint16_t irq );

/**
  * @brief  Enables the RFC interrupts.
  * @param  RFCx:  RFC peripheral
  * @param  irq: Interrupt mask to enable.
  * @retval None
  */
void seRFC_EnableInt( RFC_0_Type* RFCx, uint16_t irq );

/**
  * @brief  Clears Interrupt flags.
  * @param  RFCx:  RFC peripheral
  * @param  irq: Interrupt flags to clear.
  * @retval None
  */
void seRFC_ClearIntFlag( RFC_0_Type* RFCx, uint16_t irq );

/**
  * @brief  Set measurement counter data.
  * @param  RFCx:  RFC peripheral
  * @param  count: Measurement counter data.
  */  
void seRFC_SetMeasurementCounter( RFC_0_Type* RFCx, uint32_t count );

/**
  * @brief  Get measurement counter data.
  * @param  RFCx:  RFC peripheral
  * @retval Measurement counter data.
  */ 
uint32_t  seRFC_GetMeasurementCounter( RFC_0_Type* RFCx );

/**
  * @brief  Set Time Base Counter.
  * @param  RFCx:  RFC peripheral
  * @param  count: Measurement count.
  */ 
void seRFC_SetTimeBaseCounter( RFC_0_Type* RFCx, uint32_t count );

/**
  * @brief  Get Time Base Counter.
  * @param  RFCx:  RFC peripheral
  * @retval Measurement count.
  */ 
uint32_t  seRFC_GetTimeBaseCounter( RFC_0_Type* RFCx );

/**
  * @brief  Executes converting operation.
  * @param  RFCx:  RFC peripheral
  * @param  startCount: Start Count.
  * @param  sensorCount: Sensor Count.
  * @retval Measurement count.
  */
int16_t seRFC_RunConvertingOperation( RFC_0_Type* RFCx, uint32_t startCount, uint32_t* sensorCount );

/**
  * @brief  Configures ports for this module. Override this function to configure specific ports.
  * @param  RFCx:  RFC peripheral
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seAUX_ConfigurePortsForRFC( RFC_0_Type* RFCx, seRFC_Ports * ports );

/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */

#ifdef __cplusplus
}
#endif
#endif /*SE_RFC_H */

