/**
  ******************************************************************************
  * @file    se_rtca.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file provides all the RTCA hardware functions.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#include "se_rtca.h"

/**
  * Static functions declarations
**/
static seStatus unprotect( void );              ///< RTCA private protection mechanism
static void protect( void );                    ///< RTCA private protection mechanism

/**
  * @brief  Initializes the RTCA peripheral according to the specified
  *         parameters in the seRTCA_InitStruct.
  * @param  InitStruct: pointer to a seRTCA_InitTypeDef structure that
  *         contains the configuration information for the specified RTCA peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seRTCA_Init( seRTCA_InitTypeDef* InitStruct ) {
  
  seStatus fStatus = seSTATUS_NG;
  
  ///< Check if selected clock is enabled
  if ( se32k768HzSupplied() ) {
    fStatus = seSTATUS_OK;
  }

  if ( fStatus == seSTATUS_OK ) {
    
    ///< Stop RTCA
    seRTCA_Stop();
    
    ///< Select operation mode; 
    fStatus = seRTCA_Set12_24Mode( InitStruct->H12_24Format );
    
    ///< Select initial Date time as "01/01/00 Sunday 00:00:00"
    RTCA->YAR = 0;
    RTCA->MON_b.RTCMOH = 0;
    RTCA->MON_b.RTCMOL = 1;
    RTCA->MON_b.RTCDH  = 0;
    RTCA->MON_b.RTCDL  = 1;
    RTCA->HUR_b.RTCHH  = 0;
    RTCA->HUR_b.RTCHL  = 0;
    RTCA->HUR_b.RTCMIH = 0;
    RTCA->HUR_b.RTCMIL = 0;
    RTCA->SEC_b.RTCSH  = 0;
    RTCA->SEC_b.RTCSL  = 0;
        
    ///< Initialize interrupts.
    seRTCA_DisableInt( RTCA_INTE_ALL_Msk );
    seRTCA_ClearIntFlag( RTCA_INTF_ALL_Msk );
  } 
  
  return fStatus;
}

/**
  * @brief  Starts RTCA channel by start supplying operating clock.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seRTCA_Start( void ) {
 
  seStatus fStatus = seSTATUS_OK;
  ///< Execute software reset.
  RTCA->CTLL_b.RTCRST = 1;
  WHILE( RTCA->CTLL & RTCA_CTLL_RTCRST_Msk, fStatus );
  SANITY_CHECK( fStatus );
  RTCA->CTLL_b.RTCRUN = 1;
  
  return fStatus;
}

/**
  * @brief  Stops RTCA channel by stop supplying operating clock.
  * @retval None
  */
void seRTCA_Stop( void ) {
  
  RTCA->CTLL_b.RTCRUN = 0;
}

/**
  * @brief  Enables RTCA channel interrupt.
  * @param  irq: Interrupt to enable.
  * @retval None
  */
void seRTCA_EnableInt( uint16_t irq ) {
  
  RTCA->INTE |= irq;
}

/**
  * @brief  Disables RTCA channel interrupt.
  * @param  irq: Interrupt to disable.
  * @retval None
  */
void seRTCA_DisableInt( uint16_t irq ) {
  
  RTCA->INTE &= (uint16_t)~irq;
}

/**
  * @brief  Returns RTCA interrupt flag.
  * @param  irq: Interrupt flag to check.
  * @retval InterruptStatus
  */
seInterruptStatus seRTCA_GetIntFlag( uint16_t irq ) {
  
  seInterruptStatus fStatus = seINTERRUPT_NOT_OCCURRED;
  uint16_t flagged = RTCA->INTF;
  
  if ( irq ) {
    flagged &= irq;
    if ( flagged == irq ) {
      fStatus = seINTERRUPT_OCCURRED;
    }
  }
  
  return fStatus;
}

/**
  * @brief  Clears RTCA channel interrupt.
  * @param  irq: Interrupt flag to clear.
  * @retval None
  */
void seRTCA_ClearIntFlag( uint16_t irq ) {
  
  RTCA->INTF = irq;
}

/**
  * @brief  Start 1 sec timer.
  *         The 1-second interrupt of RTCA is enabled. User shall enable interrupt globally and
  *         provide an interrupt service routine.
  * @retval None
**/
void seRTCA_Enable1SecTimer( void ) {

  ///< Clear interrupt flag.
  seRTCA_ClearIntFlag( RTCA_INTF_T1SECIF_Msk );

  ///< The interrupt of RTCA set enable(1-second interrupt).
  RTCA->INTE_b.T1SECIE	= 1;
}

/**
  * @brief  Stops 1 sec timer.
  *         The 1-second interrupt of RTCA is disabled. User shall
  *         provide appropriate actions in the interrupt service routine.
  * @retval None
**/
void seRTCA_Disable1SecTimer( void ) {

  ///< The interrupt of RTCA set disable(1-second interrupt).
  RTCA->INTE_b.T1SECIE	= 0;
  ///< Clear interrupt flag.
  seRTCA_ClearIntFlag( RTCA_INTF_T1SECIF_Msk );
}

/**
  * @brief  Calculate theoretical regulation bits.
  *         1. Measure the frequency tolerance \"m [ppm]\"of fOSC1.
  *         2. Determine the theoretical regulation execution cycle time \"n seconds.\"
  *         The correction value for theoretical regulation can be specified within the 
  *         range from -64 to +63 and it should be written to the CTL register's RTCTRM[6:0] bits
  *         as a twos-complement number.
  * @param  m: This parameter is the frequency tolerance in ppm time ten ( to preserve accuracy in integer math. ).
  * @param  n: This param is execution cycle time in seconds.
  * @retval This function returns TRM bits.
**/
uint8_t seRTCA_CalculateTrm( int32_t m, int32_t n ) {

  ///< 256L*m*n/10000000L == 2L*m*n/78125L
  int8_t tmp8 = seClamp32( 2L*m*n/78125L, -64, 63 );
  
  return (tmp8&0x7F);
}

/**
  * @brief  Setting a year, month, day, day of the week in hardware.
  *         When a value out of the effective range is set to the year, day of the week, 
  *         the counter will be cleared to 0 at the next 
  *         count-up timing. When a such value is set to the month, day,
  *         the counter will be set to 1 at the next count-up timing.
  * @param  year: Integer year value (from 0 to 99).
  * @param  month: Integer month value (from 1 to 12).
  * @param  day: Integer day value (from 1 to 31).
  * @param  week: Integer day of the week value of type @ref seRTCA_DayOfTheWeek.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seRTCA_SetYearMonthDayWeek( uint8_t year, uint8_t month, uint8_t day, uint8_t week ) {
  
  seStatus fStatus = seSTATUS_NG;
  
  if ( !( year > 99 || month > 12 || month == 0 || day > 31 ) ) {
    fStatus = unprotect();
    RTCA->YAR_b.RTCYH = year / 10;
    RTCA->YAR_b.RTCYL = year % 10;
    RTCA->MON_b.RTCMOH = month / 10;
    RTCA->MON_b.RTCMOL = month % 10;
    RTCA->MON_b.RTCDH = day / 10;
    RTCA->MON_b.RTCDL = day % 10;
    RTCA->YAR_b.RTCWK = week;
    protect();
  }
  
  return fStatus;
}

/**
  * @brief  Getting a year, month, day, day of the week from hardware.
  * @param  year: Integer year value (from 0 to 99).
  * @param  month: Integer month value (from 1 to 12).
  * @param  day: Integer day value (from 1 to 31).
  * @param  week: Integer day of the week value of type @ref seRTCA_DayOfTheWeek.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seRTCA_GetYearMonthDayWeek( uint8_t* year, uint8_t* month, uint8_t* day, uint8_t* week ) {
  
  seStatus fStatus = unprotect();
  uint8_t value;
  
  ///< Get year.
  value = RTCA->YAR_b.RTCYH * 10;
  value += RTCA->YAR_b.RTCYL;
  *year = ( value <=99 )?value:0;
  ///< Get month
  value = RTCA->MON_b.RTCMOH * 10;
  value += RTCA->MON_b.RTCMOL;
  *month= ( value <= 12 && value > 0 )? value : 1;
  ///< Get day
  value = RTCA->MON_b.RTCDH * 10;
  value += RTCA->MON_b.RTCDL;
  *day = ( value > 0 &&  value <=31 )?value:1;
  ///< Get of theweek 
  *week = RTCA->YAR_b.RTCWK;
  protect();
  
  return fStatus;
}

/**
  * @brief  Reads 12/24 mode from hardware
  * @retval 12/24 Mode: a value of @seRTCA_Hours12_24
  */
seRTCA_Hours12_24 seRTCA_Get12_24Mode( void ) {
  
  unprotect();
  uint8_t value = RTCA->CTLL_b.RTC24H;
  protect();
  
  return (seRTCA_Hours12_24)value;
}

/**
  * @brief  Writes 12/24 mode to hardware
  * @param  mode: 12/24 mode
  * @retval Status: can be a value of @ref seStatus
  */    
seStatus seRTCA_Set12_24Mode( seRTCA_Hours12_24 mode ) {
  
  seStatus fStatus = unprotect();
  RTCA->CTLL_b.RTC24H = mode;
  protect();
  
  return fStatus;
}

/**
  * @brief  Setting an hour, minute, second in hardware.
  *         When a value out of the effective range is set to 
  *         hour (in 24H mode) counter, the counter will be cleared to 0 at the next
  *         count-up timing. When a such value is set to hour (in 12H mode) counter,
  *         the counter will be set to 1 at the next count-up timing.
  * @param  h: Integer hour value.
  * @param  m: Integer minute value.
  * @param  s: Integer second value.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seRTCA_SetHourMinuteSecond( uint8_t h, uint8_t m, uint8_t s ) {
  
  seStatus fStatus = unprotect();

  if ( fStatus ) {
    uint8_t value = RTCA->CTLL_b.RTC24H;
    if ( ( value == seRTCA_CTL_RTC24H_24 ) && !seRTCA_VALID_24HOURS(h) ) {
         fStatus = seSTATUS_NG; 
    }
    if ( (value == seRTCA_CTL_RTC24H_12) && !seRTCA_VALID_12HOURS(h) ) {
      fStatus = seSTATUS_NG;   
    }
    if ( !seRTCA_VALID_MINUTES(m) ) {
      fStatus = seSTATUS_NG;
    }
    if ( !seRTCA_VALID_MINUTES(s) ) {
      fStatus = seSTATUS_NG;
    } 
    
    RTCA->HUR_b.RTCHH = h / 10;     ///< Set hour(tenths digit).
    RTCA->HUR_b.RTCHL = h % 10;     ///< Set hour(unit digit).

    RTCA->HUR_b.RTCMIH = m / 10;    ///< Set minute(tenths digit).
    RTCA->HUR_b.RTCMIL = m % 10;    ///< Set minute(unit digit).

    RTCA->SEC_b.RTCSH = s / 10;     ///< Set second(tenths digit).
    RTCA->SEC_b.RTCSL = s % 10;     ///< Set second(unit digit).
  }
  
  protect();
  
  return fStatus;
}

/**
  * @brief  Getting an hour, minute, second from hardware.
  * @param  h: Pointer to integer hour value.
  * @param  m: Pointer to integer minute value.
  * @param  s: Pointer to integer second value.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seRTCA_GetHourMinuteSecond( uint8_t* h,uint8_t* m, uint8_t* s ) {  
  
  seStatus fStatus = unprotect();
  
  ///< Get value.
  *h = RTCA->HUR_b.RTCHH * 10;
  *h += RTCA->HUR_b.RTCHL;

  *m = RTCA->HUR_b.RTCMIH * 10;
  *m += RTCA->HUR_b.RTCMIL;
  
  *s =  RTCA->SEC_b.RTCSH * 10;
  *s += RTCA->SEC_b.RTCSL;
  protect();  
  
  return fStatus;
}

/**
  * @brief  Set AM/PM indicator.
  * @param  indicator: AM/PM indicator. @ref seRTCA_AM_PM_indicator
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seRTCA_SetAM_PM( seRTCA_AM_PM indicator ) {
   
  seStatus fStatus = unprotect(); 
  ///< Set value.
  RTCA->HUR_b.RTCAP = indicator;
  protect();  
  
  return fStatus;
}

/**
  * @brief  Get AM/PM indicator.
  * @retval indicator: @ref seRTCA_AM_PM.
  */
seRTCA_AM_PM seRTCA_GetAM_PM( void ) {
   
  unprotect();
  uint16_t value = RTCA->HUR_b.RTCAP;
  protect();
  
  return (seRTCA_AM_PM)value;
}

/**
  * @brief  Sets an alarm clock and enables alarm interrupt,
  *         user provides an interrupt service routine and enables interrupt globally.
  * @param  Integer hour value from 0 to 24 @ref seRTCA_VALID_24HOURS.
  * @param  Integer minute value from 0 to 60 @ref seRTCA_MINUTES.
  * @param  Integer second value from 0 to 60 @ref seRTCA_VALID_SECONDS.
  * @param  AM/FM settings @ref seRTCA_AM_PM.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seRTCA_SetAlarm( uint8_t h, uint8_t m, uint8_t s, seRTCA_AM_PM am ) {
  
  seStatus fStatus = seSTATUS_NG;
  
  if ( seRTCA_VALID_MINUTES(m) && seRTCA_VALID_SECONDS(s) && seRTCA_VALID_24HOURS(h)  ) {
    uint16_t mode = seRTCA_Get12_24Mode();
    if ( ( mode == seRTCA_CTL_RTC24H_12 && h <= 12 ) ||
       ( mode == seRTCA_CTL_RTC24H_24 && h <= 24 )) {

      seRTCA_DisableInt( RTCA_INTE_ALARMIE_Msk );
      RTCA->ALM1_b.RTCSHA = s/10;
      RTCA->ALM1_b.RTCSLA = s%10;
      RTCA->ALM2_b.RTCMIHA = m/10;
      RTCA->ALM2_b.RTCMILA = m%10;
      RTCA->ALM2_b.RTCAPA = am;
      RTCA->ALM2_b.RTCHHA = h/10;
      RTCA->ALM2_b.RTCHLA = h%10;
      seRTCA_ClearIntFlag( RTCA_INTF_ALARMIF_Msk );
      seRTCA_EnableInt( RTCA_INTE_ALARMIE_Msk );
      fStatus = seSTATUS_OK;
    }   
  }
  
  return fStatus;
}

/**
  * @brief  Gets an alarm clock and enables alarm interrupt,
  *         user provides an interrupt service routine and enables interrupt globally.
  * @param  h: pointer to Integer hour value from 0 to 24 @ref seRTCA_VALID_24HOURS.
  * @param  m: pointer to Integer minute value from 0 to 60 @ref seRTCA_MINUTES.
  * @param  s: pointer to Integer second value from 0 to 60 @ref seRTCA_VALID_SECONDS.
  * @param  am: pointer to AM/FM settings @ref seRTCA_AM_PM.
  * @retval None
  */
void seRTCA_GetAlarm( uint8_t *h, uint8_t *m, uint8_t *s, seRTCA_AM_PM *am ) {
  
  *s  = RTCA->ALM1_b.RTCSHA * 10;
  *s += RTCA->ALM1_b.RTCSLA; 
  *m  = RTCA->ALM2_b.RTCMIHA * 10;
  *m += RTCA->ALM2_b.RTCMILA;
  *am = (seRTCA_AM_PM)RTCA->ALM2_b.RTCAPA;
  *h  = RTCA->ALM2_b.RTCHHA * 10;
  *h += RTCA->ALM2_b.RTCHLA;
}

/**
  * @brief  Sets an alarm in number of seconds from current time.
  *         seRTCA_SetAlarm() is called.
  *         User provides an interrupt service routine and enables interrupt globally.
  * @param  alarmsec:  alarm seconds from current time.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seRTCA_SetSecondsAlarm( uint32_t alarmsec ) {

  uint8_t currhour, currminute, currsecond;
  uint32_t hours, minutes, seconds;
  uint16_t mode;
  seRTCA_AM_PM ampm;
  seStatus fStatus = seSTATUS_NG;
  
  if (alarmsec < 86400)  ///< checkt that it is less than 24 hours
  {
    seRTCA_GetHourMinuteSecond( &currhour,&currminute, &currsecond );
    ampm = seRTCA_GetAM_PM();
    mode = seRTCA_Get12_24Mode();
      
    hours = (alarmsec / 3600);
    minutes = (alarmsec - (hours * 3600)) / 60;
    seconds = (alarmsec - (hours * 3600) - (minutes * 60));
    
    seconds = seconds + currsecond;
    if (seconds >= 60)
    {
      seconds -= 60;
      minutes++;
    }
    
    minutes = minutes + currminute;
    if (minutes >= 60)
    {
      minutes -= 60;
      hours++;
    }
    
    if (mode == seRTCA_CTL_RTC24H_24)
    {
      hours = hours + currhour;
      if (hours >= 24)
      {
        hours -= 24;
      }
    }
    else
    {
      hours = hours + currhour;
      if (hours >= 24)
      {
        hours -= 12;
        if (ampm == seRTCA_HUR_RTCAP_AM)
          ampm = seRTCA_HUR_RTCAP_PM;
        else
          ampm = seRTCA_HUR_RTCAP_AM;
      }
    }
    
    fStatus = seRTCA_SetAlarm( hours, minutes, seconds, ampm );
  }

  return(fStatus);
}

/**
  * @brief  Set 30-second correction.
  *         This function is provided to set the time-of-day clock by the time signal. Writing 1 to the CTL RTCADJ
  *         bit adds 1 to the minute counter if the second counter represents 30 to 59 seconds, or clears the second counter
  *         with the minute counter left unchanged if the second counter represents 0 to 29 seconds.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seRTCA_Set30secCorrection( void ) {
  
  seStatus fStatus = seSTATUS_OK;
  if ( RTCA->CTLL_b.RTCADJ == 0 ) {
    RTCA->CTLL_b.RTCADJ  = 1;
    ///< Wait untill this bit goes away to prevent an unintentional write
    ///< of 1 by read/modify of CTLL. And return the status
    WHILE( RTCA->CTLL & RTCA_CTLL_RTCADJ_Msk, fStatus );
  } else {
    fStatus = seSTATUS_NG;
  }
  return fStatus;
}

/**
  * @brief  Start Stopwatch counter
  * @param  irqs: 1Hz, 10Hz and 100Hz Interrupt.
  * @retval None
  */
void seRTCA_StartStopWatchCount( uint16_t irqs ) {

  seRTCA_ClearIntFlag( irqs );
  seRTCA_EnableInt( irqs );
  RTCA->SWCTL_b.SWRUN = 1;
}

/**
  * @brief  Stop Stopwatch counter
  * @param  irqs: 1Hz, 10Hz and 100Hz Interrupt.
  * @retval None
  */
void seRTCA_StopStopWatchCount(  uint16_t irqs ) {
  
  seRTCA_DisableInt( irqs );
  seRTCA_ClearIntFlag( irqs );
  RTCA->SWCTL_b.SWRUN = 0;
}

/**
  * @brief  Reset Stopwatch counter and set members of StopWatchCounter to 0
  * @param  StopWatchCounter: 1Hz, 10Hz and 100Hz Interrupt, see @ref swCounter
  * @retval None
  */
void seRTCA_ResetStopWatchCount( swCounter * StopWatchCounter ) {
  
  uint16_t IsStopWatchRunning = RTCA->SWCTL_b.SWRUN;
 
  StopWatchCounter->swInt1HzCount = 0; 
  
  if ( ! IsStopWatchRunning ) {
    RTCA->SWCTL = 0x11;
    RTCA->SWCTL_b.SWRUN = 0;
  } else {
    RTCA->SWCTL_b.SWRST = 1;
  }
}

/**
  * @brief  Read the 1/10-second and 1/100-second digits of  the stopwatch counter.
  *         Note, that seconds and minutes should be assigned outside of this function.
  * @param  StopWatchCounter: pointer to StopWatchCounter value of the stopwatch counter
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seRTCA_ReadStopWatchCount( swCounter * StopWatchCounter ) {

  seStatus fStatus = seSTATUS_NG;
  uint16_t i = 0;
  
  while ( i++ < 10 ) {
    uint16_t val = RTCA->SWCTL;
    if ( val == RTCA->SWCTL ) {
      StopWatchCounter->swChar10HzDigit  = 0x000f&(val>>12);
      StopWatchCounter->swChar100HzDigit = 0x000f&(val>> 8);
      fStatus = seSTATUS_OK;
      break;
    } 
  }
   
  return fStatus;
}

/**
  * @brief  Calculate weekday from year, month, day
  * @param  y: Integer year value.
  * @param  m: Integer month value.
  * @param  d: Integer day of the month value.
  * @retval weekday: @ref seRTCA_DayOfTheWeek.
  */
const uint8_t month_code[] = {0, 6, 2, 2, 5, 0, 3, 5, 1, 4, 6, 2, 4};

uint8_t seRTCA_CalcWeekDay( uint8_t y, uint8_t m, uint8_t d ) {
  
  uint8_t yearcode;
  uint8_t daycode;
  uint8_t weekday;

  daycode = d % 7;
  yearcode = y / 4;
  yearcode = (yearcode * 5) % 7;

  if ((y % 4) == 0) {  ///< Leap year
    
    if (m < 3) {
      weekday = daycode + (month_code[m] - 1) +  yearcode;
    }  
    else {
      weekday = daycode + month_code[m] +  yearcode;
    }
    weekday = weekday % 7;
  } else {
    weekday = daycode + month_code[m] +  yearcode + (y % 4);
    weekday = weekday % 7;
  }

  return weekday;
}

static int32_t freqerrsum_mHz = 0;
static int16_t prev_freqerr_mHz = 0;
static uint32_t tsamp = 900;
static int32_t rtcTrim = 0;

/**
  * @brief  Initialize Theoretical Regulation variables and set RTC alarm for next adjustment.
  *         seRTCA_SetAlarm() is called.
  *         User provides an interrupt service routine and enables interrupt globally.
  * @param  sampling_period:  sampling period for Theorectical Regulation adjustments.
  * @param  curr_freqerr_mHz:  current OSC1 frequency error in milliHertz.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seRTCA_InitTheoreticalRegulation( uint32_t sampling_period, int16_t curr_freqerr_mHz ) {

  seStatus fStatus = seSTATUS_NG;
  
  prev_freqerr_mHz = curr_freqerr_mHz;
  freqerrsum_mHz = 0;
  tsamp = sampling_period;
  
  fStatus = seRTCA_SetSecondsAlarm(tsamp);  ///< Set alarm
  
  return fStatus;
}

/**
  * @brief  Theoretical Regulation adjustment calculation/trimming and set RTC alarm for next adjustment.
  *         seRTCA_SetAlarm() is called.
  *         User provides an interrupt service routine and enables interrupt globally.
  * @param  curr_freqerr_mHz:  current OSC1 frequency error in milliHertz.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seRTCA_TheoreticalRegulationTrim( int16_t curr_freqerr_mHz ) {
  
  seStatus fStatus = seSTATUS_NG;
  
  ///< get average frequency error between current and previous value
  if ((prev_freqerr_mHz + curr_freqerr_mHz) < 0)
    prev_freqerr_mHz = (prev_freqerr_mHz + curr_freqerr_mHz - 1) / 2;
  else
    prev_freqerr_mHz = (prev_freqerr_mHz + curr_freqerr_mHz + 1) / 2;

  ///< Add to accumulator/sum
  freqerrsum_mHz += (int32_t) prev_freqerr_mHz;
  
  ///< trim value = - [((tsamp * 256)/ 32768) * freqerrsum_mHz] / 1000;
  if (freqerrsum_mHz < 0)
    rtcTrim = ((freqerrsum_mHz * tsamp) - 64000) / 128000L;  ///< round off negative value
  else
    rtcTrim = ((freqerrsum_mHz * tsamp) + 64000) / 128000L;  ///< round off positive value

  rtcTrim = -rtcTrim;  ///< Opposite sign of time drift

  ///< Clip to max adjust values
  if ((rtcTrim < 0) && (rtcTrim < -64))
    rtcTrim = -64;
  else if ((rtcTrim >= 0) && (rtcTrim > 63))
    rtcTrim = 63;

  ///< Adjust frequency error sum
  if (rtcTrim < 0)
    freqerrsum_mHz += (((rtcTrim * 128000L) - (tsamp>>1)) / tsamp);  ///< freqerrsum_mHz = freqerrsum_mHz + (rtcTrim * 32768 * 1000) / (256 * tsamp)
  else
    freqerrsum_mHz += (((rtcTrim * 128000L) + (tsamp>>1)) / tsamp);  ///< freqerrsum_mHz = freqerrsum_mHz + (rtcTrim * 32768 * 1000) / (256 * tsamp)

  ///< Trigger trimming
  if (rtcTrim < 0)  ///< make value right for RTCTRM register
    rtcTrim = rtcTrim + 128;

  while (RTCA->CTLH_b.RTCTRMBSY);
  RTCA->CTLH_b.RTCTRM = (uint8_t) (rtcTrim & 0x7F);

  ///< Set up for next sample
  prev_freqerr_mHz = curr_freqerr_mHz;

  fStatus = seRTCA_SetSecondsAlarm(tsamp);  // Set alarm
  
  return fStatus;
}

static seStatus unprotect( void ) {

  seStatus fStatus = seSTATUS_OK;
  ///< Check busy.
  WHILE( RTCA->CTLL & RTCA_CTLL_RTCBSY_Msk, fStatus );
  SANITY_CHECK( fStatus );
  ///< Hold RTCA count-up.
  RTCA->CTLL_b.RTCHLD = 1;
  ///< When the RTCTRMBSY bit = 1, the RTCHLD bit cannot
  ///< be rewritten to 1 (as fixed at 0).
  ///< Check if it was successfull stop.
  fStatus = ( RTCA->CTLL_b.RTCHLD )?seSTATUS_OK :seSTATUS_NG;

  return fStatus;
}

static void protect( void ) {

  ///< Resume RTCA count-up.
  RTCA->CTLL_b.RTCHLD = 0;
}

