/**
  ******************************************************************************
  * @file    se_snda.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file provides all the SNDA hardware functions.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#include "se_snda.h"

/**
  * @brief  Fills each seSNDA_InitTypeDef member with its default value.
  * @param  InitStruct: pointer to an seSNDA_InitTypeDef structure which will be initialized.
  * @retval None
  */
void seSNDA_InitStruct( seSNDA_InitTypeDef* InitStruct ) {

  InitStruct->Clk          = seDEFAULT_SNDA_CLK;  ///< Set SNDA operating clock.
  InitStruct->DriveMode    = seSNDA_SingleDrive;  ///< Set output pin drive mode.
  InitStruct->EnableInt    = 0;                   ///< No interrupt
}

/**
  * @brief  Initializes the SNDA peripheral according to the specified
  *         parameters in the seSNDA_InitStruct.
  * @note   This function configures the module, and module's interrupts.
  *         It clears module's interrupts
  *         but does not enable interrupt from the module to CPU. This function
  *         enables module by start supplying operating clock.
  * @param  SNDAx: SNDA peripheral.
  * @param  InitStruct: pointer to a seSNDA_InitTypeDef structure that
  *         contains the configuration information for the specified SNDA peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seSNDA_Init( SNDA_Type* SNDAx, seSNDA_InitTypeDef* InitStruct ) {

  seStatus fStatus = seCLG_IsEnabled(seSRC(InitStruct->Clk));

  ///< Check if selected clock is enabled
  if ( fStatus == seSTATUS_OK ) {
  
    //fStatus = seAUX_ConfigurePortsForSNDA( SNDAx, &InitStruct->ports );

    if (fStatus == seSTATUS_OK) {
      ///< Stop supplying operating clock
      seSNDA_Disable(SNDAx);
      
      ///< Configure the SNDA operating clock.
      seSNDA_ConfigureClock( SNDAx, InitStruct->Clk );
      
      ///< Enable count operation clock; Start supplying operating clock.
      seSNDA_Enable(SNDAx); 
      
      ///< Set output pin drive mode
      SNDAx->SEL_b.SINV = InitStruct->DriveMode;
      
      ///< Clear interrupt flag.
      seSNDA_ClearIntFlag(SNDAx);
      
      ///< Enable interrupt.
      seSNDA_EnableInt( SNDAx, InitStruct->EnableInt );
    }
  }

  return fStatus;
}

/**
  * @brief  Enables SNDA by start supplying operating clock.
  * @param  SNDAx: SNDA peripheral.
  * @retval None
  */
void seSNDA_Enable( SNDA_Type* SNDAx ) {

  SNDAx->CTL_b.MODEN = 1;
}

/**
  * @brief  Disables SNDA by stop supplying operating clock.
  * @param  SNDAx: SNDA peripheral.
  * @retval None
  */
void seSNDA_Disable( SNDA_Type* SNDAx ) {
  
  SNDAx->CTL_b.MODEN = 0;
}

/**
  * @brief  Stops SNDA.
  * @param  SNDAx: SNDA peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seSNDA_Stop(  SNDA_Type* SNDAx  ) {

  seStatus fStatus = seSTATUS_OK;
  ///< Stop buzzer output
  SNDAx->CTL_b.SSTP = 1;  
  ///< Wait for ouput has stopped
  WHILE( SNDAx->CTL & SNDA_CTL_SSTP_Msk, fStatus );
  
  ///< Clear interrupt flag.
  SNDAx->INTF = 1;

  return fStatus;
}

/**
  * @brief  Starts SNDA in normal mode.
  * @param  SNDAx: SNDA peripheral.
  * @param  frequency: This parameter defines frequency.
  * @param  duty_ratio: This parameter defines duty ratio.
  * @retval None
  */
void seSNDA_Start( SNDA_Type* SNDAx, uint16_t frequency, uint16_t duty_ratio )  {
  
  if ( SNDAx->CTL_b.MODEN == 1 ) {
    if ( SNDAx->INTF_b.SBSY == 0 ) {
      ///< Wait for SNDA becomes available
      while( SNDAx->INTF_b.SBSY ) {
        __NOP();		/// wait ...
      }       
      ///< Set Buzzer Normal Mode
      SNDAx->SEL_b.MOSEL = 0;
      ///< Set Buzzer Output
      SNDAx->DAT_b.SLEN = duty_ratio;   ///< Set buzzer output signal duty ratio
      SNDAx->DAT_b.SFRQ = frequency;    ///< Set buzzer output signal frequency
    }
  }
}

/**
  * @brief  Starts SNDA in one-shot mode.
  * @param  SNDAx: SNDA peripheral.
  * @param  frequency: This parameter defines frequency.
  * @param  duty_ratio: This parameter defines duty ratio.
  * @param  duration: This parameter defines duty duration.
  * @retval None
  */
void seSNDA_StartOneShot( SNDA_Type* SNDAx, uint16_t frequency, uint16_t duty_ratio, uint16_t duration )  {
  
  if ( SNDAx->CTL_b.MODEN == 1 ) {  
    if ( SNDAx->INTF_b.SBSY == 0 ) {
      ///< Wait for SNDA becomes available
      while( SNDAx->INTF_b.SBSY ) {
        __NOP();    /// wait ...
      }       
      ///< One-shot buzzer mode
      SNDAx->SEL_b.MOSEL = 1;          ///< Set Buzzer one-shot mode
      SNDAx->SEL_b.STIM = duration;    ///< Set output duration
      ///< Start buzzer output
      SNDAx->DAT_b.SLEN = duty_ratio;  ///< Set buzzer output signal duty ratio
      SNDAx->DAT_b.SFRQ = frequency;   ///< Set buzzer output signal frequency
    }
  }
}

/**
  * @brief  Starts SNDA in melody mode.
  * @param  SNDAx: SNDA peripheral.
  * @param  data: This parameter defines data of music.
  * @param  size: This parameter defines size of music data by number of words.
  * @param  tempo: This parameter defines tempo of playing.
  * @retval None
  */
void seSNDA_StartMelody( SNDA_Type* SNDAx, const uint16_t * data, uint32_t size, uint16_t tempo )  {
 
  if ( SNDAx->CTL_b.MODEN == 1 ) {
    if ( SNDAx->INTF_b.SBSY == 0 ) {
      uint32_t i; 
      ///< Wait for SNDA becomes available
      while( SNDAx->INTF_b.SBSY ) {
        __NOP();    /// wait ...
      }      
      ///< Melody buzzer mode
      SNDAx->SEL_b.MOSEL = 2;      ///< Set melody mode
      SNDAx->SEL_b.STIM = tempo;   ///< Set tempo

      for ( i = 0; i < size; i++ ) {
        
        SNDAx->DAT = data[i];
        while ( SNDAx->INTF_b.EMIF == 0 ) {
          __NOP();    /// wait ...
        }
      }
    }
  }
}

/**
  * @brief  Configures clock source and clock divider.
  * @param  SNDAx: SNDA peripheral.
  * @param  clock: This parameter can be a value of @ref seSNDA_Clk.
  * @retval None
  */
void seSNDA_ConfigureClock( SNDA_Type* SNDAx, seSNDA_Clk clock ) {
  if ( SNDAx->CTL_b.MODEN == 0 ) {
    ///< Configure the SNDA operating clock.
    SNDAx->CLK_b.CLKSRC = seSRC(clock);   ///< Clock source selection
    SNDAx->CLK_b.CLKDIV = seDIV(clock);   ///< Clock division ratio selection
    SNDAx->CLK_b.DBRUN  = seDBRUN;        ///< Whether clock supplied in DEBUG mode or not.
  }
}

/**
  * @brief  Enables SNDA interrupt.
  * @param  SNDAx: SNDA peripheral.
  * @param  irq: Interrupt to enable.
  * @retval None
  */
void seSNDA_EnableInt( SNDA_Type* SNDAx, uint16_t irq ) {
  
  SNDAx->INTE |= irq;   ///< Enable interrupt.
}

/**
  * @brief  Disables SNDA interrupt.
  * @param  SNDAx: SNDA peripheral.
  * @param  irq: Interrupt to disable.
  * @retval None
  */
void seSNDA_DisableInt( SNDA_Type* SNDAx, uint16_t irq ) {
  
  SNDAx->INTE &= ~irq;  ///< Disable interrupt.
}

/**
  * @brief  Returns SNDA interrupt status.
  * @param  SNDAx: SNDA peripheral.
  * @param  irq: Interrupt to check.
  * @retval InterruptStatus: see @ref seInterruptStatus
  */
seInterruptStatus seSNDA_GetIntFlag( SNDA_Type* SNDAx, uint16_t irq ) {
  
  seInterruptStatus fStatus = seINTERRUPT_NOT_OCCURRED;
  uint16_t flagged = SNDAx->INTF;
  
  if ( flagged & irq ) {
    fStatus = seINTERRUPT_OCCURRED;
  }

  return fStatus;
}

/**
  * @brief  Clears SNDA interrupt.
  * @param  SNDAx: SNDA peripheral.
  * @retval None
  */
void seSNDA_ClearIntFlag( SNDA_Type* SNDAx ) {
  
  ///< Clear interrupt flag.
  SNDAx->INTF = SNDA_INTF_EDIF_Msk;
}

