/**
  ******************************************************************************
  * @file    se_snda.h
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file contains all the functions prototypes for the SNDA firmware
  *          library.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#ifndef SE_SNDA_H
#define SE_SNDA_H

#include "se_common.h"

#ifdef __cplusplus
 extern "C" {
#endif

/** @addtogroup sePeriphLibrary
  * @{
  */

/** @defgroup SNDA
  * @{
  * @brief SNDA is a sound generator that generates melodies and buzzer signals.
  */

/** @defgroup SNDA_Constants 
  * @{
  */

/**
 * \brief Brief description.
 */

typedef enum {
   seSNDA_SingleDrive = 1,               ///< Single Pin Drive
   seSNDA_DirectDrive = 0                ///< Direct Drive
} seSNDA_DriveMode;

typedef enum {
   seSNDA_NormalBuzzer = 0,              ///< Normal buzzer mode
   seSNDA_OneShotBuzzer = 1,             ///< One-shot buzzer mode
   seSNDA_Melody = 2                     ///< Melody mode
} seSNDA_ModeSel;

/**
  * @}
  */

/** @defgroup SNDA_Types
  * @{ 
  */

/** 
  * @brief  SNDA Init structure definition 
  */

typedef struct {
   sePPORT_Id BZOUT;            ///< BZOUT pin port definition
   sePPORT_Id NBZOUT;           ///< #BZOUT pin port definition
} seSNDA_Ports;

typedef struct {
   seSNDA_Clk Clk;
   seSNDA_DriveMode DriveMode;  ///< Specifies Pin Drive Mode
   uint16_t EnableInt;          ///< Specifies Interrupts to be enabled
   seSNDA_Ports ports;
} seSNDA_InitTypeDef;

/**
  * @}
  */ 

/** @defgroup SNDA_Functions
  * @{
  */

/**
  * @brief  Fills each seSNDA_InitTypeDef member with its default value.
  * @param  InitStruct: pointer to an seSNDA_InitTypeDef structure which will be initialized.
  * @retval None
  */
void seSNDA_InitStruct( seSNDA_InitTypeDef* InitStruct );

/**
  * @brief  Initializes the SNDA peripheral according to the specified
  *         parameters in the seSNDA_InitStruct.
  * @note   This function configures the module, and module's interrupts.
  *         It clears module's interrupts
  *         but does not enable interrupt from the module to CPU. This function
  *         enables module by start supplying operating clock.
  * @param  SNDAx: SNDA peripheral.
  * @param  InitStruct: pointer to a seSNDA_InitTypeDef structure that
  *         contains the configuration information for the specified SNDA peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seSNDA_Init( SNDA_Type* SNDAx, seSNDA_InitTypeDef* InitStruct );

/**
  * @brief  Starts SNDA in normal mode.
  * @param  SNDAx: SNDA peripheral.
  * @param  frequency: This parameter defines frequency.
  * @param  duty_ratio: This parameter defines duty ratio.
  * @retval None
  */
void seSNDA_Start( SNDA_Type* SNDAx, uint16_t frequency, uint16_t duty_ratio );

/**
  * @brief  Starts SNDA in one-shot mode.
  * @param  SNDAx: SNDA peripheral.
  * @param  frequency: This parameter defines frequency.
  * @param  duty_ratio: This parameter defines duty ratio.
  * @param  duration: This parameter defines duty duration.
  * @retval None
  */
void seSNDA_StartOneShot( SNDA_Type* SNDAx, uint16_t frequency, uint16_t duty_ratio, uint16_t duration );

/**
  * @brief  Starts SNDA in melody mode.
  * @param  SNDAx: SNDA peripheral.
  * @param  data: This parameter defines data of music.
  * @param  size: This parameter defines size of music data by number of words.
  * @param  tempo: This parameter defines tempo of playing.
  * @retval None
  */
void seSNDA_StartMelody( SNDA_Type* SNDAx, const uint16_t * data, uint32_t size, uint16_t tempo );

/**
  * @brief  Stops SNDA.
  * @param  SNDAx: SNDA peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seSNDA_Stop( SNDA_Type* SNDAx );

/**
  * @brief  Enables SNDA by start supplying operating clock.
  * @param  SNDAx: SNDA peripheral.
  * @retval None
  */
void seSNDA_Enable( SNDA_Type* SNDAx );

/**
  * @brief  Disables SNDA by stop supplying operating clock.
  * @param  SNDAx: SNDA peripheral.
  * @retval None
  */
void seSNDA_Disable( SNDA_Type* SNDAx );

/**
  * @brief  Configures clock source and clock divider.
  * @param  SNDAx: SNDA peripheral.
  * @param  clock: This parameter can be a value of @ref seSNDA_Clk.
  * @retval None
  */
void seSNDA_ConfigureClock( SNDA_Type* SNDAx, seSNDA_Clk clock);

/**
  * @brief  Enables SNDA interrupt.
  * @param  SNDAx: SNDA peripheral.
  * @param  irq: Interrupt to enable.
  * @retval None
  */
void seSNDA_EnableInt( SNDA_Type* SNDAx, uint16_t irq );

/**
  * @brief  Disables SNDA interrupt.
  * @param  SNDAx: SNDA peripheral.
  * @param  irq: Interrupt to disable.
  * @retval None
  */
void seSNDA_DisableInt( SNDA_Type* SNDAx, uint16_t irq );

/**
  * @brief  Returns SNDA interrupt status.
  * @param  SNDAx: SNDA peripheral.
  * @param  irq: Interrupt to check.
  * @retval InterruptStatus: see @ref seInterruptStatus
  */
seInterruptStatus seSNDA_GetIntFlag( SNDA_Type* SNDAx, uint16_t irq );

/**
  * @brief  Clears SNDA interrupt.
  * @param  SNDAx: SNDA peripheral.
  * @retval None
  */
void seSNDA_ClearIntFlag( SNDA_Type* SNDAx );

/**
  * @brief  Discovers SNDA clock from registers.
  * @param  SNDAx: SNDA peripheral.
  * @retval SNDA clock.
  */
uint32_t seAUX_GetClkSpeedForSNDA( SNDA_Type* SNDAx );

/**
  * @brief  Configures ports for this module.
  * @param  SNDAx:  SNDA peripheral.
  * @param  ports:  SNDA ports to configure.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seAUX_ConfigurePortsForSNDA( SNDA_Type* SNDAx, seSNDA_Ports * ports );

/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */

#ifdef __cplusplus
}
#endif
#endif // SE_SNDA_H

