/**
  ******************************************************************************
  * @file    se_spia.c
  * @author  Epson
  * @version V2.1
  * @date    25-Nov-2020
  * @brief   This file provides all the SPIA hardware functions.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#include "string.h"
#include "se_spia.h"

/**
  * @brief  Initializes the SPIx peripheral according to the specified
  *         parameters in the SPIA_InitStruct.
  * @param  SPIx: SPI peripheral
  * @param  SPIA_InitStruct: pointer to a @ref seSPIA_InitTypeDef structure that
  *         contains the configuration information for the specified SPI peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seSPIA_Init(SPIA_0_Type* SPIx, seSPIA_InitTypeDef* SPIA_InitStruct) {

  seStatus fStatus = seSTATUS_OK;

  ///< Stop SPIA.
  seSPIA_Stop( SPIx );

  ///< Select Mode.
  SPIx->MOD_b.MST = SPIA_InitStruct->MST;

  ///< Set SPIA ch.x
  SPIx->MOD_b.CPOL     = SPIA_InitStruct->CPOL;        ///< Polarity  : Active Low.
  SPIx->MOD_b.CPHA     = SPIA_InitStruct->CPHA;        ///< Phase     : 0.
  SPIx->MOD_b.LSBFST   = SPIA_InitStruct->LSBFST;      ///< Format    : MSB First.
  SPIx->MOD_b.NOCLKDIV = SPIA_InitStruct->NOCLKDIV;    ///< SPICLK0   : 16-bit timer operating clock frequency.
  SPIx->MOD_b.PUEN     = SPIA_InitStruct->PUEN;        ///< pull-up   : Disable pull-up/down.
  SPIx->MOD_b.CHLN     = SPIA_InitStruct->CHLN;        ///< length    : 8 bit.
    

  return fStatus;
}

/**
  * @brief  Initializes the SPIA_InitStruct as Master.
  * @param  SPIA_InitStruct: pointer to a seSPIA_InitTypeDef structure that
  *         contains the configuration information for the specified SPI peripheral.
  * @retval None
  */
void seSPIA_InitStructForMaster(seSPIA_InitTypeDef* SPIA_InitStruct) { 
  
  ///< Set Spi variables
  SPIA_InitStruct->CHLN     = seDATA_TR_LENGTH_08BIT;   ///< Set the bit length of transfer data.
  SPIA_InitStruct->CPHA     = seSPIA_PH_RISE;           ///< Set clock phase.
  SPIA_InitStruct->CPOL     = seSPIA_POL_LOW;           ///< Set clock polarity.
  SPIA_InitStruct->LSBFST   = seSPIA_MSB_FST;           ///< Set MSB first/LSB first.
  SPIA_InitStruct->MST      = seSPIA_MODE_MASTER;       ///< Set master/slave mode.
  SPIA_InitStruct->NOCLKDIV = seDISABLE;                ///< Set master mode operating clock.
  SPIA_InitStruct->PUEN     = seENABLE;                 ///< Enable/Disable pull-up/down.
}

/**
  * @brief  Initializes the SPIA_InitStruct as Slave.
  * @param  SPIA_InitStruct: pointer to a @ref seSPIA_InitTypeDef structure that
  *         contains the configuration information for the specified SPI peripheral.
  * @retval None
  */
void seSPIA_InitStructForSlave(seSPIA_InitTypeDef* SPIA_InitStruct) {
  
  SPIA_InitStruct->CHLN     = seDATA_TR_LENGTH_08BIT;  ///< Set the bit length of transfer data.
  SPIA_InitStruct->CPHA     = seSPIA_PH_RISE;          ///< Set clock phase.
  SPIA_InitStruct->CPOL     = seSPIA_POL_LOW;          ///< Set clock polarity.
  SPIA_InitStruct->LSBFST   = seSPIA_MSB_FST;          ///< Set MSB first/LSB first.
  SPIA_InitStruct->MST      = seSPIA_MODE_SLAVE;       ///< Set master/slave mode.
  SPIA_InitStruct->PUEN     = seENABLE;                ///< Enable/Disable pull-up/down.
}

/**
  * @brief  Starts or disables the specified SPI channel.
  * @param  SPIx: SPI peripheral
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seSPIA_Start( SPIA_0_Type* SPIx ) { 

  seStatus fStatus = seSTATUS_OK;

  ///< Start SPI
  ///< Enable T16(Master mode only).
  if ( SPIx->MOD_b.MST == 1 ) { 
    fStatus = seAUX_StartBusClockForSPI( SPIx );
  }

  if ( fStatus == seSTATUS_OK ) {
    ///< Execute software reset.
    fStatus = seSPIA_Reset( SPIx );

    ///< Enable SPI
    SPIx->CTL_b.MODEN = 1;
  }

  return fStatus;
}

/**
  * @brief  Stops or disables the specified SPI channel.
  * @param  SPIx: SPI peripheral
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seSPIA_Stop( SPIA_0_Type* SPIx ) {
  
  seStatus fStatus = seSTATUS_OK;
  ///< Stop SPI 
  ///< Disable SPIA.
  SPIx->CTL_b.MODEN = 0;

  ///< Disable T16(Master mode only).
  if ( SPIx->MOD_b.MST == 1 ) {
    fStatus = seAUX_StopBusClockForSPI( SPIx );
  }
  
  return fStatus;
}

/**
  * @brief  Software reset of the specified SPI channel.
  * @param  SPIx: SPI peripheral
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seSPIA_Reset( SPIA_0_Type* SPIx ) { 

  seStatus fStatus = seSTATUS_OK;
  ///< Execute software reset.
  SPIx->CTL_b.SFTRST = 1; 
  WHILE( SPIx->CTL & SPIA_0_CTL_SFTRST_Msk, fStatus );

  return fStatus;
}

/**
  * @brief  Sends a data through the SPIx peripheral.
  * @param  SPIx: SPI peripheral
  * @param  data: Data to be transmitted.
  * @param  size: Data size in number of bytes.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seSPIA_TxBytes( SPIA_0_Type* SPIx, uint8_t data[], uint32_t size ) {

  seStatus fStatus = seSTATUS_OK;
  
  WHILE( (SPIx->INTF & SPIA_0_INTF_TBEIF_Msk) == 0, fStatus );
  SANITY_CHECK( fStatus );

  ///< Send data.
  while ( size-- ) {
    SPIx->TXD = *data++;
    ///< Wait for transmit buffer empty.
    WHILE( (SPIx->INTF & SPIA_0_INTF_TBEIF_Msk) == 0, fStatus );
    SANITY_CHECK( fStatus );
  }

  ///< Wait for idle state.
  WHILE( SPIx->INTF & SPIA_0_INTF_BSY_Msk, fStatus );

  return fStatus;
}

/**
  * @brief  Sends a data through the SPIx peripheral.
  * @param  SPIx: SPI peripheral
  * @param  data: Data to be transmitted.
  * @param  size: Data size in number of words.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seSPIA_TxHWords( SPIA_0_Type* SPIx, uint16_t data[], uint32_t size ) {

  seStatus fStatus = seSTATUS_OK;
  
  WHILE( (SPIx->INTF & SPIA_0_INTF_TBEIF_Msk) == 0, fStatus );
  SANITY_CHECK( fStatus );
  
  while( size-- ) {
    ///< Send data.
    SPIx->TXD = *data++;
    ///< Wait for transmit buffer empty.
    WHILE( (SPIx->INTF & SPIA_0_INTF_TBEIF_Msk) == 0, fStatus );
    SANITY_CHECK( fStatus );
  }
 
  ///< Wait for idle state.
  WHILE( SPIx->INTF & SPIA_0_INTF_BSY_Msk, fStatus );

  return fStatus;
}

/**
  * @brief  Returns the received data by the SPIx peripheral.
  * @param  SPIx: SPI peripheral
  * @param  data: The received data.
  * @param  size: Data size in number of bytes. 
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seSPIA_RxBytes( SPIA_0_Type* SPIx, uint8_t data[], uint32_t size ) {
  
  uint8_t dummy = SPIx->RXD; ///< Clear RBFIF flag
  seStatus fStatus = seSTATUS_OK;

  if ( SPIx->MOD_b.MST == 1 ) {
    ///< Set transmit dummy data(Master mode only).
    ///< Check transmit buffer empty.
    WHILE( (SPIx->INTF & SPIA_0_INTF_TBEIF_Msk) == 0, fStatus );
    SANITY_CHECK( fStatus );
    
    ///< Receive data (Master).
    while ( size-- ) {
      ///< Set transmit dummy data(Master mode only).
      SPIx->TXD = dummy;    
      ///< Check receive buffer full.
      WHILE( (SPIx->INTF & SPIA_0_INTF_RBFIF_Msk) == 0, fStatus ); 
      SANITY_CHECK( fStatus );     
      *data++ = SPIx->RXD;
    }    
  } else {
    
    ///< Receive data (Slave).
    while ( size-- ) {
      ///< Check receive buffer full.
      WHILE( (SPIx->INTF & SPIA_0_INTF_RBFIF_Msk) == 0, fStatus ); 
      SANITY_CHECK( fStatus );     
      *data++ = SPIx->RXD;
    }
  }

  ///< wait for idle state.
  WHILE( SPIx->INTF & SPIA_0_INTF_BSY_Msk, fStatus );

  return fStatus;
}

/**
  * @brief  Returns the received data by the SPIx peripheral.
  * @param  SPIx: SPI peripheral
  * @param  data: The received data.
  * @param  size: Data size in number of words. 
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seSPIA_RxHWords( SPIA_0_Type* SPIx, uint16_t data[], uint32_t size ) {
  
  uint8_t dummy = SPIx->RXD; ///< Clear RBFIF flag
  seStatus fStatus = seSTATUS_OK;

  if ( SPIx->MOD_b.MST == 1 ) {
    ///< Set transmit dummy data(Master mode only).
    ///< Check transmit buffer empty.
    WHILE( (SPIx->INTF & SPIA_0_INTF_TBEIF_Msk) == 0, fStatus );
    SANITY_CHECK( fStatus );
    
    ///< Receive data (Master).
    while ( size-- ) {
      ///< Set transmit dummy data(Master mode only).
      SPIx->TXD = dummy;    
      ///< Check receive buffer full.
      WHILE( (SPIx->INTF & SPIA_0_INTF_RBFIF_Msk) == 0, fStatus ); 
      SANITY_CHECK( fStatus );     
      *data++ = SPIx->RXD;
    }    
  } else {
    
    ///< Receive data (Slave).
    while ( size-- ) {
      ///< Check receive buffer full.
      WHILE( (SPIx->INTF & SPIA_0_INTF_RBFIF_Msk) == 0, fStatus ); 
      SANITY_CHECK( fStatus );     
      *data++ = SPIx->RXD;
    }
  }

  ///< wait for idle state.
  WHILE( SPIx->INTF & SPIA_0_INTF_BSY_Msk, fStatus );

  return fStatus;
}

/**
  * @brief  Enables SPI channel interrupt.
  * @param  SPIx: SPI peripheral
  * @param  irq: interrupt to enable.
  * @retval None
  */
void seSPIA_EnableInt( SPIA_0_Type* SPIx, uint16_t irq ) {

  SPIx->INTE |= irq;
}

/**
  * @brief  Disables SPI channel interrupt.
  * @param  SPIx: SPI peripheral
  * @param  irq: interrupt to disable.
  * @retval None
  */
void seSPIA_DisableInt( SPIA_0_Type* SPIx, uint16_t irq ) {

  SPIx->INTE &= ~irq;
}

/**
  * @brief  Gets SPI channel interrupt flag.
  * @param  SPIx: SPI peripheral
  * @param  flag: interrupt to check.
  * @retval seInterruptStatus see @ref seInterruptStatus
  */
seInterruptStatus seSPIA_GetIntFlag( SPIA_0_Type* SPIx, uint16_t flag ) {

  seInterruptStatus fStatus = seINTERRUPT_NOT_OCCURRED;
  uint16_t flagged = SPIx->INTF;
  
  if ( flag ) {
    flagged &= flag;
    if ( flagged == flag ) {
      fStatus = seINTERRUPT_OCCURRED;
    }
  }
  
  return fStatus;
}

/**
  * @brief  Clears SPI channel interrupt flag.
  * @param  SPIx: SPI peripheral
  * @param  flag: interrupt to clear.
  * @retval None
  */
void seSPIA_ClearIntFlag( SPIA_0_Type* SPIx, uint16_t flag ) {
  
  SPIx->INTF = flag;
}

