/**
  ******************************************************************************
  * @file    se_spia.h
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file contains all the functions prototypes for the SPI firmware
  *          library.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#ifndef SE_SPIA_H
#define SE_SPIA_H

#include "se_common.h"
    
#ifdef __cplusplus
 extern "C" {
#endif

/** @addtogroup sePeriphLibrary
  * @{
  */ 

/** @defgroup SPIA
  * @{
  * @brief The SPIA module is a subset of the SPI bus interface
  */

/** @defgroup SPIA_Constants
  * @{
  * @brief The SPIA module exported constants
  */ 

typedef enum {
   seDATA_TR_LENGTH_16BIT = 0xf,          ///< Length of transfer data = 16bit.
   seDATA_TR_LENGTH_15BIT = 0xe,          ///< Length of transfer data = 15bit.
   seDATA_TR_LENGTH_14BIT = 0xd,          ///< Length of transfer data = 14bit.
   seDATA_TR_LENGTH_13BIT = 0xc,          ///< Length of transfer data = 13bit.
   seDATA_TR_LENGTH_12BIT = 0xb,          ///< Length of transfer data = 12bit.
   seDATA_TR_LENGTH_11BIT = 0xa,          ///< Length of transfer data = 11bit.
   seDATA_TR_LENGTH_10BIT = 0x9,          ///< Length of transfer data = 10bit.
   seDATA_TR_LENGTH_09BIT = 0x8,          ///< Length of transfer data =  9bit.
   seDATA_TR_LENGTH_08BIT = 0x7,          ///< Length of transfer data =  8bit.
   seDATA_TR_LENGTH_07BIT = 0x6,          ///< Length of transfer data =  7bit.
   seDATA_TR_LENGTH_06BIT = 0x5,          ///< Length of transfer data =  6bit.
   seDATA_TR_LENGTH_05BIT = 0x4,          ///< Length of transfer data =  5bit.
   seDATA_TR_LENGTH_04BIT = 0x3,          ///< Length of transfer data =  4bit.
   seDATA_TR_LENGTH_03BIT = 0x2,          ///< Length of transfer data =  3bit.
   seDATA_TR_LENGTH_02BIT = 0x1           ///< Length of transfer data =  2bit.
} seSPIA_DataTransferLength;

typedef enum {
   seSPIA_LSB_FST  = 1,        ///< Specify the data format (input/output permutation) LSB first
   seSPIA_MSB_FST  = 0         ///< Specify the data format (input/output permutation) MSB first
} seSPIA_Format;

typedef enum {
   seSPIA_POL_LOW  = 0,        ///< Output is low when clock off.
   seSPIA_POL_HIGH = 1         ///< Output is high when clock off.
} seSPIA_Polarity;

typedef enum {
   seSPIA_PH_RISE = 0,         ///< Triggers on positive edge of clock.
   seSPIA_PH_FALL = 1          ///< Triggers on negative edge of clock.
} seSPIA_Phase;

typedef enum {
   seSPIA_MODE_SLAVE  = 0,     ///< Specifies the SPI mode slave
   seSPIA_MODE_MASTER = 1      ///< Specifies the SPI mode master
} seSPIA_OperMode;

/**
  * @}
  */ 

/** @defgroup SPIA_Types
  * @{
  */

/** 
  * @brief  SPIA Init structure definition
  */

typedef struct {
   sePPORT_Id SPISS;    ///< SPISS# pin port definition
   sePPORT_Id SDI;      ///< SDI pin port definition
   sePPORT_Id SDO;      ///< SDO pin port definition
   sePPORT_Id SPICLK;   ///< SPICLK pin port definition
} seSPI_Ports;

typedef struct { 
   seSPIA_DataTransferLength CHLN; ///< Specifies the data transfer bit length
   seState PUEN;                   ///< Enables/Disables input pin pull-up/down
   seState NOCLKDIV;               ///< Select master mode operating clock
   seSPIA_Format LSBFST;           ///< Select MSB first/LSB first
   seSPIA_Phase CPHA;              ///< Select clock phase
   seSPIA_Polarity CPOL;           ///< Select clock polarity
   seSPIA_OperMode MST;            ///< Select master/slave mode
} seSPIA_InitTypeDef;

/**
  * @}
  */

/** @defgroup SPIA_Functions
  * @{
  */

/**
  * @brief  Initializes the SPIA_InitStruct as Master.
  * @param  SPIA_InitStruct: pointer to a seSPIA_InitTypeDef structure that
  *         contains the configuration information for the specified SPI peripheral.
  * @retval None
  */
void seSPIA_InitStructForMaster( seSPIA_InitTypeDef* SPIA_InitStruct );

/**
  * @brief  Initializes the SPIA_InitStruct as Slave.
  * @param  SPIA_InitStruct: pointer to a @ref seSPIA_InitTypeDef structure that
  *         contains the configuration information for the specified SPI peripheral.
  * @retval None
  */
void seSPIA_InitStructForSlave( seSPIA_InitTypeDef* SPIA_InitStruct );

/**
  * @brief  Initializes the SPIx peripheral according to the specified
  *         parameters in the SPIA_InitStruct.
  * @param  SPIx: SPI peripheral
  * @param  SPIA_InitStruct: pointer to a @ref seSPIA_InitTypeDef structure that
  *         contains the configuration information for the specified SPI peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seSPIA_Init(SPIA_0_Type* SPIx, seSPIA_InitTypeDef* SPIA_InitStruct);

/**
  * @brief  Starts or disables the specified SPI channel.
  * @param  SPIx: SPI peripheral
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seSPIA_Start( SPIA_0_Type* SPIx );

/**
  * @brief  Stops or disables the specified SPI channel.
  * @param  SPIx: SPI peripheral
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seSPIA_Stop( SPIA_0_Type* SPIx );

/**
  * @brief  Software reset of the specified SPI channel.
  * @param  SPIx: SPI peripheral
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seSPIA_Reset( SPIA_0_Type* SPIx );

/**
  * @brief  Sends a data through the SPIx peripheral.
  * @param  SPIx: SPI peripheral
  * @param  data: Data to be transmitted.
  * @param  size: Data size in number of words.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seSPIA_TxHWords( SPIA_0_Type* SPIx, uint16_t data[], uint32_t size );

/**
  * @brief  Sends a data through the SPIx peripheral.
  * @param  SPIx: SPI peripheral
  * @param  data: Data to be transmitted.
  * @param  size: Data size in number of bytes.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seSPIA_TxBytes( SPIA_0_Type* SPIx, uint8_t data[], uint32_t size );

/**
  * @brief  Returns the received data by the SPIx peripheral.
  * @param  SPIx: SPI peripheral
  * @param  data: The received data.
  * @param  size: Data size in number of words. 
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seSPIA_RxHWords( SPIA_0_Type* SPIx, uint16_t data[], uint32_t size );

/**
  * @brief  Returns the received data by the SPIx peripheral.
  * @param  SPIx: SPI peripheral
  * @param  data: The received data.
  * @param  size: Data size in number of bytes. 
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seSPIA_RxBytes( SPIA_0_Type* SPIx, uint8_t data[], uint32_t size );

/**
  * @brief  Enables SPI channel interrupt.
  * @param  SPIx: SPI peripheral
  * @param  irq: interrupt to enable.
  * @retval None
  */
void seSPIA_EnableInt( SPIA_0_Type* SPIx, uint16_t irq );

/**
  * @brief  Disables SPI channel interrupt.
  * @param  SPIx: SPI peripheral
  * @param  irq: interrupt to disable.
  * @retval None
  */
void seSPIA_DisableInt( SPIA_0_Type* SPIx, uint16_t irq );

/**
  * @brief  Gets SPI channel interrupt flag.
  * @param  SPIx: SPI peripheral
  * @param  flag: interrupt to check.
  * @retval seInterruptStatus see @ref seInterruptStatus
  */
seInterruptStatus seSPIA_GetIntFlag( SPIA_0_Type* SPIx, uint16_t flag );

/**
  * @brief  Clears SPI channel interrupt flag.
  * @param  SPIx: SPI peripheral
  * @param  flag: interrupt to clear.
  * @retval None
  */
void seSPIA_ClearIntFlag( SPIA_0_Type* SPIx, uint16_t flag );

/**
  * @brief  Configures ports for this module.
  * @param  SPIx: SPI peripheral
  * @param  ports: ports to configure
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seAUX_ConfigurePortsForSPI( SPIA_0_Type* SPIx, seSPI_Ports * ports );

/**
  * @brief  Configures SPICLKn frequency [Hz] (= baud rate [bps])
  * @param  SPIx: SPI peripheral
  * @param  speed: SPICLKn frequency [Hz]
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seAUX_SetBusSpeedForSPI( SPIA_0_Type* SPIx, uint32_t speed );

/**
  * @brief  Discovers SPICLKn frequency [Hz] (= baud rate [bps])
  * @param  SPIx: SPI peripheral
  * @retval SPICLKn: frequency [Hz] 
  */
uint32_t seAUX_GetBusSpeedForSPI( SPIA_0_Type* SPIx );

/**
  * @brief  Starts bus clock for SPI
  * @param  SPIx: SPI peripheral
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seAUX_StartBusClockForSPI( SPIA_0_Type* SPIx );

/**
  * @brief  Stops bus clock for SPI
  * @param  SPIx: SPI peripheral
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seAUX_StopBusClockForSPI( SPIA_0_Type* SPIx );

/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */

#ifdef __cplusplus
}
#endif
#endif // SE_SPIA_H

