/**
  ******************************************************************************
  * @file    se_src.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file provides all the functions for the System Reset Controller hardware functions.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#include "se_src.h"

/**
  * @brief  Enables the pull-up of the #RESET pin.
  * @param  None
  * @retval None
  */
void seSRC_EnablePullUp( void ) {
  
  ///< Disable write-protect.
  seProtectSys( seWRITE_PROTECT_OFF );

  SRC->SRCRESETPCTL_b.PORT_PLUP_EN = 0x1;

  ///< Enable write-protect.
  seProtectSys( seWRITE_PROTECT_ON );
}

/**
  * @brief  Disables the pull-up of the #RESET pin.
  * @param  None
  * @retval None
  */
void seSRC_DisablePullUp( void ) {
  
  ///< Disable write-protect.
  seProtectSys( seWRITE_PROTECT_OFF );
  
  SRC->SRCRESETPCTL_b.PORT_PLUP_EN = 0x0;
  
  ///< Enable write-protect.
  seProtectSys( seWRITE_PROTECT_ON );
}

/**
  * @brief  Enables the #RESET pin to input the external reset signal.
  * @param  None
  * @retval None
  */
void seSRC_EnableReset( void ) {
  
  ///< Disable write-protect.
  seProtectSys( seWRITE_PROTECT_OFF );
  
  SRC->SRCRESETPCTL_b.PORT_RESET_EN = 0x1;
  
  ///< Enable write-protect.
  seProtectSys( seWRITE_PROTECT_ON );
}

/**
  * @brief  Disables the #RESET pin to input the external reset signal.
  * @param  None
  * @retval None
  */
void seSRC_DisableReset( void ) {
  
  ///< Disable write-protect.
  seProtectSys( seWRITE_PROTECT_OFF );
  
  SRC->SRCRESETPCTL_b.PORT_RESET_EN = 0x0;
  
  ///< Enable write-protect.
  seProtectSys( seWRITE_PROTECT_ON );
}

/**
  * @brief  Clear the reset request statuses.
  * @param  requestflag: @ref seSRC_ResetRequestFlag. Clear the reset request statuses function.
  * @retval None
  */
void seSRC_ClearRequestFlag( seSRC_ResetRequestFlag requestflag ) {

  SRC->SRCRESETREQ = requestflag;
}

/**
  * @brief  Get the reset request statuses.
  * @param  None
  * @retval vale: reset request statuses.
  */
uint16_t seSRC_GetRequestFlag( void ) {
  
  return SRC->SRCRESETREQ;
}

