/**
  ******************************************************************************
  * @file    se_src.h
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file contains all the functions prototypes for the System Reset Controller firmware
  *          library.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#ifndef SE_SRC_H
#define SE_SRC_H

#ifdef __cplusplus
 extern "C" {
#endif
 
#include "se_common.h"

/** @addtogroup sePeriphLibrary
  * @{
  */

/** @defgroup SRC
  * @{
  * @brief SRC is the system reset controller that resets the internal circuits according to the requests from the reset sources to
  *        archive steady IC operations. The main features of SRC are outlined below.
  */

/** @defgroup SRC_ResetRequestFlag
  * @{
  */

typedef enum {
   seSRC_KEYRSTREQ = 0x1,            ///< Key-entry reset.
   seSRC_SVDRSTREQ = 0x2,            ///< Supply voltage detector reset.
   seSRC_WDTRSTREQ = 0x4,            ///< Watchdog timer reset.
   seSRC_XRESETREQ = 0x8,            ///< #RESET pin.
   seSRC_PORBORREQ = 0x10            ///< POR and BOR.
} seSRC_ResetRequestFlag;

/**
  * @}
  */

/** @defgroup SRC_Types
  * @{ 
  */

/**
  * @}
  */

/** @defgroup SRC_Functions
  * @{
  */

/**
  * @brief  Enables the pull-up of the #RESET pin.
  * @param  None
  * @retval None
  */
void seSRC_EnablePullUp( void );

/**
  * @brief  Disables the pull-up of the #RESET pin.
  * @param  None
  * @retval None
  */
void seSRC_DisablePullUp( void );

/**
  * @brief  Enables the #RESET pin to input the external reset signal.
  * @param  None
  * @retval None
  */
void seSRC_EnableReset( void );

/**
  * @brief  Disables the #RESET pin to input the external reset signal.
  * @param  None
  * @retval None
  */
void seSRC_DisableReset( void ) ;

/**
  * @brief  Clear the reset request statuses.
  * @param  requestflag: @ref seSRC_ResetRequestFlag. Clear the reset request statuses function.
  * @retval None
  */
void seSRC_ClearRequestFlag( seSRC_ResetRequestFlag requestflag ) ;

/**
  * @brief  Get the reset request statuses.
  * @param  None
  * @retval vale: reset request statuses.
  */
uint16_t seSRC_GetRequestFlag( void ) ;

/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */

#ifdef __cplusplus
}
#endif

#endif  //SE_SRC_H

