/**
  ******************************************************************************
  * @file    se_svd4.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file provides all the SVD4 hardware functions.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#include "string.h"
#include "se_svd4.h"

/**
  * @brief  Fills each seSVD_InitStruct member with its default value.
  * @param  InitStruct: pointer to an seSVD_InitTypeDef structure which will be initialized.
  * @retval None
  */
void seSVD_InitStruct(seSVD_InitTypeDef* InitStruct) {
  
  ///< Configure parameters
  InitStruct->Clk                   = seDEFAULT_SVD_CLK;      ///< Set the clock source of SVD4
  InitStruct->VDSEL                 = seSVD_EXSVD;            ///< Set detection voltage(0:VDD / 1:EXSVD).
  InitStruct->EXSEL                 = seSVD_EXSVD_1;          ///< Set the voltage to be detected(EXSVDn1)
  InitStruct->ResetEnable           = seDISABLE;              ///< Set disable the reset issuance function.
  InitStruct->IntermittentMode      = seSVD_Continious;       ///< Set continuous operation mode.
  InitStruct->CompareVoltage        = 1;                      ///< Set comparison voltage for detecting low voltage.
  InitStruct->SamplingResCnt        = seSVD_CTL_4_TIMES_ROW;  ///< Set the condition to generate an interrupt/reset.
}

/**
  * @brief  Initializes the SVDx peripheral according to the specified
  *         parameters in the InitStruct.
  * @param  SVDCHx: SVD channel definition of type @ref seSVD_ChannelDef
  * @param  InitStruct: pointer to a seSVD_InitTypeDef structure that
  *         contains the configuration information for the specified SVD peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seSVD_Init( SVD4_Type* SVDx, seSVD_InitTypeDef* InitStruct ) {
  
  ///< Check if selected clock is enabled
  seStatus fStatus = seCLG_IsEnabled( seSRC(InitStruct->Clk) );
  
  ///< When Clk not started, do nothing.
  if ( fStatus == seSTATUS_OK ) {

    ///< Stop SVD.
    seSVD_Stop( SVDx );

    //if ( InitStruct->VDSEL == seSVD_EXSVD ) {
    //  fStatus = seAUX_ConfigurePortsForSVD( SVDx, (InitStruct->EXSEL==seSVD_EXSVD_1) ? sePPORT_SVD_EXSVD1 : sePPORT_SVD_EXSVD0 );
    //}
    
    if ( fStatus == seSTATUS_OK ) {
      ///< Disable write-protect.
      seProtectSys( seWRITE_PROTECT_OFF );

      ///< Set clock source for SVD.
      SVDx->CLK_b.CLKSRC      = seSRC(InitStruct->Clk);    ///< Clock source selection
      SVDx->CLK_b.CLKDIV      = seDIV(InitStruct->Clk);    ///< Clock division ratio selection
      SVDx->CLK_b.DBRUN       = seDBRUN;                   ///< Whether clock supplied in DEBUG mode or not.


      ///< Select detection voltage(0:VDD / 1:EXSVD).
      SVDx->CTL_b.VDSEL       = InitStruct->VDSEL;

      ///< Select reset(0xA)/interrupt(others) mode.
      SVDx->CTL_b.SVDRE       = InitStruct->ResetEnable;

      ///< Set intermittent operation mode.
      ///< Intermittent operation mode(0x1-0x3) / Continuous operation mode(0x0).
      SVDx->CTL_b.SVDMD       = InitStruct->IntermittentMode;

      ///< Comparison voltage for detecting low voltage.
      SVDx->CTL_b.SVDC        = InitStruct->CompareVoltage;

      ///< The condition to generate an interrupt/reset.
      ///< number of successive low voltage.
      SVDx->CTL_b.SVDSC       = InitStruct->SamplingResCnt;
    
      ///< Set EXSEL detect voltage source
      SVDx->CTL_b.EXSEL        = InitStruct->EXSEL;

      ///< Enable write-protect.
      seProtectSys( seWRITE_PROTECT_ON );
    }

  }

  return fStatus;
}

/**
  * @brief  Starts the specified SVD peripherial.
  * @param  SVDx: SVD peripheral
  * @retval None
  */
void seSVD_Start( SVD4_Type* SVDx ) {

  ///< Disable write-protect.
  seProtectSys( seWRITE_PROTECT_OFF );

  ///< Clear interrupt flag.
  SVDx->INTF_b.SVDIF      = 1;

  ///< Enable interrupt.
  SVDx->INTE_b.SVDIE      = 1;

  ///< Start SVD.
  SVDx->CTL_b.MODEN       = 1;

  ///< Enable write-protect.
  seProtectSys( seWRITE_PROTECT_ON );
}

/**
  * @brief  Stops the SVDx specified peripheral.
  * @param  SVDx: SVD peripheral
  * @retval None
  */
void seSVD_Stop( SVD4_Type* SVDx ) {
  
  ///< Disable write-protect.
  seProtectSys( seWRITE_PROTECT_OFF );

  ///< Stop SVD.
  SVDx->CTL_b.MODEN       = 0;

  ///< Disable interrupt.
  SVDx->INTE_b.SVDIE      = 0;

  ///< Clear interrupt flag.
  SVDx->INTF_b.SVDIF      = 1;

  ///< Enable write-protect.
  seProtectSys( seWRITE_PROTECT_ON );
}

/**
  * @brief  Set Comparison voltage for detecting low voltage
  * @param  SVDx: SVD peripheral
  * @param  volt: Comparison voltage for detecting low voltage.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seSVD_SetComparisonVoltage( SVD4_Type* SVDx, uint8_t volt ) {
  
  seStatus fStatus = seSTATUS_NG;

  if ( SVDx->CTL_b.MODEN == 0 ) {

    ///< Disable write-protect.
    seProtectSys( seWRITE_PROTECT_OFF );

    ///< Comparison voltage for detecting low voltage.
    SVDx->CTL_b.SVDC = volt; 

    ///< Enable write-protect.
    seProtectSys( seWRITE_PROTECT_ON );

    fStatus = seSTATUS_OK;
  }

  return fStatus;
}

/**
  * @brief  Set voltage source for detecting low voltage
  * @param  SVDx: SVD peripheral
  * @param  voltagesource:  Voltage source of type @ref seSVD_VoltageSource
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seSVD_SetVoltageSource( SVD4_Type* SVDx, seSVD_VoltageSource voltagesource ) {
  
  seStatus fStatus = seSTATUS_NG;

  if ( SVDx->CTL_b.MODEN == 0 ) {

    ///< Disable write-protect.
    seProtectSys( seWRITE_PROTECT_OFF );

    ///< Voltage source selection
    SVDx->CTL_b.VDSEL = voltagesource;

    ///< Enable write-protect.
    seProtectSys( seWRITE_PROTECT_ON );

    fStatus = seSTATUS_OK;
  }

  return fStatus;
}

/**
  * @brief  Set external voltage source for detecting low voltage
  * @param  SVDx: SVD peripheral
  * @param  exvoltagesource:  Voltage source of type @ref seSVD_ExVoltageSrc
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seSVD_SetExVoltageSource( SVD4_Type* SVDx, seSVD_ExVoltageSrc exvoltagesource ) {
  
  seStatus fStatus = seSTATUS_NG;

  if ( SVDx->CTL_b.MODEN == 0 ) {

    ///< Disable write-protect.
    seProtectSys( seWRITE_PROTECT_OFF );

    ///< Voltage source selection
    SVDx->CTL_b.EXSEL = exvoltagesource;

    ///< Enable write-protect.
    seProtectSys( seWRITE_PROTECT_ON );

    fStatus = seSTATUS_OK;
  }

  return fStatus;
}

/**
  * @brief  Check SVD low voltage interrupt.
  * @param  SVDx: SVD peripheral
  * @retval Interrupt status: can be a value of @ref seInterruptStatus
  */
seInterruptStatus seSVD_IsIntLowVoltage( SVD4_Type* SVDx ) {
  
  seInterruptStatus fStatus = seINTERRUPT_NOT_OCCURRED;

  ///< Check Low voltage interrupt flag.
  if ( SVDx->INTF_b.SVDDT == 1 ) { 
    fStatus = seINTERRUPT_OCCURRED;
  }

  return fStatus;
}

/**
  * @brief  Clear SVD low voltage interrupt.
  * @param  SVDx: SVD peripheral
  * @retval None
  */
void seSVD_ClearIntLowVoltage( SVD4_Type* SVDx ) {
   
  ///< Clear interrupt flag.
  SVDx->INTF_b.SVDIF = 1;
}

/**
  * @brief  Get power supply voltage detection results.
  * @param  SVDx: SVD peripheral
  * @retval Power supply status: can be a value of @ref seSVD_PowerSupply
  */
seSVD_PowerSupply seSVD_GetVoltageDetection( SVD4_Type* SVDx ) {
  
  ///< Power supply voltage(VDD or EXSVD) >=  comparison voltage.
  seSVD_PowerSupply fStatus = seSVD_POWER_HIGH;

  ///< Check Power supply voltage(VDD or EXSVD) to comparison voltage.
  if ( SVDx->INTF_b.SVDDT == 1 ) {
    fStatus = seSVD_POWER_LOW;
  }

  return fStatus;
}

