/**
  ******************************************************************************
  * @file    se_svd4.h
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file contains all the functions prototypes for the SVD4 firmware
  *          library.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#ifndef SE_SVD4_H
#define SE_SVD4_H

#include "se_common.h"

#ifdef __cplusplus
 extern "C" {
#endif

/** @addtogroup sePeriphLibrary
  * @{
  */

/** @defgroup SVD4
  * @{
  * @brief SVD4 is a supply voltage detector to monitor the power supply voltage on the VDD pin or the voltage applied to an external pin.
  */

/** @defgroup SVD4_Constants
  * @{
  */

typedef enum {
   seSVD_EXSVD = 1,             ///< External SVD
   seSVD_VDD   = 0              ///< VDD
} seSVD_VoltageSource;          ///< Voltage Source select

typedef enum {
   seSVD_EXSVD_0 = 0,           ///< External EXSVD0
   seSVD_EXSVD_1 = 1            ///< External EXSVD1
} seSVD_ExVoltageSrc;           ///< External Voltage Source select

typedef enum {
   seSVD_SVDCLK_512 = 3,        ///< SVDCLK/512(about 16msec)
   seSVD_SVDCLK_256 = 2,        ///< SVDCLK/256(about 8msec)
   seSVD_SVDCLK_128 = 1,        ///< SVDCLK/128(about 4msec)
   seSVD_Continious  = 0        ///< continious
} seSVD_IntermittentMode;       ///< SVD intermittent mode

typedef enum {
   seSVD_CTL_8_TIMES_ROW = 3,   ///< 8 times in a row
   seSVD_CTL_4_TIMES_ROW = 2,   ///< 4 times in a row
   seSVD_CTL_2_TIMES_ROW = 1,   ///< 2 times in a row
   seSVD_CTL_1_TIMES_ROW = 0    ///< 1 times in a row
} seSVD_SamplingResCnt;

typedef enum {
   seSVD_POWER_HIGH  = 0,        ///< Power supply voltage(VDD or EXSVD) >=  comparison voltage.
   seSVD_POWER_LOW = 1           ///< Power supply voltage(VDD or EXSVD) <  comparison voltage.
} seSVD_PowerSupply;             ///< The power supply voltage detection 

/**
  * @}
  */

/** @defgroup SVD_Types
  * @{
  */

/** 
  * @brief  SVD Init structure definition
  */

typedef struct {
   seSVD_Clk              Clk;                  ///< Specifies the svd clock source and divide selection.
   seSVD_VoltageSource    VDSEL;                ///< Voltage source select
   seSVD_ExVoltageSrc     EXSEL;                ///< External Voltage source select
   uint16_t               SVDIE;                ///< SVD interrupt enable
   seState                ResetEnable;          ///< SVD reset enable
   seSVD_IntermittentMode IntermittentMode;     ///< SVD intermittent mode
   uint32_t               CompareVoltage;       ///< SVD compare voltage
   seSVD_SamplingResCnt   SamplingResCnt;       ///< SVD sampiling result count
} seSVD_InitTypeDef;

/**
  * @}
  */

/** @defgroup SVD_Functions
  * @{
  */

/**
  * @brief  Fills each seSVD_InitStruct member with its default value.
  * @param  InitStruct: pointer to an seSVD_InitTypeDef structure which will be initialized.
  * @retval None
  */
void seSVD_InitStruct(seSVD_InitTypeDef* InitStruct);

/**
  * @brief  Initializes the SVDx peripheral according to the specified
  *         parameters in the InitStruct.
  * @param  SVDCHx: SVD channel definition of type @ref seSVD_ChannelDef
  * @param  InitStruct: pointer to a seSVD_InitTypeDef structure that
  *         contains the configuration information for the specified SVD peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seSVD_Init(SVD4_Type* SVDx, seSVD_InitTypeDef* InitStruct);

/**
  * @brief  Starts the specified SVD peripherial.
  * @param  SVDx: SVD peripheral
  * @retval None
  */
void seSVD_Start( SVD4_Type* SVDx );

/**
  * @brief  Stops the SVDx specified peripheral.
  * @param  SVDx: SVD peripheral
  * @retval None
  */
void seSVD_Stop( SVD4_Type* SVDx );

/**
  * @brief  Set Comparison voltage for detecting low voltage
  * @param  SVDx: SVD peripheral
  * @param  volt: Comparison voltage for detecting low voltage.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seSVD_SetComparisonVoltage( SVD4_Type* SVDx, uint8_t volt );

/**
  * @brief  Set voltage source for detecting low voltage
  * @param  SVDx: SVD peripheral
  * @param  voltagesource:  Voltage source of type @ref seSVD_VoltageSource
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seSVD_SetVoltageSource( SVD4_Type* SVDx, seSVD_VoltageSource voltagesource );

/**
  * @brief  Set external voltage source for detecting low voltage
  * @param  SVDx: SVD peripheral
  * @param  exvoltagesource:  Voltage source of type @ref seSVD_ExVoltageSrc
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seSVD_SetExVoltageSource( SVD4_Type* SVDx, seSVD_ExVoltageSrc exvoltagesource );

/**
  * @brief  Get power supply voltage detection results.
  * @param  SVDx: SVD peripheral
  * @retval Power supply status: can be a value of @ref seSVD_PowerSupply
  */
seSVD_PowerSupply seSVD_GetVoltageDetection( SVD4_Type* SVDx );

/**
  * @brief  Check SVD low voltage interrupt.
  * @param  SVDx: SVD peripheral
  * @retval Interrupt status: can be a value of @ref seInterruptStatus
  */
seInterruptStatus seSVD_IsIntLowVoltage( SVD4_Type* SVDx );

/**
  * @brief  Clear SVD low voltage interrupt.
  * @param  SVDx: SVD peripheral
  * @retval None
  */
void seSVD_ClearIntLowVoltage( SVD4_Type* SVDx );

/**
  * @brief  Configures ports for this module. Override this function to configure specific ports.
  * @param  SVDx: SVD peripheral
  * @param  ports:  SVD ports to configure.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seAUX_ConfigurePortsForSVD( SVD4_Type* SVDx, sePPORT_Id port );

/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */

#ifdef __cplusplus
}
#endif
#endif  // SE_SVD4_H

