/**
  ******************************************************************************
  * @file    se_t16.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file provides all the T16 hardware functions.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#include "se_t16.h"

/**
  * @brief  Fills each seT16_InitTypeDef member with its default value.
  * @param  InitStruct: pointer to an @ref seT16_InitTypeDef structure which will be initialized.
  * @retval None
  */
void seT16_InitStruct( seT16_InitTypeDef* InitStruct ) {
   
  InitStruct->Clk          = seDEFAULT_T16_CLK;   ///< Set the clock source of T16.
  InitStruct->CounterMode  = seT16_RepeatMode;    ///< Set the T16 operation mode.
  InitStruct->Period       = 0xE650;              ///< Preset reload data to counter.
}

/**
  * @brief  Initializes the T16 peripheral according to the specified
  *         parameters in the seT16_InitStruct.
  * @note   This function configures the module, and module's interrupts.
  *         It clears module's interrupts
  *         but does not enable interrupt from the module to CPU. This function 
  *         enables module by start supplying operating clock.
  * @param  T16x: pointer to T16 peripheral.
  * @param  InitStruct: pointer to a @ref seT16_InitTypeDef structure that
  *         contains the configuration information for the specified T16 peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seT16_Init( T16_0_Type* T16x, seT16_InitTypeDef* InitStruct ) {
  
  seStatus fStatus = seCLG_IsEnabled(seSRC(InitStruct->Clk));
  
  ///< Check if selected clock is enabled
  if ( fStatus == seSTATUS_OK ) {
    ///< Stop T16 Ch.n Timer.
    seT16_Stop( T16x );
    
    ///< Stop supplying operating clock
    seT16_Disable( T16x );
    
    ///< Configure the T16 Ch.n operating clock.
    seT16_ConfigureClock( T16x, InitStruct->Clk );
    
    ///< Enable count operation clock; Start supplying operating clock.
    seT16_Enable( T16x );

    ///< Select operation mode; Repeat mode or One-shot mode.
    seT16_ConfigureCounterMode( T16x, InitStruct->CounterMode );

    ///< Disable interrupt.
    seT16_DisableInt( T16x );
    
    ///< Clear interrupt flag.
    seT16_ClearIntFlag( T16x );
   
    ///< Preset reload data to counter.
    seT16_SetCounter( T16x, InitStruct->Period ); 
    seT16_Enable( T16x );

  } 
    
  return fStatus;
}

/**
  * @brief  Stops Timer channel.
  * @param  T16x: pointer to T16 peripheral.
  * @retval None
  */
void seT16_Stop( T16_0_Type* T16x )  {

  ///< Stop Timer
  T16x->CTL_b.PRUN = 0;
  ///< Clear interrupt flag.
  T16x->INTF = 1;
}

/**
  * @brief  Starts Timer channel.
  * @param  T16x: pointer to T16 peripheral.
  * @retval None
  */
void seT16_Start( T16_0_Type* T16x )  {
  
  ///< Clear interrupt flag.
  T16x->INTF = 1;
  
  ///< Check if Timer is enabled
  if ( T16x->CTL_b.MODEN == 1 ) {
    ///< Start Timer
    T16x->CTL_b.PRUN = 1;
  }
} 

/**
  * @brief  Enables Timer channel by start supplying operating clock.
  * @param  T16x: pointer to T16 peripheral.
  * @retval None
  */
void seT16_Enable( T16_0_Type* T16x ) {

  T16x->CTL_b.MODEN = 1;
}

/**
  * @brief  Disables Timer channel by stop supplying operating clock.
  * @param  T16x: pointer to T16 peripheral.
  * @retval None
  */
void seT16_Disable( T16_0_Type* T16x ) {
  
  T16x->CTL_b.MODEN = 0;
}

/**
  * @brief  Configures timer clock source and clock divider. 
  * @param  T16x: This parameter defines a timer channel and can be a value of @ref T16_0_Type.
  * @param  clock: This parameter can be a value of @ref seT16_Clk.
  * @retval None
  */
void seT16_ConfigureClock( T16_0_Type* T16x, seT16_Clk clock ) {
  
  ///< Configure the T16 Ch. x operating clock.
  T16x->CLK_b.CLKSRC = seSRC(clock); ///< Clock source selection
  T16x->CLK_b.CLKDIV = seDIV(clock); ///< Clock division ratio selection
  T16x->CLK_b.DBRUN  = seDBRUN;      ///< Whether clock supplied in DEBUG mode or not.
}

/**
  * @brief  Configures timer counter mode.
  * @param  T16x: This parameter defines a timer channel and can be a value of @ref T16_0_Type.
  * @param  mode: This parameter can be a value of @ref seT16_CounterMode.
  * @retval None
  */
void seT16_ConfigureCounterMode( T16_0_Type* T16x, seT16_CounterMode mode ) {
  
  T16x->MOD_b.TRMD = mode;    ///< Set Repeat mode or One-shot mode.
}

/**
  * @brief  Sets Timer counter.
  * @param  T16x: This parameter defines a timer channel and can be a value of @ref T16_0_Type.
  * @param  counter: This parameter can be a 16-bit value.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seT16_SetCounter( T16_0_Type* T16x, uint16_t counter ) {
  
  seStatus fStatus = seSTATUS_OK;
  ///< Set the initial value to be preset to the counter.
  T16x->TR = counter;

  ///< Check if Timer enabled
  if ( T16x->CTL_b.MODEN == 1 ) {
    ///< Presets the reload data stored in the T16_0TR register to the counter.
    T16x->CTL_b.PRESET = 1; 
  }
  
  ///< It is confirmed whether the preset finished.
  WHILE( T16x->CTL & T16_0_CTL_PRESET_Msk, fStatus );
  
  return fStatus;
}

/**
  * @brief  Gets Timer counter value.
  * @param  T16x: This parameter defines a timer channel and can be a value of @ref T16_0_Type.
  * @retval 16-bit counter value.
  */
uint16_t seT16_GetCounter( T16_0_Type* T16x ) {
  
  return ( T16x->TC );
}

/**
  * @brief  Enables Timer channel interrupt.
  * @param  T16x: pointer to T16 peripheral.
  * @retval None
  */
void seT16_EnableInt( T16_0_Type* T16x ) {
  
  T16x->INTE_b.UFIE = 1;   ///< Enable interrupt.
}

/**
  * @brief  Disables Timer channel interrupt.
  * @param  T16x: pointer to T16 peripheral.
  * @retval None
  */
void seT16_DisableInt( T16_0_Type* T16x ) {
  
  T16x->INTE_b.UFIE = 0;   ///< Disable interrupt.
}

/**
  * @brief  Returns Timer interrupt flag.
  * @param  T16x: pointer to T16 peripheral.
  * @retval InterruptStatus: @ref seInterruptStatus
  */
seInterruptStatus seT16_GetIntFlag( T16_0_Type* T16x ) {
  
  ///< Get interrupt status.
  return ( T16x->INTF_b.UFIF ? seINTERRUPT_OCCURRED : seINTERRUPT_NOT_OCCURRED );
}

/**
  * @brief  Clears Timer channel interrupt.
  * @param  T16x: pointer to T16 peripheral.
  * @retval None
  */
void seT16_ClearIntFlag( T16_0_Type* T16x ) {

  ///< Clear interrupt flag as a whole register
  T16x->INTF = 0x01;
}

