/**
  ******************************************************************************
  * @file    se_t16.h
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file contains all the functions prototypes for the T16 firmware
  *          library.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#ifndef SE_T16_H
#define SE_T16_H

#include "se_common.h"

#ifdef __cplusplus
 extern "C" {
#endif

/** @addtogroup sePeriphLibrary
  * @{
  */

/** @defgroup T16
  * @{
  * @brief T16 is a 16 bit timer
  */

/** @defgroup T16_Constants
  * @{
  */

typedef enum {
   seT16_RepeatMode,                     ///< Timer operates in Repeat Mode. Select this mode
                                         ///< to generate periodic underflow interrupts or when using
                                         ///< the timer to output a clock to a peripheral circuit.
   seT16_OneShotMode                     ///< Timer operates in One-shot Mode. Select this mode to
                                         ///< stop the counter after an interrupt has occurred once.
} seT16_CounterMode;

/**
  * @}
  */

/** @defgroup T16_Types
  * @{
  */

/** 
  * @brief  T16 Init structure definition
  * @note   This structure is used with all T16 but not T16B.
  */

typedef struct {
   seT16_Clk Clk;                        ///< Specifies the timer clock source and divide selection.
   seT16_CounterMode CounterMode;        ///< Specifies the counter mode.
   uint16_t Period;                      ///< Specifies the period value to be loaded into the counter.
} seT16_InitTypeDef;

/**
  * @}
  */

/** @defgroup T16_Functions
  * @{
  */

/**
  * @brief  Fills each seT16_InitTypeDef member with its default value.
  * @param  InitStruct: pointer to an @ref seT16_InitTypeDef structure which will be initialized.
  * @retval None
  */
void seT16_InitStruct( seT16_InitTypeDef* InitStruct );

/**
  * @brief  Initializes the T16 peripheral according to the specified
  *         parameters in the seT16_InitStruct.
  * @note   This function configures the module, and module's interrupts.
  *         It clears module's interrupts
  *         but does not enable interrupt from the module to CPU. This function 
  *         enables module by start supplying operating clock.
  * @param  T16x: pointer to T16 peripheral.
  * @param  InitStruct: pointer to a @ref seT16_InitTypeDef structure that
  *         contains the configuration information for the specified T16 peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seT16_Init( T16_0_Type* T16x, seT16_InitTypeDef* InitStruct );

/**
  * @brief  Starts Timer channel.
  * @param  T16x: pointer to T16 peripheral.
  * @retval None
  */
void seT16_Start( T16_0_Type* T16x );

/**
  * @brief  Stops Timer channel.
  * @param  T16x: pointer to T16 peripheral.
  * @retval None
  */
void seT16_Stop( T16_0_Type* T16x );

/**
  * @brief  Enables Timer channel by start supplying operating clock.
  * @param  T16x: pointer to T16 peripheral.
  * @retval None
  */
void seT16_Enable( T16_0_Type* T16x );

/**
  * @brief  Disables Timer channel by stop supplying operating clock.
  * @param  T16x: pointer to T16 peripheral.
  * @retval None
  */
void seT16_Disable( T16_0_Type* T16x );

/**
  * @brief  Configures timer clock source and clock divider. 
  * @param  T16x: This parameter defines a timer channel and can be a value of @ref T16_0_Type.
  * @param  clock: This parameter can be a value of @ref seT16_Clk.
  * @retval None
  */
void seT16_ConfigureClock( T16_0_Type* T16x, seT16_Clk clock );

/**
  * @brief  Configures timer counter mode.
  * @param  T16x: This parameter defines a timer channel and can be a value of @ref T16_0_Type.
  * @param  mode: This parameter can be a value of @ref seT16_CounterMode.
  * @retval None
  */
void seT16_ConfigureCounterMode( T16_0_Type* T16x, seT16_CounterMode mode );

/**
  * @brief  Sets Timer counter.
  * @param  T16x: This parameter defines a timer channel and can be a value of @ref T16_0_Type.
  * @param  counter: This parameter can be a 16-bit value.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seT16_SetCounter( T16_0_Type* T16x, uint16_t counter );

/**
  * @brief  Gets Timer counter value.
  * @param  T16x: This parameter defines a timer channel and can be a value of @ref T16_0_Type.
  * @retval 16-bit counter value.
  */
uint16_t seT16_GetCounter( T16_0_Type* T16x );

/**
  * @brief  Enables Timer channel interrupt.
  * @param  T16x: pointer to T16 peripheral.
  * @retval None
  */
void seT16_EnableInt( T16_0_Type* T16x );

/**
  * @brief  Disables Timer channel interrupt.
  * @param  T16x: pointer to T16 peripheral.
  * @retval None
  */
void seT16_DisableInt( T16_0_Type* T16x ); 

/**
  * @brief  Returns Timer interrupt flag.
  * @param  T16x: pointer to T16 peripheral.
  * @retval InterruptStatus: @ref seInterruptStatus
  */
seInterruptStatus seT16_GetIntFlag( T16_0_Type* T16x );

/**
  * @brief  Clears Timer channel interrupt.
  * @param  T16x: pointer to T16 peripheral.
  * @retval None
  */
void seT16_ClearIntFlag( T16_0_Type* T16x );


/**
  * @brief  Discovers T16 clock from registers.
  * @param  T16x: pointer to T16 peripheral.
  * @retval T16 clock.
  */
uint32_t seAUX_GetClkSpeedForT16( T16_0_Type* T16x );

/**
  * @brief  Discovers T16 channel for a peripherial.
  * @param  PeriphAddr: pointer to a peripheral.
  * @retval T16 channel.
  */
T16_0_Type * seAUX_GetT16ch_ForPeripheral( void * PeriphAddr );

/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */

#ifdef __cplusplus
}
#endif
#endif  // SE_T16_H

