/**
  ******************************************************************************
  * @file    se_t16b.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file provides all the T16B hardware functions.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#include "se_t16b.h"

/**
  * @brief  Fills each seT16B_InitTypeDef member with its default value.
  * @param  InitStruct: pointer to an seT16B_InitTypeDef structure which will be initialized.
  * @retval None
  */
void seT16B_InitStruct( seT16B_InitTypeDef* InitStruct ) {

  uint8_t k;

  InitStruct->Clk = seDEFAULT_T16B_CLK;

  ///< Set comparator/capture circuit 0 to comparator mode (TX).
  InitStruct->CTL[0].CCMD       = seT16B_ComparatorMode;          ///< The operating mode(0=Comparator/1=Capture).
  InitStruct->CTL[0].CBUFMD     = seT16B_None;                    ///< Set Immediately after writing to the T16B0CCR0 register.
  InitStruct->CTL[0].TOUTMT     = seT16B_UseOneComparator;        ///< Set waveform generation signal(MAX/zero signals).
  InitStruct->CTL[0].TOUTMD     = seT16B_SetReset;                ///< Set signal waveform to set/reset mode.
  InitStruct->CTL[0].TOUTINV    = seT16B_Normal;                  ///< Set the TOUT00 signal polarity to normal(active high).
  InitStruct->CTL[0].TOUTO      = seT16B_LowLevelOutput;          ///< Set signal output level to Low.
  InitStruct->CTL[0].CCR        = 0xffff;                         ///< Set the counter comparison value.

  ///< Set comparator/capture circuit 1-5 to capture mode (RX).
  for (k = 1; k < 6; k++)
  {
    InitStruct->CTL[k].CCMD      =  seT16B_CaptureMode;            ///< The operating mode(0=Comparator/1=Capture).
    InitStruct->CTL[k].SCS       =  seT16B_SyncCapture;            ///< Either capture mode(0=Asynchronous/1=Synchronous).
    InitStruct->CTL[k].CAPIS     =  seT16B_Input_1;                ///< Set Software trigger signal(LOW).
    InitStruct->CTL[k].CAPTRG    =  seT16B_UpDown;                 ///< Set Trigger edge(LOW->HIGH or HIGH->LOW).
  }

  ///< Set MAX counter data.
  InitStruct->MaxCounter      = 0xffff;

  ///< Set count up operation.
  InitStruct->CNTMD        = seT16B_CountUp; 

  ///< Set repeat operation.
  InitStruct->ONEST        = seT16B_RepeatMode;
}

/**
  * @brief  Initializes the T16B peripheral according to the specified
  *         parameters in the seT16B_InitStruct.
  * @note   This function configures the module, and module's interrupts.
  *         It clears module's interrupts
  *         but does not enable interrupt from the module to CPU. This function
  *         enables module by start supplying operating clock.
  * @param  T16BCHx: T16B channel definition of type @ref seT16B_ChannelDef
  * @param  InitStruct: pointer to a seT16B_InitTypeDef structure that
  *         contains the configuration information for the specified T16B peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seT16B_Init( T16B_0_Type* T16Bx, seT16B_InitTypeDef* InitStruct ) {

  uint8_t k;
  seStatus fStatus = seCLG_IsEnabled(seSRC(InitStruct->Clk));

  ///< Check if selected clock is enabled
  if ( fStatus == seSTATUS_OK ) { 

    ///< Configure port pins for T16B
    //fStatus = seAUX_ConfigurePortsForT16B( T16Bx, &InitStruct->ports );

    if ( fStatus == seSTATUS_OK ) {
      seT16B_CCRegsDef *ccregs;
      ///< Configure operating clock.
      seT16B_ConfigureClock( T16Bx, InitStruct->Clk );
      seT16B_Enable( T16Bx );     ///< Enable T16B operations

      ///< Configure Capture/Compare registers
      for (k = 0; k < 6; k++)
      {
         ccregs = (seT16B_CCRegsDef *) (((uint32_t) &T16Bx->CCCTL0) + (k<<3));

         ///< Set comparator or capture mode
         ccregs->CCCTL_b.CCMD = InitStruct->CTL[k].CCMD;
      
         if ( InitStruct->CTL[k].CCMD == seT16B_ComparatorMode ) {
           ///< Configure compare buffer
           ccregs->CCCTL_b.CBUFMD = InitStruct->CTL[k].CBUFMD;
           ///< Select waveform generation signal
           ccregs->CCCTL_b.TOUTMT = InitStruct->CTL[k].TOUTMT;
           ///< Select TOUT signal generation mode
           ccregs->CCCTL_b.TOUTMD = InitStruct->CTL[k].TOUTMD;
           ///< Select TOUT signal polarity
           ccregs->CCCTL_b.TOUTINV = InitStruct->CTL[k].TOUTINV;
           ///< Set the counter comparison value
           ccregs->CCR = InitStruct->CTL[k].CCR;
         } else {
           ///< Set synchronous/asynchronous mode
           ccregs->CCCTL_b.SCS    = InitStruct->CTL[k].SCS;
           ///< Set trigger signal
           ccregs->CCCTL_b.CAPIS  = InitStruct->CTL[k].CAPIS;
           ///< Select trigger edge
           ccregs->CCCTL_b.CAPTRG = InitStruct->CTL[k].CAPTRG;
         }
    
      }

      ///< Set the T16BnMC register.  (Set MAX counter data)
      seT16B_SetMaxCounter( T16Bx, InitStruct->MaxCounter );
      ///< Clear interrupt flags
      T16Bx->INTF = T16B_0_INTF_ALL_Msk;
      ///< Select count up/down operation
      T16Bx->CTL_b.CNTMD = InitStruct->CNTMD;
      ///< Select one-shot/repeat operation
      T16Bx->CTL_b.ONEST = InitStruct->ONEST;
    }
  }

  return fStatus;
}

/**
  * @brief  Stops Timer channel.
  * @param  T16Bx: Pointer to T16B peripheral, a timer channel.
  * @retval None
  */
void seT16B_Stop( T16B_0_Type* T16Bx )  {

  ///< Stop counting.
  T16Bx->CTL_b.RUN = 0;
}

/**
  * @brief  Starts Timer channel.
  * @param  T16Bx: Pointer to T16B peripheral, a timer channel.
  * @retval None
  */
void seT16B_Start( T16B_0_Type* T16Bx )  {
  
  ///< Enable clock
  T16Bx->CTL_b.MODEN = 1;
  ///< Start counting
  T16Bx->CTL_b.RUN = 1; 
} 

/**
  * @brief  Enables Timer channel by start supplying operating clock.
  * @param  T16Bx: Pointer to T16B peripheral, a timer channel.
  * @retval None
  */
void seT16B_Enable( T16B_0_Type* T16Bx ) {

  T16Bx->CTL_b.MODEN = 1;
}

/**
  * @brief  Disables Timer channel by stop supplying operating clock.
  * @param  T16Bx: Pointer to T16B peripheral, a timer channel.
  * @retval None
  */
void seT16B_Disable( T16B_0_Type* T16Bx ) {
  
  T16Bx->CTL_b.MODEN = 0;
}

/**
  * @brief  Configures timer clock source and clock divider.
  * @param  T16Bx: Pointer to T16B peripheral, a timer channel.
  * @param  clock: This parameter can be a value of @ref seT16B_Clk.
  * @retval None
  */
void seT16B_ConfigureClock( T16B_0_Type* T16Bx, seT16B_Clk clock ) {
  
  ///< Configure the T16 Ch. x operating clock.
  T16Bx->CLK_b.CLKSRC = seSRC(clock); ///< Clock source selection
  T16Bx->CLK_b.CLKDIV = seDIV(clock); ///< Clock division ratio selection
  T16Bx->CLK_b.DBRUN  = seDBRUN;      ///< Whether clock supplied in DEBUG mode or not. 
}

/**
  * @brief  Configures timer counter mode.
  * @param  T16Bx: Pointer to T16B peripheral, a timer channel.
  * @param  mode: Set repeat or one shot mode, see @ref seT16B_ONEST.
  * @retval None
  */
void seT16B_ConfigureCounterMode( T16B_0_Type* T16Bx, seT16B_ONEST mode ) {
  
  ///< Set Repeat mode or One-shot mode.
  T16Bx->CTL_b.ONEST = mode;
}

/**
  * @brief  Sets Timer counter.
  * @param  T16Bx: Pointer to T16B peripheral, a timer channel.
  * @param  counter: This parameter can be a 16-bit value.
  * @retval None
  */
void seT16B_SetMaxCounter( T16B_0_Type* T16Bx, uint16_t counter ) {

  T16Bx->MC_b.MC = counter;
}

/**
  * @brief  Gets Timer counter value.
  * @param  T16Bx: Pointer to T16B peripheral, a timer channel.
  * @retval 16-bit counter value.
  */
uint16_t seT16B_GetMaxCounter( T16B_0_Type* T16Bx ) {
  
  return ( T16Bx->MC_b.MC );
}

/**
  * @brief  Get comparator count.
  * @param  T16Bx: Pointer to T16B peripheral, a timer channel.
  * @param  ccsubchan: Capcture/Compare subchannel
  * @retval Comparator count 0 value.
  */
uint16_t seT16B_GetCmpCapCnt( T16B_0_Type* T16Bx, uint8_t ccsubchan ) {

  seT16B_CCRegsDef *ccregs;

  ccregs = (seT16B_CCRegsDef *) (((uint32_t) &T16Bx->CCCTL0) + (ccsubchan<<3));

  ///< Get comparator count.
  return ccregs->CCR;
}

/**
  * @brief  Set comparator count.
  * @param  T16Bx: Pointer to T16B peripheral, a timer channel.
  * @param  ccsubchan: Capcture/Compare subchannel
  * @param  count: Value to be set.
  * @retval None
  */
void seT16B_SetCmpCapCnt( T16B_0_Type* T16Bx, uint8_t ccsubchan, uint16_t count ) {

  seT16B_CCRegsDef *ccregs;

  ccregs = (seT16B_CCRegsDef *) (((uint32_t) &T16Bx->CCCTL0) + (ccsubchan<<3));

  ///< Set comparator count.
  ccregs->CCR = count;
}

/**
  * @brief  Get timer count.
  * @param  T16Bx: Pointer to T16B peripheral, a timer channel.
  * @retval Timer count value.
  */
uint16_t seT16B_GetTimerCount( T16B_0_Type* T16Bx ) {
  
  return T16Bx->TC;
}

/**
  * @brief  Set Trigger Signal.
  * @param  T16Bx: Pointer to T16B peripheral, a timer channel.
  * @param  ccsubchan: Capcture/Compare subchannel
  * @param  Level: See @ref seT16B_CAPIS.
  * @retval None
  */
void seT16B_SetTriggerSignal( T16B_0_Type* T16Bx,  uint8_t ccsubchan, seT16B_CAPIS Level ) {
  
  seT16B_CCRegsDef *ccregs;

  ccregs = (seT16B_CCRegsDef *) (((uint32_t) &T16Bx->CCCTL0) + (ccsubchan<<3));

  ccregs->CCCTL_b.CAPIS = Level;
}

/**
  * @brief  Enables Timer channel interrupt.
  * @param  T16Bx: Pointer to T16B peripheral, a timer channel.
  * @param  irq: Interrupt to enable.
  * @retval None
  */
void seT16B_EnableInt( T16B_0_Type* T16Bx, uint16_t irq ) {
  
  T16Bx->INTE |= irq;
}

/**
  * @brief  Disables Timer channel interrupt.
  * @param  T16Bx: Pointer to T16B peripheral, a timer channel.
  * @param  irq: Interrupt to disable.
  * @retval None
  */
void seT16B_DisableInt( T16B_0_Type* T16Bx, uint16_t irq ) {
  
  T16Bx->INTE &= ~irq;
}

/**
  * @brief  Returns Timer interrupt flag.
  * @param  T16Bx: Pointer to T16B peripheral, a timer channel.
  * @param  flag: Interrupt flags to check.
  * @retval InterruptStatus: see @ref seInterruptStatus
  */
seInterruptStatus seT16B_GetIntFlag( T16B_0_Type* T16Bx, uint16_t flag ) {
  
  seInterruptStatus fStatus = seINTERRUPT_NOT_OCCURRED;
  uint16_t flagged = T16Bx->INTF;
  
  if ( flag ) {
    flagged &= flag;
    if ( flagged == flag ) {
      fStatus = seINTERRUPT_OCCURRED;
    }
  }
  
  return fStatus;
}

/**
  * @brief  Clears Timer channel interrupt.
  * @param  T16Bx: Pointer to T16B peripheral, a timer channel.
  * @param  flag: Interrupt flags to clear.
  * @retval None
  */
void seT16B_ClearIntFlag( T16B_0_Type* T16Bx, uint16_t flag ) {
  
  T16Bx->INTF = flag;
}

