/**
  ******************************************************************************
  * @file    se_t16b.h
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file contains all the functions prototypes for the T16B firmware
  *          library.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#ifndef SE_T16B_H
#define SE_T16B_H

#include "se_common.h"

#ifdef __cplusplus
 extern "C" {
#endif

/** @addtogroup sePeriphLibrary
  * @{
  */

/** @defgroup T16B
  * @{
  * @brief T16B is a 16-bit PWM timer with comparator or capture functions
  */ 

/** @defgroup T16B_Constants
  * @{
  */

/**
 * @brief Brief description.
 * 
 * T16B Detailed description
 */

typedef enum {
   seT16B_RepeatMode,                   ///< Timer operates in Repeat Mode.
   seT16B_OneShotMode                   ///< Timer operates in One-shot Mode. 
} seT16B_ONEST;

typedef enum {
   seT16B_CountUp,                      ///< Timer operates in Up Count mode (one shot or repeat mode)
   seT16B_CountDown,                    ///< Timer operates in Down Count mode (one shot or repeat mode)
   seT16B_CountUpDown                   ///< Timer operates in Up-Down Count mode (one shot or repeat mode) 
} seT16B_CNTMD;

typedef enum {                          ///< Selects either synchronous capture mode or asynchronous capture mode
   seT16B_SyncCapture  = 0x1,
   seT16B_AsyncCapture = 0x0
} seT16B_SCS;

typedef enum {                          ///< Select the timing to load the comparison value written in the T16BnCCRm register to the compare buffer
   seT16B_Clear   = 0x4,
   seT16B_Excep   = 0x3,
   seT16B_Compare = 0x2,
   seT16B_Period  = 0x1,
   seT16B_None    = 0x0
} seT16B_CBUFMD;

typedef enum {                          ///< Capture input signal select
   seT16B_Input_0 = 0x3,
   seT16B_Input_1 = 0x2,
// - 
   seT16B_CCI     = 0x0
} seT16B_CAPIS;

typedef enum {                          ///< Capture trigger selsect
   seT16B_UpDown  = 0x3,
   seT16B_Down    = 0x2,
   seT16B_Up      = 0x1,
   seT16B_DisCapt = 0x0
} seT16B_CAPTRG;

typedef enum {                          ///< Selects whether the comparator MATCH signal of another system is used
   seT16B_UseBothComparator = 0x1,
   seT16B_UseOneComparator  = 0x0
} seT16B_TOUTMT;

typedef enum {                          ///< Selects signal output level when software control mode used
   seT16B_HighLevelOutput = 0x1,
   seT16B_LowLevelOutput  = 0x0
} seT16B_TOUTO;

typedef enum {                          ///< Select TOUT signal generation mode
   seT16B_ResetSet        = 0x7,
   seT16B_ToggleSet       = 0x6,
   seT16B_Reset           = 0x5,
   seT16B_Toggle          = 0x4,
   seT16B_SetReset        = 0x3,
   seT16B_ToggleReset     = 0x2,
   seT16B_Set             = 0x1,
   seT16B_SoftwareControl = 0x0
} seT16B_TOUTMD;

typedef enum {                          ///< This bit selects the TOUTnm signal polarity
   seT16B_Inverted = 0x1,
   seT16B_Normal   = 0x0
} seT16B_TOUTINV;

typedef enum {                          ///< Selects the operating mode of the comparator/capture circuit
   seT16B_ComparatorMode = 0,
   seT16B_CaptureMode = 1
} seT16B_CCMD;

/**
  * @}
  */

/** @defgroup T16B_Types
  * @{ 
  */

typedef struct {
   seT16B_SCS            SCS;                    ///< Specifies the either synchronous capture mode or asynchronous capture mode.
   seT16B_CBUFMD         CBUFMD;                 ///< Specifies the timing to load the comparison value written in the T16B_nCCRm register to the compare buffer.
   seT16B_CAPIS          CAPIS;                  ///< Specifies the trigger signal for capturing.
   seT16B_CAPTRG         CAPTRG;                 ///< Specifies the Trigger edge
   seT16B_TOUTMT         TOUTMT;                 ///< Specifies the whether the comparator MATCH signal of another system is used
   seT16B_TOUTO          TOUTO;                  ///< Specifies the TOUTnm signal output level when software control mode
   seT16B_TOUTMD         TOUTMD;                 ///< Specifies the configure how the TOUTnm signal waveform is changed by the comparator MATCH and counter MAX/ZERO signals.
   seT16B_TOUTINV        TOUTINV;                ///< Specifies the TOUTnm signal polarity
   seT16B_CCMD           CCMD;                   ///< Specifies the comparator or capture mode.
   uint16_t              CCR;                    ///< Compare/Capture Data
} seT16B_CCCTL;

/** 
  * @brief  T16B Init structure definition
  * @note   This structure is used with all T16B.
  */

typedef struct {
   sePPORT_Id TOUTCAP[6];
   sePPORT_Id EXCL0;
   sePPORT_Id EXCL1;
} seT16B_Ports;

typedef struct {
   seT16B_Clk            Clk;                    ///< Specifies the timer clock source and divide selection.
   /* CTL */
   seT16B_CCCTL          CTL[6];
   seT16B_ONEST          ONEST;                  ///< Specifies the counter mode.
   seT16B_CNTMD          CNTMD;                  ///< Specifies the Count mode
   uint16_t              MaxCounter;             ///< Specifies the max counter value.
   uint16_t              Period;                 ///< Specifies the period value to be loaded into the counter.
} seT16B_InitTypeDef;

/** 
  * @brief  T16B Capture/Compare registers definition
  */
#if defined (__CC_ARM)
#pragma anon_unions
#endif

typedef struct {
   union {
     __IO uint16_t  CCCTL;                           /*!< Compare/Capture Control Register                                      */
    
     struct {
       __IO uint16_t  CCMD       :  1;               /*!< T16B CCA register mode select                                         */
       __IO uint16_t  TOUTINV    :  1;               /*!< Tout invert                                                           */
       __IO uint16_t  TOUTMD     :  3;               /*!< Tout Output mode select                                               */
       __IO uint16_t  TOUTO      :  1;               /*!< Tout Output select                                                    */
       __IO uint16_t  TOUTMT     :  1;               /*!< Tout Motor mode select                                                */
            uint16_t             :  1;
       __IO uint16_t  CAPTRG     :  2;               /*!< Capture trigger select                                                */
       __IO uint16_t  CAPIS      :  2;               /*!< Capture input signal select                                           */
       __IO uint16_t  CBUFMD     :  3;               /*!< Compare buffer mode select                                            */
       __IO uint16_t  SCS        :  1;               /*!< SCS register mode select                                              */
     } CCCTL_b;                                      /*!< BitSize                                                               */
   };
   
   union {
     __IO uint16_t  CCR;                             /*!< Compare/Capture Data Register                                         */
     
     struct {
       __IO uint16_t  CC         : 16;               /*!< Compare/Capture Data Register                                         */
     } CCR_b;                                        /*!< BitSize                                                               */
   };
   
   union {
     __IO uint16_t  CCDMAEN;                         /*!< CC DMA Enable Register                                                */
     
     struct {
       __IO uint16_t  CC0DMAEN   :  4;               /*!< Compare/Capture request enable                                        */
     } CCDMAEN_b;                                    /*!< BitSize                                                               */
   };
   __I  uint16_t  RESERVED4;
} seT16B_CCRegsDef;

/**
  * @}
  */

/** @defgroup T16B_Functions
  * @{
  */

/**
  * @brief  Fills each seT16B_InitTypeDef member with its default value.
  * @param  InitStruct: pointer to an seT16B_InitTypeDef structure which will be initialized.
  * @retval None
  */
void seT16B_InitStruct( seT16B_InitTypeDef* InitStruct );

/**
  * @brief  Initializes the T16B peripheral according to the specified
  *         parameters in the seT16B_InitStruct.
  * @note   This function configures the module, and module's interrupts.
  *         It clears module's interrupts
  *         but does not enable interrupt from the module to CPU. This function
  *         enables module by start supplying operating clock.
  * @param  T16BCHx: T16B channel definition of type @ref seT16B_ChannelDef
  * @param  InitStruct: pointer to a seT16B_InitTypeDef structure that
  *         contains the configuration information for the specified T16B peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seT16B_Init( T16B_0_Type* T16Bx, seT16B_InitTypeDef* InitStruct );

/**
  * @brief  Starts Timer channel.
  * @param  T16Bx: Pointer to T16B peripheral, a timer channel.
  * @retval None
  */
void seT16B_Start( T16B_0_Type* T16Bx );

/**
  * @brief  Stops Timer channel.
  * @param  T16Bx: Pointer to T16B peripheral, a timer channel.
  * @retval None
  */
void seT16B_Stop( T16B_0_Type* T16Bx );

/**
  * @brief  Enables Timer channel by start supplying operating clock.
  * @param  T16Bx: Pointer to T16B peripheral, a timer channel.
  * @retval None
  */
void seT16B_Enable( T16B_0_Type* T16Bx );

/**
  * @brief  Disables Timer channel by stop supplying operating clock.
  * @param  T16Bx: Pointer to T16B peripheral, a timer channel.
  * @retval None
  */
void seT16B_Disable( T16B_0_Type* T16Bx );

/**
  * @brief  Configures timer clock source and clock divider.
  * @param  T16Bx: Pointer to T16B peripheral, a timer channel.
  * @param  clock: This parameter can be a value of @ref seT16B_Clk.
  * @retval None
  */
void seT16B_ConfigureClock( T16B_0_Type* T16Bx, seT16B_Clk clock );

/**
  * @brief  Configures timer counter mode.
  * @param  T16Bx: Pointer to T16B peripheral, a timer channel.
  * @param  mode: Set repeat or one shot mode, see @ref seT16B_ONEST.
  * @retval None
  */
void seT16B_ConfigureCounterMode( T16B_0_Type* T16Bx, seT16B_ONEST mode );

/**
  * @brief  Get comparator count.
  * @param  T16Bx: Pointer to T16B peripheral, a timer channel.
  * @param  ccsubchan: Capcture/Compare subchannel
  * @retval Comparator count 0 value.
  */
uint16_t seT16B_GetCmpCapCnt( T16B_0_Type* T16Bx, uint8_t ccsubchan);

/**
  * @brief  Set comparator count.
  * @param  T16Bx: Pointer to T16B peripheral, a timer channel.
  * @param  ccsubchan: Capcture/Compare subchannel
  * @param  count: Value to be set.
  * @retval None
  */
void seT16B_SetCmpCapCnt( T16B_0_Type* T16Bx, uint8_t ccsubchan, uint16_t count );

/**
  * @brief  Get timer count.
  * @param  T16Bx: Pointer to T16B peripheral, a timer channel.
  * @retval Timer count value.
  */
uint16_t seT16B_GetTimerCount( T16B_0_Type* T16Bx );

/**
  * @brief  Set Trigger Signal.
  * @param  T16Bx: Pointer to T16B peripheral, a timer channel.
  * @param  ccsubchan: Capcture/Compare subchannel
  * @param  Level: See @ref seT16B_CAPIS.
  * @retval None
  */
void seT16B_SetTriggerSignal( T16B_0_Type* T16Bx,  uint8_t ccsubchan, seT16B_CAPIS Level );

/**
  * @brief  Sets Timer counter.
  * @param  T16Bx: Pointer to T16B peripheral, a timer channel.
  * @param  counter: This parameter can be a 16-bit value.
  * @retval None
  */
void seT16B_SetMaxCounter( T16B_0_Type* T16Bx, uint16_t counter );

/**
  * @brief  Gets Timer counter value.
  * @param  T16Bx: Pointer to T16B peripheral, a timer channel.
  * @retval 16-bit counter value.
  */
uint16_t seT16B_GetMaxCounter( T16B_0_Type* T16Bx );

/**
  * @brief  Enables Timer channel interrupt.
  * @param  T16Bx: Pointer to T16B peripheral, a timer channel.
  * @param  irq: Interrupt to enable.
  * @retval None
  */
void seT16B_EnableInt( T16B_0_Type* T16Bx, uint16_t irq );

/**
  * @brief  Disables Timer channel interrupt.
  * @param  T16Bx: Pointer to T16B peripheral, a timer channel.
  * @param  irq: Interrupt to disable.
  * @retval None
  */
void seT16B_DisableInt( T16B_0_Type* T16Bx, uint16_t irq ); 

/**
  * @brief  Returns Timer interrupt flag.
  * @param  T16Bx: Pointer to T16B peripheral, a timer channel.
  * @param  flag: Interrupt flags to check.
  * @retval InterruptStatus: see @ref seInterruptStatus
  */
seInterruptStatus seT16B_GetIntFlag( T16B_0_Type* T16Bx, uint16_t flag );

/**
  * @brief  Clears Timer channel interrupt.
  * @param  T16Bx: Pointer to T16B peripheral, a timer channel.
  * @param  flag: Interrupt flags to clear.
  * @retval None
  */
void seT16B_ClearIntFlag( T16B_0_Type* T16Bx, uint16_t flag );

/**
  * @brief  Configures ports for this module. Override this function to configure specific ports.
  * @param  T16BCHx: T16B channel definition of type @ref seT16B_ChannelDef
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seAUX_ConfigurePortsForT16B( T16B_0_Type* T16Bx, seT16B_Ports * ports );

/**
  * @brief  Discovers T16B clock from registers.
  * @param  T16Bx: Pointer to T16B peripheral, a timer channel.
  * @retval T16B clock.
  */
uint32_t seAUX_GetClkSpeedForT16B( T16B_0_Type* T16Bx );

/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */

#ifdef __cplusplus
}
#endif
#endif  // SE_T16B_H

