/**
  ******************************************************************************
  * @file    se_tsrvr.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file provides all the TSRVR hardware functions.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#include "se_tsrvr.h"

/**
  * Static functions declarations
  **/
static seTSRVR_VREFAMD TSRVR_VRefMode;
static uint32_t TSRVR_Startup_Dly;

/**
  * @brief  Initialize temperature sensor ADC parameters.
  * @param  InitStruct:  initialization parameters for ADC12A
  * @param  vrefmd:      VREF selection
  * @param  startupdly:  TSRVR startup delay (ms)
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seTSRVR_Init ( seADC12A_InitTypeDef* InitStruct, seTSRVR_VREFAMD vrefmd, uint32_t startupdly ) {

  TSRVR_VRefMode = vrefmd;          ///< Set the reference voltage generator output voltage.
  TSRVR_Startup_Dly = startupdly;   ///< Set TSRVR startup delay (ms).

  ///< initalize ADC12A
  seADC12A_Init( ADC12A_0, InitStruct );

  return(seSTATUS_OK);
}

/**
  * @brief  Start temperature measurement.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seTSRVR_StartTempMeas ( void ) {

  ///< Turn on VREF of temperature sensor
  seTSRVR_SetVrefMode(TSRVR_VRefMode);
  ///< Enable temperature sensor output
  seTSRVR_Enable();

  ///< Delay for settling
  seSysSleepMS(TSRVR_Startup_Dly);
  ///< start ADC12A
  seADC12A_Start(ADC12A_0);

  return(seSTATUS_OK);
}

/**
  * @brief  Stop temperature measurement
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seTSRVR_StopTempMeas ( void ) {

  ///< Stop conversion and disable ADC12A
  seADC12A_Stop( ADC12A_0 );
  ///< Disable temperature sensor
  seTSRVR_Disable();
  ///< Turn off VREF of temperature sensor
  seTSRVR_SetVrefMode(seTSRVR_VREF_HIZ);

  return(seSTATUS_OK);
}

