/**
  ******************************************************************************
  * @file    se_tsrvr.h
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file contains all the functions prototypes for the TSRVR firmware
  *          library.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#ifndef SE_TSRVR_H
#define SE_TSRVR_H

#include "se_common.h"
#include "se_adc12a.h"

#ifdef __cplusplus
 extern "C" {
#endif

/** @addtogroup sePeriphLibrary
  * @{
  */

/** @defgroup TSRVR
  * @{
  * @brief The TSRVR is a built-in temperature sensor and reference voltage generator.
  *
  */

/** @defgroup TSRVR_Constants
  * @{
  */

typedef enum {
   seTSRVR_VREF_HIZ             = 0U,     ///< VREF is Hi-Z
   seTSRVR_VREF_VDD             = 1U,     ///< VREF is VDD level
   seTSRVR_VREF_2V0             = 2U,     ///< VREF is 2.0V
   seTSRVR_VREF_2V5             = 3U      ///< VREF is 2.5V
} seTSRVR_VREFAMD;


/**
  * @}
  */

/** @defgroup TSRVR_macros
  * @{
  */

#define seTSRVR_Enable()                        ( TSRVR_0->TCTL_b.TEMPEN = 1 )
#define seTSRVR_Disable()                       ( TSRVR_0->TCTL_b.TEMPEN = 0 )
#define seTSRVR_SetVrefMode(vrefmd)             ( TSRVR_0->VCTL_b.VREFAMD = vrefmd )
#define seTSRVR_GetVrefMode()                   ( TSRVR_0->VCTL_b.VREFAMD )

/**
  * @}
  */

/** @defgroup TSRVR_Functions
  * @{
  */

/**
  * @brief  Initialize temperature sensor ADC parameters.
  * @param  InitStruct:  initialization parameters for ADC12A
  * @param  vrefmd:      VREF selection
  * @param  startupdly:  TSRVR startup delay (ms)
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seTSRVR_Init ( seADC12A_InitTypeDef* InitStruct, seTSRVR_VREFAMD vrefmd, uint32_t startupdly );

/**
  * @brief  Start temperature measurement
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seTSRVR_StartTempMeas ( void );

/**
  * @brief  Stop temperature measurement
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seTSRVR_StopTempMeas ( void );

/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */

#ifdef __cplusplus
}
#endif
#endif  // SE_TSRVR_H

