/**
  ******************************************************************************
  * @file    se_uart3.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file provides all the UART3 hardware functions.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#include <string.h>
#include "se_uart3.h"

/**
  * @brief  Initializes the UART peripheral according to the specified
  *         parameters in the seUART_InitStruct.
  * @param  UARTx: This parameter defines a uart channel and can be a value of @ref UART3_0_Type.
  * @param  InitStruct: pointer to a seUART_InitTypeDef structure that
  *         contains the configuration information for the specified UART peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seUART_Init( UART3_0_Type* UARTx, seUART_InitTypeDef* InitStruct ) {
  
  seStatus fStatus = seCLG_IsEnabled( seSRC(InitStruct->Clk) );
  
  ///< Check if selected clock is enabled
  if ( fStatus == seSTATUS_OK ) { 

    ///< Determine if the UART Channel already is disabled, then go ahead
    if ( UARTx->CTL_b.MODEN == 0 ) {
      //fStatus = seAUX_ConfigurePortsForUart( UARTx, InitStruct->RX, InitStruct->TX );
      
      if ( fStatus == seSTATUS_OK ) {
        ///< Stop supplying operating clock
        seUART_Disable( UARTx );
        
        ///< Configure the UART Ch.n operating clock.
        seUART_ConfigureClock( UARTx, InitStruct->Clk );
        
        ///< Select operation mode;
        seUART_ConfigureMode( UARTx, InitStruct->Mode );
        
        ///< Initialize interrupt.
        seUART_DisableInt( UARTx, UART3_0_INTE_ALL_Msk );
        seUART_ClearIntFlag( UARTx, UART3_0_INTF_ALL_Msk );
      }
    }
  }
  
  return fStatus;
}

/**
  * @brief  Fills each seUART_InitTypeDef member with its default value.
  * @param  UART_InitStruct: Pointer to an seUART_InitTypeDef structure which will be initialized.
  * @retval None
  */
void seUART_InitStruct( seUART_InitTypeDef* InitStruct ) {
    
  InitStruct->Clk                   =  seDEFAULT_UART_CLK;                 ///< Set clock source for UART3
  InitStruct->Mode.reg_b.stpb       =  seUART_MOD_STPB_1BIT;               ///< Select 1/2-bit stop bit length
  InitStruct->Mode.reg_b.prmd       =  seUART_MOD_PRMD_EVEN;               ///< Select even/odd parity function
  InitStruct->Mode.reg_b.pren       =  seUART_MOD_PREN_NO_PARITY;          ///< Enable parity function
  InitStruct->Mode.reg_b.chln       =  seUART_MOD_CHLN_8BIT;               ///< Set 7/8-bit data rate
  InitStruct->Mode.reg_b.irmd       =  seUART_MOD_IRMD_NO_IRDA;            ///< Enable Irda interface
  InitStruct->Mode.reg_b.outmd      =  seUART_MOD_OUTMD_PUSH_PULL;         ///< Enable USOUTn pn open-drain output
  InitStruct->Mode.reg_b.puen       =  seUART_MOD_PUEN_ENABLE;             ///< Enable USINn pin pull-up
  InitStruct->Mode.reg_b.invirtx    =  seUART_MOD_INVIRTX_NORMAL;          ///< Invert transmit IrDA signal
  InitStruct->Mode.reg_b.invirrx    =  seUART_MOD_INVIRRX_NORMAL;          ///< Invert receive IrDA signal
  InitStruct->Mode.reg_b.brdiv      =  seUART_MOD_BRDIV_16;                ///< Sets URT3 operating clock division ratio
  InitStruct->Mode.reg_b.caren      =  seUART_MOD_CAREN_DISABLE;           ///< Enable the carrier modulation function
  InitStruct->Mode.reg_b.pecar      =  seUART_MOD_PECAR_L;                 ///< Select the carrier modulation period
}

/**
  * @brief  Enables UART channel by start supplying operating clock.
  * @param  UARTx: This parameter defines a uart channel and can be a value of @ref UART3_0_Type.
  * @retval None
  */
void seUART_Enable( UART3_0_Type* UARTx) {
 
  ///< Execute software reset.
  UARTx->CTL_b.SFTRST = 1;
  WAIT( UARTx->CTL & UART3_0_CTL_SFTRST_Msk, seLONG_WAIT_TIMEOUT_MS );
  UARTx->CTL_b.MODEN = 1;
}

/**
  * @brief  Disables UART channel by stop supplying operating clock.
  * @param  UARTx: This parameter defines a uart channel and can be a value of @ref UART3_0_Type.
  * @retval None
  */
void seUART_Disable( UART3_0_Type* UARTx) {

  ///< Check if data sent out
  WAIT( (UARTx->INTF & UART3_0_INTF_TENDIF_Msk) == 0, seLONG_WAIT_TIMEOUT_MS );

  ///< Clear interrupt flag.
  seUART_ClearIntFlag( UARTx, UART3_0_INTF_TENDIF_Msk );
  UARTx->CTL_b.MODEN = 0;
}

/**
  * @brief  Configures UART clock source and clock divider.
  * @param  UARTx: This parameter defines a uart channel and can be a value of @ref UART3_0_Type.
  * @param  clock: This parameter can be a value of @ref seUART_Clk.
  * @retval None
  */
void seUART_ConfigureClock( UART3_0_Type* UARTx, seUART_Clk clock ) {  

  UARTx->CLK_b.CLKSRC = seSRC(clock);     ///< Clock source selection
  UARTx->CLK_b.CLKDIV = seDIV(clock);     ///< Clock division ratio selection
  UARTx->CLK_b.DBRUN  = seDBRUN;          ///< Whether clock supplied in DEBUG mode or not.
}

/**
  * @brief  Configures UART mode register.
  * @param  UARTx: This parameter defines a uart channel and can be a value of @ref UART3_0_Type.
  * @param  mode: This parameter can be a value of @ref seUART_Mode.
  * @retval None
  */
void seUART_ConfigureMode( UART3_0_Type* UARTx, seUART_Mode mode ) {
  
  UARTx->MOD = mode.reg;
}

/**
  * @brief  Sets UART baud rate.
  * @note   For the transfer rate range configurable in the UART, refer to
  *         \"UART Characteristics, Transfer baud rates\" in the \"Electrical
  *         Characteristics\" chapter.
  * @param  UARTx: This parameter defines a uart channel and can be a value of @ref UART3_0_Type.
  * @param  bps: This parameter can be a value of @ref seUART_BaudRate.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seUART_SetBaudRate( UART3_0_Type* UARTx, uint32_t bps ) {
  
  seStatus ret = seSTATUS_NG;
  uint32_t clk = seAUX_GetClkSpeedForUart( UARTx );
 
  if ( bps > 0 && clk > 0 ) {
    
    uint16_t brt = clk/bps/16;
    uint16_t fmd = clk/bps - brt*16;
    
    seUART_SetBaudRateReg( UARTx, brt-1, fmd );
    
    ret = seSTATUS_OK;
  }
  
  return ret;
}

/**
  * @brief  Sets UART baud rate by setting of the BRT nd FMD registers.
  * @note   Values of these registers shall be calculated in advance based on the
  *         formula.
  *
  *         bps = CLK_UART / {(BRT + 1) x 16 + FMD}
  *
  * @param  UARTx: This parameter defines a uart channel and can be a value of @ref UART3_0_Type.
  * @param  BRT: This parameter can be a value of (0 to 255).
  * @param  FMD: This parameter can be a value of (0 to 15).
  * @retval None
  */
void seUART_SetBaudRateReg( UART3_0_Type* UARTx, uint16_t BRT, uint16_t FMD ) {
  
  UARTx->BR_b.BRT = BRT;
  UARTx->BR_b.FMD = FMD;
}

/**
  * @brief  Enables UART channel interrupt.
  * @param  UARTx: This parameter defines a uart channel and can be a value of @ref UART3_0_Type.
  * @param  irq: Interrupt to enable.
  * @retval None
  */
void seUART_EnableInt( UART3_0_Type* UARTx, uint16_t irq ) {
  
  UARTx->INTE |= irq;
}

/**
  * @brief  Disables UART channel interrupt.
  * @param  UARTx: This parameter defines a uart channel and can be a value of @ref UART3_0_Type.
  * @param  irq: Interrupt to disable.
  * @retval None
  */
void seUART_DisableInt( UART3_0_Type* UARTx, uint16_t irq ) {
  
  UARTx->INTE &= ~irq;
}

/**
  * @brief  Returns UART interrupt flag status.
  * @param  UARTx: This parameter defines a uart channel and can be a value of @ref UART3_0_Type.
  * @param  irq: Interrupt flags to check.
  *              In case of combination of flags - all of them must be set in order to return
  *              \"INTERRUPT_OCCURED\" status.
  * @retval Interrupt status: can be a value of @ref seInterruptStatus
  */
seInterruptStatus seUART_GetIntFlag( UART3_0_Type* UARTx, uint16_t irq ) {
  
  seInterruptStatus fStatus = seINTERRUPT_NOT_OCCURRED;
  uint16_t flagged = UARTx->INTF;
  
  if ( irq ) {
    flagged &= irq;
    if ( flagged == irq ) {
      fStatus = seINTERRUPT_OCCURRED;
    }
  }
  
  return fStatus;
}

/**
  * @brief  Clears UART channel interrupt.
  * @param  UARTx: This parameter defines a uart channel and can be a value of @ref UART3_0_Type.
  * @param  irq: Interrupt flags to clear.
  *              except of UART3_0_INTF_TBEIF_Msk, UART3_0_INTF_RB1FIF_Msk, UART3_0_INTF_RB2FIF_Msk flags that
  *              must be cleared by reading / writing TXD, RXD registers.
  * @retval None
  */
void seUART_ClearIntFlag( UART3_0_Type* UARTx, uint16_t irq ) {
  
  UARTx->INTF = irq;       ///< Clear interrupt flag.
}

/**
  * @brief  Starts receiving data from UART receiver.
  * @note   This function can not be called from an interrupt Serice routine since it
  *         it can wait for a timeout.  Timeout value is equal to seSHORT_WAIT_TIMEOUT_MS.
  * @param  UARTx: This parameter defines a uart channel and can be a value of @ref UART3_0_Type.
  * @param  data: This parameter is a pointer to a destination byte array.
  * @param  size: This parameter is a destination array size. 
  * @retval Returns a number of received bytes.
  */
uint32_t seUART_Receive( UART3_0_Type* UARTx, uint8_t data[], uint32_t size ) {
  
  uint32_t i = size;

  if ( (data!=NULL) && (size>0) ) {
    do {
      WAIT( (UARTx->INTF & ( UART3_0_INTF_RB1FIF_Msk | UART3_0_INTF_RB2FIF_Msk )) == 0, seLONG_WAIT_TIMEOUT_MS );
      if ( (UARTx->INTF & ( UART3_0_INTF_RB1FIF_Msk | UART3_0_INTF_RB2FIF_Msk )) == 0 ) {
         break;
      }
      ///< Read received data.
      data[size-i] = UARTx->RXD;
    } while(--i);
  }
  
  return (size-i);
}

/**
  * @brief  Starts sending data to UART transmitter.
  * @note   This function can not be called from Interrupt Serice routine since it
  *         it can wait for a timeout. Timeout value is equal to seSHORT_WAIT_TIMEOUT_MS.
  * @param  UARTx: This parameter defines a uart channel and can be a value of @ref UART3_0_Type.
  * @param  data: This parameter is a pointer to a source byte array.
  * @param  size: This parameter is a number of source bytes to send.
  *               User is responsible for validation of the array size and the number of bytes to send.
  * @retval Returns a number of sent bytes.
  */
uint32_t seUART_Send( UART3_0_Type* UARTx, const uint8_t data[], uint32_t size ) {

  uint32_t i = size;

  if ( (data!=NULL) && (size>0) ) {
    do {
      WAIT( (UARTx->INTF & (1<<UART3_0_INTF_TBEIF_Pos)) == 0, seLONG_WAIT_TIMEOUT_MS );
      if ( (UARTx->INTF & (1<<UART3_0_INTF_TBEIF_Pos)) == 0 ) {
        break;
      }
      ///< Send data.
      UARTx->TXD = data[size-i];
      
    } while(--i);
  }

  return (size-i);
}

/**
  * @brief  Receives a byte or two bytes of data from UART receiver.
  * @note   This function can be called from Interrupt Serice routine since it
  *         its implementation is deterministic.
  * @param  UARTx: This parameter defines a uart channel and can be a value of @ref UART3_0_Type.
  * @retval one or two bytes packed as 16 bit word.
  */
uint16_t  seUART_GetData( UART3_0_Type* UARTx) {

  uint16_t data = 0;
  if ( seUART_GetIntFlag( UARTx, UART3_0_INTF_RB2FIF_Msk ) ) {
    data = UARTx->RXD;
  }
  data = data<<8 | UARTx->RXD;
  
  return data;
}

/**
  * @brief  Sends a byte to UART transmitter if the transmit buffer is empty.
  * @note   This function can be called from Interrupt Service routine since
  *         its implementation is deterministic.
  * @param  UARTx: This parameter defines a uart channel and can be a value of @ref UART3_0_Type.
  * @param  byte: This parameter is a byte of data.
  * @retval None
  */
void  seUART_SetData( UART3_0_Type* UARTx, uint8_t byte ) {
  
 if ( seUART_GetIntFlag( UARTx, UART3_0_INTF_TBEIF_Msk ) ) {
    UARTx->TXD = byte;
  }
}

