/**
  ******************************************************************************
  * @file    se_uart3.h
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file contains all the functions prototypes for the UART3 firmware
  *          library.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#ifndef SE_UART3_H
#define SE_UART3_H

#include "se_common.h"

#ifdef __cplusplus
 extern "C" {
#endif

/** @addtogroup sePeriphLibrary
  * @{
  */

/** @defgroup UART3
  * @{
  * @brief  The UART is an asynchronous serial interface
  */

/** @defgroup UART3_Constants
  * @{
  */

typedef enum {
   seUART_MOD_STPB_1BIT  = 0,                   ///< Stop is 1 bit.
   seUART_MOD_STPB_2BIT  = 1                    ///< Stop is 2 bit.
} seUART_MOD_Stpb;

typedef enum {
   seUART_MOD_PRMD_EVEN  = 0,                   ///< Parity is even.
   seUART_MOD_PRMD_ODD  = 1                     ///< Parity is odd.
} seUART_MOD_Prmd;

typedef enum {
   seUART_MOD_PREN_NO_PARITY  = 0,              ///< No parity.
   seUART_MOD_PREN_WITH_PARITY = 1              ///< Parity.
} seUART_MOD_Pren;

typedef enum {
   seUART_MOD_CHLN_7BIT  = 0,                   ///< Character length 7 bit.
   seUART_MOD_CHLN_8BIT = 1                     ///< Character length 8 bit.
} seUART_MOD_Chln;

typedef enum {
   seUART_MOD_IRMD_NO_IRDA  = 0,                ///< No Irda.
   seUART_MOD_IRMD_WITH_IRDA = 1                ///< Irda.
} seUART_MOD_Irmd;

typedef enum {
   seUART_MOD_OUTMD_PUSH_PULL  = 0,             ///< Push pull.
   seUART_MOD_OUTMD_OPEN_DRAIN = 1              ///< Open drain.
} seUART_MOD_Outmd;

typedef enum {
   seUART_MOD_PUEN_DISABLE  = 0,                ///< USIN pullup disable.
   seUART_MOD_PUEN_ENABLE = 1                   ///< USIN pullup enable.
} seUART_MOD_Puen;

typedef enum {
   seUART_MOD_INVIRTX_NORMAL  = 0,              ///< Invert transmit IrDA signal normal.
   seUART_MOD_INVIRTX_INVERT = 1                ///< Invert transmit IrDA signalinvert.
} seUART_MOD_Invirtx;

typedef enum {
   seUART_MOD_INVIRRX_NORMAL  = 0,              ///< Invert receive IrDA signal normal.
   seUART_MOD_INVIRRX_INVERT = 1                ///< Invert receive IrDA signal invert.
} seUART_MOD_Invirrx;

typedef enum {
   seUART_MOD_BRDIV_16  = 0,                    ///< Sets clock division ratio 1/16 for the transfer
   seUART_MOD_BRDIV_4 = 1                       ///< Sets clock division ratio 1/4 for the transfer
} seUART_MOD_BRDIV;

typedef enum {
   seUART_MOD_CAREN_DISABLE  = 0,               ///< Disable carrier modulation
   seUART_MOD_CAREN_ENABLE = 1                  ///< Enable carrier modulation
} seUART_MOD_CAREN;

typedef enum {
   seUART_MOD_PECAR_L  = 0,                     ///< Carrier modulation during L data period
   seUART_MOD_PECAR_H = 1                       ///< Carrier modulation during H data period
} seUART_MOD_PECAR;

typedef enum {
   seUART_BAUD_RATE_4800 = 4800,                ///< 4800bps
   seUART_BAUD_RATE_9600 = 9600,                ///< 9600bps
   seUART_BAUD_RATE_14400 = 14400,              ///< 14400bps
   seUART_BAUD_RATE_19200 = 19200,              ///< 19200bps
   seUART_BAUD_RATE_38400 = 38400,              ///< 38400bps
   seUART_BAUD_RATE_56000 = 56000,              ///< 56000bps
   seUART_BAUD_RATE_115200 = 115200,            ///< 115200bps
   seUART_BAUD_RATE_128000 = 128000,            ///< 128000bps
   seUART_BAUD_RATE_256000 = 256000             ///< 256000bps
} seUART_BaudRate;

/**
  * @}
  */

/** @defgroup UART_Types
  * @{ 
  */

/** 
  * @brief  UART Mode structure definition
  * @note
  */

typedef union {
   uint16_t  reg;                               ///< UART Mode
   
   struct {
     uint16_t  stpb           :  1;             ///< UART select 1/2-bit stop bit length
     uint16_t  prmd           :  1;             ///< UART select even/odd parity function
     uint16_t  pren           :  1;             ///< UART enable parity function
     uint16_t  chln           :  1;             ///< UART set 7/8-bit data rate
     uint16_t  irmd           :  1;             ///< UART enable Irda interface
     uint16_t  outmd          :  1;             ///< UART enable USOUTn pn open-drain output
     uint16_t  puen           :  1;             ///< UART enable USINn pin pull-up
     uint16_t  reserved_7     :  1;             ///< Reserved bit
     uint16_t  invirtx        :  1;             ///< Invert transmit IrDA signal
     uint16_t  invirrx        :  1;             ///< Invert receive IrDA signal
     uint16_t  brdiv          :  1;             ///< Sets URT3 operating clock division ratio
     uint16_t  caren          :  1;             ///< Enable the carrier modulation function
     uint16_t  pecar          :  1;             ///< Select the carrier modulation period
     uint16_t  reserved_15_13 :  3;             ///< Reserved bits
   } reg_b;                                     ///< BitSize
} seUART_Mode;

/** 
  * @brief  UART Init structure definition
  * @note
  */

typedef struct{
   seUART_Clk  Clk;                             ///< Specifies the uart clock source and divide selection.
   seUART_Mode Mode;                            ///< Specifies the mode of operation @ref seUART_Mode.
} seUART_InitTypeDef;

/**
  * @}
  */

/** @defgroup UART_Functions
  * @{
  */

/**
  * @brief  Initializes the UART peripheral according to the specified
  *         parameters in the seUART_InitStruct.
  * @param  UARTx: This parameter defines a uart channel and can be a value of @ref UART3_0_Type.
  * @param  InitStruct: pointer to a seUART_InitTypeDef structure that
  *         contains the configuration information for the specified UART peripheral.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seUART_Init( UART3_0_Type * UARTx, seUART_InitTypeDef* InitStruct );

/**
  * @brief  Fills each seUART_InitTypeDef member with its default value.
  * @param  UART_InitStruct: Pointer to an seUART_InitTypeDef structure which will be initialized.
  * @retval None
  */
void seUART_InitStruct( seUART_InitTypeDef * InitStruct );

/**
  * @brief  Enables UART channel by start supplying operating clock.
  * @param  UARTx: This parameter defines a uart channel and can be a value of @ref UART3_0_Type.
  * @retval None
  */
void seUART_Enable( UART3_0_Type * UARTx );

/**
  * @brief  Disables UART channel by stop supplying operating clock.
  * @param  UARTx: This parameter defines a uart channel and can be a value of @ref UART3_0_Type.
  * @retval None
  */
void seUART_Disable( UART3_0_Type * UARTx );

/**
  * @brief  Configures UART clock source and clock divider.
  * @param  UARTx: This parameter defines a uart channel and can be a value of @ref UART3_0_Type.
  * @param  clock: This parameter can be a value of @ref seUART_Clk.
  * @retval None
  */
void seUART_ConfigureClock( UART3_0_Type * UARTx, seUART_Clk clock );

/**
  * @brief  Configures UART mode register.
  * @param  UARTx: This parameter defines a uart channel and can be a value of @ref UART3_0_Type.
  * @param  mode: This parameter can be a value of @ref seUART_Mode.
  * @retval None
  */
void seUART_ConfigureMode( UART3_0_Type * UARTx, seUART_Mode mode );

/**
  * @brief  Sets UART baud rate.
  * @note   For the transfer rate range configurable in the UART, refer to
  *         \"UART Characteristics, Transfer baud rates\" in the \"Electrical
  *         Characteristics\" chapter.
  * @param  UARTx: This parameter defines a uart channel and can be a value of @ref UART3_0_Type.
  * @param  bps: This parameter can be a value of @ref seUART_BaudRate.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seUART_SetBaudRate( UART3_0_Type * UARTx, uint32_t bps );

/**
  * @brief  Sets UART baud rate by setting of the BRT nd FMD registers.
  * @note   Values of these registers shall be calculated in advance based on the
  *         formula.
  *
  *         bps = CLK_UART / {(BRT + 1) x 16 + FMD}
  *
  * @param  UARTx: This parameter defines a uart channel and can be a value of @ref UART3_0_Type.
  * @param  BRT: This parameter can be a value of (0 to 255).
  * @param  FMD: This parameter can be a value of (0 to 15).
  * @retval None
  */
void seUART_SetBaudRateReg( UART3_0_Type * UARTx, uint16_t BRT, uint16_t FMD );

/**
  * @brief  Enables UART channel interrupt.
  * @param  UARTx: This parameter defines a uart channel and can be a value of @ref UART3_0_Type.
  * @param  irq: Interrupt to enable.
  * @retval None
  */
void seUART_EnableInt( UART3_0_Type * UARTx, uint16_t irq );

/**
  * @brief  Disables UART channel interrupt.
  * @param  UARTx: This parameter defines a uart channel and can be a value of @ref UART3_0_Type.
  * @param  irq: Interrupt to disable.
  * @retval None
  */
void seUART_DisableInt( UART3_0_Type * UARTx, uint16_t irq );

/**
  * @brief  Returns UART interrupt flag status.
  * @param  UARTx: This parameter defines a uart channel and can be a value of @ref UART3_0_Type.
  * @param  irq: Interrupt flags to check.
  *              In case of combination of flags - all of them must be set in order to return
  *              \"INTERRUPT_OCCURED\" status.
  * @retval Interrupt status: can be a value of @ref seInterruptStatus
  */
seInterruptStatus seUART_GetIntFlag( UART3_0_Type * UARTx, uint16_t irq );

/**
  * @brief  Clears UART channel interrupt.
  * @param  UARTx: This parameter defines a uart channel and can be a value of @ref UART3_0_Type.
  * @param  irq: Interrupt flags to clear.
  *              except of UART3_0_INTF_TBEIF_Msk, UART3_0_INTF_RB1FIF_Msk, UART3_0_INTF_RB2FIF_Msk flags that
  *              must be cleared by reading / writing TXD, RXD registers.
  * @retval None
  */
void seUART_ClearIntFlag( UART3_0_Type * UARTx, uint16_t irq );

/**
  * @brief  Starts sending data to UART transmitter.
  * @note   This function can not be called from Interrupt Serice routine since it
  *         it can wait for a timeout. Timeout value is equal to seSHORT_WAIT_TIMEOUT_MS.
  * @param  UARTx: This parameter defines a uart channel and can be a value of @ref UART3_0_Type.
  * @param  data: This parameter is a pointer to a source byte array.
  * @param  size: This parameter is a number of source bytes to send.
  *               User is responsible for validation of the array size and the number of bytes to send.
  * @retval Returns a number of sent bytes.
  */
uint32_t seUART_Send( UART3_0_Type * UARTx, const uint8_t data[], uint32_t size );

/**
  * @brief  Starts receiving data from UART receiver.
  * @note   This function can not be called from an interrupt Serice routine since it
  *         it can wait for a timeout.  Timeout value is equal to seSHORT_WAIT_TIMEOUT_MS.
  * @param  UARTx: This parameter defines a uart channel and can be a value of @ref UART3_0_Type.
  * @param  data: This parameter is a pointer to a destination byte array.
  * @param  size: This parameter is a destination array size.
  * @retval Returns a number of received bytes.
  */
uint32_t seUART_Receive( UART3_0_Type * UARTx, uint8_t data[], uint32_t size );

/**
  * @brief  Receives a byte or two bytes of data from UART receiver.
  * @note   This function can be called from Interrupt Serice routine since it
  *         its implementation is deterministic.
  * @param  UARTx: This parameter defines a uart channel and can be a value of @ref UART3_0_Type.
  * @retval one or two bytes packed as 16 bit word.
  */
uint16_t seUART_GetData( UART3_0_Type * UARTx );

/**
  * @brief  Sends a byte to UART transmitter if the transmit buffer is empty.
  * @note   This function can be called from Interrupt Service routine since
  *         its implementation is deterministic.
  * @param  UARTx: This parameter defines a uart channel and can be a value of @ref UART3_0_Type.
  * @param  byte: This parameter is a byte of data.
  * @retval None
  */
void seUART_SetData( UART3_0_Type * UARTx, uint8_t byte );

/**
  * @brief  Discovers UART clock from registers.
  * @param  UARTx: This parameter defines a uart channel and can be a value of @ref UART3_0_Type.
  * @retval UART clock frequency to use in baud rate calculation.
  */
uint32_t seAUX_GetClkSpeedForUart( UART3_0_Type* UARTx );

/**
  * @brief  Configures ports for this module. Override this function to configure specific ports.
  * @param  UARTx: This parameter defines a uart channel and can be a value of @ref UART3_0_Type.
  * @param  InputPortId: Input port ID for UART.
  * @param  OutputPortId: Output port ID for UART.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seAUX_ConfigurePortsForUart( UART3_0_Type * UARTx, sePPORT_Id RX, sePPORT_Id TX );

/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */

#ifdef __cplusplus
}
#endif
#endif  // SE_UART3_H

