/**
  ******************************************************************************
  * @file    se_usb.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file provides all the DMA for USB hardware functions.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#include <stdio.h>
#include "se_usb.h"

/**
  * @brief  Enables USB channel by start supplying operating clock.
  * @retval None
  */
void seUSB_Enable( void ) {
 
  USB->CTL_b.USBEN = 1;
}

/**
  * @brief  Disables USB channel by stop supplying operating clock.
  * @retval None
  */
void seUSB_Disable( void ) {
  
  USB->CTL_b.USBEN = 0;
}

/**
  * @brief  Enable USB Interrupt in NVIC.
  * @param  irq: This parameter is of @ref IRQn_Type.
  * @retval None
  */
void seUSB_EnableInt( IRQn_Type irq ) {
  
  NVIC_EnableIRQ( irq );
}

/**
  * @brief  Disables USB Interrupt in NVIC.
  * @param  irq: This parameter is of @ref IRQn_Type.
  * @retval None
  */
void seUSB_DisableInt(  IRQn_Type irq ) {
  
  NVIC_DisableIRQ( irq );
}

/**
  * @brief  This function clears all interrrupt flags.
  * @note   1. Main interrupt are cleared by writing 1 to
  *            corresponding interrupt sources in Sie interrupt Flag Register,
  *            Sie interrupt Flag Register, General-Purpose endpoint interrupt
  *            Flag Register, andEP0 interrupt Flag Register.
  * @note   2. Writing 1 to reserved bits has no effect.
  * @retval None
  */
void seUSB_ClearAllIntFlags( void ) {
  
  UsbClearMainInt( 0xFF );
  UsbClearSIEInt( 0xFF );
  UsbClearGEPInt( 0xFF );
  UsbClearEP0Int( 0xFF );
  UsbClearEPaInt( 0xFF );
  UsbClearEPbInt( 0xFF );
  UsbClearEPcInt( 0xFF );
}

 /**
  * @brief  This function enables all interrrupts
  * @retval None
  */
void seUSB_EnableAllInt( void ) {

  UsbMainIntEnable( 0xFF );
  UsbSIEIntEnable( 0xFF );
  UsbGEPIntEnable( 0xFF );
  UsbEP0IntEnable( 0xFF );
  UsbEPaIntEnable( 0xFF );
  UsbEPbIntEnable( 0xFF );
  UsbEPcIntEnable( 0xFF );
}

 /**
  * @brief  This function disables all interrrupts.
  * @retval None
  */
void seUSB_DisableAllInt( void ) {

  UsbMainIntDisable( 0xFF );
  UsbSIEIntDisable( 0xFF );
  UsbGEPIntDisable( 0xFF );
  UsbEP0IntDisable( 0xFF );
  UsbEPaIntDisable( 0xFF );
  UsbEPbIntDisable( 0xFF );
  UsbEPcIntDisable( 0xFF );
}

/**
  * @brief  Shows VBUS state. It should not change over last 5 times.
  * @retval connected: 1 if Vbus connected and 0 if not.
  */
uint16_t seUSB_IsVbusConnected( void ) {

  uint16_t savestatus = USB->STAT_b.VBUSSTAT;
  int i = 0;
  
  ///< VBUS-check (de-bounce)
  for (i = 0; i < 5; i++) {
    if ( savestatus != USB->STAT_b.VBUSSTAT ) {
      savestatus = USB->STAT_b.VBUSSTAT;
      i = 0;
    }
  }

  return savestatus;
}

/**
  * @brief  The basic configurations for the USB controller.
  *         This function could  be called on Attach event after
  *         clocks are activated.
  * @retval None
  */
void seUSB_InitUsbModule( void ) {
  
  ///< Auto Nak for Ep0
  USB->EP0OCTL_b.AUTOFNAK  = 1;
  ///< 6. Check that the USBSTAT.LINESTAT[1:0] bits are set to 0x1 (J).
  ///< 7. Set the USBCTL.AUTONEGOEN bit to 1. (Enable auto-negotiation)
  USB->TRCTL_b.DPPUEN = 1;
  USB->TRCTL_b.OPMOD = seUSB_TRCTL_OPMODE_NORMAL;
  USB->CTL_b.AUTONEGOEN = 0;
  USB->CTL_b.BUSDETDIS = 1;
  if ( USB->STAT_b.LINESTAT == 1 ) {
    USB->CTL_b.AUTONEGOEN = 1;
  }
  NVIC_SetPriority( USB_IRQn, 3 );
  NVIC_EnableIRQ(USB_IRQn);
}

/**
  * @brief  Perform nessesary actions when VBus is connected.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seUSB_Attach( void ) {

  seStatus fStatus = seSTATUS_NG;
  
  seUSB_ActivateUSBCLK();
  
  if ( !seUSB_IsVbusConnected() ) {
    seUSB_Detach();
  }else {
    seUSB_DisableAllInt();
    seUSB_ClearAllIntFlags();
    UsbMainIntEnable( seUSB_MAIN_INTS( seUSB_MAIN_SIEIF_INT | seUSB_MAIN_EP0IF_INT | seUSB_MAIN_EP0SETIF_INT | seUSB_MAIN_GPEPIF_INT ) );
    UsbSIEIntEnable( seUSB_SIE_INTS( seUSB_SIE_RESETIF_INT | seUSB_SIE_SUSPENDIF_INT | seUSB_SIE_SOFIF_INT | seUSB_SIE_NONJIF_INT ) );

    seUSB_Enable();
    seUSB_ConfigureDefaultEndpoints();
    seUSB_ClearEPnFifos();
    seUSB_InitUsbModule();
    fStatus = seSTATUS_OK;
  }
  
  return fStatus;
}

/**
  * @brief  Perform nessesary actions when VBus is disconnected.
  * @retval None
  */
void seUSB_Detach( void ) {

  USB->CTL_b.AUTONEGOEN = 0;
  USB->TRCTL_b.DPPUEN = 0;
  USB->TRCTL_b.OPMOD = seUSB_TRCTL_OPMODE_NONDRIVING;
  seUSB_Disable();
  seUSB_DeactivateUSBCLK();
}

/**
  * @brief  The basic configurations for the USB endpoints.
  * @retval None
  */
void seUSB_ConfigureDefaultEndpoints( void ) {
  
  USB->EPACFG_b.EPEN = 0;
  USB->EPBCFG_b.EPEN = 0;
  USB->EPCCFG_b.EPEN = 0;
  
  ///< Set EP0 maximum packet size
  USB->EP0SIZE_b.MAXSIZE = seUSB_EPCFG_MAXSIZE_64;
  
  ///< Configure the following USBEPmCFG register bits:
  ///< - USBEPmCFG.MAXSIZE[6:0] bits (Set EPm maximum packet size)
  ///< - USBEPmCFG.DIR bit (Set EPm transaction direction)
  ///< - USBEPmCFG.EPNUM[3:0] bits (Set EPm endpoint number)
  ///< - USBEPmCFG.TGLMOD bit (Set EPm toggle mode)
  ///< - Set the USBEPmCFG.EPEN bit to 1. (Enable EPm)
  ///< Endpoint 1 In
  USB->EPAMAXSZ_b.MAXSIZE = seUSB_EPCFG_MAXSIZE_64;
  USB->EPACFG_b.DIR = seUSB_EPCFG_IN;
  USB->EPACFG_b.EPNUM = 1;
  USB->EPACFG_b.TGLMOD = seUSB_EPCFG_NORMAL;
  ///< Endpoint 2 Out
  USB->EPBMAXSZ_b.MAXSIZE = seUSB_EPCFG_MAXSIZE_64;
  USB->EPBCFG_b.DIR = seUSB_EPCFG_OUT;
  USB->EPBCFG_b.EPNUM = 2;
  USB->EPBCFG_b.TGLMOD = seUSB_EPCFG_NORMAL;
  ///< Endpoint 3 In
  USB->EPCMAXSZ_b.MAXSIZE = seUSB_EPCFG_MAXSIZE_64;
  USB->EPCCFG_b.DIR = seUSB_EPCFG_IN;
  USB->EPCCFG_b.EPNUM = 3;
  USB->EPCCFG_b.TGLMOD = seUSB_EPCFG_NORMAL;

  ///< Setting this bit to 1 configures EPm to get ready to operate.
  ///< When this bit is 0, access to EPm is neglected.
  USB->EPACFG_b.EPEN = 1;
  USB->EPBCFG_b.EPEN = 1;
  USB->EPCCFG_b.EPEN = 1;
  ///< Set the USBFIFORDCYC.RDCYC[1:0] bits. (Set number of FIFO read access cycles)
  USB->FIFORDCYC_b.RDCYC = 1;
}

/**
  * @brief  This function initializes EP0
  * @note   Auto force NAK for EP0
  * @retval None
  */
void seUSB_InitEp0(void) {

  ///< Set Auto force NAK for EP0
  USB->EP0OCTL_b.AUTOFNAK  = 1;
  UsbEP0IntEnable( seUSB_EP0_ALL_INT );
  USB->EPCTL_b.EP0FIFOCLR   = 1;
}

/**
  * @brief  This function sets EP0 direction
  * @param  dir:  1 (IN) or 0 (OUT).
  * @retval None
  */
void seUSB_SetEp0Dir( uint32_t dir ) {

  USB->EP0CFG_b.DIR = dir ? 1 : 0;
}

/**
  * @brief  This function gets EP0 direction.
  * @retval Returns 1 (IN) or 0 (OUT).
  */
uint32_t seUSB_GetEp0Dir ( void ) {

  return( USB->EP0CFG_b.DIR );
}

/**
  * @brief  This function is a place holder for a standard stack call.
  *         It is not needed with this HW.
  * @retval None
  */
void seUSB_SetupStage( void ) {}

/**
  * @brief  This function performs actions conforming with the
  *         Status Out stage.
  * @retval None
  */
void seseUSB_StatusOutStage( void ) {

  ///< The USB controller returns an ACK response.
  USB->EP0CFG_b.DIR = 0;   ///< Configure EP0 to OUT direction

  ///< Set the following OUT transaction control bit in the USBEP0OCTL register:
  USB->EP0OCTL_b.FNAK = 0; ///< Disable forced NAK response
}

/**
  * @brief  This function performs actions conforming with the
  *         Status In stage.
  * @retval None
  */
void seUSB_StatusInStage( void ) {

  ///< The USB controller returns a zero-length packet data.
  USB->EP0CFG_b.DIR = 1; ///< Configure EP0 to IN direction

  ///< Set the following IN transaction control bits in the USBEP0ICTL register
  USB->EP0ICTL_b.SPKTEN = 1; ///< Enable short packet transmission
  USB->EP0ICTL_b.FNAK = 0;   ///< Disable forced NAK response
}

/**
  * @brief  This function configures EP0 to In.
  * @retval None
  */
void seUSB_DataInStage( void ) {

  USB->EP0CFG_b.DIR = 1;     ///< Configure EP0 to in
  USB->EP0ICTL_b.FNAK = 0;   ///< Disable forced NAK response
}

/**
  * @brief  This function configures EP0 to Out.
  * @retval None
  */
void seUSB_DataOutStage( void ) {

  USB->EP0CFG_b.DIR = 0;       ///< Configure EP0 to OUT
  USB->EP0OCTL_b.FNAK = 0;     ///< Disable forced NAK response
}

/**
  * @brief  This function sets general purpose endpoints direction.
  * @param  EPNum: Endpoint number
  * @param  dir: 1 (IN) or 0 (OUT).
  * @retval None
  */
void seUSB_SetEPmDir ( uint32_t EPNum, uint32_t dir ) {
  
  switch ( EPNum  & 0xf ) {
    case 0: USB->EP0CFG_b.DIR = dir; break;
    case 1: USB->EPACFG_b.DIR = dir; break;
    case 2: USB->EPBCFG_b.DIR = dir; break;
    case 3: USB->EPCCFG_b.DIR = dir; break;
    default: break;
  } 
}

/**
  * @brief  This function gets general purpose endpoints direction.
  * @param  EPNum: Endpoint number
  * @retval Returns 1 (IN) or 0 (OUT).
  */
uint32_t seUSB_GetEPmDir ( uint32_t EPNum ) {
  
  uint32_t dir = 0;
  
  switch ( EPNum  & 0xf ) {
    case 0: dir = USB->EP0CFG_b.DIR; break;
    case 1: dir = USB->EPACFG_b.DIR; break;
    case 2: dir = USB->EPBCFG_b.DIR; break;
    case 3: dir = USB->EPCCFG_b.DIR; break;
    default: break;
  }
  
  return dir;
}

/**
  * @brief  Configures a general purpose endpoint.
  * @param  EPNum: Endpoint number
  * @param  val: Endpoint max size
  * @param  dir: Endpoint direction
  * @retval None
  */
void seUSB_ConfigureEPm( uint32_t EPNum, uint32_t val, uint32_t dir ) {

  switch ( EPNum  & 0xf ) {
    case 0:
    USB->EP0SIZE_b.MAXSIZE = val>>3; ///< Ep0 size register is different from Epm size registers.
                                     ///< See explanation in the sePeripheral Library document.
    USB->EP0CFG_b.DIR = dir;
      break;
    case 1:
    USB->EPAMAXSZ_b.MAXSIZE = val;
    USB->EPACFG_b.DIR = dir;
    USB->EPACFG_b.EPNUM = EPNum;
    USB->EPACFG_b.TGLMOD = seUSB_EPCFG_NORMAL;
      break;
    case 2:
    USB->EPBMAXSZ_b.MAXSIZE = val;
    USB->EPBCFG_b.DIR = dir;
    USB->EPBCFG_b.EPNUM = EPNum;
    USB->EPBCFG_b.TGLMOD = seUSB_EPCFG_NORMAL;
      break;
    case 3:
    USB->EPCMAXSZ_b.MAXSIZE = val;
    USB->EPCCFG_b.DIR = dir;
    USB->EPCCFG_b.EPNUM = EPNum;
    USB->EPCCFG_b.TGLMOD = seUSB_EPCFG_NORMAL;
      break;
    default: break;
  }
}

/**
  * @brief  This function resets general purpose endpoints.
  * @param  EPNum: Endpoint number
  * @retval None
*/
void seUSB_ResetEPm( uint16_t EPNum ) {

  switch ( EPNum  & 0xf ) {
    case 0: 
      break;
    case 1:
      USB->EPACTL = ( USB->EPACFG_b.DIR == seUSB_EPCFG_IN ) ?
        ( USB_EPACTL_FNAK_Msk | USB_EPACTL_AUTOFNAK_Msk | USB_EPACTL_SPKTEN_Msk | USB_EPACTL_TGLCLR_Msk ) :
        ( USB_EPACTL_TGLCLR_Msk );
      break;
    case 2:
      USB->EPBCTL = ( USB->EPBCFG_b.DIR == seUSB_EPCFG_IN ) ?
        ( USB_EPBCTL_AUTOFNAK_Msk | USB_EPACTL_AUTOFNAK_Msk| USB_EPBCTL_SPKTEN_Msk | USB_EPBCTL_TGLCLR_Msk ) :
        ( USB_EPBCTL_TGLCLR_Msk );
      break;
    case 3:
       USB->EPCCTL = ( USB->EPCCFG_b.DIR == seUSB_EPCFG_IN ) ?
        ( USB_EPCCTL_AUTOFNAK_Msk | USB_EPCCTL_SPKTEN_Msk | USB_EPCCTL_TGLCLR_Msk ) :
        ( USB_EPCCTL_TGLCLR_Msk );
      break;
    default:break;
  }
  seUSB_ClearEPmFifo( EPNum  & 0xf );
}

/**
  * @brief  Enables a general purpose endpoint.
  * @param  EPNum: Endpoint number
  * @retval None
  */
void seUSB_EnableEPm( uint32_t EPNum ) {

  switch ( EPNum  & 0xf ) {
    case 1: USB->EPACFG_b.EPEN = 1; UsbEPaIntEnable( 0x3C ); UsbGEPIntEnable( seUSB_GPE_EPAIF_INT); break;
    case 2: USB->EPBCFG_b.EPEN = 1; UsbEPbIntEnable( 0x3C ); UsbGEPIntEnable( seUSB_GPE_EPBIF_INT); break;
    case 3: USB->EPCCFG_b.EPEN = 1; UsbEPcIntEnable( 0x3C ); UsbGEPIntEnable( seUSB_GPE_EPCIF_INT); break;
    default: break;
  } 
}

/**
  * @brief  Disables a general purpose endpoint.
  * @param  EPNum: Endpoint number
  * @retval None
  */
void seUSB_DisableEPm( uint32_t EPNum ) {

  switch ( EPNum  & 0xf ) {
    case 1: USB->EPACFG_b.EPEN = 0; UsbEPaIntDisable( 0xFF ); UsbGEPIntEnable( seUSB_GPE_EPAIF_INT); break;
    case 2: USB->EPBCFG_b.EPEN = 0; UsbEPbIntDisable( 0xFF ); UsbGEPIntEnable( seUSB_GPE_EPBIF_INT); break;
    case 3: USB->EPCCFG_b.EPEN = 0; UsbEPcIntDisable( 0xFF ); UsbGEPIntEnable( seUSB_GPE_EPCIF_INT); break;
    default: break;
  }
}

/**
  * @brief  This function resets general purpose endpoints.
  * @param  EPNum: Endpoint number
  * @retval None
  */
void seUSB_Reset( void ) {
  
  USB->ADDR_b.USBADDR = 0;
}

/**
  * @brief  In suspend mode make sure the NONJIF interrrupt is enabled.
  * @retval None
  */
void seUSB_Suspend( void ) {
  
  UsbClearSIEInt( seUSB_SIE_NONJIF_INT );
  UsbSIEIntEnable( seUSB_SIE_NONJIF_INT );
}

/**
  * @brief  The host requests to return from Suspend.
  *         In this case, the USBCTL.NONJDETEN bit should be cleared.
  * @retval None
  */
void seUSB_Resume( void ) {

  if ( seUSB_IsVbusConnected() ) {
    seUSB_Snooze( seDISABLE );
    USB->CTL_b.NONJDETEN = 0;
  }
}

/**
  * @brief  The USB controller has a Snooze function to reduce current
  *         consumption when it is not active or in Suspend state.
  *         When the SNOOZE signal is negated, the USB controller resumes operating 
  *         after 5 clocks have elapsed. (The oscillation must be stabilized at this time.)
  *         Notes: Be sure to avoid accessing to the USB controller for five 48-MHz clock cycles
  *                from the SNOOZE signal being asserted/negated. Be sure to avoid accessing to the FIFO
  *                while the 48 MHz clock is stopped after the SNOOZE signal is asserted.
  * @param  en: see @ref seState
  * @retval None
  */
void seUSB_Snooze( seState en ) {

  ///< Disable write-protect.
  seProtectSys( seWRITE_PROTECT_OFF );
  if ( en == seDISABLE ) {
    ///<<<USBMISC->CTL_b.USBCLKEN = 1;
  }
  USBMISC->CTL_b.USBSNZ = en;

  ///< Avoid accessing to the USB controller 
  ///< for five 48-MHz clock cycles from the
  ///< SNOOZE signal being asserted/negated
  ///< The fastest clock CPU can execute on IOSC is 24 MHz
  ///< therefore three clocks would be sufficient (fettching from RAM, FLASH or cache).
  __NOP();
  __NOP();
  __NOP();
  
  if ( en == seENABLE ) {
    ///<<<USBMISC->CTL_b.USBCLKEN = 0;
  }
  ///< Enable write-protect.
  seProtectSys( seWRITE_PROTECT_ON );
}

/**
  * @brief  Reports USB controller snooze status.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seUSB_IsSnoozing( void ) {

  return (seStatus)USBMISC->CTL_b.USBSNZ;
}

/**
  * @brief  To resume from Suspend with a remote wake-up
  *         set controls the remote wakeup signal (K) output.
  *         The bit is set to 1 and in 15 msecs set to 0.
  * @retval None
  */
void seUSB_WakeUp( void ) {
  
  if ( seUSB_IsSnoozing() ) {
    seUSB_Snooze( seDISABLE );
  }
  
  USB->CTL_b.WAKEUP = 1;
  
  seSysSleepMS(50);
  
  USB->CTL_b.WAKEUP = 0;
  USB->CTL_b.NONJDETEN = 0;
}

/**
  * @brief  Auto address setup, adr argument ignored.
  * @param  adr: ignored
  * @param  setup: 1 set auto address mode
  * @retval None
  */
void seUSB_SetAddress ( uint32_t  adr, uint32_t setup ) {

  if ( setup ) {
    USB->ADDR_b.ATADDR  = 1; ///< Auto address setup, adr argument ignored.
  }
}

/**
  * @brief  This function reads back the device address.
  * @retval addr: Device address.
  */
uint32_t seUSB_GetAddress ( void ) {

  return USB->ADDR_b.USBADDR;
}

/**
  * @brief  This function enables USB controller operations
  *         and  enables the USB_DP pin (D+ line) pull-up resistor
  * @retval None
  */
void seUSB_Connect( void ) {

  seUSB_Enable();
  USB->TRCTL_b.DPPUEN = 1;
}

/**
  * @brief  This function disables USB controller operations
  *         and disables the USB_DP pin (D+ line) pull-up resistor
  * @retval None
  */
void seUSB_Disconnect( void ) { 

  seUSB_Disable();
  USB->TRCTL_b.DPPUEN = 0;
}

/**
  * @brief  This function clears all FIFOs at once.
  * @retval None
  */
void seUSB_ClearEPnFifos( void ) {

  USB->EPCTL_b.EPNFIFOCLR   = 1;
}

/**
  * @brief  This function clears EP0 FIFO
  * @retval None
  */
void seUSB_ClearEP0Fifo( void ) {

  USB->EPCTL_b.EP0FIFOCLR   = 1;
}

/**
  * @brief  This function clears general purpose endpoints  FIFO.
  * @param  EPNum: Endpoint number
  * @retval None
  */
void seUSB_ClearEPmFifo( uint16_t EPNum ) {

  switch ( EPNum & 0xf ) {
    case 1: USB->GPEPFIFOCLR_b.EPAFIFOCLR = 1; break;
    case 2: USB->GPEPFIFOCLR_b.EPBFIFOCLR = 1; break;
    case 3: USB->GPEPFIFOCLR_b.EPCFIFOCLR = 1; break;
    default: break;
  }
}

#ifdef __IAR_SYSTEMS_ICC__
#pragma optimize=none
#endif
#ifdef __CC_ARM
#pragma push
#pragma O0
#endif

/**
  * @brief  Set read data endpoint.
  * @param  EPNum: Endpoint number.
  * @retval None
  */
void seUSB_SetReadDataEndpoint( uint16_t EPNum ) {

  ///< Attention:
  ///< No reading from the CPU is allowed from the same endpoint
  ///< while the USB host is reading from the FIFO.

  ///< Set the USBFIFORWEN.FIFORDEN bit to 1. (Enable USBRDFIFOSEL register)
  USB->FIFORWEN_b.FIFORDEN = 1;
  USB->FIFORWEN_b.FIFOWREN = 0;
  ///< Set the USBRDFIFOSEL.EPmRD bit. (Select endpoint to read data)
  ///< before reading data, set
  ///< the USBEP0ICTL.FNAK or USBEPmCTL.FNAK bit so that the USB controller will not respond
  ///< to an IN transaction and transmit data.
  switch ( EPNum & 0xf ) {
    case 0: USB->RDFIFOSEL = 0;         USB->EP0OCTL_b.FNAK = 1; break;
    case 1: USB->RDFIFOSEL_b.EPARD = 1; USB-> EPACTL_b.FNAK = 1; break;
    case 2: USB->RDFIFOSEL_b.EPBRD = 1; USB-> EPBCTL_b.FNAK = 1; break;
    case 3: USB->RDFIFOSEL_b.EPCRD = 1; USB-> EPCCTL_b.FNAK = 1; break;
    default: break;
  }
}

/**
  * @brief  Clear read data endpoint.
  * @param  EPNum: Endpoint number.
  * @retval None
  */
void seUSB_ClearReadDataEndpoint(  uint16_t EPNum  ) {

  switch ( EPNum & 0xf ) {
    case 0: USB->EP0OCTL_b.FNAK = 0; break;
    case 1: USB->EPACTL_b.FNAK = 0; break;
    case 2: USB->EPBCTL_b.FNAK = 0; break;
    case 3: USB->EPCCTL_b.FNAK = 0; break;
    default: break;
  }
}

/**
  * @brief  Set write data endpoint.
  * @param  EPNum: Endpoint number.
  * @retval None
  */
void seUSB_SetWriteDataEndpoint( uint16_t EPNum ) {

  ///< Read the USBREMSPCCNT.REMSPC[6:0] bits.) (Check available space in FIFO)
  ///< Attention:
  ///< From the CPU, no writing is allowed to the same endpoint while the
  ///< USB host is writing data to the FIFO.
  ///< Set the USBFIFORWEN.FIFOWREN bit to 1. (Enable USBWRFIFOSEL register)
  USB->FIFORWEN_b.FIFOWREN = 1;
  USB->FIFORWEN_b.FIFORDEN = 0;

  ///< Set the USBWRFIFOSEL.EPmWR bit. (Select endpoint to write data)
  ///< before writing data, set the
  ///< USBEP0OCTL.FNAK or USBEPmCTL.FNAK bit so that the USB
  ///< controller will not receive data in an OUT transaction.
  switch ( EPNum & 0xf ) {
    case 0: USB->WRFIFOSEL = 0;        USB->EP0ICTL_b.FNAK = 1; break;
    case 1: USB->WRFIFOSEL_b.EPAWR = 1; USB->EPACTL_b.FNAK = 1; break;
    case 2: USB->WRFIFOSEL_b.EPBWR = 1; USB->EPBCTL_b.FNAK = 1; break;
    case 3: USB->WRFIFOSEL_b.EPCWR = 1; USB->EPCCTL_b.FNAK = 1; break;
    default: break;
  }

}

/**
  * @brief  Clear read data endpoint.
  * @param  EPNum: Endpoint number.
  * @param  size: written bytes.
  * @retval None
  */
void seUSB_ClearWriteDataEndpoint( uint16_t EPNum, uint16_t size ) {
  
  uint8_t SHRT = ( size < (USB->EP0SIZE_b.MAXSIZE)<<3 )?1:0;

  ///< Clear the USBEPmCTL. FNAK bit since the next data transfer is ready.
  switch ( EPNum & 0xf ) {
    case 0:  USB->EP0ICTL_b.SPKTEN = SHRT; USB->EP0ICTL_b.FNAK = 0; break;
    case 1:  USB->EPACTL_b.SPKTEN = SHRT; USB->EPACTL_b.FNAK = 0; break;
    case 2:  USB->EPBCTL_b.SPKTEN = SHRT; USB->EPBCTL_b.FNAK = 0; break;
    case 3:  USB->EPCCTL_b.SPKTEN = SHRT; USB->EPCCTL_b.FNAK = 0; break;
    default: break;
  } 
}

/**
  * @brief  Reads available data from FIFO.
  * @param  EPNum: Endpoint number.
  * @param  buf: Pointer to a destination byte array.
  * @param  size: Destination array size.
  * @retval Returns a number of bytes read.
  */
uint16_t seUSB_ReadFifo( uint16_t EPNum, uint8_t* buf, uint16_t size ) {

  ///< If the FIFO has available space for receiving data packets, the USB controller automatically responds to OUT
  ///< transactions to receive data. This enables software to perform OUT transfer without individual transaction control.
  ///< Note, however, that the USBEPmCTL.FNAK bit of the endpoint is set if a short packet is received (including
  ///< zero-length data packet) when the USBEPmCTL.AUTOFNAKDIS bit is cleared. Clear the USBEPmCTL.
  ///< FNAK bit when the next data transfer is ready.

  ///< Attention:
  ///<No reading from the CPU is allowed from the same endpoint
  ///<while the USB host is reading from the FIFO.
  ///< Set the USBRDFIFOSEL.EPmRD bit. (Select endpoint to read data)
  ///< before reading data, set
  ///< the USBEP0ICTL.FNAK or USBEPmCTL.FNAK bit so that the USB controller will not respond
  ///< to an IN transaction and transmit data.
  seUSB_SetReadDataEndpoint( EPNum );

  ///< Read the USBREMDATCNT.REMDAT[6:0] bits. (Check number of valid FIFO data)
  uint16_t count = USB->REMDATCNT_b.REMDAT;
  uint16_t i = 0;
  ///< Read FIFO data through the USBFIFODAT.FIFODAT[7:0] bits.
  ///< By repeatedly executing Step 4, data in the FIFO of the specified endpoint
  ///< can be read by one byte at a time in the order of reception
  if ( count != 0 ) {
    do {
      if ( i >= size ) {
        break;
      }
      buf[i++] = USB->FIFODAT_b.FIFODAT;
    }while ( i < count );
  }

  seUSB_ClearReadDataEndpoint( EPNum );
 
  return (i);
}

#ifdef __CC_ARM
#pragma pop
#endif

/**
  * @brief  Writes data to available space in FIFO.
  * @param  EPNum: Endpoint number.
  * @param  buf: Pointer to a source byte array.
  * @param  size: Source array size.
  * @retval Returns a number of bytes written.
  */
uint16_t seUSB_WriteFifo( uint16_t EPNum, const uint8_t* buf, uint16_t size ) {

  ///< Read the USBREMSPCCNT.REMSPC[6:0] bits.) (Check available space in FIFO)
  ///< Attention:
  ///< From the CPU, no writing is allowed to the same endpoint while the
  ///< USB host is writing data to the FIFO.
  ///< Set the USBFIFORWEN.FIFOWREN bit to 1. (Enable USBWRFIFOSEL register)
  ///< Set the USBWRFIFOSEL.EPmWR bit. (Select endpoint to write data)
  ///< before writing data, set the
  ///< USBEP0OCTL.FNAK or USBEPmCTL.FNAK bit so that the USB
  ///< controller will not receive data in an OUT transaction.
  seUSB_SetWriteDataEndpoint( EPNum );
 
  volatile uint16_t count = USB->REMSPCCNT_b.REMSPC;
  uint16_t i = 0;

  ///< Write data to the FIFO through the USBFIFODAT.FIFODAT[7:0] bits.
  ///< By repeatedly executing Step 4, data can be written to the FIFO of the specified
  ///< endpoint by one byte at a time. The data written to the FIFO will be transmitted
  ///< in data packets in the order of writing.
  if ( count != 0 ) {
    do {
      if ( i >= size ) {
        break;
      }
      USB->FIFODAT_b.FIFODAT = buf[i++];
    }while ( i < count );
  }

  if ( count != 0 ) {
    ///< Clear the USBEPmCTL. FNAK bit since the next data transfer is ready.
    seUSB_ClearWriteDataEndpoint( EPNum, size );
  }
  return i;
}

/**
  * @brief  Sets Stall.
  * @param  EPNum: Endpoint number
  * @retval None
  */
void seUSB_SetStall( uint32_t EPNum ) {

  switch ( EPNum  & 0xf ) {
    case 1: USB->EPACTL_b.FSTALL = 1; break;
    case 2: USB->EPBCTL_b.FSTALL = 1; break;
    case 3: USB->EPCCTL_b.FSTALL = 1; break;
    default: break;
  }
}

/**
  * @brief  Clears Stall.
  * @param  EPNum: Endpoint number
  * @retval None
  */
void seUSB_ClrStall( uint32_t EPNum ) { 

  switch ( EPNum & 0xf ) {
    case 1: USB->EPACTL_b.FSTALL = 0; break;
    case 2: USB->EPBCTL_b.FSTALL = 0; break;
    case 3: USB->EPCCTL_b.FSTALL = 0; break;
    default: break;
  }
}

/**
  * @brief  This function reads and parses control packet.
  * @param  packet: This parameter is of @ref USB_SETUP_PACKET
  * @retval None
  */
void seUSB_GetSetupPacket( USB_SETUP_PACKET *packet ) {

  ///< Read Request type
  *((uint8_t*)&packet->bmRequestType) = USB->EP0SETUP0;

  ///< Read Request number
  packet->bRequest = USB->EP0SETUP1_b.BREQ;

  ///< Read Value
  packet->wValueL = USB->EP0SETUP2_b.WVAL;
  packet->wValueH = USB->EP0SETUP3_b.WVAL;

  ///< Read Index
  packet->wIndexL = USB->EP0SETUP4_b.WINDX;
  packet->wIndexH = USB->EP0SETUP5_b.WINDX;

  ///< Read Length
  packet->wLength = USB->EP0SETUP6_b.WLEN;
  packet->wLength |= (USB->EP0SETUP7_b.WLEN)<<8;
}

/**
  * @brief  Turn off pll and stop supplying USB clock.
  *         This function is used When using PLL for USB circuit.
  * @retval None
  */
void seUSB_DeactivateUSBCLK( void ) {

  ///< Disable write-protect.
  seProtectSys( seWRITE_PROTECT_OFF );
  
  USBMISC->CTL = 0;
  
  ///< Enable write-protect.
  seProtectSys( seWRITE_PROTECT_ON );
}

