/**
  ******************************************************************************
  * @file    se_wdt2.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file provides all the Watchdog hardware functions.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#include "se_wdt2.h"

/**
  * @brief  Fills each seWDT_InitTypeDef member with its default value.
  * @param  InitStruct: pointer to an @ref seWDT_InitTypeDef structure which will be initialized.
  * @retval None
  */
void seWDT_InitStruct( seWDT_InitTypeDef* InitStruct ) {
   
  InitStruct->Clk  = seDEFAULT_WDT_CLK;    ///< Set the clock source of WDT2.
  InitStruct->mode = seWDT_NMIMode;        ///< Set the WDT2 operating mode.
  InitStruct->CMP  = 0x3FF;                ///< seconds
}

/**
  * @brief  Initializes the WDT peripheral according to the specified
  *         parameters in the seWDT_InitStruct.
  * @note   This function configures the module, and module's interrupts.
  *         It clears module's interrupts
  *         but does not enable interrupt from the module to CPU.
  * @param  InitStruct: pointer to a @ref seWDT_InitTypeDef structure that
  *         contains the configuration information for the specified WDT peripheral.
  * @retval Status: see @ref seStatus
  */
seStatus seWDT_Init(  seWDT_InitTypeDef* InitStruct ) {
  
  seStatus fStatus = seCLG_IsEnabled(seSRC(InitStruct->Clk));
  
  ///< Check if selected clock is enabled
  if ( fStatus == seSTATUS_OK ) { 
    ///< Stop WDT.
    seWDT_Stop();
    
    ///< Configure the WDT operating clock.
    seWDT_ConfigureClock( InitStruct->Clk );

    ///< Set desirable wdt cycle.
    seWDT_SetCMP( InitStruct->CMP ); 
    
    ///< Set the WDT2 operating mode.
    seWDT_SetMode( InitStruct->mode );
    
  } 
    
  return fStatus;
}

/**
  * @brief  Stops Watchdog Timer overflow counter.
  * @retval None
  */
void seWDT_Stop( void ) {
  
  ///< Disable write-protect.
  seProtectSys( seWRITE_PROTECT_OFF );
  ///< Stop Timer
  WDT2->CTL_b.WDTRUN = 0xA;
  ///< Enable write-protect.
  seProtectSys( seWRITE_PROTECT_ON );
}

/**
  * @brief  Starts Watchdog Timer overflow counter.
  * @retval Status: see @ref seStatus
  */
seStatus seWDT_Start( void ) {
  
  seStatus fStatus = seSTATUS_OK;
  
  ///< Disable write-protect.
  seProtectSys( seWRITE_PROTECT_OFF );
  WDT2->CTL_b.WDTCNTRST = 1;  ///< Reset WDT before resuming operations.
  WDT2->CTL_b.WDTRUN  = 0x1;  ///< Wait here while WDTRUN bit goes to 0, sampling on 32 KHz
  WHILE( WDT2->CTL & WDT2_CTL_WDTRUN_Msk, fStatus );
  ///< Enable write-protect.
  seProtectSys( seWRITE_PROTECT_ON );
  
  return fStatus;
} 

/**
  * @brief  Sets Watchdog Timer mode.
  * @param  mode: This parameter defines Watchog timer operating mode and can be a value of @ref seWDT_Mode.
  * @retval None
  */
void seWDT_SetMode( seWDT_Mode mode ) {

  ///< Disable write-protect.
  seProtectSys( seWRITE_PROTECT_OFF );
  WDT2->CTL_b.MOD = mode; 
  ///< Enable write-protect.
  seProtectSys( seWRITE_PROTECT_ON );
}

/**
  * @brief  Configures Watchdog timer clock source and clock divider.
  * @param  clock: This parameter can be a value of @ref seWDT_Clk.
  * @retval None
  */
void seWDT_ConfigureClock( seWDT_Clk clock ) {
  
  ///< Disable write-protect.
  seProtectSys( seWRITE_PROTECT_OFF );
  WDT2->CLK_b.CLKSRC = seSRC(clock);  ///< Clock source selection
  WDT2->CLK_b.CLKDIV = seDIV(clock);  ///< Clock division ratio selection
  WDT2->CLK_b.DBRUN  = seDBRUN;       ///< Whether clock supplied in DEBUG mode or not.
  ///< Enable write-protect.
  seProtectSys( seWRITE_PROTECT_ON );
}

/**
  * @brief  Gets Watchdog Timer overflow counter cycle value.
  * @retval vale: 10-bit counter cycle value in seconds.
  */
uint16_t seWDT_Get_tWDTSecs( void ) {
  
  uint32_t clk_wdt = seAUX_GetClkSpeedForWDT();
  uint16_t tWDT = 0; 
  if ( clk_wdt != 0 ) {
    tWDT = ( seWDT_GetCMP()+1 )/ clk_wdt;
  }
  
  return tWDT;
}

/**
  * @brief  Resets Watchdog Timer overflow counter.
  * @retval None
  */
void seWDT_ResetCounter( void ) {
  
  ///< Disable write-protect.
  seProtectSys( seWRITE_PROTECT_OFF );
  ///< Reset WDT.
  WDT2->CTL_b.WDTCNTRST = 1;
  ///< Enable write-protect.
  seProtectSys( seWRITE_PROTECT_ON );
}

/**
  * @brief  Sets Comparator value.
  * @param  value: This parameter defines 10-bit comparator value.
  * @retval None
  */
void seWDT_SetCMP( uint16_t value ) {
  
  ///< Disable write-protect.
  seProtectSys( seWRITE_PROTECT_OFF );
  ///< Set wdt cycle.
  WDT2->CMP_b.CMP = value;
  ///< Enable write-protect.
  seProtectSys( seWRITE_PROTECT_ON );
}

/**
  * @brief  Gets Comparator value.
  * @retval value: 10-bit comparator value.
  * @retval None
  */
uint16_t seWDT_GetCMP( void ) {
  
  return  WDT2->CMP_b.CMP;  
}

/**
  * @brief  Resets Chip and Chip hardware.
  *         The reset actions depend on the selected WDT mode.
  * @param  mode: could be a value of @ref seWDT_Mode
  * @retval None
  */
void seWDT_ChipReset( seWDT_Mode mode ) {
  
  ///< Disable write-protect.
  seProtectSys( seWRITE_PROTECT_OFF );
  ///< Stop Timer
  WDT2->CTL_b.WDTRUN = 0xA;
  ///< Set wdt cycle.
  WDT2->CMP_b.CMP = 250;
  ///< Set the WDT2 operating mode.
  WDT2->CTL_b.MOD = mode;
  ///< Reset WDT.
  WDT2->CTL_b.WDTCNTRST = 1;
  ///< Start Timer
  WDT2->CTL_b.WDTRUN = 0x1;
  ///< Enable write-protect.
  seProtectSys( seWRITE_PROTECT_ON );
}

