/**
  ******************************************************************************
  * @file    se_wdt2.h
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file contains all the functions prototypes for the Watchdog firmware
  *          library.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#ifndef SE_WDT2_H
#define SE_WDT2_H

#include "se_common.h"

#ifdef __cplusplus
 extern "C" {
#endif

/** @addtogroup sePeriphLibrary
  * @{
  */

/** @defgroup WDT2
  * @{
  * @brief WDT2 restarts the system if a problem occurs, such as when the program cannot be executed normally.
  */

/** @defgroup WDT2_Constants
  * @{
  */

typedef enum {
   seWDT_NMIMode = 1,                   ///< Watchdog Timer operates in NMI mode or Reset mode. Select this operating mode to generate overflow NMI interrupt.
   seWDT_ResetMode = 0,                 ///< Watchdog Timer operates in Reset Mode. Select this operating modeto generate reset on timer overflow event.
   seWDT_ResetAndNMIMode = 2            ///< Watchdog Timer operates in Reset and NMI Mode.
                                        ///< If the STATNMI bit is not cleared to 0 after an NMI interrupt
                                        ///< has occurred due to a counter compare match, WDT2 issues
                                        ///< a reset when the next compare match occurs.
} seWDT_Mode;

/**
  * @}
  */

/** @defgroup WDT2_Types
  * @{
  */

/** 
  * @brief  WDT Init structure definition
  * @note   This structure is used with all WDT but not T16B.
  */

typedef struct
{
   seWDT_Clk Clk;             ///< Specifies the timer clock source selection and the prescaler value used to divide the WDT clock.
                              ///< The clock frequency should be set to around 256 Hz.
   uint16_t CMP;              ///< Specifies Comparator cycle.
   seWDT_Mode mode;           ///< Choose between NMI or Reset mode.
} seWDT_InitTypeDef;

/**
  * @}
  */

/** @defgroup WDT2_Functions
  * @{
  */

/**
  * @brief  Fills each seWDT_InitTypeDef member with its default value.
  * @param  InitStruct: pointer to an @ref seWDT_InitTypeDef structure which will be initialized.
  * @retval None
  */
void seWDT_InitStruct( seWDT_InitTypeDef* InitStruct );

/**
  * @brief  Initializes the WDT peripheral according to the specified
  *         parameters in the seWDT_InitStruct.
  * @note   This function configures the module, and module's interrupts.
  *         It clears module's interrupts
  *         but does not enable interrupt from the module to CPU.
  * @param  InitStruct: pointer to a @ref seWDT_InitTypeDef structure that
  *         contains the configuration information for the specified WDT peripheral.
  * @retval Status: see @ref seStatus
  */
seStatus seWDT_Init( seWDT_InitTypeDef* InitStruct );

/**
  * @brief  Starts Watchdog Timer overflow counter.
  * @retval Status: see @ref seStatus
  */
seStatus seWDT_Start( void );

/**
  * @brief  Stops Watchdog Timer overflow counter.
  * @retval None
  */
void seWDT_Stop( void );

/**
  * @brief  Resets Watchdog Timer overflow counter.
  * @retval None
  */
void seWDT_ResetCounter( void );

/**
  * @brief  Configures Watchdog timer clock source and clock divider.
  * @param  clock: This parameter can be a value of @ref seWDT_Clk.
  * @retval None
  */
void seWDT_ConfigureClock( seWDT_Clk clock );

/**
  * @brief  Sets Watchdog Timer overflow counter cycle to provide tWDT in seconds.
  *         The function finds the selected clock dividers matching requested cycle value.
  *         No change is done to the clock dividers if desired timing is not possible.
  * @param  tWDT: This parameter can be a value less or equal to 4.
  * @retval Status: see @ref seStatus
  */
seStatus seWDT_Set_tWDTSecs( uint16_t tWDT );

/**
  * @brief  Gets Watchdog Timer overflow counter cycle value.
  * @retval vale: 10-bit counter cycle value in seconds.
  */
uint16_t seWDT_Get_tWDTSecs( void );

/**
  * @brief  Sets Watchdog Timer mode.
  * @param  mode: This parameter defines Watchog timer operating mode and can be a value of @ref seWDT_Mode.
  * @retval None
  */
void seWDT_SetMode( seWDT_Mode mode );

/**
  * @brief  Sets Comparator value.
  * @param  value: This parameter defines 10-bit comparator value.
  * @retval None
  */
void seWDT_SetCMP( uint16_t value );

/**
  * @brief  Gets Comparator value.
  * @retval value: 10-bit comparator value.
  * @retval None
  */
uint16_t seWDT_GetCMP( void );

/**
  * @brief  Resets Chip and Chip hardware.
  *         The reset actions depend on the selected WDT mode.
  * @param  mode: could be a value of @ref seWDT_Mode
  * @retval None
  */
void seWDT_ChipReset( seWDT_Mode mode );

/**
  * @brief  Discovers WDT clock from registers.
  * @retval Hz: WDT clock in Hz.
  */
uint32_t seAUX_GetClkSpeedForWDT( void );

/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */

#ifdef __cplusplus
}
#endif
#endif  //SE_WDT2_H

