/* CMSIS-DAP Interface Firmware
 * Copyright (c) 2009-2015 ARM Limited
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#include <stdio.h>
#include "RTL.h"
#include "rl_usb.h"
#include "usb_config.c"
#include "board.h"
#include "S1C31.h"
#include "port_config.h"
#include CLG_module_header
#include PPORT_module_header
#include SVD_module_header
#include USB_module_header
#include DMAC_module_header
#include DMAUSB_module_header

uint32_t gFrameNumber;
uint16_t gSetupToken;
uint16_t gRemoteWakeUpConfigured;
uint8_t  gEpSize[USBD_EP_NUM + 1];
#if defined(USB_WITH_DMA) && defined(DMAUSB_module_header)
#define seUSB_DMACH_IN seDMAC_CH1      ///< Reserve this DMAC channel to copy IN packets. 
#define seUSB_DMACH_OUT seDMAC_CH0     ///< Reserve this DMAC channel to copy OUT packets.
static uint8_t DMA_ARRAY[0x200];
#include DMAUSB_module_header
#endif


static void handleEpaInt( void );
static void handleEpbInt( void );
static void handleEpcInt( void );

/*
 *  USB Device Initialize Function
 *   Called by the User to initialize USB
 *   Function does low level initialization 
 *   by starting nessesary clocks. 
 *   Return Value:    None
 */
void USBD_Init( void ) {
  
  gEpSize[0] = USBD_MAX_PACKET0;
  
  // Set and start clock for USB
  seAUX_SetClkForUSB( seDEFAULT_USB_CLK );
  
  // Prepare USB to sense attach action by PPORT interrupt
  seCLG_Start( seSRC(seDEFAULT_PPORT_CLK) );
  sePPORT_ConfigureClock( seDEFAULT_PPORT_CLK );


  sePPORT_DisableInt(USB_PORT_ID);
  sePPORT_InitAsInput(USB_PORT_ID);
  sePPORT_EnablePullDownResistor(USB_PORT_ID);
  sePPORT_EnableInt(USB_PORT_ID, sePPORT_INT_EDGE_RISING);   
  NVIC_SetPriority (USB_PORT_IRQ, 2 );    
  NVIC_EnableIRQ(USB_PORT_IRQ);

#if defined(USB_WITH_DMA) && defined(DMAUSB_module_header)
  seDMAUSB_Init( (uint32_t)DMA_ARRAY, seUSB_DMACH_IN, seUSB_DMACH_OUT );
#endif
}

/*
 *  USB Device Connect Function
 *   Called by the User to Connect/Disconnect USB Device
 *    Parameters:      con:   Connect/Disconnect
 *    Return Value:    None
 */
void USBD_Connect( uint32_t con ) {
  
  if ( con ) 
    seUSB_Connect();
  else 
    seUSB_Disconnect();

}

/*
 *  USB Device Reset Function
 *   Called automatically on USB Device Reset
 *    Return Value:    None
 */
void USBD_Reset( void ) {
  
  gFrameNumber = 0;
  gSetupToken = 0;
  gRemoteWakeUpConfigured = 0;
  seUSB_Reset();
  seUSB_ClearEPnFifos();
  seUSB_InitEp0();
  seUSB_ClearAllIntFlags();

}

/*
 *  USB Device Suspend Function
 *   Called automatically on USB Device Suspend
 *    Return Value:    None
 */
void USBD_Suspend( void ) {

  seUSB_Suspend();

}

/*
 *  USB Device Resume Function
 *   Called automatically on USB Device Resume
 *    Return Value:    None
 */
void USBD_Resume( void ) {

  seUSB_Resume();

}

/*
 *  USB Device Remote Wakeup Function
 *   Called automatically on USB Device Remote Wakeup
 *    Return Value:    None
 */
void USBD_WakeUp( void ) {
  
  if ( gRemoteWakeUpConfigured ) {
    seUSB_WakeUp();
  }
}

/*
 *  USB Device Remote Wakeup Configuration Function
 *    Parameters:      cfg:   Device Enable/Disable
 *    Return Value:    None
 */
void USBD_WakeUpCfg( uint32_t cfg ) {
  
  gRemoteWakeUpConfigured  = cfg;

}

/*
 *  USB Device Set Address Function
 *    Parameters:      adr:   USB Device Address
 *    Return Value:    None
 */
void USBD_SetAddress( uint32_t  adr, uint32_t setup ) {

  seUSB_SetAddress( adr, setup );

}

/*
 *  USB Device Configure Function
 *    Parameters:      cfg:   Device Configure/Deconfigure
 *    Return Value:    None
 */
void USBD_Configure( uint32_t cfg ) {
  
  /* Not needed */

}

/*
 *  Configure USB Device Endpoint according to Descriptor
 *    Parameters:      pEPD:  Pointer to Device Endpoint Descriptor
 *    Return Value:    None
 */
void USBD_ConfigEP( USB_ENDPOINT_DESCRIPTOR *pEPD ) {

  uint32_t num, val, dir;

  num = pEPD->bEndpointAddress & 0x7; //  may be 0xf
  val = pEPD->wMaxPacketSize;
  dir = 1;

  if ( !( pEPD->bEndpointAddress & 0x80 ) ) {
    gEpSize[num] = val;
    dir = 0;
  }  
  seUSB_ConfigureEPm( num, val, dir );
  USBD_ResetEP( num );

}

/*
 *  Set Direction for USB Device Control Endpoint
 *    Parameters:      dir:   Out (dir == 0), In (dir = 1)
 *    Return Value:    None
 */
void USBD_DirCtrlEP( uint32_t dir ) {

  seUSB_SetEp0Dir(dir);

}

/*
 *  Enable USB Device Endpoint
 *    Parameters:      EPNum: Device Endpoint Number
 *                       EPNum.0..3: Address
 *                       EPNum.7:    Dir
 *    Return Value:    None
 */
void USBD_EnableEP( uint32_t EPNum ) {

    seUSB_EnableEPm( EPNum & 0xf );
}

/*
 *  Disable USB Endpoint
 *    Parameters:      EPNum: Endpoint Number
 *                       EPNum.0..3: Address
 *                       EPNum.7:    Dir
 *    Return Value:    None
 */
void USBD_DisableEP( uint32_t EPNum ) {

  seUSB_DisableEPm( EPNum & 0xf );
}

/*
 *  Reset USB Device Endpoint
 *    Parameters:      EPNum: Device Endpoint Number
 *                       EPNum.0..3: Address
 *                       EPNum.7:    Dir
 *    Return Value:    None
 */
void USBD_ResetEP( uint32_t EPNum ) {

  seUSB_ResetEPm( EPNum & 0xf );

}

/*
 *  Set Stall for USB Device Endpoint
 *    Parameters:      EPNum: Device Endpoint Number
 *                       EPNum.0..3: Address
 *                       EPNum.7:    Dir
 *    Return Value:    None
 */
void USBD_SetStallEP( uint32_t EPNum ) {

  seUSB_SetStall( EPNum & 0xf );

}

/*
 *  Clear Stall for USB Device Endpoint
 *    Parameters:      EPNum: Device Endpoint Number
 *                       EPNum.0..3: Address
 *                       EPNum.7:    Dir
 *    Return Value:    None
 */
void USBD_ClrStallEP( uint32_t EPNum ) {

  EPNum &= 0xf;
  seUSB_ClrStall( EPNum );
  USBD_ResetEP( EPNum );

}

/*
 *  Clear USB Device Endpoint Buffer
 *    Parameters:      EPNum: Device Endpoint Number
 *                       EPNum.0..3: Address
 *                       EPNum.7:    Dir
 *    Return Value:    None
 */
void USBD_ClearEPBuf( uint32_t EPNum ) {

  seUSB_ClearEPmFifo( EPNum & 0xf );

}

/*
 *  Read USB Device Endpoint Data
 *    Parameters:      EPNum: Device Endpoint Number
 *                       EPNum.0..3: Address
 *                       EPNum.7:    Dir
 *                     pData: Pointer to Data Buffer
 *    Return Value:    Number of bytes read
 */
uint32_t USBD_ReadEP( uint32_t EPNum, uint8_t *pData ) {

  uint32_t sz = gEpSize[EPNum & 0xf]; 
  
  if ( ( EPNum == 0) && gSetupToken ) { 
    seUSB_GetSetupPacket( (USB_SETUP_PACKET*)pData );
    gSetupToken = 0;
  } else {
#if defined(USB_WITH_DMA) && defined(DMAUSB_module_header)
    sz = seDMAUSB_ReadFifo( EPNum & 0xf, pData, sz );
#else
    sz = seUSB_ReadFifo( EPNum & 0xf, pData, sz );
#endif
  }
  return (sz);
}

/*
 *  Write USB Device Endpoint Data
 *    Parameters:      EPNum: Device Endpoint Number
 *                       EPNum.0..3: Address
 *                       EPNum.7:    Dir
 *                     pData: Pointer to Data Buffer
 *                     cnt:   Number of bytes to write
 *    Return Value:    Number of bytes written
 */
uint32_t USBD_WriteEP( uint32_t EPNum, uint8_t *pData, uint32_t cnt ) {
  
  EPNum &=0xf;
#if defined(USB_WITH_DMA) && defined(DMAUSB_module_header)
  cnt = seDMAUSB_WriteFifo( EPNum, pData, cnt );
#else
  cnt = seUSB_WriteFifo( EPNum, pData, cnt );
#endif
  return cnt;

}

/*
 *  Get USB Device Last Frame Number
 *    Parameters:      None
 *    Return Value:    Frame Number
 */
uint32_t USBD_GetFrame( void ) {
  return gFrameNumber;
}

/*
 *  USB Device Interrupt Service Routine
 */
void USB_IRQHandler( void ) {
  volatile uint32_t igp, imain, isie, ip0;
  imain = UsbGetMainInt() & UsbGetMainIntEn();
  isie = UsbGetSIEInt() &  UsbGetSIEIntEn();
  igp = UsbGetGEPInt() &  UsbGetGEPIntEn();
  //  Sie interrupt    
  if ( imain & seUSB_MAIN_SIEIF_INT ) {   
    if ( isie &  seUSB_SIE_RESETIF_INT ) {   
      USBD_Reset();
      usbd_reset_core();
      if ( USBD_P_Reset_Event ) {
        USBD_P_Reset_Event();
      }
      UsbClearSIEInt( seUSB_SIE_RESETIF_INT );
    }
    
    if ( isie &  seUSB_SIE_ATADDRI_INT ) {
      UsbClearSIEInt( seUSB_SIE_ATADDRI_INT );
    }

    if ( isie &  seUSB_SIE_SUSPENDIF_INT ) {
      USBD_Suspend();
      if ( USBD_P_Suspend_Event ) {
        USBD_P_Suspend_Event();
      } 
      UsbClearSIEInt( seUSB_SIE_SUSPENDIF_INT );
    }
    
    if ( isie &  seUSB_SIE_NONJIF_INT ) {
      USBD_Resume();
      if ( USBD_P_Resume_Event ) {
        USBD_P_Resume_Event();
      } 
      UsbClearSIEInt( seUSB_SIE_NONJIF_INT );
    }  
    
    //  Start Of Frame                                                           
    if ( isie & seUSB_SIE_SOFIF_INT ) {
      if ( USBD_P_SOF_Event ) {
        USBD_P_SOF_Event();
        gFrameNumber++;
      }
      UsbClearSIEInt( seUSB_SIE_SOFIF_INT ); 
    }
#if 0
  //  To add error handling.                                                      
#endif
    
  }
  //  Ep0 setup interrupt    
  if ( imain & seUSB_MAIN_EP0SETIF_INT ) {
    //  analize Ep0Cfg    
    gSetupToken = 1;
    USBD_P_EP[0]( USBD_EVT_SETUP );
    //  prepare for transaction direction already set in core. 
    UsbClearMainInt( seUSB_MAIN_EP0SETIF_INT ); 
    if ( USB->EP0CFG_b.DIR == 0 ) {
      seUSB_DataOutStage();
    } else {
      seUSB_DataInStage();
    }   
  }
  
  //  Ep0 interrupt    
  if ( imain & seUSB_MAIN_EP0IF_INT ) {
  if ( USB->EP0INTF_b.INNAKIF ) {
    //  Switch to status stage 
    if ( USB->EP0CFG_b.DIR == 0 ) {
    seUSB_StatusInStage();
     }
     else {
      seUSB_DataInStage();
      if ( USBD_P_EP[0] ) {
        USBD_P_EP[0]( USBD_EVT_IN );
      }
    }
    
    UsbClearEP0Int( seUSB_EP0_INNAKIF_INT );
  }
  
    if ( USB->EP0INTF_b.OUTNAKIF ) {
      //  Switch to status stage
      if ( USB->EP0CFG_b.DIR == 1 ) {
        seseUSB_StatusOutStage(); 
      } else {
        seUSB_DataOutStage();
      }
      
      UsbClearEP0Int( seUSB_EP0_OUTNAKIF_INT );
      
    }
  
    if ( USB->EP0INTF_b.OUTACKIF ) {
      if ( USBD_P_EP[0] ) {
       USBD_P_EP[0](USBD_EVT_OUT);
      }
     
      UsbClearEP0Int( seUSB_EP0_OUTACKIF_INT );
    }
  
    if ( USB->EP0INTF_b.INACKIF ) {
      if ( USBD_P_EP[0] ) {
        USBD_P_EP[0]( USBD_EVT_IN );
      }
      
      UsbClearEP0Int( seUSB_EP0_INACKIF_INT );
    }
     if ( USB->EP0INTF_b.OUTERRIF ) {
      if ( USBD_P_EP[0] ) {
       USBD_P_EP[0]( USBD_EVT_OUT_STALL );
      }
      
      UsbClearEP0Int( seUSB_EP0_OUTERRIF_INT );
    }
  
    if ( USB->EP0INTF_b.INERRIF ) {
      if ( USBD_P_EP[0] ) {
        USBD_P_EP[0]( USBD_EVT_IN_STALL );
      }
      
      UsbClearEP0Int( seUSB_EP0_INERRIF_INT );
    }
  }
  if ( igp & seUSB_GPE_EPAIF_INT ) {
    handleEpaInt();
    UsbClearEPaInt( seUSB_GPE_EPAIF_INT );
  }
  if ( igp & seUSB_GPE_EPBIF_INT ) {
    handleEpbInt();
    UsbClearEPbInt( seUSB_GPE_EPBIF_INT );
  }  
  if ( igp & seUSB_GPE_EPCIF_INT ) {
    handleEpcInt();
    UsbClearEPcInt( seUSB_GPE_EPCIF_INT );
  }
}

void handleEpaInt( void ) {
  uint8_t num = USB->EPACFG_b.EPNUM; 
  if ( USB->EPAINTF_b.INNAKIF ) {
    
    UsbClearEPaInt( seUSB_EPM_INNAKIF_INT );
  }
  if ( USB->EPAINTF_b.OUTNAKIF ) {
      
   
    UsbClearEPaInt( seUSB_EPM_OUTNAKIF_INT );
  }
   if ( USB->EPAINTF_b.INERRIF ) {
    UsbClearEPaInt( seUSB_EPM_INERRIF_INT );
  }
  
  if ( USB->EPAINTF_b.OUTACKIF ) {
    UsbClearEPaInt( seUSB_EPM_OUTACKIF_INT );
    if ( USBD_P_EP[num] ) {
     USBD_P_EP[num]( USBD_EVT_OUT );
    }
    
  }
  if ( USB->EPAINTF_b.INACKIF ) {
    UsbClearEPaInt( seUSB_EPM_INACKIF_INT );
    if ( USBD_P_EP[num] ) {
      USBD_P_EP[num]( USBD_EVT_IN );
    }
    
  }
}

void handleEpbInt( void ) {
   uint8_t num = USB->EPBCFG_b.EPNUM; 
  if ( USB->EPBINTF_b.OUTSHACKIF ) {
    UsbClearEPbInt( seUSB_EPM_OUTSHACKIF_INT );
  }
  if ( USB->EPBINTF_b.INNAKIF ) {
    
    UsbClearEPbInt( seUSB_EPM_INNAKIF_INT );
  }
  if ( USB->EPBINTF_b.OUTNAKIF ) {
    
    UsbClearEPbInt( seUSB_EPM_OUTNAKIF_INT );
  }
  if ( USB->EPBINTF_b.INERRIF ) {
    
    UsbClearEPbInt( seUSB_EPM_INERRIF_INT );
  }
  if ( USB->EPBINTF_b.OUTACKIF ) {
    UsbClearEPbInt( seUSB_EPM_OUTACKIF_INT );
    if ( USBD_P_EP[num] ) {
     USBD_P_EP[num]( USBD_EVT_OUT );
    }
    
  }
  if ( USB->EPBINTF_b.INACKIF ) {
    UsbClearEPbInt( seUSB_EPM_INACKIF_INT );
    if ( USBD_P_EP[num] ) {
      USBD_P_EP[num]( USBD_EVT_IN );
    }
    
  }
}

void handleEpcInt( void ) {
  uint8_t num = USB->EPCCFG_b.EPNUM; ;
  if ( USB->EPCINTF_b.INNAKIF ) {
     
    UsbClearEPcInt( seUSB_EPM_INNAKIF_INT );
  }
  if ( USB->EPCINTF_b.OUTNAKIF ) {
     
    UsbClearEPcInt( seUSB_EPM_OUTNAKIF_INT );
  }
  if ( USB->EPCINTF_b.OUTACKIF ) {
    UsbClearEPcInt( seUSB_EPM_OUTACKIF_INT );
    if ( USBD_P_EP[num] ) {
     USBD_P_EP[num]( USBD_EVT_OUT );
    }
    
  }
  if ( USB->EPCINTF_b.INACKIF ) {
    if ( USBD_P_EP[num] ) {
      USBD_P_EP[num]( USBD_EVT_IN );
    }
    
    UsbClearEPcInt( seUSB_EPM_INACKIF_INT );
  }

}

void BOARD_USB_PPORT_IRQHandler( void ) {
  
  if ( PPORT_USB_INTF & PPORT_USB_INTF_MSK ) {
    if ( !seUSB_IsVbusConnected() ) {
      sePPORT_EnableInt(USB_PORT_ID, sePPORT_INT_EDGE_FALLING);
      seUSB_Attach();
	  
	  if(sePPORT_GetInput(USB_PORT_ID) == 0)
	  {
		  seUSB_Detach();
          usbd_init();
      }
	}
    PPORT_USB_INTF = PPORT_USB_INTF_MSK; 
  } 

}

