/****************************************************************************
 *
 * @file	main.c
 * @brief	This is a boot loader sample program for S1C31W73 demonstration.
 *
 * @par		Copyright:
 *			Copyright (C) SEIKO EPSON CORPORATION 2019. All rights reserved.
 *
 * @warning -
 *
 * @par		History:
 *			- 06/12/2019    1.00
 *				- First revision.
 *
 ***************************************************************************/
/* include */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "srec.h"
#include "Driver_Flash.h"

#include "system_S1C31W73.h"
#include "board.h"
#include "settings.h"
#include "se_pport.h"
#include "se_common.h"
#include "se_uart3.h"
#include "se_t16.h"
#include "se_wdt2.h"

extern  ARM_DRIVER_FLASH Driver_Flash;


/* --- definition --- */
#define		BUFF_SIZE		(256)
#define		BUFF_SIZE_HALF	(BUFF_SIZE>>1)
#define		XON_CODE		(0x11)
#define		XOFF_CODE		(0x13)


#define		ROM_END			0x5FFFF			//W73
#define		RAM_END			0x20007FFF		//W73
#define		SECTOR_MAX		1536			//W73
#define		SECTOR_SIZE		0x100			//W73
#define		INDICATOR_ADDR	0x3000			//W73
#define		START_VECTOR	0x4000			//W73


#define WRITE_PROT					(*(volatile unsigned short*)0x40000000)
#define OSCTRIM1 					(*(volatile unsigned short*)0x40000052)


/* --- extern --- */
extern char __START_program[];
extern const unsigned char upgrade_indicator[4];
extern int (*flentry[])();


/* --- struct --- */
typedef struct st_data_buf {
	unsigned short end_point;				/// buff end point
    char buf[BUFF_SIZE];
} ST_DATA_S;

/* --- static variable --- */
static unsigned char do_upgrade[4] = {
	0x00, 0x00, 0x00, 0x00,
};
static unsigned char upgrade_completed[4] = {
	0xaa, 0xaa, 0xaa, 0xaa,
};

static struct srec_decode_t srecData;		/// decode data buffer
static ST_DATA_S 	oneLineData;			/// UART receive buffer

/* --- function declaration --- */
int main(void);
static int memory_write(struct srec_decode_t * srec);
static void data_copy(void);
static int get_record(void);
static int update(void);
static void start_wdt(void);
static void stop_wdt(void);
void Bootloader_IRQHandler(void);

static void start_application( uint32_t app_link_location ){
  void (*pfMainApp)( void );
  uint32_t *pointer;

  __disable_irq();

  SCB->VTOR = app_link_location;
  pointer = (uint32_t *)app_link_location;
  __set_MSP(*pointer);
  pointer++;
  pfMainApp = (void (*)(void))(*pointer);
  pfMainApp();
}
static void ConfigureDebugUART3_0( seUART_InitTypeDef* InitStruct )
{
  ///< Stop supplying operating clock
  UART3_0->CTL_b.MODEN = 0;
  
  // Configure the UPMUX directly instead of calling the sePPORT_UpMuxFunction
  // Since RAM has not been initialized yet

  // Set the USIN - P16, USOUT - P17
  PPORT->P1IOEN_b.P1IEN6 = 0;
  PPORT->P1IOEN_b.P1IEN7 = 0;
  PPORT->P1IOEN_b.P1OEN6 = 1;
  PPORT->P1IOEN_b.P1OEN7 = 1;
  PPORT->P1DAT_b.P1OUT6 = 1;
  PPORT->P1DAT_b.P1OUT7 = 1;
  PPORT->P1MODSEL_b.P1SEL6 = 0;
  PPORT->P1MODSEL_b.P1SEL7 = 0;
  UPMUX->P1MUX3_b.P16PERISEL = 3;		// UART.
  UPMUX->P1MUX3_b.P16PPFNC = 1;	        // USIN.
  UPMUX->P1MUX3_b.P16PERICH = 0;		// Ch.0.
  UPMUX->P1MUX3_b.P17PERISEL = 3;		// UART.
  UPMUX->P1MUX3_b.P17PPFNC = 2;	        // USOUT.
  UPMUX->P1MUX3_b.P17PERICH = 0;		// Ch.0.
  PPORT->P1FNCSEL_b.P16MUX      = 1;    //  UPMUX.
  PPORT->P1MODSEL_b.P1SEL6      = 1;    // Enable peripheral I/O function.
  PPORT->P1FNCSEL_b.P17MUX      = 1;    // UPMUX.
  PPORT->P1MODSEL_b.P1SEL7      = 1;    // Enable peripheral I/O function.  
  
  // Configure the UART Ch.n operating clock.
  seUART_ConfigureClock( UART3_0, InitStruct->Clk );
             
  // Select operation mode; 
  seUART_ConfigureMode( UART3_0, InitStruct->Mode );
  
  // Initialize interrupt.
  seUART_DisableInt( UART3_0, UART3_0_INTE_ALL_Msk ); 
  seUART_ClearIntFlag( UART3_0, UART3_0_INTF_ALL_Msk );
}

int main(void)
{
	int result;
	int	i;
	
	/// STOP WDT
	stop_wdt();

	//Bootloader_IRQHandler();
		
	if (memcmp(upgrade_indicator, upgrade_completed, 4) == 0) {
		/// jump to the reset vector handler of the loaded program.
		start_application( START_VECTOR );
	}

	//disable Interrupt
  	__asm("CPSID i");
	

		
	///Initialize
	Driver_Flash.Initialize(NULL);

#if	_DEBUG	
	WRITE_PROT = 0x96;
	OSCTRIM1 = 0x254A;						//32MHz Trriming
#endif	
	
	if (memcmp(upgrade_indicator, do_upgrade, 4) == 0) {
		/// erase sector (INDICATOR_ADDR - ROM_END )
		for(i=(INDICATOR_ADDR/SECTOR_SIZE);i<SECTOR_MAX;i++)
		{
			Driver_Flash.EraseSector(i*SECTOR_SIZE );
		}
	}
	
#ifdef _DEBUG	
	//test
    sePPORT_InitAsOutput( sePPORT_P01 );
    sePPORT_DisableInt( sePPORT_P01 );
#endif
	/// receive S-record and write it.
	result = update();

	//Uninitialize
  	Driver_Flash.Uninitialize();
	
	while(1)
	{};

	return result;
}

#ifdef __IAR_SYSTEMS_ICC__
#pragma section = ".data"
#pragma section = ".data_init"

#else
extern char Load$$RW_IRAM1$$RW$$Base;
extern char Load$$RW_IRAM1$$RW$$Length;
extern char Image$$RW_IRAM1$$RW$$Base;
#endif


/****************************************************************************
 * start upgrade function.
 *
 * @brief	start upgrade.
 *
 * @retval	none
 ***************************************************************************/
void Bootloader_IRQHandler(void)
{
	unsigned char data[] = {0x00, 0x00, 0x00, 0x00};

	
#ifdef __IAR_SYSTEMS_ICC__	
    char *targetAddr = __section_begin(".data");
    char *sourceAddr = __section_begin(".data_init");
    size_t size      = __section_size (".data_init");
#else
	char *targetAddr = &Image$$RW_IRAM1$$RW$$Base;
    char *sourceAddr = &Load$$RW_IRAM1$$RW$$Base;
    size_t size      = (unsigned int)&Load$$RW_IRAM1$$RW$$Length;
#endif	
	
    memcpy(targetAddr, sourceAddr, size);
	
	
	/// interrupt disable
	 __asm("CPSID i");
	 
	
	
	///Initialize
	Driver_Flash.Initialize(NULL);
		

	/// Flash Write
	Driver_Flash.ProgramData(INDICATOR_ADDR, data,4);

	
	//Uninitialize
  	Driver_Flash.Uninitialize();


}

/****************************************************************************
 * update function.
 *
 * @brief	update function.
 * UART setting
 *	port
 *	 P00 : SOUT
 *	 P01 : SIN
 *
 *	baud rate	: 230400bps
 *	data bits	: 8bit
 *	parity		: none
 *	stop bit	: 1bit
 *	flow control: xon/xoff
 *
 *
 * @retval	EXIT_SUCCESS:Success.
 * @retval	EXIT_FAILURE:Error.
 ***************************************************************************/
int update(void)
{
	short srecResult;


	/// initialize for receive buffer.
	srecData.record_length = 0;
	oneLineData.end_point = 0;

	/// initialize T16 Ch.1
	/// - IOSC=32MHz, CLKDIV=1/128 [(124+1)/(32MHz/128)=0.5msec.]		@230400bps
	
	/// - IOSC=32MHz, CLKDIV=1/128 [(124+1)/(32MHz/128)=20msec.]		@9600bps
		
	seT16_InitTypeDef T16_InitStruct;
	
  	T16_InitStruct.Clk                 = seT16_IOSC_CLKDIV_128;
  	T16_InitStruct.CounterMode            = seT16_RepeatMode;
  	T16_InitStruct.Period                 = 0x7c;						//@230400bps
	//T16_InitStruct.Period                 = 0x1387;						//@9600bps
	seT16_Init(T16_1,&T16_InitStruct ) ;
	
	//seCLG_Start( seCLG_OSC3 );
	seUART_InitTypeDef InitStruct;
	memset( &InitStruct, 0, sizeof( InitStruct ) );
  	seUART_InitStruct( &InitStruct );
  	InitStruct.Clk             = seUART_IOSC_CLKDIV_2;    
  	InitStruct.Mode.reg_b.puen = seUART_MOD_PUEN_ENABLE;
  	InitStruct.Mode.reg_b.chln = seUART_MOD_CHLN_8BIT;
  	InitStruct.Mode.reg_b.stpb = seUART_MOD_STPB_1BIT;
  	InitStruct.Mode.reg_b.pren = seUART_MOD_PREN_NO_PARITY;
  	InitStruct.Mode.reg_b.prmd = seUART_MOD_PRMD_EVEN;
  	InitStruct.Mode.reg_b.irmd = seUART_MOD_IRMD_NO_IRDA;

  	ConfigureDebugUART3_0( &InitStruct );
	seUART_SetBaudRate( BOARD_UART_CH, 230400 );
	//seUART_SetBaudRate( BOARD_UART_CH, 9600 );
 
  	seUART_Enable( BOARD_UART_CH );
  
	
	while(1)
	{
		/// Get S record from PC terminal.
		if(get_record() == EXIT_FAILURE)	{
			continue;
		}
#ifdef _DEBUG			
		sePPORT_SetOutput( sePPORT_P01, sePPORT_DATA_HIGH );
#endif		
		/// S Record => Decode
		srecResult = srec_decode_record(&srecData, oneLineData.buf);
		switch(srecResult)
		{
			case SREC_TYPE_S0:
			case SREC_TYPE_S4:
			case SREC_TYPE_S5:
			case SREC_TYPE_S6:
				break;
			case SREC_TYPE_S1:
			case SREC_TYPE_S2:
			case SREC_TYPE_S3:
				/// Flash Write(data)
				if(memory_write(&srecData) != ARM_DRIVER_OK)	{
					/// Error
					seUART_Disable(UART3_0);
					return EXIT_FAILURE;
				}
				break;
			case SREC_TYPE_S7:
			case SREC_TYPE_S8:
			case SREC_TYPE_S9:
				/// Flash Write(upgrade_completed)
				Driver_Flash.ProgramData(INDICATOR_ADDR, (unsigned char*)upgrade_completed,4);
				goto endRecv;
				break;
			case SREC_TYPE_UNKNOWN:
			case SREC_TYPE_ERROR_CHECKSUM:
			case SREC_TYPE_ERROR_LENGTH:
				/// Decode Error
				seUART_Disable(UART3_0);
				return EXIT_FAILURE;
				break;
		}
		/// remaining data copy
		data_copy();
#ifdef _DEBUG				
		sePPORT_SetOutput( sePPORT_P01, sePPORT_DATA_LOW );
#endif		
	}

endRecv:
	/// stop UART Ch.0
	seUART_Disable(UART3_0);

	/// IC reset
	start_wdt();

	return EXIT_SUCCESS;
}
/****************************************************************************
 * data_copy function.
 *
 * @brief	Remaining data copy.
 *
 * @retval	none
 ***************************************************************************/
void data_copy(void)
{
	int remainSize;

	if(srecData.record_length == 0){
		return;
	}
	remainSize = oneLineData.end_point - srecData.record_length -1;
	memcpy(oneLineData.buf, oneLineData.buf + srecData.record_length + 1, remainSize);
	srecData.record_length = 0;
	oneLineData.end_point = remainSize;

}

/****************************************************************************
 * Get S Record Receive function.
 *
 * @brief	Get S record data from PC(UART).
 *
 * @retval	EXIT_SUCCESS:Success.
 ***************************************************************************/
int get_record(void)
{
	int i;
	char * buffer;
	char data;
	unsigned short cr_flag;
	unsigned short end_point;

	buffer = oneLineData.buf + oneLineData.end_point;
	cr_flag = 0;
	end_point = oneLineData.end_point;


	/// start data receive
	UART3_0->TXD = XON_CODE;
    
	
	/// 0.5msec * 10000 = 5sec
	for (i = 0; i < 10000; i++)
	{
		///timer start
		T16_1->CTL_b.PRESET = 1;
		seT16_Start(T16_1);
		do {
			if ((UART3_0->INTF & (UART3_0_INTF_RB1FIF_Msk | UART3_0_INTF_RB2FIF_Msk)) != 0 ) {
				*buffer++ =  UART3_0->RXD;
				end_point++;
				goto START;
			}
		} while (seT16_GetIntFlag( T16_1 ) != seINTERRUPT_OCCURRED );
		  
	}

	return EXIT_FAILURE;

START:
	do {
		/// Check receive buffer one byte full interrupt.
		if ((UART3_0->INTF & (UART3_0_INTF_RB1FIF_Msk | UART3_0_INTF_RB2FIF_Msk)) != 0 ) {
			/// save receive data.
			data =  UART3_0->RXD;
			end_point++;
			if (data == 0x0d) {
				*buffer++ = 0;
				/// stop data receive
				UART3_0->TXD = XOFF_CODE;
				cr_flag = 1;
				break;
			} else {
				*buffer++ = data;
			}
			
			///timer start
			T16_1->CTL_b.PRESET = 1;
			seT16_Start(T16_1);
		}
	} while (seT16_GetIntFlag( T16_1 ) != seINTERRUPT_OCCURRED );


	/// Receive remaining data.
	do {
		/// Check receive buffer one byte full interrupt.
		if ((UART3_0->INTF & (UART3_0_INTF_RB1FIF_Msk | UART3_0_INTF_RB2FIF_Msk)) != 0 ) {
			/// save receive data.
			data =  UART3_0->RXD;
			end_point++;
			if (data == 0x0d) {
				*buffer++ = 0;
				break;
			} else {
				*buffer++ = data;
			}
			///timer start
			T16_1->CTL_b.PRESET = 1;
			seT16_Start(T16_1);
		}
	} while (seT16_GetIntFlag( T16_1 ) != seINTERRUPT_OCCURRED );


	oneLineData.end_point = end_point;
	if(cr_flag == 0){
		return EXIT_FAILURE;
	}
	else{
		return EXIT_SUCCESS;
	}
}


/****************************************************************************
 * memory_write function.
 *
 * @brief	Flash Write.
 *
 * @retval	FLS_E_SUCCESS: Success.
 ***************************************************************************/
int memory_write(struct srec_decode_t * srec)
{
	int result =ARM_DRIVER_OK;

	

	if((srec->address >= START_VECTOR)&& (srec->address < ROM_END)){
		/// Flash Write
		result = Driver_Flash.ProgramData(srec->address, srec->data, srec->length);
	}
	else if((srec->address >= 0x20001000) && (srec->address < RAM_END)){
		/// Sram Write
		memcpy((char *)srec->address, srec->data, srec->length);
		result = ARM_DRIVER_OK;
	} else {
		result = ARM_DRIVER_OK;
	}


		
	return result;
}
/****************************************************************************
 * start_wdt function.
 *
 * @brief	Reset by WDT.
 *
 * @retval	none
 ***************************************************************************/
void start_wdt(void)
{
	seWDT_InitTypeDef WDT_InitStruct;
	
	seWDT_InitStruct( &WDT_InitStruct ); //default settings
  	
  	WDT_InitStruct.Clk = seWDT_IOSC_CLKDIV_8192;
  	WDT_InitStruct.CMP = 2929;
  	seWDT_Init( &WDT_InitStruct );
	seWDT_SetMode(seWDT_ResetMode);
  	
  	
  	seWDT_Start();                       //Start WDT

}
/****************************************************************************
 * stop_wdt function.
 *
 * @brief	Reset by WDT.
 *
 * @retval	none
 ***************************************************************************/
void stop_wdt(void)
{
	seWDT_Stop();
}


