/****************************************************************************
 *
 * @file	srec.c
 * @brief	This is SRecord decode program.
 *
 * @par		Copyright:
 *			Copyright (C) SEIKO EPSON CORPORATION 2015-2016. All rights reserved.
 *
 * @warning -
 *
 * @par		History:
 *			- 21 Feb, 2016	1.00
 *				- First revision.
 *
 ***************************************************************************/
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "srec.h"

static unsigned char schars_to_sbyte(char * schars) {
	int sbyte;

	sbyte = 0;
	if (isdigit(*schars)) {
		sbyte = *schars - '0';
	} else if (isxdigit(*schars)) {
		sbyte = toupper(*schars) - 'A' + 10;
	}
	sbyte <<= 4;
	schars++;
	if (isdigit(*schars)) {
		sbyte += *schars - '0';
	} else if (isxdigit(*schars)) {
		sbyte += toupper(*schars) - 'A' + 10;
	}

	return (unsigned char)sbyte;
}

static unsigned long sbytes_get_16bit(unsigned char * sbytes) {
	unsigned long value;

	value = *sbytes++; value <<= 8;
	value += *sbytes;

	return value;
}

static unsigned long sbytes_get_24bit(unsigned char * sbytes) {
	unsigned long value;

	value = *sbytes++; value <<= 8;
	value += *sbytes++; value <<= 8;
	value += *sbytes;

	return value;
}

static unsigned long sbytes_get_32bit(unsigned char * sbytes) {
	unsigned long value;

	value = *sbytes++; value <<= 8;
	value += *sbytes++; value <<= 8;
	value += *sbytes++; value <<= 8;
	value += *sbytes;

	return value;
}

static unsigned char sbytes_checksum(unsigned char * sbytes, int length, unsigned char sum) {
	while (length > 0) {
		sum += *sbytes++;
		length--;
	}

	return sum;
}

static void sbytes_decode_schars(unsigned char * sbytes, char * schars, int length) {
	while (length > 0) {
		*sbytes++ = schars_to_sbyte(schars);
		schars += 2;
		length--;
	}
}

static const int stype_address_length[10] = {
	2, 2, 3, 4, 0, 2, 3, 4, 3, 2
};

enum srec_type_t srec_decode_record(struct srec_decode_t * srec, char * record) {
	enum srec_type_t stype;
	size_t length;
	unsigned char sbytes[6];
	unsigned long address;
	unsigned char checksum;

	stype = SREC_TYPE_UNKNOWN;

	if (toupper(record[0]) != 'S') {
		length = 0;
		goto end;
	}
	length = strspn(record + 1, "0123456789abcdefABCDEF") + 1;

	if ((length < 10) || !isdigit(record[1])) goto end;

	stype = (enum srec_type_t) (record[1] - '0');
	if (length < (1 + 1 + stype_address_length[stype] + 1) * 2) goto end;

	sbytes_decode_schars(sbytes, record + 2, 1 + stype_address_length[stype]);
	srec->length = sbytes[0] - (1 + stype_address_length[stype]);
	if (length < (1 + 1 + stype_address_length[stype] + srec->length + 1) * 2) goto end;

	switch (stype_address_length[stype]) {
	case 2: address = sbytes_get_16bit(sbytes + 1); break;
	case 3: address = sbytes_get_24bit(sbytes + 1); break;
	case 4: address = sbytes_get_32bit(sbytes + 1); break;
	default: address = 0; break;
	}
	srec->address = address;
	sbytes_decode_schars(srec->data, record + 4 + stype_address_length[stype] * 2, srec->length + 1);
	checksum = sbytes_checksum(sbytes, 1 + stype_address_length[stype], 0);
	checksum = sbytes_checksum(srec->data, srec->length + 1, checksum);
	if (0xff != checksum) {
		stype = SREC_TYPE_ERROR_CHECKSUM;
	}

end:
	srec->stype = stype;
	srec->record_length = length;
	return stype;
}
