/******************************************************************
 *		s1c31TestClk.c
 *
 *		This is the test that verify the main clock is correct or not.
 *
 * 		history	:	2009.06.04	new making
 *
 ******************************************************************/

#include "board.h"
#include "s1c31SelfTest.h"

/* prototype */
short s1c31TestClk(unsigned long baseFreq, unsigned short range);
static void s1c31InitIntTestClk(void);

/* wait till RTC stable */
#define RTC_NOT_STABLE 0			// first RTC interrupt is not accuracy.
#define RTC_STABLE 1				// so wait till RTC interrupt is stable.
#define RTC_INT_ON 2				// RTC interrupt flag on

/* RTC interrupt set up */
#define RTC_INT_LEVEL 6				// RTC interrupt level
#define RTC_MODE_SELECT 0x1			// RTC timer mode
#define RTC_INT_CYCLE 0x0			// RTC interrupt interval(1/64s)
#define RTC_INT_MODE 0x1			// RTC interrupt mode -> level

/* T8 interrupt set up */
#define T16_INT_LEVEL 4				// T8 interrupt level
#define T16_COUNT_MODE 0x0			// T8 count mode
#define T16_CLOCK_SELECT_16384 0xe	// divide clock select(1/16384)
#define T16_CLOCK_SELECT_2 0x1		// divide clock select(1/2)
#define T16_COUNT_DATA 0xff			// count data
#define T16_COUNT_WAIT 0xf5			// wait time data
#define T16_INT_WAIT 25				// number of interrupt to wait
// the value is decided by T8_CLOCK_SELECT. (2^T8_CLOCK_SELECT)
#define T16_CALC_COUNT 2

/* the value is decided by RTC interrupt cycle.
 * 1/64 s interrupt: 64
 * 1 s interrupt: 1
 */
#define RTC_CALC_COUNT 32 //64


volatile unsigned short countT16Int;		// number of 16-bit timer underflow.
volatile unsigned short rtcIntNum;		// number of RTC interrupt occurred.

static seStatus StartOsc1( void );

/**
 *	s1c31TestClk
 *
 *	@brief 	execute the main clock test.
 *
 *	@param1	baseFreq	expected value of the main clock frequency.
 *	@param2 chkNum		permissible error(from -range% to +range%).
 *
 *	@return E_OK or E_CLOCK
 *			pass the test -> E_OK
 *			failed the test -> E_CLOCK
 */
short s1c31TestClk(unsigned long baseFreq, unsigned short range)
{
	unsigned long mClkFreq;				// the main clock frequency
	unsigned long count;				// 8-bit timer counter value

	unsigned long allow;				// permissible error range

  T16_0_Type* T16x = T16_3;

	countT16Int = 0;
	rtcIntNum = RTC_NOT_STABLE;
	/* calculate permissible error range */
	allow = (baseFreq * (unsigned long)range) / 100;

  StartOsc1();
	/* initialize interrupt function */
	s1c31InitIntTestClk();

	/* disable interrupt */
	__asm("CPSID I");

	/* RTC start */
	RTCA->CTLL_b.RTCRUN = 1; //? RTC_CNTL0_RTCSTP = 0x0;

	while (rtcIntNum != RTC_INT_ON) {
		/* wait till RTC interrupt occur */
		__asm("CPSIE I");
    __WFI(); //__asm("halt"); 
		__asm("nop");
		__asm("CPSID I");
		__asm("nop");
		__asm("nop");
	}

	/* enable interrupt */
	__asm("CPSIE I");

	/* stop RTC */
	RTCA->CTLL_b.RTCRUN = 0; //? RTC_CNTL0_RTCSTP = 0x1;

	/* stop RTC interrupt */
	RTCA->INTE = 0; //? RTC_INTMODE_RTCIEN = 0x00;

	/* 16-bit timer interrupt disable */
	T16x->INTE_b.UFIE = 0; //? T8_INT_T8IE = 0x0;

	/* read the counter value */
	/* case underflow count data + 1(for interrupt occurred count data + 1) */
//	count = ((unsigned long)(T16_COUNT_DATA + 1) * (unsigned long)countT16Int) +
//			(unsigned long)(T16_COUNT_DATA - T16x->TC);
  count  = (unsigned long)(T16_COUNT_DATA + 1);
  count *= (unsigned long)countT16Int;
  count += (unsigned long)(T16_COUNT_DATA - T16x->TC);

	/* calculate main clock frequency
	 * mClkFreq = count * (clock select = 1/2)2 * (rtc interrupt interval = 1/64 s)64
	 */
	mClkFreq = count * T16_CALC_COUNT * RTC_CALC_COUNT;

	/* check main clock is normal or not normal */
	if ((mClkFreq < (baseFreq - allow)) || ((baseFreq + allow) < mClkFreq)) {
		return E_CLOCK;
	} else {
		return E_OK;
	}

}

/**
 *	s1c31InitIntTestClk
 *
 *	@brief 	initialize interrupt function 8bit timer and RTC.
 *
 */

static seStatus StartOsc1( void ) {

//  if ( seGetOsc1XtalFreq() == 32768 ) { ///< Do nothing if not 32k768H Xtal
    ///< It doesn't do at all when having already started.
    if ( CLG->OSC_b.OSC1EN == 0 ) {
      ///< Disable write-protect.
      seProtectSys( seWRITE_PROTECT_OFF );

      ///< Clear interrupt flag.

      //seCLG_ClearIntFlag( seCLG_FLGS( seCLG_OSC1STAIF | seCLG_OSC1STPIF ) );
      CLG->INTF = CLG_INTF_OSC1STAIF_Msk | CLG_INTF_OSC1STPIF_Msk; //seCLG_ClearIntFlag( CLG_INTF_OSC1STAIF_Msk | CLG_INTF_OSC1STPIF_Msk );

      CLG->OSC_b.OSC1EN = 0; ///< Stop Osc1.

      CLG->OSC1_b.INV1N  = 0; ///< Set oscillation inverter gain(0 = Min).
      CLG->OSC1_b.CGI1   = 0; ///< Set internal gate capacitor(0 = Min).
      CLG->OSC1_b.OSC1WT = 1; ///< Set oscillation stabilization waiting time(1 = 4096 clocks).

      ///< Enable write-protect.
      seProtectSys( seWRITE_PROTECT_ON );

      CLG->OSC_b.OSC1EN = 1; ///< Start oscillation.

      while ( CLG->INTF_b.OSC1STAIF == 0 ) {
        ;
      }
    }
//  }
  
  return ( CLG->OSC_b.OSC1EN==1 ) ? seSTATUS_OK : seSTATUS_NG;
}

static void s1c31InitIntTestClk(void)
{

   T16_0_Type* T16x = T16_3;

	/* wait till OSC1 is stable(3 seconds) by using 16-bit timer */
	/* initialize 16-bit timer(to wait till OSC1 is stable) */
	/* stop timer */
	T16x->CTL &= (~((unsigned short)0x0012));  //? T8_CTL &= (~((unsigned short)0x0003));

	/* 16-bit timer interrupt disable */
	T16x->INTE_b.UFIE = 0; //? T8_INT_T8IE = 0x0;

	/* 16-bit timer interrupt flag clear */
	T16x->INTF = 0x01;   //? T8_INT_T8IF = 0x1;

	/* set interrupt level.4 */
//	ITC_LV5_ILV11 = T8_INT_LEVEL;
  NVIC_SetPriority( T16_3_IRQn, T16_INT_LEVEL ); 
  NVIC_ClearPendingIRQ( T16_3_IRQn ); 
  NVIC_EnableIRQ( T16_3_IRQn );

	/* select timer count mode -> repeat */
	T16x->MOD_b.TRMD = T16_COUNT_MODE; //? T8_CTL_TRMD = T8_COUNT_MODE;

	/* select clock 1/16384 */
	//T8_CLK_DF = T8_CLOCK_SELECT_16384;
  T16x->CLK_b.CLKSRC = seSRC(seT16_IOSC_CLKDIV_16384); ///< Clock source selection
  T16x->CLK_b.CLKDIV = seDIV(seT16_IOSC_CLKDIV_16384); ///< Clock division ratio selection
  T16x->CLK_b.DBRUN  = seDBRUN;      ///< Whether clock supplied in DEBUG mode or not.

	/* set reload data 0xf5 */
	T16x->TR = T16_COUNT_WAIT;   //? T8_TR_TR = T8_COUNT_WAIT;

  T16x->CTL_b.MODEN = 1;

	/* timer reset */
	T16x->CTL_b.PRESET = 1; //? T8_CTL_PRESER = 0x1;
        ///< It is confirmed whether the preset finished.
        WHILE( T16x->CTL & T16_0_CTL_PRESET_Msk, fStatus );

	/* 16-bit timer interrupt start */
	T16x->INTE_b.UFIE = 1; //? T8_INT_T8IE = 0x1;

	/* 16-bit timer start */
	T16x->CTL_b.PRUN = 1; //? T8_CTL_PRUN = 0x01;

	/* wait 3 seconds */
	while (1) {
		/* wait till 8-bit timer interrupt occur */
    __WFI(); //__asm("halt"); //setHalt();//__WFI();//		__asm("halt");
		__asm("nop");

		if (countT16Int >= T16_INT_WAIT) {
			break;
		}
	}

	/* stop 8-bit timer */
	T16x->CTL &= (~((unsigned short)0x0012));  //? (~((unsigned short)0x0003));

	countT16Int = 0;

	/* initialize RTC interrupt */
//  StartOsc1();////seCLG_Start( seCLG_OSC1 );
	//STNBY=ENABLE
//	P2_IO_P2IO |= 0x10;					//P24 = OUTPUT
//	P2_OUT_P2OUT |= 0x10;				//P24 = HIGH

	/* RTC reset */
//	RTC_CNTL0_RTCRST = 0x1;
//	RTC_CNTL0_RTCRST = 0x0;
/////////////////

//RTCA->CTLL_b.RTCRUN = 0;

////////////////
  RTCA->CTLL_b.RTCRST = 1;
  while( RTCA->CTLL & RTCA_CTLL_RTCRST_Msk ) { //        WHILE( RTCA->CTLL & RTCA_CTLL_RTCRST_Msk, fStatus );
      __NOP();
  }
  RTCA->CTLL_b.RTCRUN = 1;  

	/* busy check */
	while (1) {
		/* counter hold control -> hold */
		RTCA->CTLL_b.RTCHLD = 1; //? RTC_CNTL1_RTCHLD = 0x01;

		if (RTCA->CTLL & RTCA_CTLL_RTCBSY_Msk) { //(RTC_CNTL1_RTCBSY == 0x1) {
			/* Busy */
		} else {
			/* Not Busy */
			break;
		}
	}     

	/* stop RTC */
	RTCA->CTLL_b.RTCRUN = 0; //? RTC_CNTL0_RTCSTP = 0x1;

	/* RTC interrupt stop */
	RTCA->INTE = 0;  //? RTC_INTMODE_RTCIEN = 0x00;

	/* RTC interrupt flag clear */
	RTCA->INTF = (~((unsigned short)(7<<9))); //? RTC_INSTAT_RTCIRQ = 0x01;

	/* set RTC interrupt level.6 */
	//ITC_LV9_ILV18 = RTC_INT_LEVEL;
  NVIC_SetPriority( RTC_IRQn, RTC_INT_LEVEL );  
  NVIC_EnableIRQ( RTC_IRQn );

	/* 12h/24h mode select */
	//RTC_CNTL0_RTC24H = RTC_MODE_SELECT;
  RTCA->CTLL_b.RTC24H = RTC_MODE_SELECT;        

	/* RTC interrupt cycle 1/64 second */
//	RTCA->INTE_b.T1_32SECIE = 1;  //1/32 //? RTC_INTMODE_RTCT = RTC_INT_CYCLE;

//	/* select RTC interrupt mode */
//	RTC_INTMODE_RTCIMD = RTC_INT_MODE;

	/* counter hold control -> running */
	RTCA->CTLL_b.RTCHLD = 0; //protect(); RTC_CNTL1_RTCHLD = 0x00;

	/* initialize 8-bit timer(to count main clock) */
	/* stop timer */
	T16x->CTL_b.PRUN = 0; //T16x->CTL &= (~((unsigned short)0x0012));  //? (~((unsigned short)0x0003));

	/* 16-bit timer interrupt disable */
	T16x->INTE_b.UFIE = 0; //? T8_INT_T8IE = 0x0;

	/* 8-bit timer interrupt flag clear */
	T16x->INTF = 0x01;   //? T8_INT_T8IF = 0x1;

	/* set interrupt level.4 */
	//ITC_LV5_ILV11 = T8_INT_LEVEL;
  NVIC_SetPriority( T16_3_IRQn, T16_INT_LEVEL ); 
  NVIC_ClearPendingIRQ( T16_3_IRQn ); 
  NVIC_EnableIRQ( T16_3_IRQn );

	/* select timer count mode -> repeat */
	T16x->MOD_b.TRMD = T16_COUNT_MODE; //? T8_CTL_TRMD = T8_COUNT_MODE;

	/* select clock 1/2 */
	//T8_CLK_DF = T8_CLOCK_SELECT_2;
  T16x->CLK_b.CLKSRC = seSRC(seT16_IOSC_CLKDIV_2); ///< Clock source selection
  T16x->CLK_b.CLKDIV = seDIV(seT16_IOSC_CLKDIV_2); ///< Clock division ratio selection

	/* set reload data 0xff */
	T16x->TR = T16_COUNT_DATA;   //? T8_TR_TR = T8_COUNT_DATA;

	/* timer reset */
	//T8_CTL_PRESER = 0x1;
	T16x->CTL_b.PRESET = 1; //? T8_CTL_PRESER = 0x1;
        ///< It is confirmed whether the preset finished.
        WHILE( T16x->CTL & T16_0_CTL_PRESET_Msk, fStatus );

	/* RTC interrupt flag clear */
	RTCA->INTF = (~((unsigned short)(7<<9))); //? RTC_INSTAT_RTCIRQ = 0x01;

	/* RTC interrupt start */
	RTCA->INTE_b.T1_32SECIE = 1; //? RTC_INTMODE_RTCIEN = 0x01;

	/* 8-bit timer interrupt start */
	T16x->INTE_b.UFIE = 1; //? T8_INT_T8IE = 0x1;

}

/**
 *	RtcInt
 *
 *	@brief 	real time clock interrupt handler.
 */
void RTCA_IRQHandler(void)
{
        T16_0_Type* T16x = T16_3;

	if (rtcIntNum == RTC_NOT_STABLE) {
		/* 8-bit timer start */
		T16x->CTL_b.PRUN = 1; //? T8_CTL_PRUN = 0x01;
	} else {
		/* stop 8-bit timer */
		T16x->CTL &= (~((unsigned short)0x0012));  //? (~((unsigned short)0x0003));
	}

	/* rtc interrupt count + 1 */
	rtcIntNum++;

	/* RTC interrupt flag clear */
	RTCA->INTF = (~((unsigned short)(7<<9))); //? RTC_INSTAT_RTCIRQ = 0x01;

}

/**
 *	T16_3_IRQHandler
 *
 *	@brief 16-bit timer interrupt handler.
 */
void T16_3_IRQHandler(void)
{
	/* 16-bit timer interrupt flag clear */
	T16_3->INTF = 1; //? T8_INT_T8IF = 0x1;
	countT16Int++;

}


