/**
  ******************************************************************************
  * @file    main.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file is ADC12A example software.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2018-2020. All rights reserved.
  ******************************************************************************
  */

/** @defgroup ADC12A
  * @{
  * @brief The ADC12A module provides a successive approximation type 12-bit A/D conversion interface.
  * 
  * \section intro_sec Introduction
  * This example shows how to program the ADC12A to do ADC conversion.
  * 
  * \section hardware_setup Hardware Setup
  *
  * Run terminal program.
  * 1. Connect SVT board to PC via USB Adapter for UART as it is explained
  *    the Sample Software Manual in Section 2.2.1 and shown on the Figure 2.2.1.3
  *    and on Figure 2.2.1.4.
  * 2. Run a terminal program.
  * 
  * \section operation_sec Operations
  * - Initializes ADC12A.
  * - Starts ADC12A.
  * - Observe data output in the terminal program window.
  * - You can change voltage value on ADC12A channels.
  *
  * \subsection output Example of output:
  *
  *  -CPU clock- seCLG_IOSC
  * ADC12A_ADIN0 = 0
  * ADC12A_ADIN1 = 0
  * ADC12A_ADIN2 = 0
  * ADC12A_ADIN3 = 0
  * ADC12A_ADIN4 = 4095
  * ADC12A_ADIN5 = 4095
  * ADC12A_ADIN6 = 4095
  * ADC12A_ADIN7 = 0
  * ADC12A_ADIN0 = 0
  * ADC12A_ADIN1 = 0
  * ADC12A_ADIN2 = 0
  * ADC12A_ADIN3 = 768
  * ADC12A_ADIN4 = 4095
  * ADC12A_ADIN5 = 4095
  * ADC12A_ADIN6 = 4095
  * ADC12A_ADIN7 = 0
  * ADC12A_ADIN0 = 2016
  * ADC12A_ADIN1 = 2016
  * ADC12A_ADIN2 = 3968
  * ADC12A_ADIN3 = 2048
  * ADC12A_ADIN4 = 4095
  * ADC12A_ADIN5 = 4095
  * ADC12A_ADIN6 = 4095
  * ADC12A_ADIN7 = 0
  * ADC12A_ADIN0 = 0
  * ADC12A_ADIN1 = 0
  * ADC12A_ADIN2 = 0
  * ADC12A_ADIN3 = 0
  * ADC12A_ADIN4 = 4095
  * ADC12A_ADIN5 = 4095
  * ADC12A_ADIN6 = 4095
  * ADC12A_ADIN7 = 0
  * ADC12A_ADIN0 = 0
  * ADC12A_ADIN1 = 0
  * ADC12A_ADIN2 = 0
  * ADC12A_ADIN3 = 0
  * ADC12A_ADIN4 = 4095
  * ADC12A_ADIN5 = 4095
  * ADC12A_ADIN6 = 4095
  * ADC12A_ADIN7 = 0
  * ADC12A_ADIN0 = 0
  * ADC12A_ADIN1 = 0
  * ADC12A_ADIN2 = 0
  * ADC12A_ADIN3 = 0
  * ADC12A_ADIN4 = 4095
  * ADC12A_ADIN5 = 4095
  * ADC12A_ADIN6 = 4095
  * ADC12A_ADIN7 = 0
  * Exit
  * 
  ******************************************************************************
  */
/**
  * @}
  */
#include <string.h>
#include <stdio.h>
#include "board.h"  
#include "port_config.h"

///< Structure for peripheral library
static seADC12A_InitTypeDef InitStruct;
static uint16_t g_adcCount = 0;
static uint16_t g_active_adin = 0;
static volatile uint16_t g_ADC12A_INTF = 0;
static volatile uint16_t g_adcValue[8] = {0,0,0,0,0,0,0,0};

int main( void )  {

  ///< Displays the frequency of the system clock.
  ShowRunningClock();
  
  seSysSleepMS(10);  ///< wait for printf characters to flush
  
  ///< Clear parameters
  memset( &InitStruct, 0, sizeof( seADC12A_InitTypeDef ) );
  
  ///< Configure parameters
  InitStruct.Clk            = BOARD_ADC12A_CLK;             ///< Configure the ADC12A operating clock.
  InitStruct.SampleClock    = seADC12A_SMPCLK_4CYC;         ///< Sample Clock in number of CLK_T16 cycles
  InitStruct.TrigSrc        = seADC12A_SWTRIG;              ///< Conversion trigger source
  InitStruct.ConversionMode = seADC12A_CONTINUOUS;          ///< Conversion mode
  InitStruct.StoreMode      = seADC12A_RIGHT_JUSTIFY;       ///< Data store mode
  InitStruct.VoltageRange   = seADC12A_VRANGE_18_55;        ///< Conversion voltage range
  InitStruct.ports.StartAIN = BOARD_ADC12A_START_ADIN;      ///< Starting ADINx analog input to sample
  InitStruct.ports.EndAIN   = BOARD_ADC12A_END_ADIN;        ///< Ending ADINx analog input to sample

  seCLG_Start(seSRC(InitStruct.Clk));
  seADC12A_Init( BOARD_ADC12A_CH, &InitStruct );

  ///< Setting of PORT ADIN00-06,VREFA0
  sePPORT_Init(sePPORT_ADC_0_IN0, ADIN00_INIT);             ///< Assign port to ADC12 ADIN00
  sePPORT_Init(sePPORT_ADC_0_IN1, ADIN01_INIT);             ///< Assign port to ADC12 ADIN01
  sePPORT_Init(sePPORT_ADC_0_IN2, ADIN02_INIT);             ///< Assign port to ADC12 ADIN02
  sePPORT_Init(sePPORT_ADC_0_IN3, ADIN03_INIT);             ///< Assign port to ADC12 ADIN03
  sePPORT_Init(sePPORT_ADC_0_IN4, ADIN04_INIT);             ///< Assign port to ADC12 ADIN04
  sePPORT_Init(sePPORT_ADC_0_IN5, ADIN05_INIT);             ///< Assign port to ADC12 ADIN05
  sePPORT_Init(sePPORT_ADC_0_IN6, ADIN06_INIT);             ///< Assign port to ADC12 ADIN06
  sePPORT_Init(sePPORT_ADC_0_VREFA, VREFA0_INIT);           ///< Assign port to ADC12 VREFA0
  
  ///< Enable ADC interrupt
  seADC12A_ClearIntFlag( BOARD_ADC12A_CH, 0xFFFF );         ///< Clear ADC interrupts
  seADC12A_EnableInt( BOARD_ADC12A_CH, BOARD_ADC12A_INTE ); ///< Enable ADC12A interrupts
  NVIC_SetPriority( ADC12A_0_IRQn, 3 );                     ///< Set interrupt level
  NVIC_EnableIRQ( ADC12A_0_IRQn );                          ///< Enable ADC interrupt
 
  ///< Find all enabled adc inputs
  g_active_adin = 0;
  for ( uint16_t i=BOARD_ADC12A_START_ADIN; i <= BOARD_ADC12A_END_ADIN; i++ ) {
    g_active_adin |= (1<<i);
  }
   
  while( g_adcCount < 30 ) {

    if ( BOARD_ADC12A_CH->CTL_b.ADST == 0 ) {
      for ( uint16_t i=BOARD_ADC12A_START_ADIN; i <= BOARD_ADC12A_END_ADIN; i++ ) {
        if ( g_ADC12A_INTF & (1<<i) ) {
          g_ADC12A_INTF ^= (1<<i);
          printf( "ADC12A_ADIN%d = %d\n",  i, g_adcValue[i]);
        }
      }

      g_adcCount++;
      
      ///< Clear ADC interrupts
      seADC12A_ClearIntFlag( BOARD_ADC12A_CH, 0xFFFF );
      ///< start ADC conversion
      seADC12A_Start(BOARD_ADC12A_CH);
    } 
  }
  
  ///< Stop ADC conversion before exit
  seADC12A_Stop(BOARD_ADC12A_CH);
  
  printf( "\nExit\n" );
  
  return seSTATUS_OK;
} 

/**
  * @brief  ADC12A Interrupt function
  * @retval None
  */
void ADC12A_IRQHandler( void ) {

  if (BOARD_ADC12A_CH->INTF_b.OVIF) {      ///< Overrun interrupt
    BOARD_ADC12A_CH->INTF = ADC12A_0_INTF_OVIF_Msk;
    seADC12A_Stop(BOARD_ADC12A_CH);
  } else  
  
  if ( BOARD_ADC12A_CH->INTF_b.AD0CIF ) {  ///< AIN0 interrupt
     BOARD_ADC12A_CH->INTF = ADC12A_0_INTF_AD0CIF_Msk;
     g_adcValue[0] = seADC12A_ReadADC(BOARD_ADC12A_CH);
     g_ADC12A_INTF |= ADC12A_0_INTF_AD0CIF_Msk;
  } else
  
  if ( BOARD_ADC12A_CH->INTF_b.AD1CIF ) {  ///< AIN1 interrupt
     BOARD_ADC12A_CH->INTF = ADC12A_0_INTF_AD1CIF_Msk;
     g_adcValue[1] = seADC12A_ReadADC(BOARD_ADC12A_CH);
     g_ADC12A_INTF |= ADC12A_0_INTF_AD1CIF_Msk;
  } else
  
  if ( BOARD_ADC12A_CH->INTF_b.AD2CIF ) {  ///< AIN2 interrupt
     BOARD_ADC12A_CH->INTF = ADC12A_0_INTF_AD2CIF_Msk;
     g_adcValue[2] = seADC12A_ReadADC(BOARD_ADC12A_CH);
     g_ADC12A_INTF |= ADC12A_0_INTF_AD2CIF_Msk;
  } else
  
  if ( BOARD_ADC12A_CH->INTF_b.AD3CIF ) {  ///< AIN3 interrupt
     BOARD_ADC12A_CH->INTF = ADC12A_0_INTF_AD3CIF_Msk;
     g_adcValue[3] = seADC12A_ReadADC(BOARD_ADC12A_CH);
     g_ADC12A_INTF |= ADC12A_0_INTF_AD3CIF_Msk;
  } else
  
  if ( BOARD_ADC12A_CH->INTF_b.AD4CIF ) {  ///< AIN4 interrupt
     BOARD_ADC12A_CH->INTF = ADC12A_0_INTF_AD4CIF_Msk;
     g_adcValue[4] = seADC12A_ReadADC(BOARD_ADC12A_CH);
     g_ADC12A_INTF |= ADC12A_0_INTF_AD4CIF_Msk;
  } else
  
  if ( BOARD_ADC12A_CH->INTF_b.AD5CIF ) {  ///< AIN5 interrupt
     BOARD_ADC12A_CH->INTF = ADC12A_0_INTF_AD5CIF_Msk;
     g_adcValue[5] = seADC12A_ReadADC(BOARD_ADC12A_CH);
     g_ADC12A_INTF |= ADC12A_0_INTF_AD5CIF_Msk;
  } else
  
  if ( BOARD_ADC12A_CH->INTF_b.AD6CIF ) {  ///< AIN6 interrupt
     BOARD_ADC12A_CH->INTF = ADC12A_0_INTF_AD6CIF_Msk;
     g_adcValue[6] = seADC12A_ReadADC(BOARD_ADC12A_CH);
     g_ADC12A_INTF |= ADC12A_0_INTF_AD6CIF_Msk;
  } else
  
  if ( BOARD_ADC12A_CH->INTF_b.AD7CIF ) {  ///< AIN7 interrupt
     BOARD_ADC12A_CH->INTF = ADC12A_0_INTF_AD7CIF_Msk;
     g_adcValue[7] = seADC12A_ReadADC(BOARD_ADC12A_CH);
     g_ADC12A_INTF |= ADC12A_0_INTF_AD7CIF_Msk;
  } 
  
  if ( g_ADC12A_INTF == g_active_adin ) {
    ///< Stop conversion when all active inputs measured
    seADC12A_Stop(BOARD_ADC12A_CH);
  }
  
}

