/**
  ******************************************************************************
  * @file    main.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file is CLG Driver example software.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */ 

/** @defgroup CLG
  * @{
  * @brief CLG module is the clock generator that controls the clock sources and manages clock supply
  *        to the CPU and the peripheral circuits
  *
  * \section intro_sec Introduction
  * This example executes various CLG functions such as starting OSCs, setting Wake up clocks and sleep modes.
  *
  * \section operation_sec Operations
  * 
  * - Initializes CLG.
  * - Run Autotrimming of IOSC.
  * - Verifies Sleep or Halt states running various clocks.
  * - Verifies Wakeup states running various clocks.
  *
  * \subsection output Example of output:
  *
  *  -CPU clock- seCLG_IOSC
  * CLG Initialization ok
  * 
  * CLG IOSC Auto-trimming ok
  * 
  * Halt    (OSC3) -actual- seCLG_OSC3
  * Wake up (OSC3) -actual- seCLG_OSC3
  * 
  * Sleep   (OSC1) -actual- seCLG_OSC1
  * Wake up (OSC3) -actual- seCLG_OSC3
  * Exit
  *
  ******************************************************************************
  */
/**
  * @}
  */ 
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "board.h"

///< Structure for peripheral library
static void ShowLocalRunningClock( void );
T16_0_Type* T16x;

int main( void ) {
#ifdef USE_GCC_LIBS
  extern void initialise_monitor_handles(void);
  initialise_monitor_handles();
#endif
  IRQn_Type IRQn = T16_2_IRQn;
  T16x = T16_2;
  
  ///< Init CLG module to configure IOSC as system clock
  seCLG_InitTypeDef ClgInitStruc;
  ///< Clear parameters
  memset( &ClgInitStruc, 0, sizeof( ClgInitStruc ) );
  
  ClgInitStruc.SysClk = seCLG_IOSC_CLKDIV_1;       ///< Sets system clock.
  ClgInitStruc.OperationInSlpEn = seENABLE;        ///< Sets Stop Clock mode used while in sleep
  ClgInitStruc.SysClkSwitchOnWkUpEn =seDISABLE;    ///< Sets SYSCLK switching function at wake-up
  
  ///< Displays the frequency of the system clock.
  ShowRunningClock();
  
  seStatus fStatus = seCLG_Init( &ClgInitStruc );
  printf ( "CLG Initialization %s\n", fStatus?"ok":"failed" );

  ///< Run Autotrimming of IOSC. Temporary switch to OSC1
  seCLG_Start( seCLG_OSC1 );
  fStatus = seCLG_RunAutoTrimming( seCLG_IOSC, seCLG_OSC1 );
  printf ( "\nCLG IOSC Auto-trimming %s\n", fStatus?"ok":"failed" );
  seSysSleepMS(10);  ///< let printf characters flush out before re-initializing clock

  ///< Clear CLG interrupts
  seCLG_ClearIntFlag( CLG_INTF_IOSCTERIF_Msk | CLG_INTF_IOSCTEDIF_Msk );
 
  ///< Configure timer interupt as a tool to wake up CPU after sleeping.
  seCLG_Start( seCLG_OSC1 );
  seT16_InitTypeDef T16_InitStruct;
  memset( &T16_InitStruct, 0, sizeof( T16_InitStruct ) );
  T16_InitStruct.Clk                    = seT16_OSC1_CLKDIV_1;   ///< Set the clock source of T16.
  T16_InitStruct.CounterMode            = seT16_OneShotMode;     ///< Set the T16 operation mode.
  T16_InitStruct.Period                 = 32767;                 ///< Preset reload data to counter. (32767+1)/32.768kHz=1sec.
  ///< Set OSC1 to keep supplying clock to T16 in sleep
  seCLG_Start(seSRC(T16_InitStruct.Clk));
  seCLG_SetOperInSlp( seCLG_OSC1, seDISABLE );
  seT16_Init( T16x, &T16_InitStruct );
  NVIC_SetPriority( IRQn, 3 );
  NVIC_ClearPendingIRQ(IRQn);
  NVIC_EnableIRQ( IRQn );
  
  ///< Halt when CPU is running on OSC3
  seCLG_Start( seCLG_OSC3 );
  fStatus = seCLG_SwitchSysClk( seCLG_OSC3_CLKDIV_1 );
  if ( fStatus == seSTATUS_NG ) {
    printf ( "Switch SysClk to OSC3 failed.\n" );
  }
  seCLG_ClearIntFlag( CLG_INTF_ALL_Msk );
  printf ( "\nHalt    (OSC3)" );
  ///< Displays of the system clock.
  ShowLocalRunningClock();
  seT16_EnableInt( T16x );
  seT16_Start( T16x );
  
  do{
  __WFI();
  }while( seINTERRUPT_OCCURRED != seT16_GetIntFlag( T16x ));

  seT16_Stop( T16x );
  seT16_ClearIntFlag( T16x );
  printf ( "Wake up (OSC3)" );
  ///< Displays of the system clock.
  ShowLocalRunningClock();
  
  ///< Switch to OSC1
  seCLG_Start( seCLG_OSC1 ); 
  fStatus = seCLG_SwitchSysClk( seCLG_OSC1_CLKDIV_1 );
  if ( fStatus == seSTATUS_NG ) {
    printf ( "ERROR:  Switch SysClk to seCLG_OSC1 failed\n" );
  }
  ///< Sleep CPU running on OSC1, wake up on OSC3
  fStatus = seCLG_SetWkUpSysClk( seCLG_OSC3_CLKDIV_1, seENABLE );
  if ( fStatus == seSTATUS_NG ) {
    printf ( "seCLG_SetWkUpSysClk(seCLG_OSC3) failed.\n" );

  }
  printf ( "\nSleep   (OSC1)" );
  ///< Displays of the system clock.
  ShowLocalRunningClock();
  seT16_EnableInt( T16x );
  seT16_Start( T16x );

  ///< Configure deep sleep
  SCB->SCR = 1<<2;
  
  do{
  __WFI();
  }while( seINTERRUPT_OCCURRED != seT16_GetIntFlag( T16x ));
  
  seT16_Stop( T16x );
  seT16_ClearIntFlag( T16x );
  ///< Disable deep sleep
  SCB->SCR = 0;
  if ( seCLG_OSC3 != seCLG_GetSysClkSrc() ) {
    printf ( "Wake up switch OSC failed failed.\n" );
  }
  printf ( "Wake up (OSC3)" );
  ///< Displays of the system clock.
  ShowLocalRunningClock();
  
  seCLG_Start( seCLG_IOSC );
  fStatus = seCLG_SwitchSysClk( seCLG_IOSC_CLKDIV_1 );
  if ( fStatus == seSTATUS_NG ) {
    seASSERT( fStatus );
  }
  //printf ( "Switched to (IOSC)" );
  //ShowLocalRunningClock(); 
  printf( "Exit\n" );
  
  return fStatus;
}

/**
  * @brief  Displays of the system clock
  * @retval None
  */
static void ShowLocalRunningClock( void ) {

  if ( seCLG_OSC1 == seCLG_GetSysClkSrc() ) {
    printf ( " -actual- seCLG_OSC1\n" );
  } else if ( seCLG_OSC3 == seCLG_GetSysClkSrc() ) {
    printf ( " -actual- seCLG_OSC3\n" );
  } else if ( seCLG_IOSC == seCLG_GetSysClkSrc() ) {
    printf ( " -actual- seCLG_IOSC\n" );
  } else {
    printf ( " - ??? Not expected state ??? \n" );
  }
}

/**
  * @brief  T16 Interrupt function
  * @retval None
  */

void T16_0_IRQHandler( void ) {
  
    seT16_DisableInt( T16x );
}

void T16_1_IRQHandler( void ) {
  
    seT16_DisableInt( T16x );
}

void T16_2_IRQHandler( void ) {
  
    seT16_DisableInt( T16x );
}

void T16_3_IRQHandler( void ) {
  
    seT16_DisableInt( T16x );
}

void T16_4_IRQHandler( void ) {
  
    seT16_DisableInt( T16x );
}

void T16_5_IRQHandler( void ) {
  
    seT16_DisableInt( T16x );
}

void T16_6_IRQHandler( void ) {
  
    seT16_DisableInt( T16x );
}

void T16_7_IRQHandler( void ) {
  
    seT16_DisableInt( T16x );
}

