/**
  ******************************************************************************
  * @file    main.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file is DMA example software.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

/** @defgroup DMA
  * @{
  * @brief The DMA module is used for data transfers between memory and peripheral devices.
  *
  * \section intro_sec Introduction
  *
  * This example shows :
  *
  * - Using DMAC in a basic memory-to-memory data transfer
  * - Using DMAC from memory to a peripheral data transfer
  * - Using DMAC from a peripheral to memory data transfer
  * - Using DMAC concurrent peripheral to memory and memory to peripheral data transfer
  *
  * For more examples of using DMAC for transfers see QSPI_DMA Example code.
  *
  * \section hardware_setup Hardware Setup
  *
  * To demonstrate UART DMA transfer, connect UART to PC by a USB adapter.
  *
  * \section operation_sec Operations
  *
  * \subsection ex1 Example 1. Memory to Memory DMA transfer
  *
  * - DMAC Channel 0 is configured to transfer the contents of a data buffer stored in RAM
  *   to the reception buffer also in RAM. Access size is Byte.
  * - DMAC Channel 1 is configured to transfer the contents of a data buffer stored in RAM
  *   to the reception buffer in RAM. Access size is Half Word.
  * - DMAC Channel 2 is configured to transfer the contents of a data buffer stored in RAM
  *   to the reception buffer in RAM. Access size is Word.
  * - The start of transfer is triggered by software.
  *
  * In this example:
  * 1. The DMAC interrupts in NVIC are not enabled.
  * 2. DMAC Channel transfer is enabled.
  * 3. Source and destination addresses incrementing is enabled.
  * 4. The transfer is started by setting the Software Request register bits.
  * 5. At the end of the transfer, a Transfer Completion interrupt flag is generated.
  * 6. Once interrupt flag is generated, the "number of transfers" is read which must be equal to 0.
  * 7. The Transfer Complete Interrupt flag is then cleared.
  * 8. A comparison between the source and destination buffers is done to check that all data have
  *    been correctly transferred.
  *
  * \subsection ex2 The Example 2. Peripheral to Memory DMA transfer.
  * 
  * - DMAC Channel 0 is configured to transfer data from a UART data register to memory.
  * - UART is configured with baud rate 115200. DMA transfers are enabled for Receive buffer full event.
  * - The start of transfer is triggered by typing 8 characters in the PC window running a terminal program.
  *
  * In this example:
  * 1. The DMAC interrupts in NVIC are not enabled.
  * 2. DMAC Channel transfer is enabled.
  * 3. DMAC Channel filtering is disabled for the selected DMAC Channel.
  * 4. Source address incrementing is disabled.
  * 5. Destination address incrementing is enabled.
  * 6. The transfer is started by an UART receive buffer becoming full.
  * 7. At the end of the transfer, a DMAC Transfer Completion interrupt flag is generated.
  * 8. Once interrupt flag is generated, the "transfer mode" is read which must be equal to 0(STOP mode).
  * 9. The Transfer Complete Interrupt flag is then cleared.
  * 10. The Memory to Peripheral transfer (ex2) is used to output characters back to the terminal window.
  * 11. Correctness of the transfer is verified by seeing correct characters displayed in the terminal window.
  *
  * \subsection ex3 The Example 3. Concurrent Peripheral to Memory and Memory to Peripheral DMA transfers 
  * 
  * - T16B_0 sub-channel 0 is configured for compare mode.
  * - T16B_0 sub-channel 1 is configured for capture mode.
  * - DMAC Channel 0 is configured to transfer data from memory to the T16B sub-channel 0 data register.
  * - DMAC Channel 1 is configured to transfer data from the T16B sub-channel 1 data register to memory.
  */
#ifdef SND_module_header
 /*
  * \subsection ex4 Example 4. Memory to Peripheral DMA transfer
  *
  * - DMAC Channel 0 is configured to transfer data from a buffer stored in RAM memory to SNDA data
  *   register.
  * - SNDA is configured to play melody.
  * - The start of transfer is triggered by SNDA.
  *
  * In this example:
  * 1. The DMAC interrupts in NVIC are not enabled.
  * 2. DMAC Channel transfer is enabled.
  * 3. DMAC Channel filtering is disabled for the selected DMAC Channel.
  * 4. Source address incrementing is enabled.
  * 5. Destination address incrementing is disabled.
  * 6. The transfer is started by setting sound buffer empty.
  * 7. At the end of the transfer, a Transfer Completion interrupt flag is generated.
  * 8. Once interrupt flag is generated, the "number of transfers" is read which must be equal to 0.
  * 9. The Transfer Complete Interrupt flag is then cleared.
  * 10. Correctness of the transfer is verified by sound of the playing melody.
  */
#endif 
 /*
  * In this example:
  * 1. The DMAC interrupts in NVIC are enabled.
  * 2. DMAC Channel transfers are enabled for two channels.
  * 3. DMAC Channel filtering is disabled for the selected DMAC channels.
  * 4. The DMA transfer is started by a compare interrupt on the T16B sub-channel 0.
  * 5. At the end of the transfer, a DMAC Transfer Completion interrupt flag is generated on the DMAC
  *    channel 0.
  * 6. Once the DMAC interrupt is generated it sets a software completion flag by software in the DMAC
  *    interrupt service routine.
  * 7. The Transfer Complete Interrupt flag is then cleared.
  * 8. The Peripheral to Memory transfer is started by a capture interrupt on T16B sub-channel 1.
  * 9. At the end of the transfer, a DMAC Transfer Completion interrupt flag is generated on the DMAC
  *    channel 1.
  * 10. Once interrupt flag is generated it sets a software completion flag in the DMAC interrupt service
  *     routine.
  * 11. The Transfer Complete Interrupt flag is then cleared.
  * 12. Software detects both transfer completion.
  *
  * \subsection output Example of output:
  *
  *  -CPU clock- seCLG_IOSC
  * DMA Initializing
  * DMA Test: Memory
  * DMA Test: UART
  * - Type 8 characters...
  * 12345678 - 8 characters have been sent back to UART
  * DMA Test: T16B
  * DMA Test: SNDA
  *
  * Exit
  * 
  ******************************************************************************
  */
/**
  * @}
  */
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "board.h"
#include "port_config.h"

///< Structure for peripheral library
uint8_t mem8[16];
uint16_t mem16[128]__attribute__((aligned (32)));
uint16_t dma_queue0;
uint16_t dma_queue1;
uint16_t dma_queue2;
uint16_t dma_queue3;
static const uint16_t Snda_Data[] __attribute__((aligned (32))) = { 
                                    0x0362, 0x0762, 0x0349, 0x0352, 0x075C, 0x036E,
                                    0x037C, 0x0783, 0x0362, 0x036E, 0x037C, 0x0793,
                                    0x0352, 0x0752, 0x037C, 0x0362, 0x0352, 0x033D,
                                    0x0341, 0x0349, 0x0F4E, 0x8B4E, 0x0362, 0x073D,
                                    0x0341, 0x033D, 0x0341, 0x0749, 0x0362, 0x0749,
                                    0x0352, 0x0349, 0x0352, 0x075C, 0x035C, 0x0762,
                                    0x0352, 0x035C, 0x0362, 0x036E, 0x037C, 0x0383,
                                    0x0F93, 0x0F93 };

static uint8_t DMA_ARRAY[0x100];

static void DmaTransfMemToMem8( const uint8_t * src, uint8_t * dst, uint32_t transfcount, seDMAC_CHANNEL chan );
static void DmaTransfMemToMem16( const uint16_t * src, uint16_t * dst, uint32_t transfcount, seDMAC_CHANNEL chan );
static void DmaTransfMemToMem32( const uint32_t * src, uint32_t * dst, uint32_t transfcount, seDMAC_CHANNEL chan );

static void DmaBlockTransfPeriphToMem( uint32_t periph, uint32_t daddress, uint32_t transfcount, seDMAC_CHANNEL chan );
static void DmaBlockTransfMemToPeriph( uint32_t saddress, uint32_t periph, uint32_t transfcount, seDMAC_CHANNEL chan );
   
static void DmaTestMem( void );
static void DmaTestUart( void );
static void TestDmaT16B( void );
#ifdef SND_module_header
static void DmaTestSnda( void );
#endif

static void ShowMemVerify( const void * src, const void * dst, int size );

int main( void ) {
  
  ///< Displays the frequency of the system clock.
  ShowRunningClock();
  
  ///< Disable DMA interrupt.
  seDMAC_DisableInt( seDMAC_ALL_INT, seDMAC_CH_ALL );
  seDMAC_ClearIntFlag( seDMAC_ALL_INT, seDMAC_CH_ALL );
  
  ///< Statically allocate memory for DMA descriptors.
  printf("DMA Initializing\n");
  seStatus fStatus = seDMAC_Init( ( (uint32_t)DMA_ARRAY + 0x80) & ~0x7FU, 3 );
  if ( fStatus ) {
    printf("DMA Test: Memory\n");
    DmaTestMem();   
    printf("DMA Test: UART\n");
    DmaTestUart();
    printf("DMA Test: T16B\n");
    TestDmaT16B();
#ifdef SND_module_header
    printf("DMA Test: SNDA\n");
    DmaTestSnda();
#endif
  }
  printf( "\nExit\n" );
  
  return fStatus;
}

static void DmaTestMem( void ) {
  
  ///< Prepare memory buffers for DMA transfers
  memset( mem16, 0x00, sizeof( Snda_Data)); 
  DmaTransfMemToMem8( (const uint8_t*)Snda_Data, (uint8_t*)mem16, sizeof( Snda_Data ), seDMAC_CH0 );
  ShowMemVerify( Snda_Data, mem16, sizeof( Snda_Data ) );

  memset( mem16, 0x00, sizeof( Snda_Data));
  DmaTransfMemToMem16( (const uint16_t*)Snda_Data, (uint16_t*)mem16, sizeof( Snda_Data )/2, seDMAC_CH1 );
  ShowMemVerify( Snda_Data, mem16, sizeof( Snda_Data ) );

  memset( mem16, 0x00, sizeof( Snda_Data));
  DmaTransfMemToMem32( (const uint32_t*)Snda_Data, (uint32_t*)mem16, sizeof( Snda_Data )/4, seDMAC_CH2 );
  ShowMemVerify( Snda_Data, mem16, sizeof( Snda_Data ) );
}

#ifdef SND_module_header
static void DmaTestSnda( void ) {

  ///< Prepare SNDA as a source agent to DMA transfer
  seSNDA_InitTypeDef SNDA_InitStruct;
  memset( &SNDA_InitStruct, 0, sizeof( SNDA_InitStruct ) );
  seSNDA_InitStruct( &SNDA_InitStruct );
  SNDA_InitStruct.ports.NBZOUT = BOARD_BUZZER_NEG;
  SNDA_InitStruct.ports.BZOUT  = BOARD_BUZZER_POS;
  SNDA_InitStruct.Clk = BOARD_SNDA_CLOCK;
  seCLG_Start( seSRC(SNDA_InitStruct.Clk) );
  seSNDA_Init( BOARD_SNDA_CH, &SNDA_InitStruct );
  ///< Setting of PORT SNDA
  sePPORT_Init(sePPORT_BZOUT, SNDA_BZOUT_INIT);
  sePPORT_Init(sePPORT_NBZOUT, SNDA_NBZOUT_INIT);
  SNDA->SEL_b.MOSEL = seSNDA_Melody;
  SNDA->SEL_b.STIM = 3;     ///< Set tempo
  DmaBlockTransfMemToPeriph( (uint32_t)Snda_Data, (uint32_t)SNDA, sizeof(Snda_Data)/2, seDMAC_CH0 );
  while( SNDA->INTF_b.SBSY ) { __NOP(); } 

}
#endif

static void DmaTestUart( void ) {
  
  printf( " - Type 8 characters...\n" );
  
  seSysSleepMS(10);  ///< let printf characters flush out before re-initializing UART
  
  ///< Prepare regions for periferal to and from memory transfers
  memset( mem8, 0x55, 16*sizeof( uint8_t));
  ///< Prepare UART Channel 0 as a source agent to DMA transfer

  seUART_InitTypeDef InitStruct;
  memset( &InitStruct, 0, sizeof( InitStruct ) );
  seUART_InitStruct( &InitStruct );
  InitStruct.Clk = BOARD_UART_CLK;
  seCLG_Start( seSRC(InitStruct.Clk) );
  seUART_Init( BOARD_UART_CH, &InitStruct );
  ///< Setting of PORT TX,RX
  sePPORT_Init(BOARD_UART_RX_PORT, UART_RX_INIT);
  sePPORT_Init(BOARD_UART_TX_PORT, UART_TX_INIT);
  
  seUART_SetBaudRate( BOARD_UART_CH, seUART_BAUD_RATE_115200 );
  seUART_Enable( BOARD_UART_CH ); 

  DmaBlockTransfPeriphToMem( (uint32_t)BOARD_UART_CH, (uint32_t)mem8, 8, seDMAC_CH0 );
  DmaBlockTransfMemToPeriph( (uint32_t)mem8, (uint32_t)BOARD_UART_CH, 8, seDMAC_CH0 );

  printf( " - 8 characters have been sent back to UART\n" );
}


// T16B channel 0 definition

static void TestDmaT16B( void ) {
  
  seT16B_InitTypeDef T16B_InitStruct;
  
  ///< Prepare regions for peripheral to and from memory transfers
  for ( int i = 0; i < 8; i++ ) {
    mem16[i] = 1000;
  }

  memset( &T16B_InitStruct, 0, sizeof( T16B_InitStruct ) );
  seT16B_InitStruct(&T16B_InitStruct);
  T16B_InitStruct.Clk = BOARD_T16B_CLOCK;
  ///< Setting of PORT T16B
  sePPORT_Init(BOARD_T16B_TOUTCAP0, T16B_TOUTCAP0_INIT);
  sePPORT_Init(BOARD_T16B_TOUTCAP1, T16B_TOUTCAP1_INIT);
  
  seCLG_Start( seSRC(T16B_InitStruct.Clk) );
  seT16B_Init( BOARD_T16B_CH, &T16B_InitStruct );  
  NVIC_EnableIRQ( T16B_0_IRQn );
  seT16B_SetMaxCounter( BOARD_T16B_CH, 0x5000 ); 
  seT16B_SetCmpCapCnt( BOARD_T16B_CH, 0, 0x2000 ); 
  seT16B_ClearIntFlag( BOARD_T16B_CH, T16B_0_INTF_ALL_Msk ); 
  seT16B_EnableInt( BOARD_T16B_CH, T16B_0_INTE_CNTMAXIE_Msk | T16B_0_INTE_CMPCAP0IE_Msk | T16B_0_INTE_CMPCAP1IE_Msk );
  seT16B_Start( BOARD_T16B_CH );
  dma_queue0 = dma_queue1 = 0;
  ///< to enable DMA event
  seDMAC_Enable( seDMAC_CNLS( seDMAC_CH0 | seDMAC_CH1 ) ); 
  seDMAC_EnableRequestMask( seDMAC_CNLS( seDMAC_CH0 | seDMAC_CH1 ) ); ///< Block events for now.
  seDMAC_PriorityIncrease( seDMAC_CH1 );
  seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CNLS( seDMAC_CH0 | seDMAC_CH1 ) );
  seDMAC_EnableInt( seDMAC_TRANSF_COMPL, seDMAC_CNLS( seDMAC_CH0 | seDMAC_CH1 ) );
  NVIC_EnableIRQ( DMAC_IRQn ); 
  volatile uint16_t* enable_trigger = 0;
  if ( ( enable_trigger = seDMAC_NonBlockTransfMemToPeriph( (uint32_t)mem16, (uint32_t)BOARD_T16B_CH, 8, seDMAC_CH0 ) ) != 0 ) {
    seDMAC_NonBlockTransfPeriphToMem( (uint32_t)BOARD_T16B_CH, (uint32_t)mem16, 8, seDMAC_CH1 );
    do {
      setEventWait(); 
    } while ( !( (dma_queue0 == 1) && (dma_queue1 == 1) ) );
    seDMAC_EnableRequestMask( seDMAC_CNLS( seDMAC_CH0 | seDMAC_CH1 ) );
    *enable_trigger &= ~seDMAC_CH0;
    seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CNLS( seDMAC_CH0 | seDMAC_CH1 ) );
    seT16B_Stop( BOARD_T16B_CH );
    NVIC_DisableIRQ( T16B_0_IRQn );
    NVIC_DisableIRQ( DMAC_IRQn );
    seDMAC_DisableInt( seDMAC_TRANSF_COMPL, seDMAC_CNLS( seDMAC_CH0 | seDMAC_CH1 ) );
  } 
}

static void DmaTransfMemToMem8( const uint8_t* src, uint8_t * dst, uint32_t transfcount, seDMAC_CHANNEL chan ) {

  if ( transfcount > 0 ) {
    do {
      uint32_t size_m1;
      if ( transfcount > (seDMAC_NM_MAX+1) ) {
        size_m1 = seDMAC_NM_MAX;
        transfcount -= (seDMAC_NM_MAX+1);
      } else {
        size_m1 = transfcount-1;
        transfcount = 0;
      }
      uint32_t cdata1 = seDMAC_cdata( seDMAC_MODE_AUTO_REQ,size_m1,0,seDMAC_SIZE_BYTE,seDMAC_SIZE_BYTE,seDMAC_INC_1,seDMAC_INC_1 );
      seDMAC_SetChannel( cdata1, ( uint32_t )(src+size_m1), ( uint32_t )(dst+size_m1), chan );
      seDMAC_Enable( chan );
      ///< to enable DMA event
      seDMAC_ClearIntFlag( seDMAC_ALL_INT, chan ); 
      seDMAC_Start( chan ); 
      do {
        setEventWait(); ///< WFE Sleep operation, wake up when an event is received
        if (seDMAC_GetIntFlag( seDMAC_TRANSF_COMPL, chan )) {
           seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, chan );
        }
      } while ( !seDMAC_GetIntFlag( seDMAC_ERR_INT, seDMAC_CH_ALL ) && seDMAC_GetNMinus1( chan ) );
      src += ( seDMAC_NM_MAX+1 );
      dst += ( seDMAC_NM_MAX+1 );
    } while ( transfcount>0 );
    seDMAC_Disable( chan );
    seDMAC_ClearIntFlag( seDMAC_ALL_INT, chan );
  }
}

static void DmaTransfMemToMem16( const uint16_t* src, uint16_t * dst, uint32_t transfcount, seDMAC_CHANNEL chan ) {

  if ( transfcount > 0 ) {
    do {
      uint32_t size_m1;
      if ( transfcount > (seDMAC_NM_MAX+1) ) {
        size_m1 = seDMAC_NM_MAX;
        transfcount -= (seDMAC_NM_MAX+1);
      } else {
        size_m1 = transfcount-1;
        transfcount = 0;
      }
      uint32_t cdata1 = seDMAC_cdata( seDMAC_MODE_AUTO_REQ,size_m1,0,seDMAC_SIZE_HALF_WORD,seDMAC_SIZE_HALF_WORD,seDMAC_INC_2,seDMAC_INC_2 );
      seDMAC_SetChannel( cdata1, ( uint32_t )(src+size_m1), ( uint32_t )(dst+size_m1), chan );
      seDMAC_Enable( chan );   
      ///< to enable DMA event
      seDMAC_ClearIntFlag( seDMAC_ALL_INT, chan );
      seDMAC_Start( chan ); 
      do {
        setEventWait(); ///< WFE Sleep operation, wake up when an event is received
        if (seDMAC_GetIntFlag( seDMAC_TRANSF_COMPL, chan )) {
           seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, chan );
        }
      } while ( !seDMAC_GetIntFlag( seDMAC_ERR_INT, seDMAC_CH_ALL ) && seDMAC_GetNMinus1( chan ) );
      src += ( seDMAC_NM_MAX+1 );
      dst += ( seDMAC_NM_MAX+1 );
    }  while ( transfcount>0 );
    seDMAC_Disable( chan );
    seDMAC_ClearIntFlag( seDMAC_ALL_INT, chan );
  }
}

static void DmaTransfMemToMem32( const uint32_t* src, uint32_t * dst, uint32_t transfcount, seDMAC_CHANNEL chan ) {

  if ( transfcount > 0 ) {
    do {
      uint32_t size_m1;
      if ( transfcount > (seDMAC_NM_MAX+1) ) {
        size_m1 = seDMAC_NM_MAX;
        transfcount -= (seDMAC_NM_MAX+1);
      } else {
        size_m1 = transfcount-1;
        transfcount = 0;
      }
      uint32_t cdata1 = seDMAC_cdata( seDMAC_MODE_AUTO_REQ,size_m1,0,seDMAC_SIZE_WORD,seDMAC_SIZE_WORD,seDMAC_INC_4,seDMAC_INC_4 );
      seDMAC_SetChannel( cdata1, ( uint32_t )(src+size_m1), ( uint32_t )(dst+size_m1), chan );
      seDMAC_Enable( chan );   
      ///< to enable DMA event
      seDMAC_ClearIntFlag( seDMAC_ALL_INT, chan );
      seDMAC_Start( chan ); 
      do {
        setEventWait(); ///< WFE Sleep operation, wake up when an event is received
        if (seDMAC_GetIntFlag( seDMAC_TRANSF_COMPL, chan )) {
           seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, chan );
        }
      } while ( !seDMAC_GetIntFlag( seDMAC_ERR_INT, seDMAC_CH_ALL ) && seDMAC_GetNMinus1( chan ) );
      src += seDMAC_NM_MAX;
      dst += seDMAC_NM_MAX;
    }  while ( transfcount>0 );
    seDMAC_Disable( chan );
    seDMAC_ClearIntFlag( seDMAC_ALL_INT, chan );
  }
}

static void DmaBlockTransfMemToPeriph( uint32_t saddress, uint32_t periph, uint32_t transfcount, seDMAC_CHANNEL chan ) {

  volatile uint16_t* enable_trigger = 0;
  
  if ( ( enable_trigger = seDMAC_NonBlockTransfMemToPeriph( saddress, periph, transfcount, chan ) ) != 0 ) {
    do {
      setEventWait(); ///< WFE Sleep operation, wake up when an event is received
      if ( seDMAC_GetIntFlag( seDMAC_TRANSF_COMPL, chan ) ) {
        *enable_trigger &= ~chan;
        seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, chan );
      }
    } while (  seDMAC_GetMode( chan ) != seDMAC_MODE_STOP );
    
    seDMAC_EnableRequestMask( chan );
    seDMAC_Disable( chan );
    *enable_trigger &= ~chan;
    seDMAC_ClearIntFlag( seDMAC_ALL_INT, chan );
  }
}

static void DmaBlockTransfPeriphToMem( uint32_t periph, uint32_t daddress, uint32_t transfcount, seDMAC_CHANNEL chan ) {

  volatile uint16_t* enable_trigger = 0;
  if ( ( enable_trigger = seDMAC_NonBlockTransfPeriphToMem( periph, daddress, transfcount, chan ) ) != 0 ) {

    do {
      setEventWait(); ///< WFE Sleep operation, wake up when an event is received
      if ( seDMAC_GetIntFlag( seDMAC_TRANSF_COMPL, chan ) ) {
        *enable_trigger &= ~chan;
        seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, chan );
      }  
    } while (  !seDMAC_GetIntFlag( seDMAC_ERR_INT, seDMAC_CH_ALL ) && seDMAC_GetMode( chan ) != seDMAC_MODE_STOP );
  }
  seDMAC_EnableRequestMask( chan );
  seDMAC_Disable( chan );
  *enable_trigger &= ~chan;
  seDMAC_ClearIntFlag( seDMAC_ALL_INT, chan );
}

static void ShowMemVerify( const void * src, const void * dst, int size ) {
  
  int i;
  
  for ( i = 0; i < size; i++ ) {
    if ( ((const uint8_t *)src)[i] != ((const uint8_t *)dst)[i] ) {
      printf( "Error! i= %d [%02x] !=  [%02x] \n", i, ((const uint8_t *)src)[i], ((const uint8_t *)dst)[i] );
      break;
    }
  }
}

/**
  * @brief  DMAC interrupt.
  * @return void
  */
void DMAC_IRQHandler( void ) {
    
  if ( seDMAC_GetIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CH0 ) ) {
    
    seDMAC_DisableInt( seDMAC_TRANSF_COMPL, seDMAC_CNLS( seDMAC_CH0 ) );
    seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CH0 );
    dma_queue0 = 1;

  }
  
  if ( seDMAC_GetIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CH1 ) ) {

    seDMAC_DisableInt( seDMAC_TRANSF_COMPL, seDMAC_CH1 );
    seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CH1 );
    dma_queue1 = 1;
  }
  
  if ( seDMAC_GetIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CH2 ) ) {
    
    seDMAC_DisableInt( seDMAC_TRANSF_COMPL, seDMAC_CH2 );
    seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CH2 );
    dma_queue2 = 1;

  }
  
  if ( seDMAC_GetIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CH3 ) ) {

    seDMAC_DisableInt( seDMAC_TRANSF_COMPL, seDMAC_CH3 );
    seDMAC_ClearIntFlag( seDMAC_TRANSF_COMPL, seDMAC_CH3 );
    dma_queue3 = 1;
  }    
}

/**
  * @brief  T16B(Ch.0) interrupt.
  * @return void
  */
void T16B_0_IRQHandler( void ) {
  
  ///< Check counter zero interrupt flag.
  if ( seINTERRUPT_OCCURRED == seT16B_GetIntFlag( T16B_0, T16B_0_INTF_CNTZEROIF_Msk ) ) {
    ///< Clear flag.
    seT16B_ClearIntFlag( T16B_0, T16B_0_INTF_CNTZEROIF_Msk );
  }
  ///< Check counter MAX interrupt flag.
  if ( seINTERRUPT_OCCURRED == seT16B_GetIntFlag( T16B_0, T16B_0_INTF_CNTMAXIF_Msk ) ) {
    ///< Clear flag.
    seT16B_ClearIntFlag( T16B_0, T16B_0_INTF_CNTMAXIF_Msk );
    ///< Set trigger signal(LOW).
    seT16B_SetTriggerSignal( T16B_0, 1, seT16B_Input_1 );
  }
  ///< Check compare/capture 0 interrupt flag.
  if ( seINTERRUPT_OCCURRED == seT16B_GetIntFlag( T16B_0, T16B_0_INTF_CMPCAP0IF_Msk ) ) {
    ///< Clear flag.
    seT16B_ClearIntFlag( T16B_0, T16B_0_INTF_CMPCAP0IF_Msk );
    ///< Set trigger signal(HIGH).
    seT16B_SetTriggerSignal( T16B_0, 1, seT16B_Input_0 );
  }
  ///< Check compare/capture 1 interrupt flag.
  if ( seINTERRUPT_OCCURRED == seT16B_GetIntFlag( T16B_0, T16B_0_INTF_CMPCAP1IF_Msk ) ) {
    ///< Clear flag.
    seT16B_ClearIntFlag( T16B_0, T16B_0_INTF_CMPCAP1IF_Msk );
  } 
}

