/**
  ******************************************************************************
  * @file    main.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file is I2C example software.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
*/ 

/** @defgroup I2C
  * @{
  * @brief I2C
  * \section intro_sec Introduction
  *
  * This example provides a description of how to use a I2C in the master mode with EEPROM 24FC512 Memory chip.
  *
  * \section hardware_setup Hardware Setup
  *
  * 1. Connect the evaluation board where the I2C sample programs are installed. Connect each port as follows:
  *
  *         S1C31xxx                                 24FC512
  *         [master]                                 [slave]
  *         BOARD_I2C_SDA_PORT------------><-----------SDA
  *
  *         BOARD_I2C_SCL_PORT------------>>-----------SCL
  *
  * \section operation_sec Operations
  *
  * This Example tests I2C_0 module with EEPROM at address 0x50.
  * This test requires 24FC512 Memory chip. 
  * 1. Test configures I2C Master mode.
  * 2. Test writes ascii string to the EEPORM memory.
  * 3. Then it reads data back from EEPORM, compares data.
  *
  * \subsection output Example of output:
  *
  *  -CPU clock- seCLG_IOSC
  * Testing I2C_0 module with EEPROM at 0x50
  * Write data: Test 0 1 2 4
  * Read data: Test 0 1 2 4
  * Status: OK
  * Exit
  *
  ******************************************************************************
  */
/**
  * @}
  */
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "board.h"
#include "port_config.h"

///< Structures for testing peripheral library
static seI2C_InitTypeDef InitStruct;

int main( void ) {

  seStatus fStatus;
  uint16_t addr = 0x50; ///< address of i2c eeprom slave
  
  ///< Displays the frequency of the system clock.
  ShowRunningClock();
  printf( "Testing I2C_0 module with EEPROM at 0x%x\n", addr );
  
  
  ///< Setting of PORT I2C
  sePPORT_Init(BOARD_I2C_SCL_PORT, I2C_SCL_INIT);
  sePPORT_Init(BOARD_I2C_SDA_PORT, I2C_SDA_INIT);
  
  ///< Clear parameters
  memset( &InitStruct, 0, sizeof( InitStruct ) );
  ///< Configure parameters
  seI2C_InitStructForMaster(&InitStruct);
  ///< Set transfer rate( bps = CLK_I2C / (BRT + 3) x 2 ).
  ///< 24.00MHz / 60 = 400000 bps.
  InitStruct.Clk = BOARD_I2C_MASTER_CLOCK;
  InitStruct.BRT           = 27;
  seCLG_Start( seSRC(InitStruct.Clk) );
  fStatus = seI2C_Init( BOARD_I2C_CH, &InitStruct );
  
  if ( fStatus == seSTATUS_OK ) {
    
    fStatus = seI2C_Enable( BOARD_I2C_CH );
    
    if ( fStatus == seSTATUS_OK ) {

      uint8_t w_data[] = "\0\0Test 0 1 2 4"; ///< The first two figures is memory address inside of i2c eeprom
      uint8_t r_data[sizeof(w_data)]; 
      uint32_t size = sizeof(w_data);
      
      ///< Clear receive data buffer.
      memset( r_data, 0, sizeof(w_data) );

      ///< Write
      do {
        fStatus = seI2C_MstSendData( BOARD_I2C_CH, addr, w_data, size, seDISABLE );
      } while (fStatus == seSTATUS_NG);
      seSysSleepMS(5);
      if ( fStatus == seSTATUS_OK ) {
        seI2C_Enable( BOARD_I2C_CH );
        
        ///< Set eeprom read address from
        do {
          fStatus= seI2C_MstSendData( BOARD_I2C_CH, addr, w_data, 2, seENABLE );
          
        } while (fStatus == seSTATUS_NG );
        
        if ( fStatus == seSTATUS_OK ) {
          
          printf( "Write data: %s\n", w_data+2 );
          ///< Read
          do {
          fStatus = seI2C_MstReceiveData( BOARD_I2C_CH, addr, r_data, size-2, seDISABLE );
          } while (fStatus == seSTATUS_NG);
          
          printf( "Read data: %s\n", r_data );
          
          if ( fStatus == seSTATUS_OK ) {
            ///< Check receive data buffer.
            if ( memcmp( w_data+2, r_data, sizeof(w_data)-2 ) ) {
              fStatus = seSTATUS_NG;
            }
          }
        }
        seI2C_Disable( BOARD_I2C_CH );
      }
    }
  }
  
  printf( "Status: %s\n", fStatus==seSTATUS_OK ? "OK" : "failed" );
  printf( "Exit\n" );
  
  return fStatus;
}

