/**
  ******************************************************************************
  * @file    main.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file is LCD Driver example software.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

/** @defgroup LCD32C
  * @{
  * @brief LCD32C is an LCD driver module which can drive a dot matrix LCD of up to 80 segment x 32 common lines.
  *
  * \section intro_sec Introduction
  * This example displays various patterns on a 72x32 LCD display.
  *
  * \section operation_sec Operations
  * 
  * 1. Initializes LCD32C.
  * 2. Shows different patterns on LCD display.
  * 3. Demonstrates changing of the contrast.
  * 4. Displays text using different fonts.
  * 5. Detects frame buffer interrupt.
  *
  * \subsection output Example of output:
  * 
  *  -CPU clock- seCLG_IOSC
  * External voltage mode selected
  * Set panel contrast high (dark)
  * Show all zeros, area0
  * Show all ones, area0
  * Show both ones and zeros, area0
  * Show checkboard pattern, area0
  * Set display inverse state NORMAL
  * Hit '+' to increase and '-' to decrease contrast, 'q' to exit
  * Set panel contrast 0
  * Set panel contrast 1
  * Set panel contrast 2
  * Set panel contrast 3
  * Set panel contrast 4
  * Set panel contrast 5
  * Set panel contrast 6
  * Set panel contrast 7
  * Set panel contrast 8
  * Set panel contrast 9
  * Set panel contrast 10
  * Set panel contrast 11
  * Set panel contrast 10
  * Set panel contrast 9
  * Set panel contrast 8
  * Set panel contrast 7
  * Set panel contrast 6
  * Set panel contrast 5
  * Set panel contrast 4
  * Set panel contrast 3
  * Set panel contrast 2
  * Set panel contrast 1
  * Set panel contrast 0
  * Set panel contrast 1
  * Set panel contrast 2
  * Set panel contrast 3
  * Set panel contrast 4
  * Set panel contrast 5
  * Set panel contrast 6
  * 
  * LCD32C waiting for interrupt....
  *   CLK -- 0x0150
  *   CTL -- 0x0003
  *  TIM1 -- 0x011f
  *  TIM2 -- 0x0300
  *   PWR -- 0x8615
  *   DSP -- 0x0075
  * COMC0 -- 0xffff
  * COMC1 -- 0xffff
  *  INTF -- 0x0001
  *  INTE -- 0x0000
  *  TEST -- 0x0000
  * Exit
  *
  ******************************************************************************
  */
/**
  * @}
  */
#include <string.h>
#include <stdio.h>
#include <S1C31.h>
#include "board.h"
#include "port_config.h"
#include "settings.h"
#include "se_lcd32c.h"
#include "se_common.h"

#include "console_font_4x6.h"
#include "console_font_4x7.h"
#include "console_font_5x8.h"
#include "console_font_5x12.h"
#include "console_font_6x8.h"
#include "selcd_font.h"

///< Structure for peripheral library
static seLCD32C_InitTypeDef LCD32C_InitStruct;
static volatile uint16_t InterruptOccured = 0;
static void setLcd32cCheckerboardPatternLarge( uint8_t area );
static void setLcd32cBytesPattern( uint8_t area, const uint8_t byte1, uint8_t byte2 );
static void setLcd32cCheckerboardPatternMedium( uint8_t area );
static void showLcdReg( void );

///< LCD display variables
static seFONTSET font4x6 = {4, 6, console_font_4x6};
static seFONTSET font6x8 = {6, 8, console_font_6x8};
static seLCDOBJ dispobj = {(unsigned char *) seLCD32C_SEGRAM_TOP_ADDR_AREA1, 72, 32};
static const char* RegNames[] = { "CLK", "CTL", "TIM1", "TIM2", "PWR", "DSP", "COMC0", "COMC1", "INTF", "INTE", "TEST" };

int main( void ) {
  
  ///< Displays the frequency of the system clock.
  ShowRunningClock();
  
  //seCLG_Start( seCLG_OSC1 );

  ///< Clear parameters
  memset( &LCD32C_InitStruct, 0, sizeof( LCD32C_InitStruct ) );

  ///< Setting of PORT COMx-SEGx
  sePPORT_Init(sePPORT_P50, LCD32C_SEG95_INIT);             ///< Assign port to LCD32C COM16/SEG95
  sePPORT_Init(sePPORT_P51, LCD32C_SEG94_INIT);             ///< Assign port to LCD32C COM17/SEG94
  sePPORT_Init(sePPORT_P52, LCD32C_SEG93_INIT);             ///< Assign port to LCD32C COM18/SEG93
  sePPORT_Init(sePPORT_P53, LCD32C_SEG92_INIT);             ///< Assign port to LCD32C COM19/SEG92
  sePPORT_Init(sePPORT_P54, LCD32C_SEG91_INIT);             ///< Assign port to LCD32C COM20/SEG91
  sePPORT_Init(sePPORT_P55, LCD32C_SEG90_INIT);             ///< Assign port to LCD32C COM21/SEG90
  sePPORT_Init(sePPORT_P56, LCD32C_SEG89_INIT);             ///< Assign port to LCD32C COM22/SEG89
  sePPORT_Init(sePPORT_P57, LCD32C_SEG88_INIT);             ///< Assign port to LCD32C COM23/SEG88

  sePPORT_Init(sePPORT_P60, LCD32C_SEG87_INIT);             ///< Assign port to LCD32C COM24/SEG87
  sePPORT_Init(sePPORT_P61, LCD32C_SEG86_INIT);             ///< Assign port to LCD32C COM25/SEG86
  sePPORT_Init(sePPORT_P62, LCD32C_SEG85_INIT);             ///< Assign port to LCD32C COM26/SEG85
  sePPORT_Init(sePPORT_P63, LCD32C_SEG84_INIT);             ///< Assign port to LCD32C COM27/SEG84
  sePPORT_Init(sePPORT_P64, LCD32C_SEG83_INIT);             ///< Assign port to LCD32C COM28/SEG83
  sePPORT_Init(sePPORT_P65, LCD32C_SEG82_INIT);             ///< Assign port to LCD32C COM29/SEG82
  sePPORT_Init(sePPORT_P66, LCD32C_SEG81_INIT);             ///< Assign port to LCD32C COM30/SEG81
  sePPORT_Init(sePPORT_P67, LCD32C_SEG80_INIT);             ///< Assign port to LCD32C COM31/SEG80

  sePPORT_Init(sePPORT_P70, LCD32C_SEG79_INIT);             ///< Assign port to LCD32C SEG79
  sePPORT_Init(sePPORT_P71, LCD32C_SEG78_INIT);             ///< Assign port to LCD32C SEG78
  sePPORT_Init(sePPORT_P72, LCD32C_SEG77_INIT);             ///< Assign port to LCD32C SEG77
  sePPORT_Init(sePPORT_P73, LCD32C_SEG76_INIT);             ///< Assign port to LCD32C SEG76
  sePPORT_Init(sePPORT_P74, LCD32C_SEG75_INIT);             ///< Assign port to LCD32C SEG75
  sePPORT_Init(sePPORT_P75, LCD32C_SEG74_INIT);             ///< Assign port to LCD32C SEG74
  sePPORT_Init(sePPORT_P76, LCD32C_SEG73_INIT);             ///< Assign port to LCD32C SEG73
  sePPORT_Init(sePPORT_P77, LCD32C_SEG72_INIT);             ///< Assign port to LCD32C SEG72

  sePPORT_Init(sePPORT_P80, LCD32C_SEG71_INIT);             ///< Assign port to LCD32C SEG71
  sePPORT_Init(sePPORT_P81, LCD32C_SEG70_INIT);             ///< Assign port to LCD32C SEG70
  sePPORT_Init(sePPORT_P82, LCD32C_SEG69_INIT);             ///< Assign port to LCD32C SEG69
  sePPORT_Init(sePPORT_P83, LCD32C_SEG68_INIT);             ///< Assign port to LCD32C SEG68
  sePPORT_Init(sePPORT_P84, LCD32C_SEG67_INIT);             ///< Assign port to LCD32C SEG67
  sePPORT_Init(sePPORT_P85, LCD32C_SEG66_INIT);             ///< Assign port to LCD32C SEG66
  sePPORT_Init(sePPORT_P86, LCD32C_SEG65_INIT);             ///< Assign port to LCD32C SEG65
  sePPORT_Init(sePPORT_P87, LCD32C_SEG64_INIT);             ///< Assign port to LCD32C SEG64

  ///< Configure parameters
  LCD32C_InitStruct.Clk = BOARD_LCD32C_CLOCK; 
  LCD32C_InitStruct.CtlLcdIoDischargeEn = seENABLE;
  LCD32C_InitStruct.DspSelComPinDir = 1;
  LCD32C_InitStruct.DspSelectArea = seLCD32C_DISPLAY_AREA0;
  LCD32C_InitStruct.DspSelSegPinDir = 1;
  LCD32C_InitStruct.PwrBoosterEn = seENABLE;
  LCD32C_InitStruct.PwrBiasSel = seLCD32C_PWR_BIAS_1_5;
  LCD32C_InitStruct.PwrHeavyLoadProtectionModeEn = seENABLE;
  LCD32C_InitStruct.PwrVoltageRegulatorEn = seENABLE;
  LCD32C_InitStruct.PwrExternalVcSel = seENABLE;
  LCD32C_InitStruct.Tim1Duty = seLCD32C_DUTY_1_32;
  LCD32C_InitStruct.Tim1FrameCount = 1;
  LCD32C_InitStruct.Tim2BoostClk = seLCD32C_FLCLK_DIV_4;
  LCD32C_InitStruct.Tim2Nline = 0;

  seCLG_Start(seSRC(LCD32C_InitStruct.Clk));
  seLCD32C_Init( &LCD32C_InitStruct );
  
  printf( "%s mode selected\n", (LCD32C->PWR_b.EXVCSEL)?"External voltage":
            "Internal generation" );
  
  printf( "Set panel contrast high (dark)\n" );
  seLCD32C_SetPanelContrast( seLCD32C_CONTRAST_LEVEL15 );
  
  printf( "Show all zeros, area0\n" );
  setLcd32cBytesPattern( seLCD32C_DISPLAY_AREA0, 0x00, 0x00 );
  seLCD32C_SetDisplayArea( seLCD32C_DISPLAY_AREA0 );

  ///< Turn on Display
  seLCD32C_SetDisplayState( seLCD32C_DISP_STATE_NORMAL );
  seSysSleepMS( 1000 );
  
  printf( "Show all ones, area0\n" );
  setLcd32cBytesPattern( seLCD32C_DISPLAY_AREA0, 0xFF, 0xFF );
  seLCD32C_SetDisplayArea( seLCD32C_DISPLAY_AREA0 );
 
  seSysSleepMS( 1000 );
   
  printf( "Show both ones and zeros, area0\n" );
  setLcd32cBytesPattern( seLCD32C_DISPLAY_AREA0, 0xFF, 0x00 );
  seLCD32C_SetDisplayArea( seLCD32C_DISPLAY_AREA0 );
 
  seSysSleepMS( 1000 );
     
  printf( "Show checkboard pattern, area0\n" );
  setLcd32cCheckerboardPatternLarge( seLCD32C_DISPLAY_AREA0 );
  seLCD32C_SetDisplayArea( seLCD32C_DISPLAY_AREA0 );
    
  seSysSleepMS( 1000 );
 
  printf( "Set display inverse state NORMAL\n" );
  seLCD32C_SetDisplayInvState( seLCD32C_DISP_NORMAL );
  
  seSysSleepMS( 1000 );
  
  int contrast = seLCD32C_CONTRAST_LEVEL8;
  int ctrl = 0;
  seLCD32C_SetDisplayArea( seLCD32C_DISPLAY_AREA0 );
  seLCD32C_SetDisplayInvState( seLCD32C_DISP_NORMAL );
  seLCD32C_SetDisplayState( seLCD32C_DISP_STATE_NORMAL );
  
  printf( "Hit '+' to increase and '-' to decrease contrast, 'q' to exit\n" );
  
  do {
    printf( "Set panel contrast %d\n", contrast );   
    seLCD32C_SetPanelContrast( (seLCD32C_Contrast)contrast );  
    ctrl = getchar();
		if (ctrl == EOF)
			break;
    switch (ctrl) {
      
    case '+': contrast = (contrast < seLCD32C_CONTRAST_LEVEL15)?(contrast+1):contrast; break;
    case '-': contrast = (contrast > seLCD32C_CONTRAST_LEVEL0)?(contrast-1):contrast; break;
    default: break;
    }
 
  } while (  ctrl != 'q' );
  
  seLCD32C_SetDisplayMemory( seLCD32C_DISPLAY_AREA0, 0 );
  
  setLcd32cBytesPattern( seLCD32C_DISPLAY_AREA1, 0x00, 0x00 );

  seLcdPutStr( &dispobj, &font6x8, 20, 7, "EPSON" );
  
  seLcdPutStr( &dispobj, &font4x6, 0, 16, "EXCEED YOUR VISION" );

  seLcdPutStr( &dispobj, &font6x8, 12,  28, "S1C31" );
  
  seLCD32C_SetDisplayArea( seLCD32C_DISPLAY_AREA1 );
  
  seLCD32C_EnableInt();
  
  NVIC_SetPriority( LCD_IRQn, 3 );  
  NVIC_EnableIRQ( LCD_IRQn );
  printf( "\nLCD32C waiting for interrupt....\n" );
  do {
    ///< Wait For Interrupt
    setHalt();
  } while ( InterruptOccured != seINTERRUPT_OCCURRED );

  seLCD32C_DisableInt();
  NVIC_DisableIRQ( LCD_IRQn );
  NVIC_ClearPendingIRQ( LCD_IRQn );
  showLcdReg();

  printf( "Exit\n" );
  
  return seSTATUS_OK;
}

/**
  * @brief  LCD32C Interrupt function
  * @retval None
  */
void LCD32C_IRQHandler( void ) {
  
  seLCD32C_ClearIntFlag();
  InterruptOccured = 1;
}

/**
  * @brief  Set Display Data RAM
  * @retval None
  */
static void setLcd32cBytesPattern( uint8_t area, const uint8_t byte1, uint8_t byte2 ) {
  
  uint16_t i;
  volatile uint8_t *segram = (uint8_t *)seLCD32C_SEGRAM_TOP_ADDR_AREA0;

  if ( area == seLCD32C_DISPLAY_AREA1 ) {
    segram = (unsigned char *)seLCD32C_SEGRAM_TOP_ADDR_AREA1;
  }

  for ( i = 0; i < seLCD32C_SEGRAM_SIZE; i = i + 2 ) {
    if ( i % 32 < 16 ) {
      segram[ i + 0 ] = byte1;
      segram[ i + 1 ] = byte1;
    } else {
      segram[ i + 0 ] = byte2;
      segram[ i + 1 ] = byte2;
    }
  }
}

/**
  * @brief  Set Display Data RAM (Checker board Pattern Large)
  * @retval None
  */
static void setLcd32cCheckerboardPatternLarge( uint8_t area ) {
  
  uint16_t i;
  
  volatile uint8_t *segram = (uint8_t *)seLCD32C_SEGRAM_TOP_ADDR_AREA0;

  if ( area == seLCD32C_DISPLAY_AREA1 ) {
    segram = (unsigned char *)seLCD32C_SEGRAM_TOP_ADDR_AREA1;
  }

  for ( i = 0; i < seLCD32C_SEGRAM_SIZE; i = i + 2 ) {

    if ( i % 32 < 16 ) {
      segram[ i + 1 ] = 0x0F;
      segram[ i + 0 ] = 0x0F;
    } else {
      segram[ i + 1 ] = 0xF0;
      segram[ i + 0 ] = 0xF0;
    }
  }
}

/**
  * @brief  Set Display Data RAM (Checker board Pattern Medium)
  * @retval None
  */
static void setLcd32cCheckerboardPatternMedium( uint8_t area ) {
  
  uint16_t i;
  
  volatile uint8_t *segram = (uint8_t *)seLCD32C_SEGRAM_TOP_ADDR_AREA0;

  if ( area == seLCD32C_DISPLAY_AREA1 ) {
    segram = (unsigned char *)seLCD32C_SEGRAM_TOP_ADDR_AREA1;
  }

  for ( i = 0; i < seLCD32C_SEGRAM_SIZE/2; i = i + 1 ) {
    if ( i % 8 < 4 ) {
      segram[ i ] = 0x33;
    } else {
      segram[ i ] = 0xCC;
    }
  }
}

/**
  * @brief  Displays of the LCD32C Control Registers
  * @retval None
  */
static void showLcdReg( void ) {
  
  int i = 0;
  uint16_t* pReg = ( uint16_t*)LCD32C_BASE;
  for ( i = 0; i < 11; i++ ) {
    printf( "%5s -- 0x%04x\n", RegNames[i], pReg[i] );
  }  
}
