/**
  ******************************************************************************
  * @file    main.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file is PPORT example software.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

/** @defgroup PPORT
  * @{
  * @brief The PPORT module controls the general-purpose input/output (GPIO) pins of the S1C31xxx.
  *
  ******************************************************************************
  */
/**
  * @}
  */ 
#include "main.h"

#include <string.h>
#include <stdio.h>
#include "board.h"
#include "port_config.h"

///<  Structure for peripheral library
static seInterruptStatus IntStatus;
static volatile uint8_t outData;
static volatile uint8_t inData;

int main( void )  {
  
  seStatus fStatus;
  
  ///< Displays the frequency of the system clock.
  ShowRunningClock();
  
  ///< Initialize and start BOARD_PPORT_CLOCK. Switch CPU to BOARD_PPORT_CLOCK.
  seCLG_Start( seSRC(BOARD_PPORT_CLOCK) );
  fStatus = seCLG_SwitchSysClk( (seCLG_Clk)BOARD_PPORT_CLOCK ); 

  if ( fStatus != seSTATUS_NG ) {

    if ( seCLG_IOSC != seSRC(BOARD_PPORT_CLOCK) ) {
      seCLG_Stop( seCLG_IOSC );
    }

    ///< Configure the PPORT operating clock.
    sePPORT_ConfigureClock( BOARD_PPORT_CLOCK );
    ///< Continue operation state before SLEEP.
    seCLG_SetOperInSlp( seSRC(BOARD_PPORT_CLOCK), seDISABLE );
    
    ///< Enable PORT interrupt
    NVIC_SetPriority( BOARD_PPORT_IRQn, 2 );
    NVIC_ClearPendingIRQ( BOARD_PPORT_IRQn );
    NVIC_EnableIRQ( BOARD_PPORT_IRQn );
    
    ///< Setting of PORT
    sePPORT_Init(BOARD_OUTPORT, OUTPORT_INIT);
    sePPORT_Init(BOARD_INPORT, INPORT_INIT);
    
    
    ///< Disable interrupt.
    sePPORT_DisableInt( BOARD_OUTPORT );
    ///< Clear interrupt flag.
    sePPORT_ClearIntFlag( BOARD_INPORT );
    IntStatus = seINTERRUPT_NOT_OCCURRED;
    ///< Set the interrupt(LOW by HIGH).
    sePPORT_ClearIntFlag( BOARD_INPORT );
    sePPORT_EnableInt( BOARD_INPORT, sePPORT_INT_EDGE_RISING );
    outData = sePPORT_GetOutput( BOARD_OUTPORT );
    inData = sePPORT_GetInput( BOARD_INPORT );

    printf( GET_NAME(BOARD_OUTPORT) " output state: %s\n", outData ? "High" : "Low" );
    printf( GET_NAME(BOARD_INPORT) " input state: %s\n", inData ? "High" : "Low" );
    printf( GET_NAME(BOARD_INPORT) " input waits for RISING edge interrupt\n" );
    printf( GET_NAME(BOARD_OUTPORT) " output is going to HIGH...\n" );
 
    ///< Set output data(HIGH).
    sePPORT_SetOutput( BOARD_OUTPORT, sePPORT_DATA_HIGH );
    ///< Wait interrupt.
    while( IntStatus == seINTERRUPT_NOT_OCCURRED ) {
      setHalt();
    }
    
    printf( "Interrupt on " GET_NAME(BOARD_INPORT) " occurred\n" );
    outData = sePPORT_GetOutput( BOARD_OUTPORT );
    inData = sePPORT_GetInput( BOARD_INPORT );
    printf( GET_NAME(BOARD_OUTPORT) " output state: %s\n", outData ? "High" : "Low" );
    printf( GET_NAME(BOARD_INPORT) " input state: %s\n\n", inData ? "High" : "Low" );
    
    ///< disable interrupt.
    sePPORT_DisableInt( BOARD_INPORT );
    sePPORT_DisableChatteringFilter( BOARD_INPORT );
    ///< clear interrupt flag.
    sePPORT_ClearIntFlag( BOARD_INPORT );
    IntStatus = seINTERRUPT_NOT_OCCURRED;
    ///< set the interrupt(HIGH by LOW).
    sePPORT_ClearIntFlag( BOARD_INPORT );
    sePPORT_EnableInt( BOARD_INPORT, sePPORT_INT_EDGE_FALLING );
    outData = sePPORT_GetOutput( BOARD_OUTPORT );
    inData = sePPORT_GetInput( BOARD_INPORT );

    printf( GET_NAME(BOARD_OUTPORT) " output state: %s\n", outData ? "High" : "Low" );
    printf( GET_NAME(BOARD_INPORT) " input state: %s\n", inData ? "High" : "Low" );
    printf( GET_NAME(BOARD_INPORT) " input waits for FALLING edge interrupt\n" );
    printf( GET_NAME(BOARD_OUTPORT) " output is going to LOW...\n" );    
    
    ///< Set output data(LOW).
    sePPORT_SetOutput( BOARD_OUTPORT, sePPORT_DATA_LOW );
    ///< Wait interrupt.
    while( IntStatus == seINTERRUPT_NOT_OCCURRED ) {
      setSleep();
    }
    
    printf( "Interrupt on " GET_NAME(BOARD_INPORT) " occurred\n" );
    outData = sePPORT_GetOutput( BOARD_OUTPORT );
    inData = sePPORT_GetInput( BOARD_INPORT );
    printf( GET_NAME(BOARD_OUTPORT) " output state: %s\n", outData ? "High" : "Low" );
    printf( GET_NAME(BOARD_INPORT) " input state: %s\n\n", inData ? "High" : "Low" );
  }
  
  printf( "Exit\n" );
  
  return seSTATUS_OK;
} 

/**
  * @brief  PORT Interrupt function
  * @retval None
  */
void BOARD_PPORT_IRQHandler( void ) {
  
  IntStatus = sePPORT_GetIntFlag( BOARD_INPORT );
  
  if ( IntStatus == seINTERRUPT_OCCURRED ) {
    sePPORT_ClearIntFlag( BOARD_INPORT );
  }
}

