/**
  ******************************************************************************
  * @file    main.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file is QSPI example software.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */
  
/** @defgroup QSPI
  * @{
  * @brief QSPI module is a Quad Synchronous Serial Interface supporting both master and slave modes.
  *
  ******************************************************************************
  */
/**
  * @}
  */ 
#include "main.h"

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "board.h"
#include "port_config.h"

#define BUF_SIZE        64
#define QSPI_DMA ///< Uncomment this line to read/write data by DMA

///< Structures for testing peripheral library
static seT16_InitTypeDef T16_InitStruct;
static seQSPI_InitTypeDef QSPI_InitStruct;

static seQSPI_TransferMode tmod[] = { seQSPI_MODE_SINGLE, seQSPI_MODE_DUAL, seQSPI_MODE_QUAD };
static seQSPI_Clocks chln[]       = { seQSPI_08CLK,       seQSPI_04CLK,     seQSPI_02CLK     };
static seQSPI_Clocks chdl[]       = { seQSPI_08CLK,       seQSPI_04CLK,     seQSPI_02CLK     };
static char * strMode[]           = { "SINGLE",           "DUAL",           "QUAD"           };

#ifdef QSPI_DMA
static uint8_t DMA_ARRAY[0x100];
#endif

static uint8_t r_data[BUF_SIZE];
static uint8_t w_data[BUF_SIZE];

///<  Function declarations
static void SignalMasterTransmit( void ) { sePPORT_SetOutput( BOARD_QSPI_HANDSHAKE_PORT, sePPORT_DATA_HIGH ); }
static void SignalMasterReceive( void ) { sePPORT_SetOutput( BOARD_QSPI_HANDSHAKE_PORT, sePPORT_DATA_LOW ); }

int main( void ) {

  seStatus fStatus;
  
  ///< Displays the frequency of the system clock.
  ShowRunningClock();
  
  ///< Clear parameters
  memset( &T16_InitStruct, 0, sizeof( T16_InitStruct ) );
  memset( &QSPI_InitStruct, 0, sizeof( QSPI_InitStruct ) );

#ifdef QSPI_DMA
  seDMAC_Init( ( (uint32_t)DMA_ARRAY + 0x80) & ~0x7FU, 3 );
#endif
  
  ///< Init T16
  seT16_InitStruct(&T16_InitStruct);
  T16_InitStruct.Clk = BOARD_QSPI_T16_CLOCK;
  T16_InitStruct.Period = 80;
  seCLG_Start( seSRC(T16_InitStruct.Clk) );
  seT16_Init( seAUX_GetT16ch_ForPeripheral(BOARD_QSPI_CH), &T16_InitStruct );
  
  ///< Init QSPI as Master
  seQSPI_InitStructForMaster(&QSPI_InitStruct);
#ifdef QSPI_MODE_SINGLE
  QSPI_InitStruct.TMOD = seQSPI_MODE_SINGLE;
  ///< Setting of PORT QSPI
  sePPORT_Init(sePPORT_QSPI_0_CLK, QSPI_0_CLK_INIT);
  sePPORT_Init(sePPORT_QSPI_0_D0, QSPI_0_D0_INIT);
  sePPORT_Init(sePPORT_QSPI_0_D1, QSPI_0_D1_INIT);
  sePPORT_Init(sePPORT_QSPI_0_SS, QSPI_0_SS_INIT);
#else
  QSPI_InitStruct.TMOD = seQSPI_MODE_DUAL;
  ///< Setting of PORT QSPI
  sePPORT_Init(sePPORT_QSPI_0_CLK, QSPI_0_CLK_INIT);
  sePPORT_Init(sePPORT_QSPI_0_D0, QSPI_0_D0_INIT);
  sePPORT_Init(sePPORT_QSPI_0_D1, QSPI_0_D1_INIT);
  sePPORT_Init(sePPORT_QSPI_0_D2, QSPI_0_D2_INIT);
  sePPORT_Init(sePPORT_QSPI_0_D3, QSPI_0_D3_INIT);
  sePPORT_Init(sePPORT_QSPI_0_SS, QSPI_0_SS_INIT);
#endif
  seQSPI_Init( BOARD_QSPI_CH, &QSPI_InitStruct );
  
  printf( "Get bus speed %u \n", seAUX_GetBusSpeedForQSPI(BOARD_QSPI_CH) );
  
  uint32_t busspeed = BOARD_QSPI_BUS_SPEED_HZ;
  printf( "Set bus speed %u \n", busspeed );
  seAUX_SetBusSpeedForQSPI( BOARD_QSPI_CH, busspeed );
  printf( "Get bus speed %u \n", seAUX_GetBusSpeedForQSPI(BOARD_QSPI_CH) );
  
  ///< Assign GPIO output to set Read/Write command to Slave
  sePPORT_InitAsOutput( BOARD_QSPI_HANDSHAKE_PORT );
  sePPORT_SetOutput( BOARD_QSPI_HANDSHAKE_PORT, sePPORT_DATA_LOW );
  
  fStatus = seQSPI_Start( BOARD_QSPI_CH );
  if ( fStatus == seSTATUS_OK ) {
    uint16_t mode, cnt;
    
    while ( 1 ) {
    
      uint32_t test_OK = 0, test_NG = 0;
    
#ifdef QSPI_MODE_SINGLE
      for ( mode = seQSPI_MODE_SINGLE; mode < seQSPI_MODE_SINGLE+1; mode++ ) {
#else
      for ( mode = seQSPI_MODE_DUAL; mode < seQSPI_MODE_QUAD+1; mode++ ) {
#endif
        printf( "Set QSPI_0 in %s mode.\n", strMode[mode] );
        seQSPI_SetMode( BOARD_QSPI_CH, tmod[mode], chln[mode], chdl[mode] );
        cnt = 10;
        while( cnt ) {
          
          for ( int i = 0; i < BUF_SIZE; i++ ) {
            w_data[i] = rand();
          }
          cnt--;
          SignalMasterTransmit();
          seQSPI_SetIO( BOARD_QSPI_CH, seQSPI_Output );
          seQSPI_ASSERT_MST_CS0();
#ifdef QSPI_DMA
          fStatus = seDMAQSPI_TxBytes( BOARD_QSPI_CH, w_data, BUF_SIZE );
#else
          ///< Non DMA transfer can be used here as well
          fStatus = seQSPI_TxBytes( BOARD_QSPI_CH, w_data, BUF_SIZE );
#endif
          seQSPI_NEGATE_MST_CS0();
          
          if ( fStatus == seSTATUS_OK ) {
            
            seQSPI_SetIO( BOARD_QSPI_CH, seQSPI_Input );
            SignalMasterReceive();
            seQSPI_ASSERT_MST_CS0();
#ifdef QSPI_DMA
          fStatus = seDMAQSPI_RxBytes( BOARD_QSPI_CH, r_data, BUF_SIZE );
#else
          ///< Non DMA transfer can be used here as well 
          fStatus = seQSPI_RxBytes( BOARD_QSPI_CH, r_data, BUF_SIZE );
#endif
            seQSPI_NEGATE_MST_CS0();
            
            ///<  Compare W/R data
            if ( fStatus == seSTATUS_OK ) {
              if ( ! memcmp( r_data, w_data, BUF_SIZE ) ) {
                printf( " - OK(%u), NG(%u)\n", ++test_OK, test_NG );
              } else {
                uint16_t i;
                fStatus = seSTATUS_NG;
                printf( "QSPI_0 read/write check failed (%u)\n", ++test_NG );
                printf( "READ[WRITE]: " );
                for ( i = 0; i < BUF_SIZE; i++ ) {
                  printf( "%02x[%02x] ", r_data[i], w_data[i] );
                }
                printf( "\n");
                
                if ( seINTERRUPT_OCCURRED == seQSPI_GetIntFlag( BOARD_QSPI_CH, QSPI_0_INTF_OEIF_Msk/*seQSPI_OEIF*/ ) ) {
                  seQSPI_ClearIntFlag( BOARD_QSPI_CH, QSPI_0_INTF_OEIF_Msk/*seQSPI_OEIF*/ );
                  seQSPI_Reset( BOARD_QSPI_CH );
                }
              }
            } else {
              printf( "QSPI_0 read failed (%u)\n", ++test_NG );
            }
          }
        }
      }
    }
  } else {
    printf( "ERROR: seQSPI_Start failed.\n" );
  }

  printf( "Exit\n" );
  
  return fStatus;
}

