/** @defgroup QSPI_MASTER
 * @{
 * @brief QSPI module is a Quad Synchronous Serial Interface supporting both master and slave modes.
 *
 * \section intro_sec Introduction
 *
 * This example provides a description of how to use a QSPI in the master mode.
 * 
 * \section hardware_setup Hardware Setup
 *
 * 1. The EVT31W73 board has an onboard serial flash device attached to the QSPI interface.
 *    In order to run the QSPI master/slave sample programs, the serial flash on each board must
 *    be isolated from the QSPI interface by removing 0-ohm resistors R157, R158, R159, R160,
 *    R163, and R168.
 *    NOTE:  Ensure that the 0-ohm connections for these resistors are restored after
 *    evaluating the QSPI master/slave sample programs.
 *
 * 2. Connect the evaluation boards where the QSPI master/slave sample programs are installed. 
 *    Connect each port as follows:
 * 
 *                     DUAL or QUAD mode
 *         [master]                         [slave]
 *
 *   TH14   QSDI0n3------------><-----------QSDIOn3   TH14
 *
 *   TH13   QSDI0n2------------><-----------QSDIOn2   TH13
 *
 *   TH12   QSDI0n1------------><-----------QSDIOn1   TH12
 *
 *   TH11   QSDI0n0------------><-----------QSDIOn0   TH11
 *
 *   TH10  QSPICLKn------------>>-----------QSPICLKn  TH10
 *
 *   TH15  #QSPISSn------------>>-----------#QSPISSn  TH15
 *
 *   TH1        P00------------>>-----------P00       TH1
 *
 *   TH9        GND------------><-----------GND       TH9
 *
 *   For definition of THx see EVT31W73 schematic in EVT31W73 manual.
 *   Please note that the connections are different for single mode.
 *   Uncomment out QSPI_MODE_SINGLE in settings.h for the single mode setup.
 *                      
 *                        SINGLE mode
 *                       
 *         [master]                         [slave]
 *
 *   TH11   QSDI0n0------------->>-----------QSDIOn1  TH12
 *
 *   TH12   QSDI0n1------------><-----------QSDIOn0   TH11
 *
 *   TH10  QSPICLKn------------>>-----------QSPICLKn  TH10
 *
 *   TH15  #QSPISSn------------>>-----------#QSPISSn  TH15
 *
 *   TH1        P00------------>>-----------P00       TH1
 *
 *   TH9        GND------------><-----------GND       TH9
 * 
 *   For definition of THx see EVT31W73 schematic in EVT31W73 manual.
 *
 * 3. Launch the slave example program first, then the master program
 * 
 * \section operation_sec Operation
 * 
 * 1. Initialize the QSPI module in master mode as below:
 *      - Data length is 8bit
 *      - Data format is MSB first
 *      - Use 16-bit timer T16_CH2 for baud rate generator.
 * 2. Set bus speed to 100000.
 * 3. Assign GPIO P00 as output to set Read/Write command to the slave.
 * 4. Send the 'BUF_SIZE' bytes of random data to the slave.
 * 5. Receive the 'BUF_SIZE' bytes of data from the slave.
 * 6. Compare whether the received data is the same as the sent data and exit.
 *
 * \subsection output Example of output:
 *
 * For Double/Quad Mode:
 *  -CPU clock- seCLG_IOSC (20000000)
 * Get bus speed 123456
 * Set bus speed 100000
 * Get bus speed 100000
 * Set QSPI_0 in DUAL mode.
 *  - OK(1), NG(0)
 *  - OK(2), NG(0)
 *  - OK(3), NG(0)
 *  - OK(4), NG(0)
 *  - OK(5), NG(0)
 *  - OK(6), NG(0)
 *  - OK(7), NG(0)
 *  - OK(8), NG(0)
 *  - OK(9), NG(0)
 *  - OK(10), NG(0)
 * Set QSPI_0 in QUAD mode.
 *  - OK(11), NG(0)
 *  - OK(12), NG(0)
 *  - OK(13), NG(0)
 *  - OK(14), NG(0)
 *  - OK(15), NG(0)
 *  - OK(16), NG(0)
 *  - OK(17), NG(0)
 *  - OK(18), NG(0)
 *  - OK(19), NG(0)
 *  - OK(20), NG(0)
 *
 * For Single Mode:
 *  -CPU clock- seCLG_IOSC (20000000)
 * Get bus speed 123456
 * Set bus speed 100000
 * Get bus speed 100000
 * Set QSPI_0 in SINGLE mode.
 *  - OK(1), NG(0)
 *  - OK(2), NG(0)
 *  - OK(3), NG(0)
 *  - OK(4), NG(0)
 *  - OK(5), NG(0)
 *  - OK(6), NG(0)
 *  - OK(7), NG(0)
 *  - OK(8), NG(0)
 *  - OK(9), NG(0)
 *  - OK(10), NG(0)
 *
 ***************************************************************************
 */
/**
  * @}
  */ 


