/**
  ******************************************************************************
  * @file    main.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file is QSPI example software.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

/** @defgroup QSPI
  * @{
  * @brief QSPI module is a Quad Synchronous Serial Interface supporting both master and slave modes.
  *
  ******************************************************************************
  */
/**
  * @}
  */ 
#include "main.h"

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "board.h"
#include "port_config.h"

#define BUF_SIZE        64

///<  Structures for testing peripheral library
static seQSPI_InitTypeDef QSPI_InitStruct;

static seQSPI_TransferMode tmod[] = { seQSPI_MODE_SINGLE, seQSPI_MODE_DUAL, seQSPI_MODE_QUAD };
static seQSPI_Clocks chln[]       = { seQSPI_08CLK,       seQSPI_04CLK,     seQSPI_02CLK     };
static seQSPI_Clocks chdl[]       = { seQSPI_08CLK,       seQSPI_04CLK,     seQSPI_02CLK     };

uint8_t data[BUF_SIZE];
uint16_t cnt;

int main( void ) {

  seStatus fStatus;
  
  ///< Clear parameters
  memset( &QSPI_InitStruct, 0, sizeof( QSPI_InitStruct ) );
  
  ///< Init QSPI as Slave
  seQSPI_InitStructForSlave(&QSPI_InitStruct);
  
#ifdef QSPI_MODE_SINGLE
  QSPI_InitStruct.TMOD = seQSPI_MODE_SINGLE;
  ///< Setting of PORT QSPI
  sePPORT_Init(sePPORT_QSPI_0_CLK, QSPI_0_CLK_INIT);
  sePPORT_Init(sePPORT_QSPI_0_D0, QSPI_0_D0_INIT);
  sePPORT_Init(sePPORT_QSPI_0_D1, QSPI_0_D1_INIT);
  sePPORT_Init(sePPORT_QSPI_0_SS, QSPI_0_SS_INIT);  
#else
  QSPI_InitStruct.TMOD = seQSPI_MODE_DUAL;
  ///< Setting of PORT QSPI
  sePPORT_Init(sePPORT_QSPI_0_CLK, QSPI_0_CLK_INIT);
  sePPORT_Init(sePPORT_QSPI_0_D0, QSPI_0_D0_INIT);
  sePPORT_Init(sePPORT_QSPI_0_D1, QSPI_0_D1_INIT);
  sePPORT_Init(sePPORT_QSPI_0_D2, QSPI_0_D2_INIT);
  sePPORT_Init(sePPORT_QSPI_0_D3, QSPI_0_D3_INIT);
  sePPORT_Init(sePPORT_QSPI_0_SS, QSPI_0_SS_INIT);
#endif
  seQSPI_Init( BOARD_QSPI_CH, &QSPI_InitStruct );
  
  ///< Init GPIO as R/W communication flag
  ///< Assign GPIO port as input to sync with Master
  sePPORT_InitAsInput( BOARD_QSPI_HANDSHAKE_PORT );
  
  ///< Enable Port interrupt
  sePPORT_ClearIntFlag( BOARD_QSPI_HANDSHAKE_PORT );
  sePPORT_EnableInt( BOARD_QSPI_HANDSHAKE_PORT, sePPORT_INT_EDGE_RISING );
  NVIC_EnableIRQ( BOARD_QSPI_PPORT_IRQn );
  
  fStatus = seQSPI_Start( BOARD_QSPI_CH );
  if ( fStatus == seSTATUS_OK ) {
    
    uint16_t mode;
    
    while ( 1 ) {
#ifdef QSPI_MODE_SINGLE
      for ( mode = seQSPI_MODE_SINGLE; mode < seQSPI_MODE_SINGLE+1; mode++ ) {
#else
      for ( mode = seQSPI_MODE_DUAL; mode < seQSPI_MODE_QUAD+1; mode++ ) {
#endif      
        seQSPI_SetMode( BOARD_QSPI_CH, tmod[mode], chln[mode], chdl[mode] );
        cnt = 10;
        while( cnt ) {
          if ( seINTERRUPT_OCCURRED == seQSPI_GetIntFlag( BOARD_QSPI_CH, QSPI_0_INTF_OEIF_Msk/*seQSPI_OEIF*/ ) ) {
            seQSPI_ClearIntFlag( BOARD_QSPI_CH, QSPI_0_INTF_OEIF_Msk/*seQSPI_OEIF*/ );
            seQSPI_Reset( BOARD_QSPI_CH );
            fStatus = seSTATUS_NG;
          }
        }
      }
    }
  }
  
  return fStatus;
} 

/**
  * @brief  PORT Interrupt function
  * @retval None
  */
void BOARD_QSPI_PPORT_IRQHandler( void ) {
  
  if (sePPORT_GetIntFlag( BOARD_QSPI_HANDSHAKE_PORT ) == seINTERRUPT_OCCURRED) {
    cnt--;
    ///< Slave receive
    seQSPI_SetIO( BOARD_QSPI_CH, seQSPI_Input );
    seQSPI_RxBytes( BOARD_QSPI_CH, data, BUF_SIZE );
    ///< Slave send
    seQSPI_SetIO( BOARD_QSPI_CH, seQSPI_Output );
    seQSPI_TxBytes( BOARD_QSPI_CH, data, BUF_SIZE );
    memset( data, 0, BUF_SIZE );
    sePPORT_ClearIntFlag( BOARD_QSPI_HANDSHAKE_PORT );
  }
}

