/**
  ******************************************************************************
  * @file    main.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file is REMC3 Driver example software.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */
  
/** @defgroup REMC3
  * @{
  * @brief The REMC3 circuit generates infrared remote control output signals.
  *
  * \section intro_sec Introduction
  * This example programs the REMC3 to generate various IR pulses.
  *
  * \section hardware_setup Hardware Setup
  *
  * This sample program is for the evaluation board. For information on
  * a connection refer to the manual.
  *
  * \section operation_sec Operations
  * 
  * This example software uses the NEC format for output.
  * 
  * 1. REMC3 determines the data waveform using two parameters: APLEN (data signal duty)
  *    and DBLEN (data signal cycle). Each of APLEN and DBLEN defines the waveform with 34 frame
  *    signals and 2 repeat signals. APLEN and DBLEN pair allocates the leader, customer code,
  *    and repeat signals at initialization time. Those signals are common to different scenarios
  *    when switches Btn1 to Btn2 are pushed. Data may vary according to selection of the Btn1 to Btn2.
  *    Data is allocated when a switch interrupt occurs.
  * 
  * 2. Further initialization consists of configuration of switches Btn1 to Btn2, allocation ports
  *    for REMO, and initialization of BOARD_REMC_T16 Ch. Then CPU enters into the halt mode to wait for interrupts
  *    caused by pushing switches Btn1 to Btn2.
  * 
  * 3. When an interrupt occurs allocate data's APLEN and DBLEN values. Since REMC3 uses a buffer mode,
  *    REMC3 should be activated after initialization. Also, activate BOARD_REMC_T16 Ch and generate a BOARD_REMC_T16 Ch
  *    interrupt after about 108ms.
  * 
  * 4. When a compare DB interrupt occurs in REMC3, write the following APLEN and DBLEN data into register.
  *    When writing last but one data element, change REMC3 to a one-shot mode to prevent the whisker-like
  *    signal output. After last data element, stop the REMC3 operation.
  * 
  * 5. When a BOARD_REMC_T16 Ch interrupt occurred, check whether the same switch Btn1 to Btn2 was pushed.
  *    If the same switch was pushed, set the register values of REMC3 so that a repeat waveform 
  *    is generated, initialize REMC3 before activation, and activate REMC3. Otherwise, stop the BOARD_REMC_T16 Ch
  *    operation.
  * 
  ******************************************************************************
  */
/**
  * @}
  */
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "board.h"
#include "port_config.h"

///< NEC format:  T=562us

#define FRAME_LEADER_APLEN  35967   ///< 16*T=8.992ms -> (35967+1)/4000kHz=8.992ms
#define FRAME_LEADER_SPACE  17983   ///<  8*T=4.496ms -> (17983+1)/4000kHz=4.496ms
#define FRAME_LEADER_DBLEN  ( FRAME_LEADER_APLEN + FRAME_LEADER_SPACE )

#define REPEAT_LEADER_APLEN 35967   ///< 16*T=8.992ms -> (35967+1)/4000kHz=8.992ms
#define REPEAT_LEADER_SPACE 8991    ///<  4*T=2.248ms ->  (8991+1)/4000kHz=2.248ms
#define REPEAT_LEADER_DBLEN ( REPEAT_LEADER_APLEN + REPEAT_LEADER_SPACE )

#define DATA0_APLEN         2247    ///<  1*T=0.562ms ->  (2247+1)/4000kHz=0.562ms
#define DATA0_SPACE         2247    ///<  1*T=0.562ms ->  (2247+1)/4000kHz=0.562ms
#define DATA0_DBLEN         ( DATA0_APLEN + DATA0_SPACE )

#define DATA1_APLEN         2247    ///<  1*T=0.562ms ->  (2247+1)/4000kHz=0.562ms
#define DATA1_SPACE         6743    ///<  3*T=1.682ms ->  (6743+1)/4000kHz=1.682ms
#define DATA1_DBLEN         ( DATA1_APLEN + DATA1_SPACE )

#define STOP_APLEN          2247    ///<  1*T=0.562ms ->  (2247+1)/4000kHz=0.562ms
#define STOP_SPACE          2247    ///<  1*T for interrupt operation.
#define STOP_DBLEN          ( STOP_APLEN + STOP_SPACE )

#define REMC_CRPER          104     ///< 4000kHz/(104+1)=38.1kHz
#define REMC_CRDTY          34      ///< (34+1)/(104+1)=1/3
#define REMC_CARR           ( ( REMC_CRDTY << 8 ) | REMC_CRPER )

#define INTERVAL_CYCLE      26999   ///< 108ms -> (26999+1)/(4000kHz/16)=108ms

#define FRAME_DATA_LEN      34      ///< Leader(1)+CustomerCode(16)+Data(8)+^Data(8)+stop(1)
#define REPEAT_DATA_LEN     2       ///< Leader(1)+stop(1)

///< Structure for peripheral library
static uint16_t frameDataAplen[ FRAME_DATA_LEN + 2 ], frameDataDblen[ FRAME_DATA_LEN + 2 ];
static uint16_t repeatDataAplen[ REPEAT_DATA_LEN ], repeatDataDblen[ REPEAT_DATA_LEN ];
static uint16_t lastBtn, dataPtr, repeating; 
static seREMC_InitTypeDef InitStructure;

const unsigned char customerCode[ 2 ] = { 0x50, 0xaf };         ///< Hitachi-1
const unsigned char tvData[ 4 ] = { 0x17, 0x19, 0x12, 0x15 };   ///< power, ch+, vol+, vol-

int main( void ) {
  
  ///< Displays the frequency of the system clock.
  ShowRunningClock();
  
  uint16_t i, bit, src, *dstAplen, *dstDblen;

  lastBtn = 0;		///< Btn is OFF.

  frameDataAplen[ 0 ] = FRAME_LEADER_APLEN;
  frameDataDblen[ 0 ] = FRAME_LEADER_DBLEN;

  ///< Configure parameters
  bit = 0x0001;
  src = customerCode[ 0 ];
  dstAplen = &frameDataAplen[ 1 ];
  dstDblen = &frameDataDblen[ 1 ];
  for ( i = 0; i < 16; i++ ) {
    if ( ( src & bit ) == 0 ) {
      *dstAplen = DATA0_APLEN;
      *dstDblen = DATA0_DBLEN;
    } else {
      *dstAplen = DATA1_APLEN;
      *dstDblen = DATA1_DBLEN;
    }
    ++dstAplen;
    ++dstDblen;
    if ( i == 7 ) {
      src = customerCode[ 1 ];
      bit = 0x0001;
    } else {
      bit <<= 1;
    }
  }

  frameDataAplen[ 33 ] = STOP_APLEN;
  frameDataDblen[ 33 ] = STOP_DBLEN;

  repeatDataAplen[ 0 ] = REPEAT_LEADER_APLEN;
  repeatDataDblen[ 0 ] = REPEAT_LEADER_DBLEN;
  repeatDataAplen[ 1 ] = STOP_APLEN;
  repeatDataDblen[ 1 ] = STOP_DBLEN;

  seCLG_SetIoscFreqSel( BOARD_REMC_IOSC_CLK_FREQ );
  SystemCoreClockUpdate();
  
  ///< Displays the frequency of the system clock.
  ShowRunningClock();
  
  seSysSleepMS(1);  ///< wait for printf characters to flush


  ///< Setting of PORT REMO
  sePPORT_Init(sePPORT_REMC_REMO, REMC_REMO_INIT);
  
  ///< Init REMC3
  InitStructure.Clk = BOARD_REMC_CLOCK;
  InitStructure.carr = REMC_CARR;

  seCLG_Start( seSRC(InitStructure.Clk) ); 
  seCLG_SetOperInSlp( seSRC(InitStructure.Clk), seDISABLE );
  seStatus fstatus = seREMC_Init( BOARD_REMC_CH, &InitStructure ); 
  if ( fstatus ) {

    ///< Enable REMC3 interrupt
    NVIC_SetPriority( REMC3_IRQn, 3 );
    NVIC_ClearPendingIRQ( REMC3_IRQn );
    NVIC_EnableIRQ( REMC3_IRQn );
       
    ///< Enable PORT interrupt
    NVIC_SetPriority( BOARD_REMC_PPORT_IRQn, 3 );
    NVIC_ClearPendingIRQ( BOARD_REMC_PPORT_IRQn );
    NVIC_EnableIRQ( BOARD_REMC_PPORT_IRQn );
    
    ///< Assign push buttons ports.
    sePPORT_DisableChatteringFilter( BOARD_REMC_BTN2_PORT );
    sePPORT_DisableChatteringFilter( BOARD_REMC_BTN1_PORT );
    
    sePPORT_InitAsInput( BOARD_REMC_BTN2_PORT );
    sePPORT_InitAsInput( BOARD_REMC_BTN1_PORT );
    
    sePPORT_EnablePullUpResistor( BOARD_REMC_BTN2_PORT );
    sePPORT_EnablePullUpResistor( BOARD_REMC_BTN1_PORT );
    
    sePPORT_EnableInt( BOARD_REMC_BTN2_PORT, sePPORT_INT_EDGE_FALLING );
    sePPORT_EnableInt( BOARD_REMC_BTN1_PORT, sePPORT_INT_EDGE_FALLING );
   
    ///< Init T16
    seT16_InitTypeDef T16_InitStruct;
    ///< Clear parameters
    memset( &T16_InitStruct, 0, sizeof( T16_InitStruct ) );
    T16_InitStruct.Clk                    = BOARD_REMC_T16_CLOCK;
    T16_InitStruct.CounterMode            = seT16_RepeatMode;
    T16_InitStruct.Period                 = INTERVAL_CYCLE;   ///< 108ms -> (26999+1)/(16000kHz/64)=108ms
    seCLG_Start( seSRC(T16_InitStruct.Clk) );
    ///< Set IOSC to keep supplying clock to T16 in sleep
    seCLG_SetOperInSlp( seSRC(T16_InitStruct.Clk), seDISABLE );
    seT16_Init( BOARD_REMC_T16, &T16_InitStruct );
    seT16_ClearIntFlag( BOARD_REMC_T16 );
    seT16_EnableInt( BOARD_REMC_T16 ); 
    NVIC_SetPriority( BOARD_REMC_T16_IRQn, 3 );
    NVIC_ClearPendingIRQ(BOARD_REMC_T16_IRQn);  
    NVIC_EnableIRQ( BOARD_REMC_T16_IRQn );

    ///< Sleep.  Wake up from Btn1 - Btn2 interrupt.
    do{
      setHalt();
    }while( 1 );  
  }  
  
  return seSTATUS_OK; 
}

/**
  * @brief  PORT Interrupt function
  * @retval None
  */
void BOARD_REMC_PPORT_IRQHandler( void ) {
  
  seInterruptStatus Btn1, Btn2;
  
  Btn1 = sePPORT_GetIntFlag( BOARD_REMC_BTN1_PORT );
  Btn2 = sePPORT_GetIntFlag( BOARD_REMC_BTN2_PORT );
  
  if ( ( Btn1 == seINTERRUPT_OCCURRED ) || ( Btn2 == seINTERRUPT_OCCURRED ) ) {

    unsigned short newBtn;
    
    if ( Btn1 == seINTERRUPT_OCCURRED ) {
      newBtn = 1;
    } else if ( Btn2 == seINTERRUPT_OCCURRED ) {
      newBtn = 2;
    } else {
      newBtn = 0;
    }

    if ( lastBtn != newBtn ) {	///< New Btn is pressed

      unsigned short i, bit, src, *dstAplen, *dstDblen;
      
      lastBtn = newBtn;

      ///< Set APLEN and DBLEN
      bit = 0x0001;
      src = tvData[ lastBtn - 1 ];
      dstAplen = &frameDataAplen[ 17 ];
      dstDblen = &frameDataDblen[ 17 ];
      for ( i = 0; i < 16; i++ ) {
        if ( ( src & bit ) == 0 ) {
          *dstAplen = DATA0_APLEN;
          *dstDblen = DATA0_DBLEN;
        } else {
          *dstAplen = DATA1_APLEN;
          *dstDblen = DATA1_DBLEN;
        }
        ++dstAplen;
        ++dstDblen;
        if ( i == 7 ) {
          src = ~src;
          bit = 0x0001;
        } else {
          bit <<= 1;
        }
      }

      dataPtr = 2;
      repeating = 0;
      seREMC_Init( BOARD_REMC_CH, &InitStructure );
      seREMC_Start( BOARD_REMC_CH, frameDataAplen[ 0 ], frameDataDblen[ 0 ] );
      BOARD_REMC_CH->APLEN = frameDataAplen[ 1 ];	///< Set data bit active pulse length.
      BOARD_REMC_CH->DBLEN = frameDataDblen[ 1 ];	///< Set data bit length.
      BOARD_REMC_T16->CTL = 0x0103;			///< ----.---1.----.--11  PRUN=1 PRESET=0 MODEN=1  Presets the reload data and count start.

    }
    
    ///< Else interrupted by chattering
    sePPORT_ClearIntFlag( BOARD_REMC_BTN1_PORT );
    sePPORT_ClearIntFlag( BOARD_REMC_BTN2_PORT );
  }
}

/**
  * @brief  REMC3 Interrupt function
  * @retval None
  */
void REMC3_IRQHandler( void ) {
  
  if ( ( BOARD_REMC_CH->INTF & REMC3_INTF_DBIF_Msk/*seREMC_DBIF*/ ) != 0 ) {	///< ----.----.----.--1-	REMINTF_DBIF==1
    if ( repeating != 0 ) {			///< Now repeating.
      if ( dataPtr == REPEAT_DATA_LEN ) {
        BOARD_REMC_CH->DBCTL	= 0x010d;		///< ----.--01.---0.11-1	PRESET=0 PRUN=1 REMOINV=0(norm) BUFEN=1(enable) TRMD=1(one shot) MODEN=1  Preset and start counter.
      }
      if ( dataPtr == ( REPEAT_DATA_LEN + 1 ) ) {
        seREMC_Stop(BOARD_REMC_CH);
      }
      ++dataPtr;
    } else {
      BOARD_REMC_CH->APLEN = frameDataAplen[ dataPtr ];	// Set data bit active pulse length.
      BOARD_REMC_CH->DBLEN = frameDataDblen[ dataPtr ];	// Set data bit length.
      if ( dataPtr == FRAME_DATA_LEN ) {
        BOARD_REMC_CH->DBCTL	= 0x010d;	        ///< ----.--01.---0.11-1	PRESET=0 PRUN=1 REMOINV=0(norm) BUFEN=1(enable) TRMD=1(one shot) MODEN=1  Preset and start counter.
      }
      if ( dataPtr == ( FRAME_DATA_LEN + 1 ) ) {
        seREMC_Stop(BOARD_REMC_CH);
        repeating = 1;
      }
      ++dataPtr;
    }
    ///< Clear interrupt flag.
    BOARD_REMC_CH->INTF = REMC3_INTF_DBIF_Msk/*seREMC_DBIF*/;			///< ----.----.----.--1-	REMINTF_DBIF==1(clear)
  }
}

/**
  * @brief  T16 Interrupt function
  * @retval None
  */
void BOARD_REMC_T16_IRQHandler( void ) {
  
  sePPORT_Data StatBtn1, StatBtn2;

  if ( ( BOARD_REMC_T16->INTF & 0x0001 ) != 0 ) {	///< ----.----.----.---1	UFIE==1
    uint16_t BtnPressed = 0;

    StatBtn1 = sePPORT_GetInput( BOARD_REMC_BTN1_PORT );
    StatBtn2 = sePPORT_GetInput( BOARD_REMC_BTN2_PORT );
    
    
    if ( StatBtn1 == sePPORT_DATA_HIGH ) {
      if ( lastBtn == 1 ) {
        BtnPressed = 1;
      }
    } else if ( StatBtn2 == sePPORT_DATA_HIGH ) {
      if ( lastBtn == 2 ) {
        BtnPressed = 1;
      }
    }

    if ( BtnPressed != 0 ) {

      dataPtr = 2; 
      seREMC_Init( BOARD_REMC_CH, &InitStructure ); 
      seREMC_Start( BOARD_REMC_CH, repeatDataAplen[ 0 ], repeatDataDblen[ 0 ] );
      BOARD_REMC_CH->APLEN = repeatDataAplen[ 1 ]; ///< Set data bit active pulse length.
      BOARD_REMC_CH->DBLEN = repeatDataDblen[ 1 ]; ///< Set data bit length.

    } else {

      BOARD_REMC_T16->CTL = 0x0001; ///< ----.---0.----.--01  PRUN=0 PRESET=0 MODEN=1  Count stop.
      lastBtn = 0;

    }

    ///< Clear interrupt flag.
    BOARD_REMC_T16->INTF = 0x0001; ///< ----.----.----.---1	T16_0INTF_UFIF==1(clear)
  }
}

