/**
  ******************************************************************************
  * @file    main.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file is Template example software.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */ 

/** @defgroup RFC
  * @{
  * @brief The RFC is a CR oscillation
  *        type A/D converter (R/F converter).
  *
  * \section intro_sec Introduction
  * This shows an example of programming the RFC to take measurements.
  * 
  * \section hardware_setup Hardware Setup
  * 
  *  Before the use of this sample program, connect the board as follows
  *
  *   SENA0 ---------------------Resistive sensor (DC bias)
  *
  *   REF0  ---------------------Reference resistor
  *
  *   RFIN0 ---------------------Reference resistor and Reference capacitor
  *
  * \section operation_sec Operations
  * 
  * 1. Initializes RFC.
  * 2. Runs RFC and gets measurement counter.
  * 3. Displays conversion status and counter number.
  *
  * \subsection output Example of output:
  * 
  *  -CPU clock- seCLG_IOSC
  * Starting R/F Conversion operation...
  * R/F Converting operation finished with OK status, count = 253430
  * Exit
  * 
  ******************************************************************************
  */
/**
  * @}
  */
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "board.h"
#include "port_config.h"

int main( void ) {
  
  ///< Displays the frequency of the system clock.
  ShowRunningClock();

  seRFC_InitTypeDef InitStructure;
  
  ///< Setting of PORT RFC
  sePPORT_Init(sePPORT_RFC_SENB0, RFC_SENB0_INIT);
  sePPORT_Init(sePPORT_RFC_SENA0, RFC_SENA0_INIT);
  sePPORT_Init(sePPORT_RFC_REF0, RFC_REF0_INIT);
  sePPORT_Init(sePPORT_RFC_RFIN0, RFC_RFIN0_INIT);

  ///< Configure parameters
  InitStructure.Clk = BOARD_RFC_CLOCK;
  InitStructure.StopInSleep = seDISABLE;
  InitStructure.EnableExternClockInput = seDISABLE;
  InitStructure.OscMode = seRFC_DcMode;
  
  ///< Initialize RFC.
  seCLG_Start( seSRC(InitStructure.Clk) );
  seStatus fstatus = seRFC_Init( BOARD_RFC_CH, &InitStructure );
  if ( fstatus ) {
    uint32_t count = 0;
    printf( "Starting R/F Conversion operation...\n" );
    ///< Start RFC.
    seRFC_Start(BOARD_RFC_CH);
    ///< Run RFC and get measurement counter.
    int16_t result = seRFC_RunConvertingOperation( BOARD_RFC_CH, 0x00fc0000UL , &count );
    fstatus = ( result == RFC_STS_CONV_SEN_A || result == RFC_STS_CONV_SEN_B ) ? seSTATUS_OK : seSTATUS_NG;
    printf( "R/F Converting operation finished with %s status, count = %u\n", fstatus?"OK":"NG", count );
  }

  printf( "Exit\n" );
  
  return fstatus;
}

