/**
  ******************************************************************************
  * @file    main.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file is RTCA example software.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

/** @defgroup RTCA
  * @{
  * @brief RTCA module is a real-time clock with a perpetual calendar function
  *
  * \section intro_sec Introduction
  * This example shows how to program and read the various functions of the real-time clock
  * such as the time/date, stopwatch, alarm, and trimming.
  * 
  * \section operation_sec Operations
  * 
  * 1. Initializes RTCA.
  * 2. Starts RTCA.
  * 3. Calculates TRM.
  * 4. Sets time and date, and reads it back.
  * 5. Sets an alarm and CPU goes to sleep. Expects interrupt while sleeping.
  * 6. Starts 1 second timer to perform trimming.
  * 7. Checks stop watch operations.
  *
  * \subsection output Example of output:
  *
  *  -CPU clock- seCLG_IOSC
  * RTCA TRM bits 0x43
  * RTCA Set hour 4 (PM), minute 17, second 0
  * RTCA Get hour 4 (PM), minute 17, second 0
  * RTCA Set year 15, month 2, day 5, Thursday
  * RTCA Get year, month, day. 2015, 2, 5, Thursday
  * RTCA Set Alarm hour 4 (PM), minute 17, second 5
  * RTCA waiting for alarm....
  * RTCA Alarm occurred : hour 4(PM), minute 17, second 5
  * RTCA Start Trimming
  * RTCA waiting for timer....
  * RTCA Trimming performed : hour 4(PM), minute 17, second 11
  * RTCA StopWatch (start) : hour 4(PM), minute 17, second 11
  * RTCA sleep 5 seconds waiting for Stop....
  * RTCA interrupts disabled for count reading
  * SysTick 5000
  * StopWatchHW: 5.14
  * Exit
  * 
  ******************************************************************************
  */
/**
  * @}
  */
#include <string.h>
#include <stdio.h>
#include "board.h"

///< Structures and variables for testing 
static seRTCA_InitTypeDef InitStruct;
static const char* DayOfWeek[] = {  "Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", };
static const char* AM_PMIndicator[] = { "AM", "PM", };
static uint16_t AlarmOccured = 0;
static int8_t trm_bits = 0;
static uint32_t seconds_counter = 4090;
static swCounter StopWatchHW = { 0, };
uint32_t lasttick, systicks;

int main( void ) {
  
  seStatus fStatus = seSTATUS_OK; 
  uint8_t h=0, m=0, s=0;
  uint8_t y=0, mo=0, m1=0, d=0;
  seRTCA_AM_PM ind;
  uint8_t w;
  uint32_t Timer;
  
  ///< Displays the frequency of the system clock.
  ShowRunningClock();
  
  seconds_counter = 4090;
  
  ///< Init RTCA module
  InitStruct.ClkSupldInDebugMode = seENABLE;
  InitStruct.H12_24Format = seRTCA_CTL_RTC24H_12;
  seCLG_Start( seCLG_OSC1 );
  seRTCA_Init( &InitStruct );
  
  ///< RTCA started 
  seRTCA_Start( );
  
  ///< Calculate TRM. Correction rate : -58.2 ppm(Bits = 0x43).
  ///< Execution cycle time: 4096 seconds.
  trm_bits = seRTCA_CalculateTrm( -582, 4096 );
  printf( "RTCA TRM bits 0x%x\r\n", trm_bits );
  
  ///< Set time and date, read it back
  h = 4; m = 17; s = 0; ind = seRTCA_HUR_RTCAP_PM;
  printf( "RTCA Set hour %d (%s), minute %d, second %d \n", h, AM_PMIndicator[ind], m, s );
  seRTCA_SetAM_PM( ind );
  seRTCA_SetHourMinuteSecond( h, m, s );
  seRTCA_GetHourMinuteSecond( &h, &m, &s);
  ind = seRTCA_GetAM_PM();
  printf( "RTCA Get hour %d (%s), minute %d, second %d \n",h, AM_PMIndicator[ind], m, s );
  ///< set date. Read it back
  y = 15; mo = 2; d = 5; w = seRTCA_YAR_RTCWK_THU;
  printf( "RTCA Set year %d, month %d, day %d, %s \n", y, mo, d, DayOfWeek[w] );
  seRTCA_SetYearMonthDayWeek( y, mo, d, w );
  seRTCA_GetYearMonthDayWeek( &y, &mo, &d, &w );  
  printf( "RTCA Get year, month, day. 20%d, %d, %d, %s  \n", y, mo, d, DayOfWeek[w] );
  
  ///< Set an alarm and sleep. Expect interrrupt while sleeping.
  h = 4; m = 17; s = 5; ind = seRTCA_HUR_RTCAP_PM;
  printf( "RTCA Set Alarm hour %d (%s), minute %d, second %d \n", h, AM_PMIndicator[ind], m, s );
  seRTCA_SetAlarm( h, m, s, ind );
  NVIC_SetPriority( RTC_IRQn, 3 );  
  NVIC_EnableIRQ( RTC_IRQn );
  printf( "RTCA waiting for alarm....\n" );
  do {
    ///< Wait For Interrupt
    setHalt();
  } while ( AlarmOccured != seINTERRUPT_OCCURRED );
  AlarmOccured = seINTERRUPT_NOT_OCCURRED;
  m1 = m;
  seRTCA_GetHourMinuteSecond( &h, &m, & s);
  printf( "RTCA Alarm occured : hour %d(%s), minute %d, second %d \n", 
         h,( seRTCA_GetAM_PM() == seRTCA_HUR_RTCAP_AM ) ? "AM" : "PM", m, s );
  ///< Reflect operation in the status.
  if ( fStatus ) {
    fStatus = ( m1 == m ) ? seSTATUS_OK : seSTATUS_NG;
  } 
  
  printf( "RTCA Start Trimming\n" );
  ///< Start 1 second timer to perform trimming
  seRTCA_Enable1SecTimer( );
  printf( "RTCA waiting for timer...\n" );
  ///< Wait in the loop with 5 secs timeout
  Timer = seSysGetTimeMS();
  do {
    ///< Request Wait For Interrupt 
    setHalt();
    if ( (seSysGetTimeMS() - Timer) >= 10000 ) {
      break;
    }
  } while ( seconds_counter != 0 );
  
  seRTCA_DisableInt( RTCA_INTE_T1SECIE_Msk );
  
   ///< Reflect operation in the status.
  if ( fStatus ) {
    fStatus = ( seconds_counter == 0 ) ? seSTATUS_OK : seSTATUS_NG;
  }
  seRTCA_GetHourMinuteSecond( &h, &m, & s);
  printf( "RTCA Trimming performed : hour %d(%s), minute %d, second %d \n",
         h,( seRTCA_GetAM_PM() == seRTCA_HUR_RTCAP_AM ) ? "AM" : "PM", m, s );
  
  
  ///< Check stop watch operations
  ///< Start
  seRTCA_GetHourMinuteSecond( &h, &m, & s);
  printf( "RTCA StopWatch (start) : hour %d(%s), minute %d, second %d \n",
         h,( seRTCA_GetAM_PM() == seRTCA_HUR_RTCAP_AM ) ? "AM" : "PM", m, s );
  
  ///< Reset
  seRTCA_ResetStopWatchCount( &StopWatchHW );
  // Start
  seRTCA_StartStopWatchCount( RTCA_INTE_SW1IE_Msk );
  
  ///< Sleep 5 seconds
  printf( "RTCA sleep about 5 seconds waiting for Stop....\n" );
  lasttick = seSysGetTimeMS();
  ///< For accurate result define IOSC_AUTOTRIMMING_ON in settings.h
  seSysSleepMS( 5000 );
  
  ///< disable interrupts and read hw count
  systicks = seSysGetTimeMS() - lasttick;
  printf( "RTCA interrupts disabled for count reading\n");
  NVIC_DisableIRQ( RTC_IRQn );
  seRTCA_StopStopWatchCount( RTCA_INTE_SW1IE_Msk ); 
  if ( seSTATUS_OK == seRTCA_ReadStopWatchCount( &StopWatchHW ) ) {
    
    printf( "SysTick %u\n", systicks );
    printf( "StopWatchHW: %d.%d%d \n", StopWatchHW.swInt1HzCount,
           StopWatchHW.swChar10HzDigit, StopWatchHW.swChar100HzDigit );
    
  } else {
    printf( "seRTCA_ReadStopWatchCount failed" );
    fStatus = seSTATUS_NG;
  }
  
  printf( "Exit\n" );
  
  return fStatus;
}

/**
  * @brief  RTCA Interrupt function
  * @retval None
  */
void RTCA_IRQHandler( void ) {
  
  ///< alarm interrupt
  if( RTCA->INTE & RTCA_INTE_ALARMIE_Msk ) {
    if( RTCA->INTF & RTCA_INTF_ALARMIF_Msk ) {
      AlarmOccured = seINTERRUPT_OCCURRED;
      RTCA->INTF = RTCA_INTF_ALARMIF_Msk;
    }
  }
  ///< 1 second interrupt
  if( RTCA->INTE & RTCA_INTE_T1SECIE_Msk ) {
    if( RTCA->INTF & RTCA_INTF_T1SECIF_Msk ) {
      seconds_counter++;
      RTCA->INTF = RTCA_INTF_T1SECIF_Msk;
      if ( seconds_counter >= 4096 ) {
        if ( RTCA->CTLH_b.RTCTRMBSY == 0 ) {
          RTCA->CTLH_b.RTCTRM = trm_bits;
          seconds_counter = 0;
        }
      }
    }
  }
  ///< Stopwatch 1Hz interrupt
  if( RTCA->INTE & RTCA_INTE_SW1IE_Msk ) {
    if( RTCA->INTF & RTCA_INTF_SW1IF_Msk ) {
      StopWatchHW.swInt1HzCount++;
      RTCA->INTF = RTCA_INTF_SW1IF_Msk;
    }
  }
}

