/**
  ******************************************************************************
  * @file    main.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file is SNDA example software.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
*/

/** @defgroup SNDA
  * @{
  * @brief SNDA is a sound generator that generates melodies and buzzer signals.
  *
  * \section intro_sec Introduction
  * This example programs the SNDA to generate various tones and music.
  * 
  * \section operation_sec Operations
  * 
  * 1. Example starts OSC1 and initializes SNDA.
  * 2. SNDA runs in normal buzzer mode for 5 sec.
  * 3. SNDA runs in one-shot buzzer mode for 250msecs.
  * 4. SNDA runs in melody mode.
  *
  * \subsection output Example of output:
  *
  *  -CPU clock- seCLG_IOSC
  * Start SNDA in normal buzzer mode for 5 sec
  * Start SNDA in one-shot buzzer mode, 250ms
  * Start SNDA in melody mode
  * Start SNDA playing of 'Postoj Parovoz' music
  * Exit
  *
  ******************************************************************************
 */
/**
  * @}
  */
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "board.h"
#include "port_config.h"

///< Structures for testing peripheral library
static seSNDA_InitTypeDef SNDA_InitStruct;
static const uint16_t data[] = {
                           0x00f8, 0x00ea, 0x00dd, 0x00d1, 0x00c5, 0x00ba, 0x00af,
                           0x00a5, 0x009c, 0x0093, 0x008b, 0x0083, 0x007c, 0x0075,
                           0x006e, 0x0068, 0x0062, 0x005c, 0x0057, 0x0052, 0x004e,
                           0x0049, 0x0045, 0x0041, 0x003d, 0x003a, 0x0037, 0x0033,
                           0x0030, 0x002e, 0x002b, 0x0029, 0x0026, 0x0024, 0x0022,
                           0x0020, 0x001e };

static const uint16_t PostojParovoz[] = {
                                    0x0362, 0x0762, 0x0349, 0x0352, 0x075C, 0x036E,
                                    0x037C, 0x0783, 0x0362, 0x036E, 0x037C, 0x0793,
                                    0x0352, 0x0752, 0x037C, 0x0362, 0x0352, 0x033D,
                                    0x0341, 0x0349, 0x0F4E, 0x8B4E, 0x0362, 0x073D,
                                    0x0341, 0x033D, 0x0341, 0x0749, 0x0362, 0x0749,
                                    0x0352, 0x0349, 0x0352, 0x075C, 0x035C, 0x0762,
                                    0x0352, 0x035C, 0x0362, 0x036E, 0x037C, 0x0383,
                                    0x0F93 };

int main( void ) {

  seStatus fStatus; 
  
  ///< Displays the frequency of the system clock.
  ShowRunningClock();
  
  ///< Clear parameters
  memset( &SNDA_InitStruct, 0, sizeof( SNDA_InitStruct ) );
  
  ///< Setting of PORT SNDA
  sePPORT_Init(sePPORT_BZOUT, SNDA_BZOUT_INIT);
  sePPORT_Init(sePPORT_NBZOUT, SNDA_NBZOUT_INIT);

  ///< Configure parameters
  seSNDA_InitStruct( &SNDA_InitStruct );
  SNDA_InitStruct.Clk          = BOARD_SNDA_CLOCK;
  seCLG_Start( seSRC(SNDA_InitStruct.Clk) );
  fStatus = seSNDA_Init( BOARD_SNDA_CH, &SNDA_InitStruct );

  if ( fStatus == seSTATUS_OK ) {
    
    printf( "Start SNDA in normal buzzer mode for 5 sec\n" );
    ///<  Start SNDA in normal buzzer mode
    seSNDA_Start( BOARD_SNDA_CH, 0x1f, 0x0f );
    ///<  Lets SNDA to buzzer for 5 sec before stop it
    seSysSleepMS(5000); 
    ///<  Stop SNDA
    seSNDA_Stop(BOARD_SNDA_CH);
    
    ///<  Separate buzzer sounds with 2000 ms of silence
    seSysSleepMS( 2000 ); 
    
    printf( "Start SNDA in one-shot buzzer mode, 250ms\n" );
    ///<  Start SNDA in one-shot buzzer mode
    seSNDA_StartOneShot( BOARD_SNDA_CH, 0x1f, 0x0f, 0x0f );
    
    ///<  Separate buzzer sounds with 2000 ms of silence
    seSysSleepMS( 2000 ); 
    
    printf( "Start SNDA in melody mode\n" );
    ///<  Start SNDA in melody mode 
    seSNDA_StartMelody( BOARD_SNDA_CH, data, sizeof(data)/2, 0x07 );
    
    ///<  Separate buzzer sounds with 2000 ms of silence
    seSysSleepMS( 2000 );
    
    printf( "Start SNDA playing of 'Postoj Parovoz' music\n" );
    ///<  Play music Postoj Parovoz
    seSNDA_StartMelody( BOARD_SNDA_CH, PostojParovoz, sizeof(PostojParovoz)/2, 0x03 );
  }

  printf( "Exit\n" );
  
  return fStatus;
} 

