/**
  ******************************************************************************
  * @file    main.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file is SPIA example software.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
*/

/** @defgroup SPIA_MASTER
  * @{
  * @brief SPIA module is is a synchronous serial interface supporting both master and slave modes.
  *
  * \section intro_sec Introduction
  *
  * This example provides a description of how to use a SPIA channel in the master mode to transfer data buffer.
  *
  * \section hardware_setup Hardware Setup
  *
  * 1. Connect the evaluation boards where the SPIA master/slave sample programs are installed. Connect each port as follows:
  *
  *      [master]                                                                         [slave]
  *      BOARD_SPI_SDI_PORT         SDIn------------<<-----------SDOn          BOARD_SPI_SDO_PORT
  *
  *      BOARD_SPI_SDO_PORT         SDOn------------>>-----------SDIn          BOARD_SPI_SDI_PORT
  *
  *      BOARD_SPI_CLK_PORT      SPICLKn------------>>-----------SPICLKn       BOARD_SPI_CLK_PORT
  *
  *      BOARD_MASTER_CS_PORT          .------------>>-----------#SPISSn        BOARD_SPI_SS_PORT
  *
  *      BOARD_SPI_HANDSHAKE_PORT      .------------>>-----------.       BOARD_SPI_HANDSHAKE_PORT
  *
  *                                  GND------------><-----------GND
  *
  * 2. Launch the slave example program first, then the master program
  * 
  * \section operation_sec Operation
  * 
  * 1. Initialize the SPIA module in master mode as below:
  *    - Data length is 8bit
  *    - Data format is MSB first
  *    - Use 16-bit timer T16_1 for baud rate generator.
  * 2. Set bus speed to BOARD_SPI_BUS_SPEED_HZ.
  * 3. Assign GPIO BOARD_SPI_HANDSHAKE_PORT as output to set Read/Write command to the slave.
  * 4. Send the 'BUF_SIZE' bytes of random data to the slave.
  * 5. Receive the 'BUF_SIZE' bytes of data from the slave.
  * 6. Compare whether the received data is the same as the sent data and exit.
  *
  * \subsection output Example of output:
  *
  *  -CPU clock- seCLG_IOSC
  * Set bus speed 1000000
  * Get bus speed 1000000
  * OK(1), NG(0)
  * OK(2), NG(0)
  * OK(3), NG(0)
  * OK(4), NG(0)
  * OK(5), NG(0)
  * OK(6), NG(0)
  *
  * The SPIA software shows methods of calling the Peripheral Library modules to interface Hardware
  * components on this chip. The sample programs can be built and executed on the Epson SVT board.
  *
  ******************************************************************************
  */
/**
  * @}
  */
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "board.h"
#include "port_config.h"

#define BUF_SIZE 64

#ifdef SPIA_DMA
static uint8_t DMA_ARRAY[0x100];
#endif

///< Structures for testing peripheral library
static seT16_InitTypeDef T16_InitStruct;
static seSPIA_InitTypeDef SPI_InitStruct;

///< static functions
static void MasterTransmit( void ) { sePPORT_SetOutput( BOARD_SPI_HANDSHAKE_PORT, sePPORT_DATA_HIGH ); }
static void MasterReceive( void )  { sePPORT_SetOutput( BOARD_SPI_HANDSHAKE_PORT,  sePPORT_DATA_LOW ); }

static void ConfigureSpiMasterCS( void ); 
static void AssertSpiMasterCS( void ); 
static void NegateSpiMasterCS( void ); 

#ifdef SPIA_DMA
static uint16_t r16_data[BUF_SIZE/2];
static uint16_t w16_data[BUF_SIZE/2];
#else
static uint8_t r_data[BUF_SIZE];
static uint8_t w_data[BUF_SIZE];
#endif

int main( void ) {

  seStatus fStatus;

  ///< Displays the frequency of the system clock.
  ShowRunningClock();
  
  ///< Clear parameters
  memset( &T16_InitStruct, 0, sizeof( T16_InitStruct ) );
  memset( &SPI_InitStruct, 0, sizeof( SPI_InitStruct ) );

#ifdef SPIA_DMA
  seDMAC_Init( ( (uint32_t)DMA_ARRAY + 0x80) & ~0x7FU, 3 );
#endif
  
  ///< Init T16
  seT16_InitStruct(&T16_InitStruct);
  T16_InitStruct.Clk = BOARD_SPI_T16_CLOCK;
  T16_InitStruct.Period = 80;
  seCLG_Start( seSRC(T16_InitStruct.Clk) );
  seT16_Init( seAUX_GetT16ch_ForPeripheral(BOARD_SPI_CH), &T16_InitStruct );
  
  ///< Init Spi as Master
  ///< Setting of PORT SPI
  //sePPORT_Init(BOARD_SPI_SS_PORT, SPI_SS_PORT_INIT);
  sePPORT_Init(BOARD_SPI_SDI_PORT, SPI_SDI_PORT_INIT);
  sePPORT_Init(BOARD_SPI_SDO_PORT, SPI_SDO_PORT_INIT);
  sePPORT_Init(BOARD_SPI_CLK_PORT, SPI_CLK_PORT_INIT);
  
  seSPIA_InitStructForMaster(&SPI_InitStruct);
  
#ifdef SPIA_DMA
  SPI_InitStruct.CHLN = seDATA_TR_LENGTH_16BIT;
#endif
  seSPIA_Init( BOARD_SPI_CH, &SPI_InitStruct );
  ConfigureSpiMasterCS(); ///< Configure GPIO for SS use
  
  uint32_t busspeed = BOARD_SPI_BUS_SPEED_HZ;
  printf( "Set bus speed %u \n", busspeed );
  seAUX_SetBusSpeedForSPI( BOARD_SPI_CH, busspeed );
  printf( "Get bus speed %u \n", seAUX_GetBusSpeedForSPI(BOARD_SPI_CH) );
  
  ///< Assign GPIO output to set Read/Write command to Slave
  sePPORT_InitAsOutput( BOARD_SPI_HANDSHAKE_PORT );
  sePPORT_SetOutput( BOARD_SPI_HANDSHAKE_PORT, sePPORT_DATA_LOW );
  
  fStatus = seSPIA_Start( BOARD_SPI_CH );
  if ( fStatus == seSTATUS_OK ) {
    
    uint32_t test_OK = 0, test_NG = 0;
    
    while ( 1 ) {
      
#ifdef SPIA_DMA
      for ( int i = 0; i < BUF_SIZE/2; i++ ) {
          w16_data[i] = rand();
      } 
#else
      ///< Non DMA transfer can be used here as well
      for ( int i = 0; i < BUF_SIZE; i++ ) {
          w_data[i] = rand();
      }
#endif
      
      MasterTransmit(); ///< Signal to Slave to receive
      AssertSpiMasterCS();
      
#ifdef SPIA_DMA
      fStatus = seDMASPI_TxHWords( BOARD_SPI_CH, w16_data, BUF_SIZE/2 );
#else
      ///< Non DMA transfer can be used here as well 
      fStatus = seSPIA_TxBytes( BOARD_SPI_CH, w_data, BUF_SIZE );
#endif
      
      NegateSpiMasterCS();
      
      seSysSleepMS(1); ///< Turn-around delay
      
      if ( fStatus == seSTATUS_OK ) {
        
        MasterReceive(); ///< Prepare for next signal
        AssertSpiMasterCS();
        
#ifdef SPIA_DMA
        fStatus = seDMASPI_RxHWords( BOARD_SPI_CH, r16_data, BUF_SIZE/2 );
#else
        ///< Non DMA transfer can be used here as well
        fStatus = seSPIA_RxBytes( BOARD_SPI_CH, r_data, BUF_SIZE );
#endif
        
        NegateSpiMasterCS();
        
        ///< Compare writes and reads
        if ( fStatus == seSTATUS_OK ) {
        
#ifdef SPIA_DMA
          if ( memcmp( r16_data, w16_data, BUF_SIZE/2 ) ) {
            fStatus = seSTATUS_NG;
            printf( "SPI read/write check failed (%u)\n", ++test_NG );
            if ( seINTERRUPT_OCCURRED == seSPIA_GetIntFlag( BOARD_SPI_CH, SPIA_0_INTF_OEIF_Msk ) ) {
              seSPIA_Reset( BOARD_SPI_CH );
            }
          }
#else
          ///< Non DMA transfer can be used here as well
          if ( memcmp( r_data, w_data, BUF_SIZE ) ) {
            fStatus = seSTATUS_NG;
            printf( "SPI read/write check failed (%u)\n", ++test_NG );
            if ( seINTERRUPT_OCCURRED == seSPIA_GetIntFlag( BOARD_SPI_CH, SPIA_0_INTF_OEIF_Msk ) ) {
              seSPIA_Reset( BOARD_SPI_CH );
            }
          }
#endif
          
          if ( fStatus == seSTATUS_OK ) {
            printf( " - OK(%u), NG(%u)\n", ++test_OK, test_NG );
          }
        }
      }
    }
  }
  
  return fStatus;
}

/**
  * @brief  Assign SPIA slave select signal input pin
  * @retval None
  */
static void ConfigureSpiMasterCS( void ) {
  sePPORT_InitAsOutput( BOARD_MASTER_CS_PORT );
  sePPORT_SetOutput( BOARD_MASTER_CS_PORT, sePPORT_DATA_HIGH );
}

/**
  * @brief  Assert SPIA(Master mode only)
  * @retval None
  */
static void AssertSpiMasterCS( void ) {
  sePPORT_SetOutput( BOARD_MASTER_CS_PORT, sePPORT_DATA_LOW );
  ///< Wait...
  __NOP();
}

/**
  * @brief  Negate SPIA(Master mode only)
  * @retval None
  */
static void NegateSpiMasterCS( void ) {
  sePPORT_SetOutput( BOARD_MASTER_CS_PORT, sePPORT_DATA_HIGH );
  ///< Wait...
  __NOP();
}

