/**
  ******************************************************************************
  * @file    main.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file is SPIA example software.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

/** @defgroup SPIA_SLAVE
  * @{
  * @brief SPIA module is is a synchronous serial interface supporting both master and slave modes. Only one SPIA channel is available in S1C31 microcontroller.
  *
  * \section intro_sec Introduction
  *
  * This example provides a description of how to use a SPIA channel in the slave mode to transfer data buffer.
  * 
  * \section hardware_setup Hardware Setup
  *
  * 1. Connect the evaluation boards where the SPIA master/slave sample programs are installed. Connect each port as follows:
  * 
  *      [master]                                                                         [slave]
  *      BOARD_SPI_SDI_PORT         SDIn------------<<-----------SDOn          BOARD_SPI_SDO_PORT
  *
  *      BOARD_SPI_SDO_PORT         SDOn------------>>-----------SDIn          BOARD_SPI_SDI_PORT
  *
  *      BOARD_SPI_CLK_PORT      SPICLKn------------>>-----------SPICLKn       BOARD_SPI_CLK_PORT
  *
  *      BOARD_MASTER_CS_PORT          .------------>>-----------#SPISSn        BOARD_SPI_SS_PORT
  *
  *      BOARD_SPI_HANDSHAKE_PORT      .------------>>-----------.       BOARD_SPI_HANDSHAKE_PORT
  *
  *                                  GND------------><-----------GND
  * 
  * 2. Launch the slave example program first, then the master program
  * 
  * \section operation_sec Operation
  * 
  * 1. Initialize the SPIA module in slave mode as below:
  *    - Data length is 8bit
  *    - Data format is MSB first
  * 2. Assign GPIO BOARD_SPI_HANDSHAKE_PORT as input to recognize Read/Write command from master.
  * 3. Receive the 'BUF_SIZE' bytes of data from the master.
  * 4. Send the 'BUF_SIZE' bytes of random data to the master.
  * 
  ******************************************************************************
  */
/**
  * @}
  */
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "board.h"
#include "port_config.h"

///< Structures for SPI testing
#define BUF_SIZE        64

#ifdef SPIA_DMA
static uint8_t DMA_ARRAY[0x100];
#endif

uint8_t data[BUF_SIZE];

uint16_t data_16 [BUF_SIZE/2];
 
///< Structures for testing peripheral library
static seSPIA_InitTypeDef SPI_InitStruct;

int main( void ) {
  
  seStatus fStatus;
  
  ///< Clear parameters
  memset( &SPI_InitStruct, 0, sizeof( SPI_InitStruct ) );
  
#ifdef SPIA_DMA
  seDMAC_Init( ( (uint32_t)DMA_ARRAY + 0x80) & ~0x7FU, 3 );
#endif
  
  seCLG_Start( seCLG_IOSC ); 
  
  ///< Init Spi as Slave
  ///< Setting of PORT QSPI
  sePPORT_Init(BOARD_SPI_SS_PORT,  SPI_SS_PORT_INIT);
  sePPORT_Init(BOARD_SPI_SDI_PORT, SPI_SDI_PORT_INIT);
  sePPORT_Init(BOARD_SPI_SDO_PORT, SPI_SDO_PORT_INIT);
  sePPORT_Init(BOARD_SPI_CLK_PORT, SPI_CLK_PORT_INIT);
  seSPIA_InitStructForSlave(&SPI_InitStruct);
  
#ifdef SPIA_DMA
  
  SPI_InitStruct.CHLN = seDATA_TR_LENGTH_16BIT;
  
#endif
  
  seSPIA_Init( BOARD_SPI_CH, &SPI_InitStruct );
  
  ///< Init GPIO as R/W communication flag
  ///< Assign GPIO port as input to sync with Master
  sePPORT_InitAsInput( BOARD_SPI_HANDSHAKE_PORT );
  NVIC_ClearPendingIRQ( BOARD_SPI_PPORT_IRQn );
  sePPORT_ClearIntFlag( BOARD_SPI_HANDSHAKE_PORT );
 
  ///< Enable IRQ
  NVIC_SetPriority( BOARD_SPI_PPORT_IRQn , 3 );
  NVIC_EnableIRQ( BOARD_SPI_PPORT_IRQn );
  ///< Port interrupt
  sePPORT_ClearIntFlag( BOARD_SPI_HANDSHAKE_PORT );
  sePPORT_EnableInt( BOARD_SPI_HANDSHAKE_PORT, sePPORT_INT_EDGE_RISING );
  
  fStatus = seSPIA_Start( BOARD_SPI_CH );
  if ( fStatus == seSTATUS_OK ) {
  
    while ( 1 ) {

#ifdef SPIA_DMA
      memset( data_16, 0, BUF_SIZE );
#else
      memset( data, 0, BUF_SIZE );
#endif
      if ( seINTERRUPT_OCCURRED == seSPIA_GetIntFlag( BOARD_SPI_CH, SPIA_0_INTF_OEIF_Msk ) ) {
        seSPIA_Reset( BOARD_SPI_CH );
	fStatus = seSTATUS_NG;
      }
    }
  }
  
  return fStatus;
}

/**
  * @brief  PORT Interrupt function
  * @retval None
  */
void BOARD_SPI_PPORT_IRQHandler( void ) {
  
  if (sePPORT_GetIntFlag( BOARD_SPI_HANDSHAKE_PORT ) == seINTERRUPT_OCCURRED) {
  
#ifdef SPIA_DMA
    ///< Slave receive
    seDMASPI_RxHWords( BOARD_SPI_CH, data_16, BUF_SIZE/2 );
    ///< Slave send
    seDMASPI_TxHWords( BOARD_SPI_CH, data_16, BUF_SIZE/2 );
#else
    ///< Slave receive
    seSPIA_RxBytes( BOARD_SPI_CH, data, BUF_SIZE );
    // Slave send
    seSPIA_TxBytes( BOARD_SPI_CH, data, BUF_SIZE );
#endif
    ///< Clear Interrupt flag
    sePPORT_ClearIntFlag( BOARD_SPI_HANDSHAKE_PORT );
    NVIC_ClearPendingIRQ( BOARD_SPI_PPORT_IRQn );
  }
}

