/**
  ******************************************************************************
  * @file    main.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file is SVD4 Driver example software.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

/** @defgroup SVD4_CH0
  * @{
  *
  * @brief SVD4 is a supply voltage detector to monitor the power supply voltage on the VDD pin or
  *        the voltage applied to an external pin.
  *
  * \section intro_sec Introduction
  * This example shows VDD voltage drop detection.
  *
  * \section hardware_setup Hardware Setup
  * 
  * Example requires evaluation board configured to power from the debug probe.
  * The PC's USB port is connected to VDD on evaluation board with USB cable.
  *
  * \section operation_sec Operations
  * 
  * 1. Software prepares to watch the VDD pin dropping the voltage below some limit.
  * 2. USB cable is disconnected.
  * 3. Software detects voltage drop and reports it on the terminal.
  *
  * \subsection output Example of output:
  *
  * - CPU clock- seCLG_IOSC
  *
  * SVD is configured to detect the voltage drop.
  * Please Disconnect USB cable.
  * Power voltage drop detected
  * Interrupt Low voltage occurred
  * IRQ handler called
  * Exit
  *
  ******************************************************************************
  */
/**
  * @}
  */
#include <string.h>
#include <stdio.h>
#include <board.h>
#include "port_config.h"

///< Structure for peripheral library
static seSVD_InitTypeDef SVD_InitStruct;

///< Interrupt flag.
static uint16_t svdIntLowVoltage = seINTERRUPT_NOT_OCCURRED;
static uint16_t irq = seINTERRUPT_NOT_OCCURRED;

int main( void ) {
  
  seStatus fStatus;

  ///< Displays the frequency of the system clock.
  ShowRunningClock();

  ///< Setting of PORT SVD
  //sePPORT_Init(sePPORT_SVD_EXSVD0, SVD_EXSVD0_INIT);
  sePPORT_Init(sePPORT_SVD_EXSVD1, SVD_EXSVD1_INIT);


  ///< Configure parameters
  seSVD_InitStruct(&SVD_InitStruct);
  SVD_InitStruct.Clk               = BOARD_SVD_CLOCK;
  SVD_InitStruct.IntermittentMode  = seSVD_Continious;
  SVD_InitStruct.SamplingResCnt    = seSVD_CTL_8_TIMES_ROW;
  seCLG_Start( seSRC(SVD_InitStruct.Clk) );
  seSVD_Init( BOARD_SVD_CH, &SVD_InitStruct );
  NVIC_ClearPendingIRQ( BOARD_SVD_IRQn );
  NVIC_EnableIRQ( BOARD_SVD_IRQn );
  fStatus = seSVD_SetComparisonVoltage( BOARD_SVD_CH, 1 );
 
  if ( fStatus == seSTATUS_OK ) {
    
    seSVD_PowerSupply PowerVoltageStatus;
    uint32_t Timer, TimeoutMS = 500000;
    
    printf( "\nSVD is configured to detect the voltage drop.\nPlease drop voltage on " GET_NAME(BOARD_EXSVD_PORT) ".\n" );
    seSVD_Start( BOARD_SVD_CH );
  
    Timer = seSysGetTimeMS();
   
    while ( (seSysGetTimeMS() - Timer) < TimeoutMS ) {

      if ( seINTERRUPT_OCCURRED == seSVD_IsIntLowVoltage( BOARD_SVD_CH ) ) {
        svdIntLowVoltage = seINTERRUPT_OCCURRED;
        seSVD_ClearIntLowVoltage( BOARD_SVD_CH );
      }
      
      PowerVoltageStatus = seSVD_GetVoltageDetection( BOARD_SVD_CH );
      if ( PowerVoltageStatus == seSVD_POWER_LOW ) {
          break;
      }
      setHalt();
    }
    
    if ( PowerVoltageStatus == seSVD_POWER_LOW ) {
      printf( "Power voltage drop detected\n" );
    } else {
      printf( "Power voltage OK\n" );
    }
    
    if ( svdIntLowVoltage == seINTERRUPT_OCCURRED ) {
      printf( "Interrupt Low voltage occurred\n" );
    } else {
      printf( "Interrupt Low voltage not occurred\n" );
    }
    
    if ( irq == seINTERRUPT_OCCURRED ) {
      printf( "IRQ handler called\n" );
    } else {
      printf( "IRQ handler not called\n" );
    }
    
    fStatus = seSTATUS_OK;
  } else {
    printf( "Failed to set comparision vaoltage\n" );
  }
  
  printf( "Exit\n" );
  
  return fStatus;
}

/**
  * @brief  SVD Interrupt function
  * @retval None
  */
void BOARD_SVD_IRQHandler( void ) {

  irq = seINTERRUPT_OCCURRED;
  BOARD_SVD_CH->INTE_b.SVDIE = 0;
}

