/**
  ******************************************************************************
  * @file    main.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file is T16 timer example software.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

/** @defgroup T16
  * @{
  * @brief T16 module is a 16-bit timer.
  *
  * \section intro_sec Introduction
  * This example programs the T16 channel x timer to interrupt periodically and counts the
  * number of interrupts that occurred within a 5-second interval.
  * 
  * \section operation_sec Operations
  *
  * 1. BOARD_TRIM_CLG is selected as timer clock source. Example code does BOARD_TRIM_CLG auto-trimming to provide clock accuracy for the timer clock.
  * 2. Initializes T16 channel x.
  * 3. Configures T16 channel x interrupts.
  * 4. Enables T16 interrupts.
  * 5. Counts a number of interrupts happened during 5 second interval.
  *
  * \subsection output Example of output:
  *
  *  -CPU clock- seCLG_IOSC
  * CLG IOSC Auto-trimming ok
  * IRQ interrupted: 1695 times during 5 sec
  * Exit
  *
  ******************************************************************************
  */
/**
  * @}
  */
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "board.h"


static seT16_InitTypeDef T16_InitStruct;
static uint32_t t16IntCount;

int main( void ) {
  
  T16_0_Type* T16x = BOARD_T16_CH;
  seStatus fStatus;
  
  ///< Displays the frequency of the system clock.
  ShowRunningClock();
  
  seSysSleepMS(1);  ///< wait for printf characters to flush
  
  ///< Run Autotrimming of BOARD_TRIM_CLG. Temporary switch to BOARD_TEMP_CLG
  seCLG_Start( BOARD_TEMP_CLG ); 
  fStatus = seCLG_RunAutoTrimming( BOARD_TRIM_CLG, BOARD_TEMP_CLG );
  printf ( "\nCLG " GET_NAME(BOARD_TRIM_CLG) " Auto-trimming %s\n", fStatus?"ok":"failed" );
  seCLG_ClearIntFlag( BOARD_TRIM_TERIF | BOARD_TRIM_TEDIF ); 
  
  ///< Configure parameters
  seT16_InitStruct( &T16_InitStruct );
  T16_InitStruct.Clk                = BOARD_T16_CLOCK;
  T16_InitStruct.CounterMode        = seT16_RepeatMode;
  T16_InitStruct.Period             = 0xE650;
  seCLG_Start( seSRC(T16_InitStruct.Clk) );
  seT16_Init( T16x, &T16_InitStruct );
  seT16_EnableInt( T16x );
  
  NVIC_EnableIRQ( BOARD_T16_CH_IRQn );
  
  
  if ( fStatus == seSTATUS_OK ) {
    
    uint32_t TimeoutMS = 5000;
    t16IntCount = 0;
    seT16_Start(  T16x );
    seSysSleepMS( TimeoutMS );
    seT16_Stop(  T16x );
    
    printf( "IRQ interrupted: %u times during %u sec\n", t16IntCount, TimeoutMS/1000 );
  }
  
  printf( "Exit\n" );
        
  return fStatus;
} 

/**
  * @brief  T16 Interrupt function
  * @retval None
  */
void BOARD_T16_CH_IRQHandler( void ) {
  
  t16IntCount++;
  seT16_ClearIntFlag( BOARD_T16_CH );
}

