/**
  ******************************************************************************
  * @file    main.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file is T16B example software.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

/** @defgroup T16B
  * @{
  * @brief T16B module is a 16-bit PWM timer with comparator/capture functions.
  * 
  * \section intro_sec Introduction
  * This example exercises the various comparator/capture functions of the T16B.
  * 
  * \section operation_sec Operations
  * 
  * 1. Set the 16bit PWM timer as follows and start the T16-bit PWM timer.
  *      - Mode: Repeat up-count mode
  *      - Maximum counter value: 0x5000
  *      - Comparator/capture circuit 0: Comparator mode (compare buffer: 0x2000)
  *      - Comparator/capture circuit 1: Number of capture interrupts in capture mode (trigger signal: LOW)
  *
  * 2. Start the 16-bit PWM timer and put CPU in a Halt state.
  * 3. When an interrupt occurs in the 16-bit PWM timer, release CPU from the Halt state.
  * 4. Each time an interrupt occurs in the 16-bit PWM timer, get the number of interrupts below:
  *      - Number of compare interrupts
  *      - Number of capture interrupts
  *      - Number of maximum-count interrupts
  *      - Number of count-zero interrupts
  *
  * 5. When a compare interrupt occurs, set the trigger signal for capture to HIGH.
  * 6. When a maximum-count interrupt occurs, set the trigger signal for capture to LOW.
  * 
  * \subsection output Example of output:
  *
  *  -CPU clock- seCLG_IOSC
  * Comparator Count            = 0x2000
  * Timer Count                 = 0x0010
  * Max Count                   = 0x5000
  * Interrupt Max Count         = 10
  * Interrupt Zero Count        = 11
  * Interrupt Comparator Count  = 10
  * Interrupt Capture Count     = 20
  *  - CaptureCountData[0] = 0x2007
  *  - CaptureCountData[1] = 0x0005
  *  - CaptureCountData[2] = 0x2007
  *  - CaptureCountData[3] = 0x0005
  *  - CaptureCountData[4] = 0x2007
  *  - CaptureCountData[5] = 0x0005
  *  - CaptureCountData[6] = 0x2007
  *  - CaptureCountData[7] = 0x0005
  *  - CaptureCountData[8] = 0x2007
  *  - CaptureCountData[9] = 0x0005
  *  - CaptureCountData[10] = 0x2007
  *  - CaptureCountData[11] = 0x0005
  *  - CaptureCountData[12] = 0x2007
  *  - CaptureCountData[13] = 0x0005
  *  - CaptureCountData[14] = 0x2007
  *  - CaptureCountData[15] = 0x0005
  *  - CaptureCountData[16] = 0x2007
  *  - CaptureCountData[17] = 0x0005
  *  - CaptureCountData[18] = 0x2007
  *  - CaptureCountData[19] = 0x0005
  * Exit
  *
  ******************************************************************************
  */
/**
  * @}
  */
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "board.h"
#include "port_config.h"

///< Compare interrupt counter data max.
#define T16B_CAP_DATA_MAX	( 20 )

///< Interrupt counters.
static uint16_t t16bIntComparator = 0;	///< Comparator interrupt count.
static uint16_t t16bIntCapture    = 0;	///< Compare interrupt count.
static uint16_t t16bIntMax        = 0;	///< Counter MAX interrupt count.
static uint16_t t16bIntZero       = 0;	///< Counter zero interrupt count.

///< Compare interrupt counter data.
static uint16_t t16bIntCaptureData[ T16B_CAP_DATA_MAX ];

///< Compare interrupt counter data index.
uint16_t t16bIntCaptureIndex;

///< Structures for peripheral library
static seT16B_InitTypeDef T16B_InitStruct;


///< Pipes from Interrupt service routine to the application.
void T16B_IRQWork( T16B_0_Type* T16Bx );
uint16_t getT16bIntComparatorCount( void );
uint16_t getT16bIntCaptureCountData( uint16_t index );
uint16_t getT16bIntCaptureCount( void );
uint16_t getT16bIntZeroCount( void );
uint16_t getT16bIntMaxCount( void );
void resetT16bIntCount( void );

int main( void ) {
  
  seStatus fStatus;
  
  ///< Displays the frequency of the system clock.
  ShowRunningClock();
  
  ///< Setting of PORT T16B
  sePPORT_Init(BOARD_T16B_TOUTCAP0, T16B_TOUTCAP0_INIT);
  sePPORT_Init(BOARD_T16B_TOUTCAP1, T16B_TOUTCAP1_INIT);
  
  ///< Clear parameters
  memset( &T16B_InitStruct, 0, sizeof( T16B_InitStruct ) );
  
  ///< Init T16B
  seT16B_InitStruct(&T16B_InitStruct);
  T16B_InitStruct.Clk = BOARD_T16B_CLOCK;
  seCLG_Start( seSRC(T16B_InitStruct.Clk) );
  fStatus = seT16B_Init( BOARD_T16B_CH, &T16B_InitStruct );
  
  ///< Enable T16B interrupt
  if ( fStatus == seSTATUS_OK ) {
    if ( BOARD_T16B_CH == T16B_0 ) {
      NVIC_EnableIRQ( T16B_0_IRQn );
    } else if ( BOARD_T16B_CH == (T16B_0_Type*)T16B_1 ) {
      NVIC_EnableIRQ( T16B_1_IRQn );
    } else {
      fStatus = seSTATUS_NG;
    }
  }
  
  if ( fStatus == seSTATUS_OK ) {
  
    uint16_t i, data, count = 0;
    
    ///< Set max count.
    seT16B_SetMaxCounter( BOARD_T16B_CH, 0x5000 );
    ///< Set comparator count.
    seT16B_SetCmpCapCnt( BOARD_T16B_CH, 0, 0x2000 );
    seT16B_ClearIntFlag( BOARD_T16B_CH, T16B_0_INTF_ALL_Msk );
    seT16B_EnableInt( BOARD_T16B_CH, T16B_0_INTE_CNTZEROIE_Msk |
                                     T16B_0_INTE_CNTMAXIE_Msk |
                                     T16B_0_INTE_CMPCAP0IE_Msk |
                                     T16B_0_INTE_CMPCAP1IE_Msk );
    
    ///< Reset counter(global variable).
    resetT16bIntCount();
    
    seT16B_Start( BOARD_T16B_CH );

    do {
      ///< Wait For Interrupt
      setHalt();
      ///< Get counter MAX interrupt count.
      count = getT16bIntMaxCount();
    } while ( count < 10 );
    
    ///< Stop T16B.
    seT16B_Stop( BOARD_T16B_CH );

    ///< Get interrupt counter.
    data = seT16B_GetCmpCapCnt( BOARD_T16B_CH, 0 );
    printf("Comparator Count            = 0x%04X\n", data);
    data = seT16B_GetTimerCount( BOARD_T16B_CH ); 
    printf("Timer Count                 = 0x%04X\n", data);
    data = seT16B_GetMaxCounter( BOARD_T16B_CH ); 
    printf("Max Count                   = 0x%04X\n", data);
    data = getT16bIntMaxCount();
    printf("Interrupt Max Count         = %d\n", data);
    data = getT16bIntZeroCount();
    printf("Interrupt Zero Count        = %d\n", data);
    data = getT16bIntComparatorCount();
    printf("Interrupt Comparator Count  = %d\n", data);
    data = getT16bIntCaptureCount();
    printf("Interrupt Capture Count     = %d\n", data);

    for ( i = 0; i < T16B_CAP_DATA_MAX; i++ ) {
      data = getT16bIntCaptureCountData(i);
      printf(" - CaptureCountData[%d] = 0x%04X\n", i, data);
    }
  }

  printf( "Exit\n" );
  
  return seSTATUS_OK;
} 

/**
  * @brief  Reset T16B interrupt counter.
  * @retval None
  */
void resetT16bIntCount( void ) {
  
  unsigned int i;

  ///< Reset counter
  t16bIntComparator	= 0;	///< Comparator interrupt count.
  t16bIntCapture	= 0;	///< Compare interrupt count.
  t16bIntMax		= 0;	///< Counter MAX interrupt count.
  t16bIntZero	= 0;	    ///< Counter zero interrupt count.

  ///< Reset compare interrupt counter data.
  t16bIntCaptureIndex	= 0;
  for ( i = 0; i < T16B_CAP_DATA_MAX; i++ ) {
    t16bIntCaptureData[i] = 0;
  }
}

/**
  * @brief  Get counter MAX interrupt count.
  * @retval Counter MAX interrupt count.
  */
uint16_t getT16bIntMaxCount( void ) {
  
  return ( t16bIntMax );
}

/**
  * @brief  Get counter zero interrupt count.
  * @retval Counter zero interrupt count.
  */
uint16_t getT16bIntZeroCount( void ) {
  
  return ( t16bIntZero );
}

/**
  * @brief  Get compare interrupt count.
  * @retval Compare interrupt count.
  */
uint16_t getT16bIntCaptureCount( void ) {
  
  return ( t16bIntCapture );
}

/**
  * @brief  Get count value when compare interrupt.
  * @param  index:Compare interrupt index.
  * @return Count value when compare interrupt.
  */
uint16_t getT16bIntCaptureCountData( uint16_t index ) {
  
  uint16_t data;
  
  if ( index >= T16B_CAP_DATA_MAX ) {
    data = 0;
  } else {
    data = t16bIntCaptureData[ index ];
  }

  return ( data );
}

/**
  * @brief  Get comparator interrupt count.
  * @return Comparator interrupt count.
  */
uint16_t getT16bIntComparatorCount( void ) {
  
  return ( t16bIntComparator );
}

/**
  * @brief  T16B(Ch.0) interrupt.
  * @return void
  */
void T16B_0_IRQHandler( void ) {
  
  T16B_IRQWork( T16B_0 );
}

/**
  * @brief  T16B(Ch.1) interrupt.
  * @return void
  */
void T16B_1_IRQHandler( void ) {
  
  T16B_IRQWork( (T16B_0_Type*)T16B_1 );
}

/**
  * @brief  T16B Ch.n interrupt processing.
  * @return void
  */
void T16B_IRQWork( T16B_0_Type* T16Bx ) {
  
  ///< Check counter zero interrupt flag.
  if ( seINTERRUPT_OCCURRED == seT16B_GetIntFlag( T16Bx, T16B_0_INTF_CNTZEROIF_Msk ) ) {
    ///< Clear flag.
    seT16B_ClearIntFlag( T16Bx, T16B_0_INTF_CNTZEROIF_Msk );
    ///< The increment does the counter zero interrupt count.
    t16bIntZero++;
  }

  ///< Check counter MAX interrupt flag.
  if ( seINTERRUPT_OCCURRED == seT16B_GetIntFlag( T16Bx, T16B_0_INTF_CNTMAXIF_Msk ) ) {
    ///< Clear flag.
    seT16B_ClearIntFlag( T16Bx, T16B_0_INTF_CNTMAXIF_Msk );
    ///< The increment does the counter zero interrupt count.
    t16bIntMax++;

    ///< Set trigger signal(LOW).
    seT16B_SetTriggerSignal( T16Bx,  1, seT16B_Input_1 );
  }

  ///< Check compare/capture 0 interrupt flag.
  if ( seINTERRUPT_OCCURRED == seT16B_GetIntFlag( T16Bx, T16B_0_INTF_CMPCAP0IF_Msk ) ) {
    ///< Clear flag.
    seT16B_ClearIntFlag( T16Bx, T16B_0_INTF_CMPCAP0IF_Msk );
    ///< The increment does the comparator interrupt count.
    t16bIntComparator++;

    ///< Set trigger signal(HIGH).
    seT16B_SetTriggerSignal( T16Bx, 1, seT16B_Input_0 );
  }

  ///< Check compare/capture 1 interrupt flag.
  if ( seINTERRUPT_OCCURRED == seT16B_GetIntFlag( T16Bx, T16B_0_INTF_CMPCAP1IF_Msk ) ) {
    ///< Clear flag.
    seT16B_ClearIntFlag( T16Bx, T16B_0_INTF_CMPCAP1IF_Msk );
    ///< The increment does the compare interrupt count.
    t16bIntCapture++;
    ///< Get timer counter data.
    if ( t16bIntCaptureIndex < T16B_CAP_DATA_MAX ) {
      t16bIntCaptureData[ t16bIntCaptureIndex ] = seT16B_GetCmpCapCnt( T16Bx, 1 );
      t16bIntCaptureIndex++;
    }
  }
}

