/**
  ******************************************************************************
  * @file    tsrvr_temperature.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file provides example routines to use the TSRVR sensor
  *          to read temperature.
  * Routines include:
  *
  *    initTempTSRVR() - initialize temperature reading using TSRVR
  *    readTempTSRVR() - read temperature using TSRVR
  *
  *    Calculation of temperature from ADC reading:
  *
  *         Tsen = [((ADD - ADCref) * 65536) / M] + Tref
  *
  *         where:
  *             Tsen     = Calculated temperature from ADC reading
  *             ADD      = ADC12 reading
  *             ADCref[] = Reference ADC12 value
  *             M[]      = constant based on the temperature sensor output voltage characteristic
  *             Tref[]= Reference temperature at which ADCref was taken
  *
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#include "tsrvr_temperature.h"
#include "board.h"

/**
  * Static functions declarations
  **/
static seADC12A_InitTypeDef TSRVR_ADC_Init;

static uint32_t adcsum;
static volatile int32_t Tsen;
static seTSRVR_VREFAMD VrefIdx;
static uint32_t VrefStartupDly;
static uint16_t adcValue;
uint8_t adcCount;


/** Temperature sensor estimated calibration data.
  * Needs calibration.
  */
const uint16_t Tref[]        = {    30,      30,     30,     30 };
const uint16_t ADCref[]      = {     0,    1491,   2236,   1787 };
const int32_t  M[]           = {     1, 3134370, 484159, 386447 };

/**
  * @brief  Initialize temperature reading using TSRVR.
  * @param  vrefsel:  VREF selection.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus initTempTSRVR ( seTSRVR_VREFAMD vrefsel, uint32_t startupdly ) {

  ///< Configure parameters
  TSRVR_ADC_Init.Clk            = BOARD_ADC12A_CLK;               ///< Set the clock source of ADC12.
  TSRVR_ADC_Init.VoltageRange   = seADC12A_VRANGE_18_55;          ///< Conversion voltage range
  TSRVR_ADC_Init.ports.StartAIN = BOARD_ADC12A_TEMPERATURE_ADIN;  ///< TSRVR is ADC channel
  TSRVR_ADC_Init.ports.EndAIN   = BOARD_ADC12A_TEMPERATURE_ADIN;  ///< TSRVR is ADC channel
  TSRVR_ADC_Init.StoreMode      = seADC12A_RIGHT_JUSTIFY;         ///< Lower bits are significant
  TSRVR_ADC_Init.ConversionMode = seADC12A_ONETIME;               ///< One-time conversion
  TSRVR_ADC_Init.TrigSrc        = seADC12A_SWTRIG;                ///< Software trigger
  TSRVR_ADC_Init.SampleClock    = seADC12A_SMPCLK_4CYC;           ///< 4 cycles of CLK_T16

  seCLG_Start(seSRC(TSRVR_ADC_Init.Clk)); ///< make sure that selected clock is on

  VrefIdx = vrefsel;
  VrefStartupDly = startupdly;
  // seTSRVR_Init( &TSRVR_ADC_Init, VrefIdx, VrefStartupDly );

  ///< Enable ADC interrupt
  seADC12A_ClearIntFlag( BOARD_ADC12A_CH, 0xFFFF );
  seADC12A_EnableInt( BOARD_ADC12A_CH, BOARD_ADC12A_TEMPERATURE_INTE ); 
  NVIC_SetPriority( ADC12A_0_IRQn, 3 );  
  NVIC_EnableIRQ( ADC12A_0_IRQn );

  return(seSTATUS_OK);
}

/**
  * @brief  Read/calculate temperature using TSRVR.
  * @param  numsamples:  number of samples to average
  * @retval temperature in degrees C
  */
int16_t readTempTSRVR ( uint8_t numsamples ) {

  //uint8_t i;

  ///< Initialize TSRVR and ADC12A
  seTSRVR_Init( &TSRVR_ADC_Init, VrefIdx, VrefStartupDly );
  
  ///< Take average ADC reading
  adcsum = 0;
  for ( uint8_t i = 0; i < numsamples; i++ )
  {
      adcCount = 0;

      ///< Start temperature measurement
      seTSRVR_StartTempMeas();

      while (adcCount < 1)
         setHalt();

      ///< Stop temperature measurement
      seTSRVR_StopTempMeas();

      adcsum = adcsum + adcValue;
  }
  
  ///< Stop conversion and disable ADC12A
  seADC12A_Disable(BOARD_ADC12A_CH);

  Tsen = (adcsum / numsamples);  ///< Average ADC reading

  Tsen = Tsen - ADCref[VrefIdx];
  Tsen = (Tsen << 16);

  if (Tsen < 0)  ///< Rounding
     Tsen = Tsen - (M[VrefIdx]>>1);
  else
     Tsen = Tsen + (M[VrefIdx]>>1);

  Tsen = (Tsen / M[VrefIdx]) + Tref[VrefIdx];

  return((int16_t) Tsen);
}


/**
  * @brief  ADC12A Interrupt function
  * @retval None
  */
void ADC12A_IRQHandler( void ) {

  if (BOARD_ADC12A_CH->INTF_b.OVIF) { ///< Overrun interrupt
     BOARD_ADC12A_CH->INTF = 0xFFFF;  ///< Clear all interrupts

     ///< Stop and restart ADC
     seADC12A_Stop(BOARD_ADC12A_CH);
     seADC12A_Start(BOARD_ADC12A_CH);

  } else if (BOARD_ADC12A_CH->INTF & seADC12A_INTF_TEMPERATURE) {  ///< Temperature interrupt
     BOARD_ADC12A_CH->INTF = seADC12A_INTF_TEMPERATURE;  ///< Clear interrupt

     adcCount++;
     adcValue = seADC12A_ReadADC(BOARD_ADC12A_CH);
  }
}

