/**
  ******************************************************************************
  * @file    main.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file is UART3 example software.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

/** @defgroup UART3
  * @{
  * @brief The UART3 module provides an asynchrounous serial interface.
  * 
  * \section intro_sec Introduction
  * This example shows how to program the UART3 to send and receive characters when connected
  * to a PC through a USB-to-UART adapter.
  * 
  * \section hardware_setup Hardware Setup
  *
  * Run terminal program.
  * 1. Connect evaluation board to PC via USB Adapter for UART as it is explained
  *    the Sample Software Manual in Section 2.2.1 and shown on the Figure 2.2.1.3
  *    and on Figure 2.2.1.4.
  * 2. Run a terminal program.
  * 3. Configure terminal program for baud rate 115200, 8 data bits, 1 stop bit, no parity.
  * 
  * \section operation_sec Operations
  * 
  * - Start UART3 Example
  * - ASCII characters are transmitted from UART3 an displayed in the terminal program window.
  * - Then the Example program waits for the input of 8 characters.
  * - When an 8 character input is received, the characters are then sent back to the terminal program window.
  *
  * \subsection output Example of output:
  *
  *  -CPU clock- seCLG_IOSC
  * abcefghiabcefghi ... waiting for console input...
  * 1234567812345678
  * Exit
  * 
  ******************************************************************************
  */
/**
  * @}
  */
#include <string.h>
#include <stdio.h>
#include "board.h"  
#include "port_config.h"

///<  Structure for peripheral library
static seUART_InitTypeDef InitStruct;

static uint8_t buf[8] = { 0x61, 0x62, 0x63, 0x65, 0x66, 0x67, 0x68, 0x69 };
static uint16_t local_index = 0;

int main( void )  {

  ///< Displays the frequency of the system clock.
  ShowRunningClock();

  seUART_Disable(BOARD_UART_CH);

  seSysSleepMS(10);  // let printf characters flush out before re-initializing UART
  
  ///< Clear parameters
  local_index = 0;
  memset( &InitStruct, 0, sizeof( seUART_InitTypeDef ) );
  
  ///< Setting of PORT TX,RX
  sePPORT_Init(BOARD_UART_RX_PORT, UART_RX_INIT);
  sePPORT_Init(BOARD_UART_TX_PORT, UART_TX_INIT);

  ///< Configure parameters
  InitStruct.Clk             = BOARD_UART_CLK;
  InitStruct.Mode.reg_b.puen = seUART_MOD_PUEN_ENABLE;
  InitStruct.Mode.reg_b.chln = seUART_MOD_CHLN_8BIT;
  InitStruct.Mode.reg_b.stpb = seUART_MOD_STPB_1BIT;
  InitStruct.Mode.reg_b.pren = seUART_MOD_PREN_NO_PARITY;
  InitStruct.Mode.reg_b.prmd = seUART_MOD_PRMD_EVEN;
  InitStruct.Mode.reg_b.irmd = seUART_MOD_IRMD_NO_IRDA;
  
  ///< Init UART
  seCLG_Start( seSRC(InitStruct.Clk) );
  seUART_Init( BOARD_UART_CH, &InitStruct );
  seUART_SetBaudRate( BOARD_UART_CH, seUART_BAUD_RATE_115200 );
  seUART_Enable( BOARD_UART_CH );

  ///< Enable UART interrupt
  seUART_EnableInt( BOARD_UART_CH, UART3_0_INTE_RB1FIE_Msk | UART3_0_INTE_RB2FIE_Msk );
  NVIC_SetPriority(BOARD_UART_CH_IRQn, 3 );
  NVIC_EnableIRQ(BOARD_UART_CH_IRQn );
  seUART_Send( BOARD_UART_CH, buf, 8 );
 
  printf( "%8s ... waiting for console input...\n", buf );
  printf( " ... waiting for console input...\n" );
  
  while ( local_index != 8 ) {
    ;
  }

  seUART_Send( BOARD_UART_CH, buf, 8 );
  //printf( "%8s\n", buf );
  local_index = 0;
  memset( buf, 0, 8 );
  
  printf( "\nExit\n" );
  
  return seSTATUS_OK;
} 

/**
  * @brief  UART Interrupt function
  * @retval None
  */
void BOARD_UART_CH_IRQHandler( void ) {
  
  if ( local_index < 8 ) {
    if ( seUART_GetIntFlag( BOARD_UART_CH, UART3_0_INTE_RB2FIE_Msk ) ) {
      ///<  Read received data.
      buf[local_index++] = BOARD_UART_CH->RXD;
    }
    buf[local_index++] = BOARD_UART_CH->RXD;
  }
  if ( local_index >= 8 ) {
    seUART_DisableInt( BOARD_UART_CH, UART3_0_INTE_RB1FIE_Msk | UART3_0_INTE_RB2FIE_Msk );
  }
}

