/**
  ******************************************************************************
  * @file    main.c
  * @author  Epson
  * @version V2.0
  * @date    01-JUNE-2020
  * @brief   This demo(test) demonstrates one of the USB HID use cases.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */
/** @defgroup USB
 * @{
 * @brief USB module is a USB target device controller that supports FS mode based on the USB 2.0 standard.
 *        The Example code for USB module is a Human Interface Device (HID) device.  
 *        The ARM mbed USB stack is used for the HID example software.
 *
 * HID devices must meet a few general requirements that are imposed to keep the HID interface standardized
 * and efficient. All HID devices must have a control endpoint (Endpoint 0), and an interrupt IN endpoint. 
 * Many devices also use an interrupt OUT endpoint. In most cases, HID devices are not allowed to have more 
 * than one OUT and one IN endpoint. All data transferred must be formatted as reports whose structure is defined 
 * in the report descriptor. HID devices must respond to standard HID requests in addition to all standard USB requests.
 *
 * \section hardware_setup Hardware Setup
 *
 * Connect evaluation board to PC by USB cable.
 * 
 * \section operation_sec Operations
 * 
 * 1. On the PC side, start the Tools\HidClient.exe utility. 
 * 2. In the utilities window select EPSON CMSIS-HID
 * 3. Click on outputs to observe bits on the SVT board display.
 * 4. Push SW100, SW101 buttons on the board to observe inputs in the HidClient.exe utility window.
 * 5. Verify Wakeup by PC. Set PC to sleep. Wake up PC. Observe HidClient.exe continues operation.
 * 6. Verify Remote Wakeup. Enable Power management in Windows Device Manager. Set PC to sleep.
 * 7. Wake up PC by pressing SW102. 
 * 8. Check snooze. Do sleep mode first, then trigger SNOOZE by pressing SW103.
 *
 * \subsection output Example of output:
 *
 * - Click LED0-LED7 outputs and observe bits set on the EVT31W73 board display.
 * - Push SW100, SW101 buttons on the board to observe the "PRESSED" status in the utility's window.
 * - Verify Remote Wakeup. Enable "Allow this device to wake up the computer" in the Power management 
 *   tab of Device Manager. Refer to Windows documentation for details of Power management. Set PC to sleep. 
 *   Wake up PC by pressing SW102. 
 * Note: The Wakeup functions have only been tested under Windows 7.
 * 
 *************************************************************************** 
 */
/**
  * @}
  */ 

#include <string.h>
#include <stdio.h>

#include "RTL.h"
#include "rl_usb.h"

#include <S1C31.h>
#include "board.h"
#include "se_usb.h"
#include "se_common.h"
#include "se_lcd32c.h"
#include "se_clg.h"
#include "port_config.h"

#include "console_font_6x8.h"
#include "selcd_font.h"
#include "selcd_draw.h"

/**
  * @brief HID Report Descriptor (user copy).
  */
const U8 USBD_HID_ReportDescriptor[] = {
  HID_UsagePageVendor( 0x00                      ),
  HID_Usage          ( 0x01                      ),
  HID_Collection     ( HID_Application           ),
    HID_LogicalMin   ( 0                         ), /* value range: 0 - 0xFF */
    HID_LogicalMaxS  ( 0xFF                      ),
    HID_ReportSize   ( 8                         ), /* 8 bits */
    HID_ReportCount  ( 64  ),
    HID_Usage        ( 0x01                      ),
    HID_Input        ( HID_Data | HID_Variable | HID_Absolute ),
    HID_ReportCount  ( 64 ),
    HID_Usage        ( 0x01                      ),
    HID_Output       ( HID_Data | HID_Variable | HID_Absolute ),
    HID_ReportCount  ( 1 ),
    HID_Usage        ( 0x01                      ),
    HID_Feature      ( HID_Data | HID_Variable | HID_Absolute ),
  HID_EndCollection,
};
/**
  * @brief User data.
  */
#define USBD_HID_EP_INTIN           1
#define USBD_HID_EP_INTOUT          2
uint8_t feature;
uint8_t set;
uint8_t get;

/**
  * @brief LCD display variables
  */
seFONTSET font6x8  = {6, 8, console_font_6x8};
seLCDOBJ dispobj   = {(uint8_t *) seLCD32C_SEGRAM_TOP_ADDR_AREA0, 72, 32 };

/**
  * @brief Various demo functions. 
  */
static void DemoInit( void );
static void DemoDisplay( void );
static void DemoPushButton( void );
static void InitPushButtons( void );
static uint8_t GetPushButtons( void );
static char *ByteToBinary( uint8_t x, char* b );

int main( void ) {
  
  usbd_init();
  while (!usbd_configured()) {
  }
  DemoInit();
  
  do {
    DemoDisplay();
    DemoPushButton();
    seSysSleepMS( 100 );

  } while (1);
  
  return seSTATUS_OK;
} 

/**
  * @brief The function usbd_hid_init initializes the USB Device HID class support. 
  * This function is called automatically and needs no invocation in the code.
  *
  * The function usbd_hid_init is part of the USB Device Function Driver layer of 
  * the RL-USB Device Software Stack.
  **/
void  usbd_hid_init(void) {
  
  get = 0;
}

/**
  * @brief The function usbd_hid_get_report prepares data that 
  * will be returned to the USB Host. The function is called on any 
  * of following events; after previous report was sent, 
  * if idle period has expired or if 
  * report was requested through control endpoint.  
  * 
  * @param Report type
  * @param Report ID
  * @param Pointer to the buffer to report data
  * @param Request type
  * @retval Loaded report size in bytes
  **/
int usbd_hid_get_report(U8  rtype, U8 rid, U8 *buf, U8  req) { 
  
    switch (rtype) {
    case HID_REPORT_INPUT:
      switch (rid) {
        case 0:
          switch (req) {
            case USBD_HID_REQ_EP_CTRL:
            case USBD_HID_REQ_PERIOD_UPDATE:
              buf[0] = 100;
              return (1);
              break;
            case USBD_HID_REQ_EP_INT:
              buf[0] = get;
              return (64);
              break;
          }
          break;
      }
      break;
    case HID_REPORT_FEATURE:
      buf[0] = feature;
      return (1);
      break;
  }
  
  return (0);
}

/** 
  * @brief The function usbd_hid_set_report handles data that was received from the USB Host. 
  * The function is called when report was received.
  * @param Report type
  * @param Report ID
  * @param Pointer to the buffer to report data
  * @param Request type
  **/
void  usbd_hid_set_report   (U8  rtype, U8 rid, U8 *buf, int len, U8 req) {
  
  switch (rtype) {
    case HID_REPORT_INPUT:
      break;
    case HID_REPORT_OUTPUT:
      set = *buf;
      break;
    case HID_REPORT_FEATURE:
      feature = buf[0];
      break;
  }
}

/**
  * @brief The function prepares the hardware to display "set" value 
* and obtain the "get" value. 
  */
static void DemoInit( void ) {
  
  seCLG_Start( seCLG_OSC1 );
  seCLG_SetOperInSlp( seCLG_OSC1, seDISABLE );
  seLCD32C_InitTypeDef LCD32C_InitStruct;

  ///< Clear parameters
  memset( &LCD32C_InitStruct, 0, sizeof( LCD32C_InitStruct ) );

  ///< Setting of PORT COMx-SEGx
  sePPORT_Init(sePPORT_P50, LCD32C_SEG95_INIT);             ///< Assign port to LCD32C COM16/SEG95
  sePPORT_Init(sePPORT_P51, LCD32C_SEG94_INIT);             ///< Assign port to LCD32C COM17/SEG94
  sePPORT_Init(sePPORT_P52, LCD32C_SEG93_INIT);             ///< Assign port to LCD32C COM18/SEG93
  sePPORT_Init(sePPORT_P53, LCD32C_SEG92_INIT);             ///< Assign port to LCD32C COM19/SEG92
  sePPORT_Init(sePPORT_P54, LCD32C_SEG91_INIT);             ///< Assign port to LCD32C COM20/SEG91
  sePPORT_Init(sePPORT_P55, LCD32C_SEG90_INIT);             ///< Assign port to LCD32C COM21/SEG90
  sePPORT_Init(sePPORT_P56, LCD32C_SEG89_INIT);             ///< Assign port to LCD32C COM22/SEG89
  sePPORT_Init(sePPORT_P57, LCD32C_SEG88_INIT);             ///< Assign port to LCD32C COM23/SEG88

  sePPORT_Init(sePPORT_P60, LCD32C_SEG87_INIT);             ///< Assign port to LCD32C COM24/SEG87
  sePPORT_Init(sePPORT_P61, LCD32C_SEG86_INIT);             ///< Assign port to LCD32C COM25/SEG86
  sePPORT_Init(sePPORT_P62, LCD32C_SEG85_INIT);             ///< Assign port to LCD32C COM26/SEG85
  sePPORT_Init(sePPORT_P63, LCD32C_SEG84_INIT);             ///< Assign port to LCD32C COM27/SEG84
  sePPORT_Init(sePPORT_P64, LCD32C_SEG83_INIT);             ///< Assign port to LCD32C COM28/SEG83
  sePPORT_Init(sePPORT_P65, LCD32C_SEG82_INIT);             ///< Assign port to LCD32C COM29/SEG82
  sePPORT_Init(sePPORT_P66, LCD32C_SEG81_INIT);             ///< Assign port to LCD32C COM30/SEG81
  sePPORT_Init(sePPORT_P67, LCD32C_SEG80_INIT);             ///< Assign port to LCD32C COM31/SEG80

  sePPORT_Init(sePPORT_P70, LCD32C_SEG79_INIT);             ///< Assign port to LCD32C SEG79
  sePPORT_Init(sePPORT_P71, LCD32C_SEG78_INIT);             ///< Assign port to LCD32C SEG78
  sePPORT_Init(sePPORT_P72, LCD32C_SEG77_INIT);             ///< Assign port to LCD32C SEG77
  sePPORT_Init(sePPORT_P73, LCD32C_SEG76_INIT);             ///< Assign port to LCD32C SEG76
  sePPORT_Init(sePPORT_P74, LCD32C_SEG75_INIT);             ///< Assign port to LCD32C SEG75
  sePPORT_Init(sePPORT_P75, LCD32C_SEG74_INIT);             ///< Assign port to LCD32C SEG74
  sePPORT_Init(sePPORT_P76, LCD32C_SEG73_INIT);             ///< Assign port to LCD32C SEG73
  sePPORT_Init(sePPORT_P77, LCD32C_SEG72_INIT);             ///< Assign port to LCD32C SEG72

  sePPORT_Init(sePPORT_P80, LCD32C_SEG71_INIT);             ///< Assign port to LCD32C SEG71
  sePPORT_Init(sePPORT_P81, LCD32C_SEG70_INIT);             ///< Assign port to LCD32C SEG70
  sePPORT_Init(sePPORT_P82, LCD32C_SEG69_INIT);             ///< Assign port to LCD32C SEG69
  sePPORT_Init(sePPORT_P83, LCD32C_SEG68_INIT);             ///< Assign port to LCD32C SEG68
  sePPORT_Init(sePPORT_P84, LCD32C_SEG67_INIT);             ///< Assign port to LCD32C SEG67
  sePPORT_Init(sePPORT_P85, LCD32C_SEG66_INIT);             ///< Assign port to LCD32C SEG66
  sePPORT_Init(sePPORT_P86, LCD32C_SEG65_INIT);             ///< Assign port to LCD32C SEG65
  sePPORT_Init(sePPORT_P87, LCD32C_SEG64_INIT);             ///< Assign port to LCD32C SEG64

  ///< Configure parameters
  LCD32C_InitStruct.Clk = seLCD32C_OSC1_CLKDIV_1; 

  LCD32C_InitStruct.CtlLcdIoDischargeEn = seENABLE;
  LCD32C_InitStruct.DspSelComPinDir = 1;
  LCD32C_InitStruct.DspSelectArea = seLCD32C_DISPLAY_AREA0;
  LCD32C_InitStruct.DspSelSegPinDir = 1;
  LCD32C_InitStruct.PwrBoosterEn = seENABLE;
  LCD32C_InitStruct.PwrHeavyLoadProtectionModeEn = seENABLE;
  LCD32C_InitStruct.PwrVoltageRegulatorEn = seENABLE;
  LCD32C_InitStruct.Tim1Duty = seLCD32C_DUTY_1_32;
  LCD32C_InitStruct.Tim1FrameCount = 1;
  LCD32C_InitStruct.Tim2BoostClk = seLCD32C_FLCLK_DIV_4;
  LCD32C_InitStruct.Tim2Nline = 0;

  seLCD32C_Init( &LCD32C_InitStruct );
  seLCD32C_SetPanelContrast( seLCD32C_CONTRAST_LEVEL15 );
  seLCD32C_SetDisplayMemory( seLCD32C_DISPLAY_AREA0, 0 );
  seLCD32C_SetDisplayArea( seLCD32C_DISPLAY_AREA0 );
  seLCD32C_SetDisplayInvState( seLCD32C_DISP_NORMAL );

  seLCD32C_SetDisplayState( seLCD32C_DISP_STATE_NORMAL );

  InitPushButtons( );
  
  // Start INput report
  if ( USB->EPACFG_b.EPNUM == USBD_HID_EP_INTIN ) {
    USB->EPACTL &= ~USB_EPACTL_FNAK_Msk;  
  }
  else  if ( USB->EPBCFG_b.EPNUM == USBD_HID_EP_INTIN ) {
    USB->EPBCTL &= ~USB_EPBCTL_FNAK_Msk;  
  }
  else if ( USB->EPCCFG_b.EPNUM == USBD_HID_EP_INTIN ){
    USB->EPCCTL &= ~USB_EPCCTL_FNAK_Msk;  
  }
}


static char *ByteToBinary( uint8_t x, char* b ) {
  
  b[0] = 0;
  int z;
  for (z = 128; z > 0; z >>= 1) {
    strcat( b, ((x & z) == z) ? "1" : "0");
  }
  
  return b;
}

static void DemoDisplay( void ) {
  
  char strBinary[9] = {0, };
  ByteToBinary( set, strBinary );
  seLcdPutStr( &dispobj, &font6x8, 0, 19, strBinary );
}

static void DemoPushButton( void ) {
  
  uint8_t btn = GetPushButtons();

  if ( btn != 0 && btn != 3 && btn != 4 && btn != get ) {
    get = btn;
  }

  if (  btn == 4 ) {
    USBD_WakeUp();
  }
  if (  btn == 8 ) {
  
    seUSB_Snooze( seENABLE ); 
  } 

}

static void InitPushButtons( void ) {

  sePPORT_InitAsInput ( sePPORT_P11 );
  sePPORT_ClearIntFlag( sePPORT_P11 );
  sePPORT_InitAsInput ( sePPORT_P12 );
  sePPORT_ClearIntFlag( sePPORT_P12 );
  sePPORT_InitAsInput ( sePPORT_P13 );
  sePPORT_ClearIntFlag( sePPORT_P13 );
  sePPORT_InitAsInput ( sePPORT_P14 );
  sePPORT_ClearIntFlag( sePPORT_P14 );  
}

static uint8_t GetPushButtons( void ) {
  
  uint8_t btn = 0;
    
  btn |= PPORT->P1INTF_b.P1IF1 << 0;    //< SW100  left
  btn |= PPORT->P1INTF_b.P1IF2 << 1;    //< SW101  center
  btn |= PPORT->P1INTF_b.P1IF3 << 2;    //< SW102  right
  btn |= PPORT->P1INTF_b.P1IF4 << 3;    //< SW103 bottom

  //> clear interrupts if any
  PPORT->P1INTF = 0x1E & ( btn << 1 );

  return btn;
}

/**
  * @brief When RTX is not used following functions are called in Interrupt context.
  * Uncomment printfs only for debugging. Alternatively the event functions can set 
  * the global event flags.
  */

void USBD_Reset_Event       (void) { /* printf( "Reset\n");  */ }
void USBD_Suspend_Event     (void) { /* printf( "Suspend\n"); */}
void USBD_Resume_Event      (void) { /* printf( "Resume\n" ); */}
