/**
  ******************************************************************************
  * @file    main.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file is WDT2 example software.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

/** @defgroup WDT2
  * @{
  * @brief WDT2 module restarts the system if a problem occurs.
  *
  * \section intro_sec Introduction
  * This example shows how to program the WDT2 to generate an NMI interrupt and a reset.
  * 
  * \section operation_sec Operations:
  * 
  * 1. Initializes WDT2.
  * 2. OSC1 is set as WTD2's clock source. OSC1 is started and configured to operate in sleep mode.
  * 3. Shows NMI interrupts due to comparator match.
  * 4. Second scenario shows Reset after NMI mode when WDT2 is reset.
  * 5. Third scenario shows Reset after NMI mode when WDT2 is not reset.
  *
  * \subsection output Example of output:
  *
  *  -CPU clock- seCLG_IOSC
  * WDT Clock source = (1)
  * WDT Clock divider = (0)
  * WDT CLK = 250 Hz. CMP count = 1023
  * WDT 4 secs cycle is set
  * NMI interrupts due to comparator match in NMI Mode.
  * WDT NMI interrupts = 1
  * 
  * Watchdog timer was reset in Reset and NMI Mode.
  * Chip was alive after NMI
  * WDT Clock source = (1)
  * WDT Clock divider = (0)
  * WDT CLK = 250 Hz. CMP count = 250
  * WDT 1 secs cycle is set
  * WDT NMI interrupts = 5
  * 
  * Watchdog timer was not reset in Reset and NMI Mode.
  * Chip was reset on next comparator match.
  * WDT Clock source = (1)
  * WDT Clock divider = (0)
  * WDT CLK = 250 Hz. CMP count = 250
  * WDT 1 secs cycle is set
  *
  ******************************************************************************
  */
/** 
  * @}
  */
#include <string.h>
#include <stdio.h>
#include "board.h"

///< Structures to support the peripheral library interface
static seWDT_InitTypeDef WDT_InitStruct;


///< A variable to control NMI interrupt handler.
///< When it is set to 1 NMI handler resets wDT.
static uint16_t wdt_reset;

///< Shows cunnent number of NMI interrupts
static uint32_t WdtIntCount;

int main( void ) {

  seStatus fStatus = seSTATUS_OK; 
  uint32_t Timeout = 6000UL;
  
  ///< Displays the frequency of the system clock.
  ShowRunningClock();
  
  ///< Init WDT2
  seWDT_InitStruct( &WDT_InitStruct );
  WDT_InitStruct.Clk = BOARD_WDT_CLOCK;
  seCLG_Start( seSRC(WDT_InitStruct.Clk) );
  seWDT_Init( &WDT_InitStruct );
  seCLG_SetOperInSlp( seSRC(WDT_InitStruct.Clk), seDISABLE );
  
  printf( "WDT Clock source = (%d)\n", WDT2->CLK_b.CLKSRC );
  printf( "WDT Clock divider = (%d)\n", WDT2->CLK_b.CLKDIV );
  printf( "WDT CLK = %u Hz. CMP count = %d\nWDT %u secs cycle is set\n",
                seAUX_GetClkSpeedForWDT(), seWDT_GetCMP(), seWDT_Get_tWDTSecs() );

  
  ///< First scenario shows NMI interrupts due to comparator match
  wdt_reset = 1;
  WdtIntCount = 0;   ///< Reset NMI counter
  seWDT_Start();     ///< Start WDT
  do{
    setHalt();
  } while ( WdtIntCount == 0 );
  seWDT_Stop();                        ///< Stop WDT
  printf( "NMI interrupts due to comparator match in NMI Mode.\n" );
  printf( "WDT NMI interrupts = %u\n\n", WdtIntCount );
  
  ///< Second scenario shows "Reset and NMI" mode when WDT is updated (reset)
  WdtIntCount = 0; ///< Zero NMI counter
  wdt_reset = 1;
  seWDT_ChipReset( seWDT_ResetAndNMIMode );
 
  do{
    setHalt();
  } while ( WdtIntCount <= 4 );
  
  seWDT_Stop();
  
  printf( "Watchdog timer was reset in Reset and NMI Mode.\nChip was alive after NMI\n" );
  printf( "WDT Clock source = (%d)\n", WDT2->CLK_b.CLKSRC );
  printf( "WDT Clock divider = (%d)\n", WDT2->CLK_b.CLKDIV );
  printf( "WDT CLK = %u Hz. CMP count = %d\nWDT %u secs cycle is set\n",
                seAUX_GetClkSpeedForWDT(), seWDT_GetCMP(), seWDT_Get_tWDTSecs() );
  printf( "WDT NMI interrupts = %u\n\n", WdtIntCount );  
  
  ///< Third scenario shows "Reset and NMI" mode when WDT is not updated(reset)
  printf( "Watchdog timer was not reset in Reset and NMI Mode.\nChip was reset on next comparator match.\n" );
  printf( "WDT Clock source = (%d)\n", WDT2->CLK_b.CLKSRC );
  printf( "WDT Clock divider = (%d)\n", WDT2->CLK_b.CLKDIV );
  printf( "WDT CLK = %u Hz. CMP count = %d\nWDT %u secs cycle is set\n", 
                seAUX_GetClkSpeedForWDT(), seWDT_GetCMP(), seWDT_Get_tWDTSecs() );

  
  wdt_reset = 0; ///< For int handler. When set to 0 NMI handler do not reset WDT.
  seWDT_ChipReset( seWDT_ResetAndNMIMode );
  
  Timeout = 1000000UL ;         ///< Wait in the loop untill chip is reset by WDT.
  while ( Timeout ){
    putchar( '!' );
    seSysSleepMS( 100 );
    Timeout -= 100;
  }
  printf( "Exit\n" );
  
  return fStatus;
} 

/**
  * @brief  NMI Interrupt function
  * @retval None
  */
void NMI_Handler( void ){
  
  if ( wdt_reset ) {
    if ( WDT2->CTL_b.STATNMI == 1 ) {
      WdtIntCount++;
      seWDT_ResetCounter();
    }
  }
}

